Package: tidyexposomics
Title: Integrated Exposure-Omics Analysis Powered by Tidy Principles
Version: 1.0.0
Authors@R: 
    c(
    person(given = "Jason",
    family  = "Laird",
    email = "jasonlaird5353@gmail.com",
    role = c("aut", "cre"),
    comment = c(ORCID = "0009-0000-5994-2236")
    ),
    person(given = "Thomas",
    family  = "Hartung",
    email = "thartun1@jhu.edu",
    role = c("ctb"),
    comment = c(ORCID = "0000-0003-1359-7689")
    ),
    person(given = "Fenna",
    family  = "Sillé",
    email = "fsille1@jhu.edu",
    role = c("ctb"),
    comment = c(ORCID = "0000-0003-4305-2049")
    ),
    person(given = "Alexandra",
    family  = "Maertens",
    email = "amaerte1@jhu.edu",
    role = c("ctb"),
    comment = c(ORCID = "0000-0002-2077-2011")
    ),
    person("JHU Discovery Award", role = "fnd")
    )
Description: The tidyexposomics package is designed to facilitate the integration of exposure and omics data to identify exposure-omics associations. We structure our commands to fit into the tidyverse framework, where commands are designed to be simplified and intuitive. Here we provide functionality to perform quality control, sample and exposure association analysis, differential abundance analysis, multi-omics integration, and functional enrichment analysis.
License: MIT + file LICENSE
URL: https://bionomad.github.io/tidyexposomics/
BugReports: https://github.com/BioNomad/tidyexposomics/issues
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
biocViews: Software, Transcriptomics, GeneExpression, Epigenetics,
        Proteomics, DifferentialExpression, DifferentialMethylation,
        QualityControl, GraphAndNetwork, MultipleComparison,
        Regression, StatisticalMethod, Visualization, WorkflowStep
Imports: BiocFileCache, broom, cluster, dplyr, DT, factoextra, fenr,
        ggplot2 (>= 3.4.0), ggpubr, ggrepel, Hmisc, httr, igraph,
        jsonlite, limma, MASS, methods, mixOmics, naniar, purrr, readr,
        RGCCA, rlang, S4Vectors, scales, shiny, stats, stringr,
        SummarizedExperiment, tibble, tidybulk, tidyr, utils
Depends: R (>= 4.5.0), MultiAssayExperiment
Suggests: BiocStyle, circlize, curl, densityClust, DiagrammeR,
        dynamicTreeCut, edgeR, forcats, ggh4x, ggnewscale, ggraph,
        ggridges, ggsci, ggvenn, grid, gridExtra, impute, janitor,
        knitr, matrixStats, mice, mirt, missForest, MOFA2, nipalsMCIA,
        openxlsx, patchwork, reticulate, rmarkdown, testthat (>=
        3.0.0), tidygraph, tidyHeatmap, tidytext, tidyverse
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/tidyexposomics
git_branch: RELEASE_3_23
git_last_commit: b47c045
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:11:56 UTC; biocbuild
Author: Jason Laird [aut, cre] (ORCID: <https://orcid.org/0009-0000-5994-2236>),
  Thomas Hartung [ctb] (ORCID: <https://orcid.org/0000-0003-1359-7689>),
  Fenna Sillé [ctb] (ORCID: <https://orcid.org/0000-0003-4305-2049>),
  Alexandra Maertens [ctb] (ORCID:
    <https://orcid.org/0000-0002-2077-2011>),
  JHU Discovery Award [fnd]
Maintainer: Jason Laird <jasonlaird5353@gmail.com>

# Generated by roxygen2: do not edit by hand

export(build_ont_annot_app)
export(create_exposomicset)
export(download_dataset)
export(extract_omics_exposure_df)
export(extract_results)
export(extract_results_excel)
export(extract_top_factor_features)
export(filter_missing)
export(filter_non_normal)
export(filter_omics)
export(filter_sample_outliers)
export(load_annotation_data)
export(make_example_data)
export(pivot_exp)
export(pivot_feature)
export(pivot_sample)
export(plot_association)
export(plot_circos_correlation)
export(plot_correlation_summary)
export(plot_correlation_tile)
export(plot_enrichment)
export(plot_exposure_impact)
export(plot_exposure_omics_association)
export(plot_exposures)
export(plot_factor_summary)
export(plot_manhattan)
export(plot_missing)
export(plot_network)
export(plot_normality_summary)
export(plot_pca)
export(plot_sample_clusters)
export(plot_sensitivity_summary)
export(plot_top_factor_features)
export(plot_volcano)
export(run_association)
export(run_cluster_samples)
export(run_correlation)
export(run_create_network)
export(run_differential_abundance)
export(run_enrichment)
export(run_exposome_score)
export(run_exposure_impact)
export(run_exposure_omics_association)
export(run_factor_overlap)
export(run_impute_missing)
export(run_multiomics_integration)
export(run_normality_check)
export(run_pca)
export(run_pipeline_summary)
export(run_sensitivity_analysis)
export(run_summarize_exposures)
export(transform_exposure)
import(dplyr)
import(fenr)
import(ggplot2)
import(stringr)
import(tibble)
import(tidyr)
importFrom(BiocFileCache,BiocFileCache)
importFrom(BiocFileCache,bfcrpath)
importFrom(DT,DTOutput)
importFrom(DT,dataTableProxy)
importFrom(DT,datatable)
importFrom(DT,formatStyle)
importFrom(DT,renderDT)
importFrom(DT,replaceData)
importFrom(DT,styleEqual)
importFrom(Hmisc,rcorr)
importFrom(MASS,boxcox)
importFrom(MultiAssayExperiment,colData)
importFrom(MultiAssayExperiment,experiments)
importFrom(MultiAssayExperiment,metadata)
importFrom(RGCCA,rgcca)
importFrom(S4Vectors,DataFrame)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(broom,tidy)
importFrom(cluster,clusGap)
importFrom(cluster,daisy)
importFrom(cluster,diana)
importFrom(cluster,maxSE)
importFrom(dplyr,across)
importFrom(dplyr,add_count)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,select_if)
importFrom(dplyr,semi_join)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,sym)
importFrom(dplyr,top_n)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(factoextra,fviz_eig)
importFrom(factoextra,fviz_nbclust)
importFrom(factoextra,hcut)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggpubr,get_palette)
importFrom(ggpubr,rotate_x_text)
importFrom(ggpubr,theme_pubclean)
importFrom(ggpubr,theme_pubr)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(httr,GET)
importFrom(httr,status_code)
importFrom(httr,timeout)
importFrom(igraph,cluster_louvain)
importFrom(igraph,gorder)
importFrom(igraph,graph_from_data_frame)
importFrom(jsonlite,fromJSON)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(methods,as)
importFrom(mixOmics,block.splsda)
importFrom(naniar,impute_mean_all)
importFrom(naniar,impute_median_all)
importFrom(naniar,miss_var_summary)
importFrom(purrr,compact)
importFrom(purrr,discard)
importFrom(purrr,imap)
importFrom(purrr,iwalk)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,pluck)
importFrom(purrr,reduce)
importFrom(readr,parse_number)
importFrom(rlang,.data)
importFrom(rlang,`%||%`)
importFrom(rlang,set_names)
importFrom(rlang,sym)
importFrom(scales,alpha)
importFrom(shiny,HTML)
importFrom(shiny,actionButton)
importFrom(shiny,addResourcePath)
importFrom(shiny,column)
importFrom(shiny,div)
importFrom(shiny,downloadButton)
importFrom(shiny,fileInput)
importFrom(shiny,fluidPage)
importFrom(shiny,fluidRow)
importFrom(shiny,h4)
importFrom(shiny,hr)
importFrom(shiny,img)
importFrom(shiny,modalDialog)
importFrom(shiny,numericInput)
importFrom(shiny,observeEvent)
importFrom(shiny,reactiveValues)
importFrom(shiny,renderUI)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,selectizeInput)
importFrom(shiny,shinyApp)
importFrom(shiny,showModal)
importFrom(shiny,tags)
importFrom(shiny,titlePanel)
importFrom(shiny,uiOutput)
importFrom(shiny,updateSelectizeInput)
importFrom(stats,IQR)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,end)
importFrom(stats,family)
importFrom(stats,glm)
importFrom(stats,hclust)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,qchisq)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,start)
importFrom(stats,var)
importFrom(stringr,str_extract)
importFrom(stringr,str_remove)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_trim)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,enframe)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,separate_rows)
importFrom(tidyr,unnest)
importFrom(tools,md5sum)
importFrom(utils,URLencode)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,tail)
importMethodsFrom(tidybulk,identify_abundant)
importMethodsFrom(tidybulk,pivot_sample)
importMethodsFrom(tidybulk,pivot_transcript)
importMethodsFrom(tidybulk,test_differential_abundance)

<!-- badges: start -->
  [![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](LICENSE)
<!-- badges: end -->
  
# tidyexposomics <a href="#"><img src="./vignettes/logo.png" align="right" height="50" /></a>

<br>
<br>
<br>
<br>


- **Website:** https://bionomad.github.io/tidyexposomics/index.html

## Overview

The `tidyexposomics` package is designed to facilitate the integration of exposure and omics data to identify exposure-omics associations. Functions follow the tidyverse framework, where commands are designed to be simplified and intuitive. The tidyexposomics package provides functionality to perform quality control, sample and exposure association analysis, differential abundance analysis, multi-omics integration, and functional enrichment analysis.

<p align="center" width="100%">
    <img width="80%" src="vignettes/overview.png">
</p>


## Command Structure

To make the package more user-friendly, we have named our functions to be more intuitive. For example, we use the following naming conventions:

<p align="center" width="100%">
    <img width="80%" src="vignettes/command_str.png">
</p>

Results can be added to the `MultiAssayExperiment` object or returned directly with `action = 'get'`. We suggest adding results, given that pipeline steps are tracked and can be output to the R console, plotted as a workflow diagram, or exported to an excel worksheet.


## Installation

The `tidyexposomics` package depends on R (>= 4.5.0) and can be installed using the following code:

```R
# Install using Bioconductor
BiocManager::install("tidyexposomics")

# Install using devtools
devtools::install_github("bionomad/tidyexposomics")
```

## Getting Started

* [Getting Started](vignettes/tidyexposomics.Rmd)

#' Internal - load ontologies
#'
#' @keywords internal
#' @param use_demo Logical; if TRUE, load packaged demo objects \code{hpo}, \code{ecto}, \code{chebi}.
#' @param ... Optional named overrides: \code{hpo}, \code{ecto}, \code{chebi}.
#' @return A list with elements \code{hpo}, \code{ecto}, \code{chebi}.
#' @importFrom utils data modifyList
.load_ontologies <- function(use_demo = TRUE, ...) {
    dots <- list(...)
    if (use_demo && !all(c("hpo", "ecto", "chebi") %in% names(dots))) {
        annot <- tidyexposomics::load_annotation_data()
        dots <- modifyList(annot, dots)
    }
    dots[c("hpo", "ecto", "chebi")]
}


#' Internal - categorizes to ontology roots / depth
#'
#' @keywords internal
#' @param data Data frame with an ID column.
#' @param id_col Column name containing ontology term IDs.
#' @param ontologyDF Processed ontology data.frame (with list columns for relationships and \code{depth}).
#' @param root_level Either a character vector of explicit root IDs, a numeric depth, or (default) top-level roots.
#' @param assign_label If TRUE, return input with new columns; otherwise return assigned IDs.
#' @return Data frame with \code{root_id}, \code{root_label}, \code{category}, \code{category_source} (if \code{assign_label}).
#' @importFrom dplyr mutate select any_of
#' @importFrom stats setNames
.run_categorize_ontology <- function(data, id_col, ontologyDF, root_level = 0, assign_label = TRUE) {
    id_to_ancestors <- stats::setNames(ontologyDF$ancestors, ontologyDF$id)
    id_to_depth <- stats::setNames(ontologyDF$depth, ontologyDF$id)
    id_to_name <- stats::setNames(ontologyDF$name, ontologyDF$id)

    assign_to_root <- switch(class(root_level),
        "character" = {
            root_nodes <- root_level
            function(term_id) {
                anc <- c(term_id, id_to_ancestors[[term_id]])
                matched <- intersect(anc, root_nodes)
                if (length(matched)) matched[1] else NA_character_
            }
        },
        "numeric" = {
            function(term_id) {
                anc <- c(term_id, id_to_ancestors[[term_id]])
                depths <- id_to_depth[anc]
                idxs <- which(depths == root_level)
                if (length(idxs)) anc[idxs[1]] else NA_character_
            }
        },
        {
            root_nodes <- ontologyDF$id[lengths(ontologyDF$parents) == 0]
            function(term_id) {
                anc <- c(term_id, id_to_ancestors[[term_id]])
                matched <- intersect(anc, root_nodes)
                if (length(matched)) matched[1] else NA_character_
            }
        }
    )

    term_ids <- data[[id_col]]
    assigned_ids <- vapply(term_ids, assign_to_root, character(1))
    assigned_labels <- id_to_name[assigned_ids]

    if (assign_label) {
        dplyr::select(data, -dplyr::any_of(c("root_id", "root_label", "category", "category_source"))) |>
            dplyr::mutate(
                root_id         = assigned_ids,
                root_label      = assigned_labels,
                category        = ifelse(is.na(assigned_labels), "Unmapped", assigned_labels),
                category_source = ifelse(is.na(assigned_labels), "manual", "ontology")
            )
    } else {
        assigned_ids
    }
}

#' Internal - fetch OLS description for a term
#'
#' @keywords internal
#' @param ontology_id Term ID (e.g., \code{"HP:0004322"}).
#' @param ontology_prefix Prefix (e.g., \code{"HP"}).
#' @return A character scalar description or \code{NA_character_}.
#' @importFrom utils URLencode
#' @importFrom httr GET timeout status_code
#' @importFrom jsonlite fromJSON
.get_ols_description <- function(ontology_id, ontology_prefix) {
    iri <- paste0("http://purl.obolibrary.org/obo/", gsub(":", "_", ontology_id))
    double_encode <- function(url) {
        tmp <- utils::URLencode(url, reserved = TRUE)
        gsub("%", "%25", tmp, fixed = TRUE)
    }
    enc <- double_encode(iri)
    url <- paste0(
        "https://www.ebi.ac.uk/ols4/api/ontologies/",
        tolower(ontology_prefix), "/terms/", enc
    )

    if (!.can_use_network()) {
        return(NA_character_)
    }

    res <- try(httr::GET(url, httr::timeout(3)), silent = TRUE)
    if (inherits(res, "try-error") || httr::status_code(res) != 200) {
        return(NA_character_)
    }
    j <- jsonlite::fromJSON(rawToChar(res$content))
    desc <- j$description
    if (!is.null(desc) && length(desc) > 0 && nzchar(desc[[1]])) {
        desc[[1]]
    } else if (!is.null(j$annotation$description) && length(j$annotation$description) > 0) {
        j$annotation$description[[1]]
    } else {
        NA_character_
    }
}

#' Internal - should we allow network access now?
#'
#' Controlled by the R option \code{tidyexposomics.allow_network} and \code{interactive()}.
#'
#' @keywords internal
#' @return Logical scalar.
.can_use_network <- function() {
    getOption("tidyexposomics.allow_network", TRUE) && interactive()
}

test_that(".load_ontologies returns a list with hpo/ecto/chebi", {
    ont <- .load_ontologies(use_demo = TRUE)
    expect_type(ont, "list")
    expect_true(all(c("hpo", "ecto", "chebi") %in% names(ont)))
})


test_that(".run_categorize_ontology assigns a root_id", {
    toy <- data.frame(
        id = c("A", "B"),
        name = c("root", "child"),
        ancestors = I(list(character(0), "A")),
        parents = I(list(character(0), "A")),
        children = I(list("B", character(0))),
        depth = c(0, 1)
    )
    df <- data.frame(selected_ontology_id = "B")
    out <- .run_categorize_ontology(df, id_col = "selected_ontology_id", ontologyDF = toy)
    expect_equal(unname(out$root_id), "A")
})

test_that(".can_use_network respects option", {
    old <- getOption("tidyexposomics.allow_network")
    options(tidyexposomics.allow_network = FALSE)
    expect_false(.can_use_network())
    options(tidyexposomics.allow_network = old)
})


test_that("build_ont_annot_app returns shiny.appobj", {
    app <- build_ont_annot_app(use_demo = TRUE)
    expect_s3_class(app, "shiny.appobj")
})

#' Run Correlation Analysis
#'
#' Computes correlations between exposures and feature types including DEGs,
#'  omics, latent factors,
#' top factor features, or principal components (PCs). Optionally computes
#' feature-feature correlations
#' to support network analysis.
#'
#' @param exposomicset A `MultiAssayExperiment` object.
#' @param feature_type Type of features to correlate. One of `"degs"`,
#'  `"omics"`, `"factors"`, `"factor_features"`, `"exposures"`, or `"pcs"`.
#' @param exposure_cols Optional vector of exposure column names
#' (from `colData`) to use.
#' @param variable_map Optional mapping of features to include by assay
#'  for `omics` mode.
#' @param n_pcs Number of PCs to use when `feature_type = "pcs"`.
#' @param feature_cors Logical; if `TRUE`, compute correlations between
#' features rather than with exposures.
#' @param robust Logical; restrict DEGs to those passing sensitivity threshold.
#' @param score_col Column name in sensitivity analysis with feature
#' stability score.
#' @param score_thresh Threshold for filtering robust features.
#' @param correlation_method One of `"pearson"`, `"spearman"`, or `"kendall"`.
#' @param correlation_cutoff Minimum absolute correlation to retain.
#' @param cor_pval_column Column in output to filter by p-value
#' (default: `"p.value"`).
#' @param pval_cutoff Maximum p-value or FDR threshold to retain a correlation.
#' @param deg_pval_col Column with DEG adjusted p-values.
#' @param deg_logfc_col Column with DEG log fold-changes.
#' @param deg_pval_thresh P-value cutoff for DEGs.
#' @param deg_logfc_thresh Log fold-change cutoff for DEGs.
#' @param batch_size Number of features to process per batch (default: 1500).
#' @param action Whether to `"add"` results to metadata or `"get"`
#' as a data frame.
#'
#' @return If `action = "add"`, returns updated `MultiAssayExperiment`
#'  with results added to metadata.
#'         If `action = "get"`, returns a tidy `data.frame` of correlations.
#'
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run correlation analysis
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "exposures",
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' @export
run_correlation <- function(
  exposomicset,
  feature_type = c(
      "degs",
      "omics",
      "factors",
      "factor_features",
      "exposures",
      "pcs"
  ),
  exposure_cols = NULL,
  variable_map = NULL,
  n_pcs = NULL,
  feature_cors = FALSE,
  robust = FALSE,
  score_col = "stability_score",
  score_thresh = NULL,
  correlation_method = "spearman",
  correlation_cutoff = 0.3,
  cor_pval_column = "p.value",
  pval_cutoff = 0.05,
  deg_pval_col = "adj.P.Val",
  deg_logfc_col = "logFC",
  deg_pval_thresh = 0.05,
  deg_logfc_thresh = log2(1.5),
  batch_size = 1500,
  action = c("add", "get")
) {
    feature_type <- match.arg(feature_type)
    action <- match.arg(action)

    col_df <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame()
    # dplyr::select(-dplyr::starts_with("PC"))

    exposures <- col_df |>
        dplyr::select(where(is.numeric))

    if (!is.null(exposure_cols)) {
        exposures <- exposures[, intersect(
            colnames(exposures),
            exposure_cols
        ),
        drop = FALSE
        ]
    }
    if (ncol(exposures) == 0) stop("No numeric exposures found.")

    feature_matrix <- switch(feature_type,
        degs = .extract_deg_matrix(
            exposomicset,
            robust,
            score_col,
            score_thresh,
            deg_pval_col,
            deg_logfc_col,
            deg_pval_thresh,
            deg_logfc_thresh
        ),
        omics = .extract_omics_matrix(
            exposomicset,
            variable_map
        ),
        factors = .extract_factor_matrix(exposomicset),
        exposures = .extract_exposure_matrix(
            col_df,
            exposure_cols
        ),
        factor_features = .extract_factor_feature_matrix(exposomicset),
        pcs = .extract_pc_matrix(col_df, n_pcs = n_pcs)
    )

    exposures <- exposures |>
        tibble::rownames_to_column("id")

    if (feature_type == "exposures") {
        merged_data <- exposures
        exposure_vars <- setdiff(colnames(merged_data), "id")
        feature_vars <- exposure_vars
    } else {
        if (!"id" %in% colnames(feature_matrix)) {
            feature_matrix <- feature_matrix |>
                tibble::rownames_to_column("id")
        }
        merged_data <- dplyr::left_join(exposures, feature_matrix, by = "id") |>
            na.omit()
        exposure_vars <- setdiff(colnames(exposures), "id")
        feature_vars <- setdiff(colnames(feature_matrix), "id")
    }


    correlation_df <- .run_and_clean_correlation(
        exposomicset = exposomicset,
        merged_data = merged_data,
        feature_vars = feature_vars,
        exposure_vars = exposure_vars,
        feature_type = feature_type,
        feature_cors = feature_cors,
        correlation_method = correlation_method,
        correlation_cutoff = correlation_cutoff,
        cor_pval_column = cor_pval_column,
        pval_cutoff = pval_cutoff,
        batch_size = batch_size
    )

    if (action == "add") {
        if (feature_cors) {
            all_metadata <- MultiAssayExperiment::metadata(exposomicset)
            res_name <- paste0(feature_type, "_feature_cor")
            all_metadata$correlation[[res_name]] <- correlation_df
            MultiAssayExperiment::metadata(exposomicset) <- all_metadata

            step_record <- list(
                run_correlation = list(
                    timestamp = Sys.time(),
                    params = list(
                        feature_type = feature_type,
                        correlation_method = correlation_method,
                        correlation_cutoff = correlation_cutoff,
                        pval_cutoff = pval_cutoff
                    ),
                    notes = paste0(
                        "Correlated ",
                        feature_type,
                        " features with features."
                    )
                )
            )
            names(step_record) <- paste0("run_correlation_", feature_type)
        } else {
            all_metadata <- MultiAssayExperiment::metadata(exposomicset)
            all_metadata$correlation[[feature_type]] <- correlation_df
            MultiAssayExperiment::metadata(exposomicset) <- all_metadata

            step_record <- list(
                run_correlation = list(
                    timestamp = Sys.time(),
                    params = list(
                        feature_type = feature_type,
                        correlation_method = correlation_method,
                        correlation_cutoff = correlation_cutoff,
                        pval_cutoff = pval_cutoff
                    ),
                    notes = paste0(
                        "Correlated ",
                        feature_type,
                        " features with exposures."
                    )
                )
            )
            names(step_record) <- paste0("run_correlation_", feature_type)
        }


        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )
        return(exposomicset)
    } else {
        return(correlation_df)
    }
}

# --- Run exposure-feature correlation function ----------
#' Compute exposure-feature correlations in batches
#'
#' Internal helper that correlates a set of exposure variables against a set
#' of feature variables by processing features in batches. Uses
#' \code{Hmisc::rcorr()} to compute correlation coefficients and p-values, then
#' filters by absolute correlation magnitude and a p-value/FDR column.
#'
#' @param merged_data A data frame/matrix with one row per sample and columns
#'   for all \code{exposure_vars} and \code{feature_vars}. Must be numeric for
#'   selected columns. Rows with non-finite values across the selected columns
#'   are removed prior to correlation.
#' @param exposure_vars Character vector of exposure column names (subset of
#'   \code{colnames(merged_data)}).
#' @param feature_vars Character vector of feature column names (subset of
#'   \code{colnames(merged_data)}).
#' @param correlation_method Correlation method passed to \code{Hmisc::rcorr()},
#'   typically \code{"pearson"} or \code{"spearman"}.
#' @param correlation_cutoff Numeric; retain pairs with
#'   \eqn{|r| >} \code{correlation_cutoff}.
#' @param cor_pval_column Name of the p-value column to filter on after FDR
#'   computation; usually \code{"p.value"} or \code{"FDR"}.
#' @param pval_cutoff Numeric; maximum allowed value for \code{cor_pval_column}.
#' @param batch_size Integer; number of feature columns processed per batch.
#'
#' @return A \code{data.frame} with one row per retained exposure-feature pair
#' and columns: \code{var1} (exposure), \code{var2} (feature),
#' \code{correlation}, \code{p.value}, and \code{FDR}.
#' Returns an empty data frame when no pairs pass filters.
#'
#' @details
#' Correlations are computed within each batch, then results are row-bound.
#' The FDR is computed within each batch; if you require global FDR across all
#' pairs, compute \code{p.adjust} on the combined result after binding rows.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom Hmisc rcorr
#' @importFrom dplyr inner_join filter mutate bind_rows
#' @importFrom tibble rownames_to_column
#' @importFrom tidyr pivot_longer
#' @importFrom rlang sym
#' @importFrom stats p.adjust
.run_correlation_batches <- function(
  merged_data,
  exposure_vars,
  feature_vars,
  correlation_method,
  correlation_cutoff,
  cor_pval_column,
  pval_cutoff,
  batch_size
) {
    correlation_results <- list()
    batches <- split(feature_vars, ceiling(seq_along(feature_vars) / batch_size))

    for (i in seq_along(batches)) {
        features <- batches[[i]]
        cols_to_use <- intersect(c(exposure_vars, features), colnames(merged_data))
        mat <- as.matrix(merged_data[, cols_to_use, drop = FALSE])
        mat <- mat[apply(mat, 1, function(x) all(is.finite(x))), , drop = FALSE]
        if (nrow(mat) < 3) next

        corr_mat <- Hmisc::rcorr(mat, type = correlation_method)

        corr_df <- as.data.frame(corr_mat$r) |>
            tibble::rownames_to_column("var1") |>
            tidyr::pivot_longer(-var1,
                names_to = "var2",
                values_to = "correlation"
            )
        pval_df <- as.data.frame(corr_mat$P) |>
            tibble::rownames_to_column("var1") |>
            tidyr::pivot_longer(-var1,
                names_to = "var2",
                values_to = "p.value"
            )

        merged <- dplyr::inner_join(corr_df,
            pval_df,
            by = c("var1", "var2")
        ) |>
            dplyr::filter(
                var1 %in% exposure_vars,
                var2 %in% features,
                abs(correlation) > correlation_cutoff
            )

        correlation_results[[i]] <- merged
    }

    dplyr::bind_rows(correlation_results) |>
        dplyr::mutate(FDR = p.adjust(p.value, method = "fdr")) |>
        dplyr::filter(!!rlang::sym(cor_pval_column) < pval_cutoff)
}


# --- Run feature-feature correlation function ------------
#' Compute feature-feature correlations in batches
#'
#' Internal helper that correlates a (potentially large) set of feature
#' variables against itself by processing features in batches. Uses
#' \code{Hmisc::rcorr()} to compute correlation coefficients and p-values,
#' then filters by absolute correlation magnitude and an adjusted p-value
#' (FDR) or raw p-value column.
#'
#' @param feature_data A data frame/matrix with one row per sample and columns
#'   for all \code{feature_vars}. Selected columns must be numeric. Rows with
#'   non-finite values across selected columns are removed prior to
#'   correlation.
#' @param feature_vars Character vector of feature column names (subset of
#'   \code{colnames(feature_data)}).
#' @param correlation_method Correlation method passed to \code{Hmisc::rcorr()},
#'   typically \code{"pearson"} or \code{"spearman"}.
#' @param correlation_cutoff Numeric; retain pairs with
#'   \eqn{|r| >} \code{correlation_cutoff}.
#' @param cor_pval_column Name of the p-value column to filter on after FDR
#'   computation; usually \code{"p.value"} or \code{"FDR"}.
#' @param pval_cutoff Numeric; maximum allowed value for \code{cor_pval_column}.
#' @param batch_size Integer; number of feature columns processed per batch.
#'
#' @return A \code{data.frame} with one row per retained feature-feature pair
#'   and columns: \code{var1}, \code{var2}, \code{correlation}, \code{p.value},
#'   and \code{FDR}. Returns an empty data frame when no pairs pass filters.
#'
#' @details
#' Correlations are computed within each batch, then results are row-bound.
#' The FDR is computed within each batch; if you require global FDR across all
#' pairs, recompute \code{p.adjust} on the combined result after binding.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom Hmisc rcorr
#' @importFrom dplyr inner_join filter mutate bind_rows
#' @importFrom tibble rownames_to_column
#' @importFrom tidyr pivot_longer
#' @importFrom rlang sym
#' @importFrom stats p.adjust
.run_correlation_omics_batches <- function(
  feature_data,
  feature_vars,
  correlation_method,
  correlation_cutoff,
  cor_pval_column,
  pval_cutoff,
  batch_size
) {
    correlation_results <- list()
    batches <- split(feature_vars, ceiling(seq_along(feature_vars) / batch_size))

    for (i in seq_along(batches)) {
        features <- batches[[i]]
        cols_to_use <- intersect(features, colnames(feature_data))
        mat <- as.matrix(feature_data[, cols_to_use, drop = FALSE])
        mat <- mat[apply(mat, 1, function(x) all(is.finite(x))), , drop = FALSE]
        if (nrow(mat) < 3) next

        corr_mat <- Hmisc::rcorr(mat, type = correlation_method)

        corr_df <- as.data.frame(corr_mat$r) |>
            tibble::rownames_to_column("var1") |>
            tidyr::pivot_longer(-var1,
                names_to = "var2",
                values_to = "correlation"
            )
        pval_df <- as.data.frame(corr_mat$P) |>
            tibble::rownames_to_column("var1") |>
            tidyr::pivot_longer(-var1,
                names_to = "var2",
                values_to = "p.value"
            )

        merged <- dplyr::inner_join(corr_df,
            pval_df,
            by = c("var1", "var2")
        ) |>
            dplyr::filter(abs(correlation) > correlation_cutoff)

        correlation_results[[i]] <- merged
    }

    dplyr::bind_rows(correlation_results) |>
        dplyr::mutate(FDR = p.adjust(p.value, method = "fdr")) |>
        dplyr::filter(!!rlang::sym(cor_pval_column) < pval_cutoff)
}

# --- Run and Clean Correlation Function --------------
#' Run correlation and normalize variable names
#'
#' Internal helper that dispatches to the appropriate correlation routine
#' (exposure-feature vs feature-feature), then normalizes variable names by
#' stripping assay/experiment prefixes from feature columns. When correlating
#' exposures with features, the function also joins the codebook metadata.
#'
#' The output keeps \code{var1}/\code{var2} for downstream compatibility and,
#' when \code{feature_cors = FALSE}, adds convenience columns
#' \code{exposure} (== \code{var1}) and \code{feature} (== \code{var2}).
#' It also annotates \code{var1_type} and \code{var2_type}.
#'
#' @param exposomicset A \code{MultiAssayExperiment}.
#' @param merged_data Data frame used for correlation (rows = samples).
#' @param feature_vars Character vector of feature column names.
#' @param exposure_vars Character vector of exposure column names.
#' @param feature_type One of \code{"degs"}, \code{"omics"}, \code{"factors"},
#'   \code{"factor_features"}, \code{"exposures"}, or \code{"pcs"}.
#' @param feature_cors Logical; if \code{TRUE}, do feature-feature correlation.
#' @param correlation_method Correlation method for \code{Hmisc::rcorr()}.
#' @param correlation_cutoff Keep pairs with \eqn{|r| >} this threshold.
#' @param cor_pval_column Name of p-value column to filter (e.g., "p.value",
#'   "FDR").
#' @param pval_cutoff Numeric cutoff applied to \code{cor_pval_column}.
#' @param batch_size Integer batch size for correlation.
#'
#' @return A data frame with \code{var1}, \code{var2}, \code{correlation},
#'   \code{p.value}, \code{FDR}, and, when applicable, \code{exposure},
#'   \code{feature}, \code{exp_name}. Includes \code{var1_type} and
#'   \code{var2_type}.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom MultiAssayExperiment experiments metadata
#' @importFrom dplyr left_join mutate
#' @importFrom stringr str_extract str_remove str_replace_all
.run_and_clean_correlation <- function(
  exposomicset,
  merged_data,
  feature_vars,
  exposure_vars,
  feature_type,
  feature_cors,
  correlation_method,
  correlation_cutoff,
  cor_pval_column,
  pval_cutoff,
  batch_size
) {
    if (feature_cors && feature_type %in% c(
        "degs",
        "omics",
        "factors",
        "factor_features"
    )) {
        # Run correlation on features
        correlation_df <- .run_correlation_omics_batches(
            merged_data,
            feature_vars,
            correlation_method,
            correlation_cutoff,
            cor_pval_column,
            pval_cutoff,
            batch_size
        )

        # Grab experiment names
        exp_names <- names(MultiAssayExperiment::experiments(exposomicset))

        # Change names of the columns to match column type
        correlation_df <- correlation_df |>
            # Add in exp_name for the assay
            dplyr::mutate(
                exp_name_1 = stringr::str_extract(
                    var1,
                    paste0("^(", paste0(
                        stringr::str_replace_all(
                            exp_names, " ", "[ _]"
                        ),
                        collapse = "|"
                    ), ")")
                ),
                # Remove matched exp_name and any underscore or space after it
                var1 = dplyr::case_when(
                    !is.na(exp_name_1) ~ stringr::str_remove(
                        var1, paste0("^", exp_name_1, "[ _]?")
                    ),
                    TRUE ~ var1
                ),
                exp_name_2 = stringr::str_extract(
                    var2,
                    paste0(
                        "^(",
                        paste0(
                            stringr::str_replace_all(
                                exp_names, " ", "[ _]"
                            ),
                            collapse = "|"
                        ), ")"
                    )
                ),
                # Remove matched exp_name and any underscore or space after it
                var2 = dplyr::case_when(
                    !is.na(exp_name_2) ~ stringr::str_remove(
                        var2, paste0("^", exp_name_2, "[ _]?")
                    ),
                    TRUE ~ var2
                )
            )
    } else {
        correlation_df <- .run_correlation_batches(
            merged_data,
            exposure_vars,
            feature_vars,
            correlation_method,
            correlation_cutoff,
            cor_pval_column,
            pval_cutoff,
            batch_size
        )
    }


    if (!feature_cors && feature_type %in% c(
        "degs",
        "omics",
        "factors",
        "factor_features"
    )) {
        # Grab experiment names
        exp_names <- names(MultiAssayExperiment::experiments(exposomicset))

        # Change names of the columns to match column type
        correlation_df <- correlation_df |>
            dplyr::rename(
                exposure = var1,
                feature = var2
            ) |>
            # Add in exp_name for the assay
            dplyr::mutate(
                exp_name = stringr::str_extract(
                    feature,
                    paste0(
                        "^(",
                        paste0(stringr::str_replace_all(exp_names, " ", "[ _]"),
                            collapse = "|"
                        ), ")"
                    )
                ),
                # Remove matched exp_name and any underscore or space after it
                feature = dplyr::case_when(
                    !is.na(exp_name) ~ stringr::str_remove(
                        feature, paste0("^", exp_name, "[ _]?")
                    ),
                    TRUE ~ feature
                )
            )


        # Merge with exposure metadata
        correlation_df <- correlation_df |>
            dplyr::left_join(
                MultiAssayExperiment::metadata(exposomicset)$codebook,
                by = c("exposure" = "variable")
            )
    }
    return(correlation_df)
}

# --- Extract DEGs to Correlate Function -----------
#' Extract DEG expression matrix from a MultiAssayExperiment
#'
#' Internal helper that extracts log2-normalized expression values for
#' differentially expressed genes (DEGs) across assays in a
#' \code{MultiAssayExperiment}, optionally restricted to robust features
#' based on sensitivity analysis.
#'
#' @param exposomicset A \code{MultiAssayExperiment} containing DEG and (optional)
#'   sensitivity metadata.
#' @param robust Logical; if \code{TRUE}, filter to robust features using
#'   sensitivity scores.
#' @param score_col Name of the column with stability scores.
#' @param score_thresh Threshold for stability score filtering.
#' @param deg_pval_col Name of column containing DEG p-values.
#' @param deg_logfc_col Name of column containing DEG log fold-changes.
#' @param deg_pval_thresh P-value threshold for DEG filtering.
#' @param deg_logfc_thresh Absolute logFC threshold for DEG filtering.
#'
#' @return A data frame of log2-normalized expression values for DEGs
#'   (columns) with one row per sample and a column \code{"id"} for sample ID.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom purrr pluck
#' @importFrom dplyr filter pull semi_join
#' @importFrom SummarizedExperiment assay
#' @importFrom tibble rownames_to_column
#' @importFrom dplyr full_join
.extract_deg_matrix <- function(
  exposomicset,
  robust,
  score_col,
  score_thresh,
  deg_pval_col,
  deg_logfc_col,
  deg_pval_thresh,
  deg_logfc_thresh
) {
    da <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "differential_analysis",
            "differential_abundance"
        )
    if (is.null(da)) stop("No differential_abundance in metadata")

    da <- da |>
        dplyr::filter(
            !!rlang::sym(deg_pval_col) < deg_pval_thresh,
            abs(!!rlang::sym(deg_logfc_col)) > deg_logfc_thresh
        )

    if (robust) {
        sens <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck(
                "differential_analysis",
                "sensitivity_analysis"
            )
        if (is.null(sens)) stop("No sensitivity_analysis in metadata")

        stable_feats <- sens$feature_stability |>
            dplyr::filter(
                !!dplyr::sym(score_col) > (score_thresh %||% sens$score_thresh)
            )
        da <- dplyr::semi_join(da,
            stable_feats,
            by = c("exp_name", "feature")
        )
    }

    mats <- lapply(unique(da$exp_name), function(name) {
        se <- .update_assay_colData(exposomicset, name)
        feats <- da |>
            dplyr::filter(exp_name == name) |>
            dplyr::pull(feature)
        se <- se[rownames(se) %in% feats, , drop = FALSE]
        mat <- SummarizedExperiment::assay(se) |>
            t() |>
            as.data.frame()
        colnames(mat) <- paste(name, colnames(mat), sep = "_")
        tibble::rownames_to_column(mat, "id")
    })

    purrr::reduce(mats, dplyr::full_join, by = "id")
}


# --- Extract User-specified Omics to Correlate Function ---------
#' Extract omics expression matrix from MultiAssayExperiment
#'
#' Internal helper that extracts log2-transformed expression matrices for
#' omics features in a \code{MultiAssayExperiment}, optionally restricted to
#' a subset defined by a variable mapping table.
#'
#' @param exposomicset A \code{MultiAssayExperiment} object with omics assays.
#' @param variable_map Optional data frame with columns \code{variable} and
#'   \code{exp_name} indicating which features to extract from which assays.
#'
#' @return A data frame with one row per sample and one column per omics
#'   feature, prefixed by assay name. Includes a column \code{"id"} for sample ID.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom MultiAssayExperiment experiments
#' @importFrom purrr imap compact reduce
#' @importFrom SummarizedExperiment assay
#' @importFrom tibble rownames_to_column
#' @importFrom dplyr full_join
.extract_omics_matrix <- function(exposomicset, variable_map) {
    log2_omics <- .log2_multiassay(exposomicset)
    selected <- if (!is.null(variable_map)) {
        split(
            variable_map$variable,
            variable_map$exp_name
        )
    } else {
        lapply(MultiAssayExperiment::experiments(log2_omics), rownames)
    }

    dfs <- purrr::imap(
        MultiAssayExperiment::experiments(log2_omics),
        function(se, name) {
            feats <- selected[[name]]
            if (is.null(feats)) {
                return(NULL)
            }
            se <- se[rownames(se) %in% feats, , drop = FALSE]
            df <- SummarizedExperiment::assay(se) |>
                t() |>
                as.data.frame()
            colnames(df) <- paste(name, colnames(df), sep = "_")
            tibble::rownames_to_column(df, "id")
        }
    ) |> purrr::compact()

    purrr::reduce(dfs,
        dplyr::full_join,
        by = "id"
    )
}

# --- Extract Latent Factor to Correlate Function ---------
#' Extract sample x factor matrix from multi-omics integration
#'
#' Internal helper that retrieves sample-level factor scores from the stored
#' multi-omics integration results in a \code{MultiAssayExperiment}. Supports
#' \code{MOFA}, \code{MCIA}, and \code{MCCA}. Returns a data frame with one row
#' per sample and a column \code{"id"} for sample IDs.
#'
#' @param exposomicset A \code{MultiAssayExperiment} with integration results
#'   saved in \code{metadata(exposomicset)$multiomics_integration$integration_results}.
#'
#' @return A data frame of sample x factor scores with an \code{"id"} column.
#'
#' @details
#' For MOFA, factors are obtained via \code{MOFA2::get_factors()} (first view).
#' For MCIA, the \code{@global_scores} slot is used. For MCCA, the
#' \code{@sample_scores} slot is used. An error is thrown for unsupported
#' methods.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom purrr pluck
#' @importFrom tibble rownames_to_column
.extract_factor_matrix <- function(exposomicset) {
    result <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "multiomics_integration",
            "integration_results"
        )
    mat <- if (result$method == "MOFA") {
        .check_suggested("MOFA2")
        MOFA2::get_factors(result$result)[[1]]
    } else if (result$method == "MCIA") {
        result$result@global_scores
    } else if (result$method == "MCCA") {
        result$result@sample_scores
    } else {
        stop("Unsupported integration method")
    }
    as.data.frame(mat) |> tibble::rownames_to_column("id")
}

# --- Extract Latent Factor Features to Correlate Function ---------
#' Extract matrix for top factor features across assays
#'
#' Internal helper that builds a sample x feature matrix for features selected
#' from multi-omics integration (stored in
#' \code{metadata(exposomicset)$multiomics_integration$top_factor_features}).
#' For each assay, it subsets to the specified features, extracts log2 values,
#' and returns a joined data frame with an \code{"id"} column.
#'
#' @param exposomicset A \code{MultiAssayExperiment} containing omics assays and
#'   multi-omics integration metadata with a \code{top_factor_features} table
#'   that includes \code{exp_name} and \code{feature} columns.
#'
#' @return A data frame with one row per sample and columns for selected
#'   features (prefixed by assay name), plus an \code{"id"} column.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom MultiAssayExperiment metadata experiments
#' @importFrom purrr pluck imap compact reduce
#' @importFrom SummarizedExperiment assay
#' @importFrom tibble rownames_to_column
#' @importFrom dplyr full_join
.extract_factor_feature_matrix <- function(exposomicset) {
    top_feats <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "multiomics_integration",
            "top_factor_features"
        )

    if (is.null(top_feats)) stop("No top_factor_features found in metadata.")
    if (!all(c("exp_name", "feature") %in% colnames(top_feats))) {
        stop("top_factor_features must contain 'exp_name' and 'feature' columns.")
    }

    log2_omics <- .log2_multiassay(exposomicset)
    selected <- split(top_feats$feature, top_feats$exp_name)

    dfs <- purrr::imap(
        MultiAssayExperiment::experiments(log2_omics),
        function(se, name) {
            feats <- selected[[name]]
            if (is.null(feats)) {
                return(NULL)
            }
            se <- se[rownames(se) %in% feats, , drop = FALSE]
            df <- SummarizedExperiment::assay(se) |>
                t() |>
                as.data.frame()
            colnames(df) <- paste(name, colnames(df), sep = "_")
            tibble::rownames_to_column(df, "id")
        }
    ) |> purrr::compact()

    purrr::reduce(dfs, dplyr::full_join, by = "id")
}

# --- Extract Exposure Data to Correlate Function -----------
#' Extract numeric exposures from colData
#'
#' Internal helper that selects numeric exposure variables from a sample-level
#' data frame (e.g., \code{colData(exposomicset)}). Optionally restricts to a
#' user-provided set of exposure column names.
#'
#' @param col_df A data frame of sample metadata (rows = samples).
#' @param exposure_cols Optional character vector of exposure column names to
#'   retain. If \code{NULL}, all numeric columns are used.
#'
#' @return A data frame with one row per sample containing the selected numeric
#'   exposure columns and an \code{"id"} column of sample IDs.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom dplyr select where
#' @importFrom tibble rownames_to_column
.extract_exposure_matrix <- function(col_df, exposure_cols) {
    df <- col_df |> dplyr::select(where(is.numeric))
    if (!is.null(exposure_cols)) df <- df[, exposure_cols, drop = FALSE]
    tibble::rownames_to_column(df, "id")
}

# --- Extract Principal Components to Correlate Function ---------
#' Extract principal component (PC) matrix from metadata
#'
#' Internal helper that selects numeric columns named like \code{PC1}, \code{PC2},
#' etc., from a sample-level data frame (e.g., \code{colData}). Optionally
#' restricts to the first \code{n_pcs} PCs by numeric index.
#'
#' @param col_df A data frame of sample metadata (rows = samples).
#' @param n_pcs Optional integer. If provided, keep only the first \code{n_pcs}
#'   PCs by their numeric suffix.
#'
#' @return A data frame with one row per sample containing selected PC columns
#'   and an \code{"id"} column of sample IDs.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom dplyr select where matches
#' @importFrom readr parse_number
#' @importFrom tibble rownames_to_column
.extract_pc_matrix <- function(col_df, n_pcs = NULL) {
    all_pcs <- col_df |>
        dplyr::select(where(is.numeric)) |>
        dplyr::select(matches("^PC\\d+$"))

    if (!is.null(n_pcs)) {
        pc_names <- colnames(all_pcs)
        # Sort by numeric PC number
        pc_names_sorted <- pc_names[order(readr::parse_number(pc_names))]
        selected <- pc_names_sorted[seq_len(min(n_pcs, length(pc_names_sorted)))]
        all_pcs <- all_pcs[, selected, drop = FALSE]
    }

    tibble::rownames_to_column(all_pcs, "id")
}

test_that("run_create_network builds igraph object from omics_feature_cor", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 30)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # compute feature-feature correlations (required for network)
    mae <- run_correlation(
        exposomicset = mae,
        feature_type = "omics",
        feature_cors = TRUE,
        correlation_method = "spearman",
        correlation_cutoff = 0.1,
        pval_cutoff = 1,
        action = "add"
    )

    # build correlation network
    mae_net <- run_create_network(
        exposomicset = mae,
        feature_type = "omics_feature_cor",
        action = "add"
    )

    # Check that network was added to metadata
    net_obj <- MultiAssayExperiment::metadata(mae_net)$network$network_omics_feature_cor

    # make sure the graph and graph summary stats are there
    expect_true("graph" %in% names(net_obj))
    expect_true("summary" %in% names(net_obj))

    # make sure the graph and summary stats are the right class
    expect_s3_class(net_obj$graph, "igraph")
    expect_s3_class(net_obj$summary, "tbl_df")

    # Check that analysis step was recorded
    steps <- MultiAssayExperiment::metadata(mae_net)$summary$steps
    expect_true("run_create_network" %in% names(steps))
    expect_match(steps$run_create_network$notes, "Created undirected network")
})

test_that("run_create_network returns igraph object when action = 'get'", {
    dummy <- make_example_data(n_samples = 30)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    mae <- run_correlation(
        exposomicset = mae,
        feature_type = "omics",
        feature_cors = TRUE,
        correlation_method = "spearman",
        correlation_cutoff = 0.1,
        pval_cutoff = 1,
        action = "add"
    )

    net <- run_create_network(
        exposomicset = mae,
        feature_type = "omics_feature_cor",
        action = "get"
    )

    # ensure that we get the graph returned
    expect_true("graph" %in% names(net))

    # ensure the class is correct
    expect_s3_class(net$graph, "igraph")
})

test_that("run_correlation computes correlations between exposures and omics", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 30)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Run correlation
    correlated <- run_correlation(
        exposomicset = mae,
        feature_type = "omics",
        correlation_method = "spearman",
        correlation_cutoff = 0.1,
        pval_cutoff = 1,
        action = "add"
    )

    # Confirm metadata contains the correlation results
    corr_meta <- MultiAssayExperiment::metadata(correlated)$correlation$omics

    # does it deliver a data frame
    expect_s3_class(corr_meta, "data.frame")

    # are all the expected columns present
    expect_true(all(c("exposure", "feature", "correlation", "p.value") %in% colnames(corr_meta)))

    # Confirm a record was added to the summary steps
    steps <- MultiAssayExperiment::metadata(correlated)$summary$steps

    # is the step added
    expect_true("run_correlation_omics" %in% names(steps))
    expect_match(steps$run_correlation_omics$notes, "Correlated omics features with exposures")
})

test_that("run_correlation computes correlations between exposures", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 30)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Run correlation
    correlated <- run_correlation(
        exposomicset = mae,
        feature_type = "exposures",
        correlation_method = "spearman",
        correlation_cutoff = 0.1,
        pval_cutoff = 1,
        action = "add"
    )

    # Confirm metadata contains the correlation results
    corr_meta <- MultiAssayExperiment::metadata(correlated)$correlation$exposures

    # does it deliver a data frame
    expect_s3_class(corr_meta, "data.frame")

    # are all the expected columns present
    expect_true(all(c("var1", "var2", "correlation", "p.value") %in% colnames(corr_meta)))

    # Confirm a record was added to the summary steps
    steps <- MultiAssayExperiment::metadata(correlated)$summary$steps

    # is the step added
    expect_true("run_correlation_exposures" %in% names(steps))
    expect_match(steps$run_correlation_exposures$notes, "Correlated exposures features with exposures")
})

test_that("run_exposure_impact computes exposure-level centrality scores", {
    library(MultiAssayExperiment)

    # Create dummy exposomicset
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # mae differential abundance
    mae <- run_differential_abundance(
        exposomicset = mae,
        formula = ~ smoker + sex,
        abundance_col = "counts",
        method = "limma_voom",
        action = "add"
    )

    # Run DEG association + correlation
    mae <- run_correlation(mae,
        feature_type = "degs",
        exposure_cols = c("age", "bmi", "exposure_pm25"),
        correlation_cutoff = 0,
        feature_cors = T,
        deg_pval_thresh = 0.05,
        deg_pval_col = "P.Value",
        pval_cutoff = 0.1,
        action = "add"
    )

    mae <- run_correlation(mae,
        feature_type = "degs",
        exposure_cols = c("age", "bmi", "exposure_pm25"),
        correlation_cutoff = 0,
        feature_cors = F,
        deg_pval_thresh = 0.05,
        deg_pval_col = "P.Value",
        pval_cutoff = 0.1,
        action = "add"
    )

    # Build networks
    mae <- run_create_network(mae,
        feature_type = "degs",
        action = "add"
    )

    mae <- run_create_network(mae,
        feature_type = "degs_feature_cor",
        action = "add"
    )

    # Run exposure impact
    mae <- run_exposure_impact(mae,
        feature_type = "degs",
        action = "add"
    )

    # Extract result
    impact <- metadata(mae)$network$exposure_impact$degs$exposure_impact

    # Expectations
    # ensure it is a data frame
    expect_s3_class(impact, "data.frame")

    # ensure the expected columns are present
    expect_true(all(c("exposure", "mean_degree", "n_features") %in% colnames(impact)))
})

#' Run Association Analysis
#'
#' Perform GLM-based association testing between a specified outcome and
#' features from exposures, omics, or latent factors. Automatically adjusts
#' for covariates and supports both Gaussian and binomial models.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing data
#' and metadata.
#' @param outcome The outcome variable name (must be in `colData`).
#' @param source Source of features to test. One of `"omics"`,
#' `"exposures"`, `"factors"`.
#' @param covariates Optional vector of covariate names to include in the model.
#' @param feature_set Optional character vector of exposure or GO terms to test.
#' @param log_trans Optional boolean value dictating whether or not to log2
#' transform omics features.
#' @param top_n Optional integer: if using omics source, select top `n`
#'  most variable features.
#' @param family GLM family; `"gaussian"` or `"binomial"`.
#' @param correction_method Method for p-value adjustment (default: `"fdr"`).
#' @param action If `"add"` (default), saves results to metadata;
#' else returns results as list.
#' @param feature_col The column in `rowData`
#' for matching gene symbols or IDs.
#' @param mirna_assays Optional character vector of assays to exclude
#' when extracting GO terms.
#'
#' @return If `action = "add"`, returns updated `MultiAssayExperiment`.
#'  Otherwise, returns a list of:
#' - `results_df`: tidy summary of associations
#' - `covariates`: the covariates used
#' - `model_data`: model matrix used in the GLMs
#'
#'
#' @examples
#'
#' #' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run association tests
#' mae <- mae |>
#'     run_association(
#'         source = "exposures",
#'         feature_set = c("exposure_pm25", "exposure_no2"),
#'         outcome = "smoker",
#'         covariates = c("age"),
#'         family = "binomial"
#'     )
#' @importFrom stats binomial glm logLik as.formula coef family
#' @importFrom rlang .data
#' @export
run_association <- function(
  exposomicset,
  outcome,
  source = c("omics", "exposures", "factors"),
  covariates = NULL,
  feature_set = NULL,
  log_trans = TRUE,
  top_n = NULL,
  family = "gaussian",
  correction_method = "fdr",
  action = "add",
  feature_col = NULL,
  mirna_assays = NULL
) {
    # Validate inputs
    source <- match.arg(source)

    # grab coldata and scale if numeric
    data <- exposomicset |>
        MultiAssayExperiment::colData() |>
        as.data.frame() |>
        dplyr::mutate(dplyr::across(
            where(is.numeric) &
                !all_of(outcome),
            ~ as.numeric(scale(.))
        ))
    # dplyr::mutate_if(is.numeric, ~ as.numeric(scale(.)))

    # switch based on input
    features_df <- switch(source,
        omics = .extract_omics_features(
            exposomicset,
            log_trans = log_trans,
            top_n
        ),
        exposures = .extract_exposures(
            data,
            feature_set
        ),
        factors = .extract_latent_factors(exposomicset)
    )

    # create the model data
    if (source == "exposures") {
        model_data <- data
        feature_cols <- feature_set
    } else {
        model_data <- dplyr::left_join(
            tibble::rownames_to_column(data, "id"),
            features_df,
            by = "id"
        ) |>
            tibble::column_to_rownames("id")

        # feature_cols <- setdiff(colnames(features_df), "id")
    }


    # ensure outcome variable is factor if binomial
    if (family == "binomial") {
        model_data[[outcome]] <- as.factor(model_data[[outcome]])

        if (length(unique(model_data[[outcome]])) != 2) {
            stop("Binary outcome must have exactly 2 levels")
        }
    }

    # ensure that covariates are have more than one value
    if (!is.null(covariates)) {
        covariates <- covariates[covariates %in% colnames(model_data)]
        if (length(covariates) == 0) {
            stop("No valid covariates provided.")
        }
        for (cov in covariates) {
            if (length(unique(model_data[[cov]])) <= 1) {
                # stop(paste("Covariate", cov, "must have more than one unique value."))
                stop(sprintf("Covariate %s must have more than one unique value.", cov))
            }
        }
    }

    message("Running GLMs.")

    feature_cols <- setdiff(colnames(features_df), "id")

    results <- purrr::map_dfr(feature_cols, function(fcol) {
        fmla <- as.formula(
            paste(
                outcome, "~", fcol,
                if (!is.null(covariates)) {
                    paste("+", paste(covariates, collapse = "+"))
                } else {
                    ""
                }
            )
        )

        model <- tryCatch(
            glm(fmla, data = model_data, family = family),
            error = function(e) NULL
        )

        # Only continue if model is valid
        if (is.null(model)) {
            return(NULL)
        }

        # extract results
        model_summary <- broom::tidy(model)
        model_filtered <- model_summary |>
            dplyr::filter(term == fcol)

        if (nrow(model_filtered) == 0) {
            message(sprintf(
                "Skipping %s - term not found in model (possibly dropped)",
                fcol
            ))
            return(NULL)
        }


        if (source %in% c("exposures", "factors")) {}
        # Compute R^2
        model_filtered$r2 <- .calc_r2(
            model,
            model_data,
            family = family,
            outcome = outcome
        )$r2

        if (!is.null(covariates)) {
            model_filtered$adj_r2 <- .calc_r2(
                model,
                model_data,
                family = family,
                outcome = outcome
            )$adj_r2
        }

        return(model_filtered)
    }) |>
        dplyr::mutate(
            p_adjust = p.adjust(
                p.value,
                method = correction_method
            )
        ) |>
        dplyr::mutate(outcome = outcome)

    # Annotate the results based on the source
    results <- .annotate_results_by_source(
        results = results,
        source = source,
        exposomicset = exposomicset
    )

    if (
        (paste0("assoc_", source) %in%
            names(MultiAssayExperiment::metadata(exposomicset)$association)) &&
            identical(
                covariates,
                MultiAssayExperiment::metadata(exposomicset)$association[[
                    paste0("assoc_", source)
                ]]$covariates
            )) {
        if (any(results$term %in%
            MultiAssayExperiment::metadata(exposomicset)$association[[
                paste0("assoc_", source)
            ]]$results_df$term)) {
            stop("Association results for this feature already exists.")
        } else {
            results <- exposomicset |>
                MultiAssayExperiment::metadata() |>
                purrr::pluck("association") |>
                purrr::pluck(paste0("assoc_", source)) |>
                purrr::pluck("results_df") |>
                bind_rows(results)
        }
    }

    # add results to metadata
    if (action == "add") {
        MultiAssayExperiment::metadata(exposomicset)$association[[
            paste0("assoc_", source)
        ]] <- list(
            results_df = results,
            covariates = covariates
        )

        # Add in a step record
        step_record <- list(
            run_association = list(
                timestamp = Sys.time(),
                params = list(
                    outcome = outcome,
                    source = source,
                    covariates = covariates,
                    feature_set = feature_set,
                    top_n = top_n,
                    family = family,
                    correction_method = correction_method,
                    feature_col = feature_col,
                    mirna_assays = mirna_assays
                ),
                notes = paste0(
                    "Performed association analysis using source: ",
                    source
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else {
        return(list(
            results_df = results,
            covariates = covariates,
            model_data = model_data
        ))
    }
}

# --- Extract Omics Features Function ----------
#' Extract Omics Features
#'
#' Helper for extracting and optionally selecting top omics features from a
#'  MultiAssayExperiment
#' for modeling.
#'
#' @keywords internal
#' @noRd
.extract_omics_features <- function(exposomicset,
                                    log_trans = TRUE,
                                    top_n = NULL) {
    if (log_trans) {
        log2_assays <- .log2_multiassay(exposomicset)
    } else {
        log2_assays <- exposomicset
    }

    selected <- if (!is.null(top_n)) {
        .top_var_multiassay(log2_assays, n = top_n)
    } else {
        lapply(MultiAssayExperiment::experiments(log2_assays), rownames)
    }

    scaled <- .scale_multiassay(log2_assays,
        log2 = FALSE
    )

    dfs <- purrr::imap(
        MultiAssayExperiment::experiments(scaled),
        function(se, name) {
            df <- SummarizedExperiment::assay(
                se[selected[[name]], , drop = FALSE]
            ) |>
                t() |>
                as.data.frame()

            names(df) <- paste0(name, "_", names(df)) |>
                (\(chr) gsub(" |-", "_", chr))()

            tibble::rownames_to_column(df, "id")
        }
    )

    purrr::reduce(dfs, dplyr::full_join, by = "id")
}

# --- Extract Exposures Function ----------
#' Extract Exposure Features
#'
#' Subset exposure features (from colData) for modeling.
#'
#' @keywords internal
#' @noRd
.extract_exposures <- function(data, feature_set) {
    df <- data[, feature_set, drop = FALSE] |> as.data.frame()
    tibble::rownames_to_column(df, "id")
}

# --- Extract Latent Factors Function -------------
#' Extract Latent Factors
#'
#' Retrieve latent factors (MOFA, MCIA, MCCA, DIABLO, or RGCCA) for
#' modeling from metadata.
#'
#' @keywords internal
#' @noRd
.extract_latent_factors <- function(exposomicset) {
    result <- exposomicset |>
        MultiAssayExperiment::metadata() |>
        purrr::pluck("multiomics_integration") |>
        purrr::pluck("integration_results")

    mat <- switch(result$method,
        "MOFA" = {
            .check_suggested("MOFA2")
            MOFA2::get_factors(result$result)[[1]]
        },
        "MCIA" = {
            scores <- result$result@global_scores
            colnames(scores) <- paste0("Factor", seq_len(ncol(scores)))
            scores
        },
        "MCCA" = {
            result$result$sample_scores |>
                purrr::map(~ .x |>
                    as.data.frame() |>
                    tibble::rownames_to_column("sample") |>
                    tidyr::pivot_longer(-sample,
                        names_to = "factor",
                        values_to = "weight"
                    )) |>
                dplyr::bind_rows(.id = "exp_name") |>
                dplyr::group_by(factor, sample) |>
                dplyr::reframe(weight = mean(weight), .groups = "drop") |>
                tidyr::pivot_wider(names_from = "factor", values_from = "weight") |>
                tibble::column_to_rownames("sample") |>
                (\(df) {
                    colnames(df) <- gsub(" ", "_", colnames(df))
                    df
                })()
        },
        "DIABLO" = {
            result$result$variates |>
                (\(lst) lst[names(lst) != "Y"])() |>
                purrr::imap_dfc(~ {
                    comp_names <- paste(.y, colnames(.x), sep = " ")
                    df <- as.data.frame(.x)
                    colnames(df) <- comp_names
                    df
                }) |>
                tibble::rownames_to_column("id") |>
                tibble::column_to_rownames("id") |>
                (\(df) {
                    colnames(df) <- gsub(" ", "_", colnames(df))
                    df
                })()
        },
        "RGCCA" = {
            purrr::imap_dfc(result$result$Y, ~ {
                comp_names <- paste(.y, colnames(.x), sep = " ")
                df <- as.data.frame(.x)
                colnames(df) <- comp_names
                df
            }) |>
                tibble::rownames_to_column("id") |>
                tibble::column_to_rownames("id") |>
                (\(df) {
                    colnames(df) <- gsub(" ", "_", colnames(df))
                    df
                })()
        },
        stop("Unsupported integration method.")
    )

    mat <- as.data.frame(mat)
    tibble::rownames_to_column(mat, "id")
}


# --- R2 Function ---------
#' Calculate R-squared and adjusted R-squared from a GLM
#'
#' This internal helper function computes the R-squared and adjusted R-squared
#' values from a fitted generalized linear model (GLM) object.
#'
#' @param model A fitted \code{glm} object.
#' @param n Optional integer. Number of observations used to fit the model.
#'   If \code{NULL}, it is inferred from \code{model$model}.
#'
#' @return A list with two elements:
#'   - \code{r2}: R-squared
#'   - \code{adj_r2}: Adjusted R-squared
#'
#' @details
#' The adjusted R-squared is calculated as:
#' \deqn{1 - \frac{(n - 1)}{(n - p)}(1 - R^2)}
#' where \code{n} is the number of observations and \code{p} is
#' the number of parameters (including the intercept).
#'
#' @keywords internal
#' @noRd
.calc_r2 <- function(
  model,
  model_data,
  family = NULL,
  outcome = NULL
) {
    fam <- if (is.null(family)) stats::family(model)$family else family
    n <- nrow(model_data)
    p <- length(stats::coef(model)) # includes intercept

    if (identical(fam, "gaussian")) {
        # regular adjusted R2
        r2 <- 1 - model$deviance / model$null.deviance
        adj_r2 <- 1 - ((n - 1) / (n - p)) * (1 - r2)
        return(list(r2 = r2, adj_r2 = adj_r2))
    }

    if (identical(fam, "binomial")) {
        if (is.null(outcome)) stop("Provide `outcome` for binomial metrics.")
        ll_full <- as.numeric(stats::logLik(model))
        mod_null <- stats::glm(stats::as.formula(paste(outcome, "~ 1")),
            data = model_data, family = stats::binomial()
        )
        ll_null <- as.numeric(stats::logLik(mod_null))

        # McFadden and adjusted McFadden R2
        r2 <- 1 - (ll_full / ll_null)
        adj_r2 <- 1 - ((ll_full - p) / ll_null)

        return(list(r2 = r2, adj_r2 = adj_r2))
    }

    list(r2 = NA_real_, adj_r2 = NA_real_)
}


# --- Annotate the Model Results By Model Source ---------
#' Annotate association results based on feature source
#'
#' This internal helper function adds metadata annotations to the association
#' results table based on the data source (`omics`, `factors`, or `exposures`).
#'
#' For `omics` and `factors`, it:
#' - Extracts experiment names from the `MultiAssayExperiment` and parses them
#'   from the `term` column.
#' - Adds a `category` column representing the experiment name.
#' - For `factors` from DIABLO or RGCCA integration, prefixes the factor names
#'   with the experiment category.
#'
#' For `exposures`, it:
#' - Joins the `codebook` from the `MultiAssayExperiment` metadata to add
#'   variable annotations.
#'
#' For `omics`, it also:
#' - Joins feature-level metadata from `rowData` across all assays.
#'
#' @param results A data frame of association results, with at least a
#'   `term` column.
#' @param source Character string: the feature source (`"omics"`, `"factors"`,
#'   or `"exposures"`).
#' @param exposomicset A `MultiAssayExperiment` containing experiments, metadata,
#'   and codebook information.
#'
#' @return A data frame of association results with additional metadata columns
#'   (e.g., `category`, joined feature metadata).
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom dplyr mutate case_when select rename inner_join left_join
#' bind_rows
#' @importFrom stringr str_extract
#' @importFrom purrr pluck
#' @importFrom tibble rownames_to_column
#' @importFrom MultiAssayExperiment experiments metadata
#' @importFrom SummarizedExperiment rowData
.annotate_results_by_source <- function(
  results,
  source,
  exposomicset
) {
    if (source == "factors") {
        method <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck("multiomics_integration", "integration_results", "method")

        if (method %in% c("DIABLO", "RGCCA")) {
            # These have assay prefixes in term names
            exp_names <- names(MultiAssayExperiment::experiments(exposomicset)) |>
                (\(chr) gsub(" ", "_", chr))()
            matched <- data.frame(
                exp_name = exp_names,
                exp_name_clean = names(MultiAssayExperiment::experiments(exposomicset))
            )
            results <- results |>
                dplyr::mutate(
                    exp_name = stringr::str_extract(
                        term, paste0("^(", paste(exp_names, collapse = "|"), ")")
                    ),
                    term = dplyr::case_when(
                        grepl(paste(exp_names, collapse = "|"), term) ~
                            gsub(paste0("(", paste0(exp_names, "_", collapse = "|"), ")"), "", term),
                        .default = term
                    )
                ) |>
                dplyr::inner_join(matched, by = "exp_name") |>
                dplyr::select(-exp_name) |>
                dplyr::rename(category = exp_name_clean)

            results <- dplyr::mutate(results, term = paste(category, term, sep = " "))
        } else {
            # MOFA, MCIA, MCCA,  no assay prefix
            # so leaving out exp_name/category
            results <- results
        }

        return(results)
    }

    if (source == "omics") {
        exp_names <- names(MultiAssayExperiment::experiments(exposomicset)) |>
            (\(chr) gsub(" ", "_", chr))()
        matched <- data.frame(
            exp_name = exp_names,
            exp_name_clean = names(MultiAssayExperiment::experiments(exposomicset))
        )
        results <- results |>
            dplyr::mutate(
                exp_name = stringr::str_extract(
                    term, paste0("^(", paste(exp_names, collapse = "|"), ")")
                ),
                term = dplyr::case_when(
                    grepl(paste(exp_names, collapse = "|"), term) ~
                        gsub(paste0("(", paste0(exp_names, "_", collapse = "|"), ")"), "", term),
                    .default = term
                )
            ) |>
            dplyr::inner_join(matched, by = "exp_name") |>
            dplyr::select(-exp_name) |>
            dplyr::rename(category = exp_name_clean) |>
            dplyr::mutate(category = gsub("_", " ", category))

        # Add feature metadata
        feature_df <- lapply(
            names(MultiAssayExperiment::experiments(exposomicset)),
            function(name) {
                SummarizedExperiment::rowData(
                    MultiAssayExperiment::experiments(exposomicset)[[name]]
                ) |>
                    as.data.frame() |>
                    tibble::rownames_to_column(".feature") |>
                    dplyr::mutate(exp_name = name)
            }
        ) |>
            dplyr::bind_rows()

        results <- results |>
            dplyr::left_join(feature_df, by = c("term" = ".feature", "category" = "exp_name"))

        return(results)
    }

    if (source == "exposures") {
        results <- results |>
            dplyr::left_join(
                MultiAssayExperiment::metadata(exposomicset)$codebook,
                by = c("term" = "variable")
            )
        return(results)
    }

    results
}

# .annotate_results_by_source <- function(
#   results,
#   source,
#   exposomicset
# ) {
#     if (source %in% c("omics", "factors")) {
#         exp_names <- names(MultiAssayExperiment::experiments(exposomicset)) |>
#             (\(chr) gsub(" ", "_", chr))()
#
#         matched <- data.frame(
#             exp_name = exp_names,
#             exp_name_clean = names(MultiAssayExperiment::experiments(exposomicset))
#         )
#
#         results <- results |>
#             dplyr::mutate(
#                 exp_name = stringr::str_extract(
#                     term, paste0("^(", paste(exp_names, collapse = "|"), ")")
#                 ),
#                 term = dplyr::case_when(
#                     grepl(paste(exp_names, collapse = "|"), term) ~
#                         gsub(paste0(
#                             "(", paste0(exp_names, "_", collapse = "|"), ")"
#                         ), "", term),
#                     .default = term
#                 )
#             ) |>
#             dplyr::inner_join(matched, by = "exp_name") |>
#             dplyr::select(-exp_name) |>
#             dplyr::rename(category = exp_name_clean)
#
#         if (source == "factors") {
#             method <- MultiAssayExperiment::metadata(exposomicset) |>
#                 purrr::pluck(
#                     "multiomics_integration",
#                     "integration_results",
#                     "method"
#                 )
#
#             if (method %in% c("DIABLO", "RGCCA")) {
#                 results <- dplyr::mutate(
#                     results,
#                     term = paste(category, term, sep = " ")
#                 )
#             }
#         }
#     }
#
#     if (source == "exposures") {
#         results <- results |>
#             dplyr::left_join(
#                 MultiAssayExperiment::metadata(exposomicset)$codebook,
#                 by = c("term" = "variable")
#             )
#     }
#
#     if (source == "omics") {
#         feature_df <- lapply(
#             names(MultiAssayExperiment::experiments(exposomicset)),
#             function(name) {
#                 SummarizedExperiment::rowData(
#                     MultiAssayExperiment::experiments(exposomicset)[[name]]
#                 ) |>
#                     as.data.frame() |>
#                     tibble::rownames_to_column(".feature") |>
#                     dplyr::mutate(exp_name = name)
#             }
#         ) |>
#             dplyr::bind_rows()
#
#         results <- results |>
#             dplyr::mutate(category = gsub("_", " ", category)) |>
#             dplyr::left_join(
#                 feature_df,
#                 by = c("term" = ".feature",
#                        "category" = "exp_name")
#             )
#     }
#
#     return(results)
# }

test_that("run_association works with exposures as source and guassian", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 30)

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Select a few exposures to associate
    feature_set <- intersect(c("age", "bmi", "alcohol"), colnames(dummy$exposure))

    # Run association analysis
    assoc_mae <- run_association(
        exposomicset = mae,
        outcome = "exposure_pm25",
        source = "exposures",
        covariates = "sex",
        feature_set = feature_set,
        family = "gaussian",
        action = "add"
    )

    # Confirm metadata was added
    assoc_meta <- MultiAssayExperiment::metadata(assoc_mae)$association$assoc_exposures
    expect_true("results_df" %in% names(assoc_meta))
    expect_true("covariates" %in% names(assoc_meta))
    expect_true(all(c("term", "estimate", "p.value", "r2", "adj_r2") %in% colnames(assoc_meta$results_df)))

    # Confirm step was recorded
    steps <- MultiAssayExperiment::metadata(assoc_mae)$summary$steps
    expect_true("run_association" %in% names(steps))
    expect_match(steps$run_association$notes, "Performed association analysis")
})


test_that("run_association works with exposures as source and binomial", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 30)

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Select a few exposures to associate
    feature_set <- intersect(c("age", "bmi", "alcohol"), colnames(dummy$exposure))

    # Run association analysis
    assoc_mae <- run_association(
        exposomicset = mae,
        outcome = "smoker",
        source = "exposures",
        covariates = "sex",
        feature_set = feature_set,
        family = "binomial",
        action = "add"
    )

    # Confirm metadata was added
    assoc_meta <- MultiAssayExperiment::metadata(assoc_mae)$association$assoc_exposures
    expect_true("results_df" %in% names(assoc_meta))
    expect_true("covariates" %in% names(assoc_meta))
    expect_true(all(c("term", "estimate", "p.value", "r2", "adj_r2") %in% colnames(assoc_meta$results_df)))

    # Confirm step was recorded
    steps <- MultiAssayExperiment::metadata(assoc_mae)$summary$steps
    expect_true("run_association" %in% names(steps))
    expect_match(steps$run_association$notes, "Performed association analysis")
})

#' Download and cache a tidyexposomics dataset
#'
#' @param name Dataset name: one of "omics_list", "fdata", "meta",
#'   "annotated_cb", "expom_1", "chebi", "ecto", "hpo".
#' @param verbose Logical; print messages.
#' @param validate Logical; validate MD5 checksum.
#' @return A list or object loaded from the cached .RData.
#' @importFrom BiocFileCache BiocFileCache bfcrpath
#' @importFrom tools md5sum
#' @export
download_dataset <- function(
  name = c(
      "omics_list", "fdata", "meta", "annotated_cb",
      "expom_1", "chebi", "ecto", "hpo"
  ),
  verbose = TRUE,
  validate = TRUE
) {
    name <- match.arg(name)

    checksums <- c(
        omics_list = "0e0b4bc175da647363478954039b2b9d",
        fdata = "561e9eb4136e4d6116914adb9fc5a30f",
        meta = "db317363ba547eaeb6543acca850ac92",
        annotated_cb = "5ca6a27a29ea50004b1e8dcff50a5487",
        expom_1 = "5e84c508469a4db77a22f03f27ab9ccd",
        chebi = "8a5703b5ac54e63006c002c834a3f703",
        ecto = "0a267757ea47d5d99f5f4929f26186c6",
        hpo = "9948e6b99815da81daa2d9a2e586a9ae"
    )

    zip_name <- paste0(name, ".zip")
    url <- paste0("https://zenodo.org/records/17137560/files/", zip_name)

    bfc <- BiocFileCache::BiocFileCache()
    cached_zip <- BiocFileCache::bfcrpath(bfc, url)

    if (validate) {
        observed <- tools::md5sum(cached_zip)
        expected <- checksums[[name]]
        if (observed != expected) {
            stop(
                "Checksum mismatch for ", name, ".\n",
                "Expected: ", expected, "\n",
                "Observed: ", observed,
                call. = FALSE
            )
        }
        if (verbose) message("Checksum validated for ", name)
    }

    dest_dir <- tempfile(pattern = paste0("tidyexposomics_", name, "_"))
    utils::unzip(cached_zip, exdir = dest_dir)
    rdata_path <- file.path(dest_dir, "data", paste0(name, ".RData"))

    env <- new.env()
    load(rdata_path, envir = env)
    get(name, envir = env)
}

#' Load Ontology Data
#'
#' Downloads and loads `chebi`, `ecto`, and `hpo`.
#'
#' @param to_load Character vector indicating which ontology to load.
#' @param verbose Logical; print messages.
#' @param validate Logical; validate MD5 checksum.
#'
#' @return A named list of ontology objects.
#'
#' @examples
#' \dontrun{
#' # load single ontology
#' onts <- load_annotation_data(to_load = "ecto")
#'
#' # load all ontologies
#' onts <- load_annotation_data(to_load = "all")
#' }
#'
#' @export
load_annotation_data <- function(
  to_load = c("all", "chebi", "ecto", "hpo"),
  verbose = TRUE,
  validate = TRUE
) {
    to_load <- match.arg(to_load)
    names <- if (to_load == "all") c("chebi", "ecto", "hpo") else to_load
    res <- lapply(names, \(nm) download_dataset(nm, verbose = verbose, validate = validate))
    names(res) <- names
    res
}

#' Plot a Manhattan-style ExWAS summary across omics categories
#'
#' This function generates a multi-faceted Manhattan plot from the results of
#' `associate_all_outcome()`, visualizing the significance of associations
#' across omics features, grouped by category. Significant features can be
#' highlighted and labeled, and strip backgrounds can be colored per facet.
#'
#' @param exposomicset A `MultiAssayExperiment` object that has
#' already been processed by `associate_all_outcome()`.
#' @param pval_thresh Numeric threshold for significance (default = 0.05).
#' @param feature_col A character string indicating the column name to use
#' for feature labeling and highlighting (e.g., `"term"` or `"feature"`).
#' Default is `"term"`.
#' @param alpha Transparency applied to facet strip colors (default = 0.5).
#' @param min_per_cat Minimum number of features per category to be shown
#' (default = 1).
#' @param vars_to_label Optional character vector of variable names to label
#' explicitly, matched against the `feature_col` column.
#' @param sig_color Color used for significant points (default = `"magenta2"`).
#' @param non_sig_cols Character vector of alternating colors for
#' non-significant points (default = `c("grey25", "grey75")`).
#' @param pval_thresh_line_col Color of the horizontal significance
#' threshold line (default = `"grey25"`).
#' @param panel_sizes Numeric vector passed to `ggh4x::force_panelsizes()`
#' to control panel widths (default = `c(1,1,1,1,1)`).
#' @param linetype Line type for the horizontal threshold (default = `"dashed"`).
#' @param facet_cols Optional vector of colors to use for
#' facet strip backgrounds.
#' @param label_size Numeric size of the feature label text (default = 3.5).
#' @param facet_angle Angle (in degrees) for strip text rotation (default = 90).
#' @param facet_text_face Font face for facet strip labels
#' (default = `"bold.italic"`).
#'
#' @return A `ggplot` object showing the Manhattan-style faceted plot.
#'
#' @details
#' - This function expects `associate_all_outcome()` to have been run first.
#' - Facets represent omics categories, and points represent features.
#' - Points below the significance threshold are colored using `non_sig_cols`,
#'   while significant ones are colored with `sig_color` and optionally labeled.
#' - Uses `ggrepel` to avoid overlapping labels and `ggh4x` for
#' enhanced faceting.
#' - The `feature_col` argument allows customization of which column is used
#'   to label or identify features, enabling compatibility with different
#'   result formats.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run association tests
#' mae <- mae |>
#'     run_association(
#'         source = "omics",
#'         top_n = 20,
#'         feature_set = c("exposure_pm25", "exposure_no2"),
#'         outcome = "smoker",
#'         covariates = c("age"),
#'         family = "binomial"
#'     )
#'
#' # create the manhattan plot
#' manhattan_p <- mae |>
#'     plot_manhattan(feature_col = "term")
#'
#' @importFrom ggplot2 ggplot aes geom_point geom_vline
#'   scale_color_identity labs theme element_text element_rect
#' @importFrom dplyr filter mutate group_by ungroup n bind_rows pull
#' @importFrom purrr pluck
#' @importFrom ggrepel geom_label_repel
#' @importFrom scales alpha
#' @importFrom ggpubr theme_pubr
#'
#' @export
plot_manhattan <- function(
  exposomicset,
  pval_thresh = 0.05,
  feature_col = "term",
  alpha = 0.5,
  min_per_cat = 1,
  vars_to_label = NULL,
  sig_color = "magenta2",
  non_sig_cols = c("grey25", "grey75"),
  pval_thresh_line_col = "grey25",
  panel_sizes = c(1, 1, 1, 1, 1),
  linetype = "dashed",
  facet_cols = NULL,
  label_size = 3.5,
  facet_angle = 90,
  facet_text_face = "bold.italic"
) {
    # require(ggplot2)
    .check_suggested(pkg = "forcats")
    .check_suggested(pkg = "ggh4x")

    # Check if "correlation" is a name in metadata
    if (!("association" %in% names(MultiAssayExperiment::metadata(exposomicset)))) {
        stop("Please run `run_association() first.`")
    }

    # If exp_names in manhattan_data, replace with experiment names
    exposure_res_df <- exposomicset |>
        MultiAssayExperiment::metadata() |>
        purrr::pluck("association") |>
        purrr::pluck("assoc_exposures") |>
        purrr::pluck("results_df")

    exposure_res_df[[feature_col]] <- exposure_res_df$term

    manhattan_data <- exposomicset |>
        MultiAssayExperiment::metadata() |>
        purrr::pluck("association") |>
        purrr::pluck("assoc_omics") |>
        purrr::pluck("results_df") |>
        bind_rows(
            exposure_res_df
        ) |>
        dplyr::filter(!is.na(category)) |>
        dplyr::mutate(category = gsub("_", " ", category)) |>
        dplyr::mutate(
            var = forcats::fct_reorder(as.character(term), category),
            thresh_met = ifelse(p.value < pval_thresh, "yes", "no")
        )

    if (!is.null(vars_to_label)) {
        manhattan_data <- manhattan_data |>
            dplyr::mutate(label = ifelse(
                var %in% vars_to_label,
                as.character(!!dplyr::sym(feature_col)), NA
            )) |>
            dplyr::mutate(
                label = dplyr::case_when(
                    !is.na(label) ~ paste0("italic(", label, ")"),
                    .default = NA_character_
                )
            )
    } else {
        manhattan_data <- manhattan_data |>
            dplyr::mutate(
                label = ifelse(thresh_met == "yes",
                    as.character(!!dplyr::sym(feature_col)),
                    NA
                )
            )
    }

    # Filter out categories with fewer than min_per_cat significant features
    manhattan_data <- manhattan_data |>
        dplyr::group_by(category) |>
        dplyr::filter(dplyr::n() >= min_per_cat) |>
        dplyr::ungroup()

    # Get unique ordered categories
    ordered_cats <- manhattan_data$category |>
        unique() |>
        sort()

    # Alternating point colors for non-significant values
    nonsig_greys <- rep(non_sig_cols, length.out = length(ordered_cats))
    category_grey_map <- setNames(nonsig_greys, ordered_cats)

    # Add color column to data
    manhattan_data <- manhattan_data |>
        dplyr::mutate(
            cols = ifelse(
                thresh_met == "yes",
                sig_color,
                category_grey_map[category]
            )
        )


    # Facet colors
    if (is.null(facet_cols)) {
        facet_cols <- tidy_exp_pal[
            seq_len(length(unique(
                manhattan_data |>
                    dplyr::filter(!is.na(category)) |>
                    dplyr::pull(category)
            )))
        ]
    } else {
        facet_cols
    }

    manhattan <- manhattan_data |>
        ggplot(aes(
            y = var,
            x = -log10(p.value),
            color = cols
        )) +
        geom_point() +
        geom_vline(
            xintercept = -log10(pval_thresh),
            linetype = linetype,
            color = pval_thresh_line_col
        ) +
        ggrepel::geom_label_repel(
            data = manhattan_data,
            aes(
                label = label,
                fontface = "bold",
                fill = "white"
            ),
            parse = TRUE,
            size = label_size,
            fill = "white",
            max.overlaps = 5,
            min.segment.length = .5,
            box.padding = 0.6,
            point.padding = 0.2,
            color = "black",
            na.rm = TRUE
        ) +
        ggh4x::facet_grid2(
            category ~ .,
            scales = "free_y",
            space = "free_y",
            strip = ggh4x::strip_themed(
                background_y = ggh4x::elem_list_rect(
                    fill = scales::alpha(
                        facet_cols,
                        alpha
                    )
                )
            )
        ) +
        ggh4x::force_panelsizes(rows = panel_sizes) +
        scale_color_identity() +
        ggpubr::theme_pubr(legend = "none") +
        theme(
            axis.ticks.y = element_blank(),
            axis.text.y = element_blank(),
            strip.placement = "outside",
            strip.background = element_rect(linewidth = 0, colour = NA),
            strip.text.y = element_text(
                angle = facet_angle,
                face = facet_text_face
            ),
            panel.spacing = unit(0, "lines")
        ) +
        labs(
            y = "",
            x = expression(-Log[10] * "P")
        )
    return(manhattan)
}

#' Run Differential Abundance Analysis
#'
#' Performs differential abundance testing across all assays in a
#' `MultiAssayExperiment` object using a specified statistical method.
#' The function updates each assay with its
#' corresponding `colData`, fits the model using the provided formula,
#' and combines the results into a unified table.
#'
#' @param exposomicset A `MultiAssayExperiment` containing assays to test.
#' @param formula A model formula for the differential analysis
#' (e.g., ~ group + batch).
#' @param abundance_col Character. The name of the assay matrix to use
#' for abundance values. Default is `"counts"`.
#' @param method Character. Differential analysis method to use.
#' Currently supports `"limma_trend"` (default).
#' @param contrasts A named list of contrasts for pairwise comparisons.
#'  Default is `NULL` (uses default group comparisons).
#' @param scaling_method Character. Scaling method to apply before modeling.
#' Options include `"none"` (default), `"zscore"`, etc.
#' @param action Character. Whether to `"add"` results to `exposomicset`
#'  metadata or `"get"` the results as a data frame. Default is `"add"`.
#'
#' @return Either the updated `MultiAssayExperiment` (if `action = "add"`)
#'  or a tibble with differential abundance results (if `action = "get"`).
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # perform differential abundance analysis
#' mae <- run_differential_abundance(
#'     exposomicset = mae,
#'     formula = ~ smoker + sex,
#'     abundance_col = "counts",
#'     method = "limma_trend",
#'     action = "add"
#' )
#'
#' @importFrom limma lmFit eBayes topTable makeContrasts contrasts.fit
#' @importFrom stats binomial
#' @importFrom rlang .data
#' @export
run_differential_abundance <- function(
  exposomicset,
  formula,
  abundance_col = "counts",
  method = "limma_trend",
  contrasts = NULL,
  scaling_method = "none",
  action = "add"
) {
    message("Running differential abundance testing.")
    .check_suggested(pkg = "edgeR")
    .check_suggested(pkg = "limma")

    # Initialize a data frame to store results
    da_results_df <- list()

    # Iterate through assays in exposomicset
    for (exp_name in names(MultiAssayExperiment::experiments(exposomicset))) {
        message("Processing assay: ", exp_name)

        # Update assay with colData
        exp <- .update_assay_colData(exposomicset, exp_name)

        # Run differential analysis using `.run_se_differential_abundance`
        res <- .run_se_differential_abundance(
            se = exp,
            formula = formula,
            abundance_col = abundance_col,
            method = method,
            scaling_method = scaling_method,
            contrasts = contrasts
        )

        # If results exist, append assay name and store them
        if (!is.null(res) && nrow(res) > 0) {
            res <- res |>
                dplyr::mutate(exp_name = exp_name)
            da_results_df[[exp_name]] <- res
        } else {
            warning("No significant results found for assay: ", exp_name)
        }
    }

    # Combine results across assays
    final_results <- da_results_df |>
        dplyr::bind_rows() |>
        as_tibble()

    message("Differential abundance testing completed.")

    if (action == "add") {
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$differential_analysis$differential_abundance <- final_results
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        # Add step record to metadata
        step_record <- list(
            run_differential_abundance = list(
                timestamp = Sys.time(),
                params = list(
                    formula = format(formula),
                    method = method,
                    scaling_method = scaling_method,
                    abundance_col = abundance_col,
                    contrasts = contrasts
                ),
                notes = "Performed differential abundance analysis across all assays."
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )
        return(exposomicset)
    } else if (action == "get") {
        return(final_results)
    } else {
        stop("Invalid action specified. Use 'add' or 'get'.")
    }
    return(exposomicset)
}


# --- Limma Trend Option ----------------------
#' Run Differential Abundance Analysis using limma-trend
#'
#' Performs differential abundance analysis on a `SummarizedExperiment`
#' using the limma-trend method. Limma-trend can be used for preprocessed
#' (e.g. log-transformed) abundance data such as logCPM, log-intensity,
#' or log2(TPM+1).
#'
#' @importFrom stats binomial
#' @importFrom rlang .data
#' @importFrom limma lmFit eBayes topTable makeContrasts contrasts.fit
#' @importMethodsFrom tidybulk pivot_transcript
#' @keywords internal
#' @noRd
.run_limma_trend <- function(
  se,
  formula,
  abundance_col,
  contrasts = NULL,
  scaling_method = "none",
  robust = TRUE
) {
    .check_suggested("limma")

    mat <- SummarizedExperiment::assay(se, abundance_col)
    if (!is.matrix(mat)) mat <- as.matrix(mat)

    # Optional scaling
    if (identical(scaling_method, "zscore")) {
        mat <- t(scale(t(mat)))
    }

    # Log2 transform if needed
    if (max(mat, na.rm = TRUE) > 50) {
        mat <- log2(mat + 1)
    }

    df <- as.data.frame(SummarizedExperiment::colData(se))
    design <- stats::model.matrix(formula, data = df)

    fit <- limma::lmFit(mat, design)
    fit <- limma::eBayes(fit, trend = TRUE, robust = robust)

    results <- list()
    if (!is.null(contrasts)) {
        cm <- limma::makeContrasts(contrasts = contrasts, levels = design)
        fit2 <- limma::contrasts.fit(fit, cm)
        fit2 <- limma::eBayes(fit2, trend = TRUE, robust = robust)
        for (i in seq_len(ncol(cm))) {
            tb <- limma::topTable(fit2, coef = i, number = Inf, sort.by = "none")
            tb$contrast <- colnames(cm)[i]
            results[[i]] <- tb
        }
    } else {
        coef_idx <- setdiff(
            seq_len(ncol(design)),
            which(colnames(design) == "(Intercept)")
        )
        for (k in coef_idx) {
            tb <- limma::topTable(fit, coef = k, number = Inf, sort.by = "none")
            tb$contrast <- colnames(design)[k]
            results[[length(results) + 1]] <- tb
        }
    }

    out <- dplyr::bind_rows(results)

    out <- out |>
        tibble::rownames_to_column("feature") |>
        dplyr::mutate(
            feature = gsub("\\.\\.\\..*", "", feature),
            method = "limma_trend",
            .abundant = TRUE,
            scaling = scaling_method
        ) |>
        dplyr::left_join(
            se |>
                pivot_transcript(),
            by = c("feature" = ".feature")
        ) |>
        dplyr::filter(grepl(all.vars(formula)[1], contrast)) |>
        tibble::as_tibble()

    return(out)
}

test_that("run_sensitivity_analysis returns results with action = 'get'", {
    skip_if_not_installed("limma")

    # create dummy data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Run differential abundance
    mae <- run_differential_abundance(
        exposomicset = mae,
        formula = ~ smoker + sex,
        abundance_col = "counts",
        method = "limma_voom",
        action = "add"
    )

    # Run the sensitivity analysis
    mae <- run_sensitivity_analysis(
        exposomicset = mae,
        base_formula = ~ smoker + sex,
        methods = c("limma_voom"),
        scaling_methods = c("none"),
        covariates_to_remove = "sex",
        pval_col = "P.Value",
        logfc_col = "logFC",
        pval_threshold = 0.05,
        logFC_threshold = 0,
        bootstrap_n = 3,
        action = "add"
    )

    result <- mae@metadata$differential_analysis$sensitivity_analysis

    # check that the results are a list
    expect_type(result, "list")

    # check that each item is the expected data type
    expect_s3_class(result$sensitivity_df, "data.frame")
    expect_s3_class(result$feature_stability, "data.frame")
    expect_type(result$score_thresh, "double")

    # grab rowdata column names
    rowdata_cols <- mae |>
        pivot_feature() |>
        dplyr::select(-c(.exp_name, .feature)) |>
        colnames()

    # check that the expected rowdata column names are present
    expect_contains(colnames(result$sensitivity_df), rowdata_cols)

    # check that the condition column names are present
    expect_contains(
        colnames(result$sensitivity_df),
        c(
            "feature", "contrast", "method",
            "scaling", "model", "exp_name",
            "bootstrap_id"
        )
    )

    # ensure that the scores were calculated
    scores <- c("presence_rate", "effect_consistency", "stability_score", "mean_log_p", "logp_weighted_score", "sd_logFC", "iqr_logFC", "cv_logFC", "sign_flip_freq", "sd_log_p")
    lapply(scores, function(score) {
        expect_type(result$feature_stability[[score]], "double")
    })
})

test_that("run_factor_overlap() returns expected object and annotations", {
    skip_if_not_installed("MultiAssayExperiment")

    # Create dummy data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Run differential abundance
    mae <- run_differential_abundance(
        exposomicset = mae,
        formula = ~ smoker + sex,
        abundance_col = "counts",
        method = "limma_voom",
        action = "add"
    )

    # Perform multi-omics integration and extract top features
    mae <- run_multiomics_integration(mae,
        method = "DIABLO",
        outcome = "smoker",
        n_factors = 3
    )

    mae <- extract_top_factor_features(mae,
        percentile = .5,
        factors = c("V1", "V2", "V3")
    )

    # Run function
    mae <- run_factor_overlap(mae,
        pval_col = "P.Value",
        robust = F,
        action = "add"
    )

    # Check output structure
    expect_s4_class(mae, "MultiAssayExperiment")

    # are the common factor features added to the metadata
    expect_true("common_top_factor_features" %in%
        names(mae@metadata$multiomics_integration))

    res_df <- run_factor_overlap(mae,
        robust = F,
        pval_col = "P.Value",
        action = "get"
    )

    # is the result a data frame
    expect_s3_class(res_df, "data.frame")

    # are the expected column names present
    expect_true(all(c("exp_name", "feature", "factor", "is_deg") %in% names(res_df)))

    # check if the is_deg column populated correctly
    expect_true(is.logical(res_df$is_deg))
})

test_that("run_enrichment identifies GO terms for DEGs and updates metadata", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 30)

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Run differential abundance to generate DEGs
    mae <- run_differential_abundance(
        exposomicset = mae,
        formula = ~ smoker + sex,
        abundance_col = "counts",
        method = "limma_voom",
        action = "add"
    )

    # Run enrichment on DEGs using GO
    enriched <- run_enrichment(
        exposomicset = mae,
        feature_type = "degs",
        feature_col = "symbol",
        deg_pval_col = "P.Value",
        deg_pval_threshold = 0.5,
        clustering_approach = "diana",
        deg_logfc_col = "logFC",
        deg_logfc_threshold = log2(1.5),
        db = "GO",
        species = "goa_human",
        action = "add"
    )

    # Confirm enrichment results added to metadata
    enr_res <- MultiAssayExperiment::metadata(enriched)$enrichment$degs
    expect_s3_class(enr_res, "data.frame")


    # Confirm a step record was logged
    steps <- MultiAssayExperiment::metadata(enriched)$summary$steps
    expect_true("run_enrichment" %in% names(steps))
    expect_match(steps$run_enrichment$notes, "Performed GO enrichment on degs features")
})

test_that("run_differential_abundance returns expected results and updates metadata", {
    skip_if_not_installed("limma")
    # Create dummy data
    dummy <- make_example_data(n_samples = 20)

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Run differential abundance
    mae_da <- run_differential_abundance(
        exposomicset = mae,
        formula = ~ smoker + sex,
        abundance_col = "counts",
        method = "limma_voom",
        action = "add"
    )

    # Check that results were added to metadata
    da_results <- MultiAssayExperiment::metadata(mae_da)$differential_analysis$differential_abundance

    # is it the right class
    expect_s3_class(da_results, "tbl_df")

    # do we get the expected column names
    expect_true(all(c("feature", "logFC", "adj.P.Val", "exp_name") %in% colnames(da_results)))

    # Confirm a step was recorded
    steps <- MultiAssayExperiment::metadata(mae_da)$summary$steps
    expect_true("run_differential_abundance" %in% names(steps))
    expect_match(steps$run_differential_abundance$notes, "Performed differential abundance analysis")
})

#' Extract Top Contributing Features for Factors
#'
#' Identifies the most influential features for specified factors
#' using multiomics integration results.
#' Features are selected based on either a percentile cutoff
#' or an absolute loading threshold.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing
#' integration results.
#' @param factors A character vector specifying the factors of interest.
#' If `NULL`, factors are automatically selected from the association results
#' using the `pval_col` and `pval_thresh` criteria.
#' @param pval_col A string specifying the column name of the p-value or
#' adjusted p-value used for factor selection if `factors` is `NULL`.
#' Default is `"p_adjust"`.
#' @param pval_thresh A numeric value specifying the significance threshold
#' for selecting factors from association results when `factors` is `NULL`.
#' Default is `0.05`.
#' @param method A character string specifying the feature selection method
#' (`"percentile"` or `"threshold"`). Default is `"percentile"`.
#' @param percentile A numeric value between 0 and 1 indicating the
#' percentile threshold for feature selection when `method = "percentile"`.
#' Default is `0.9`.
#' @param threshold A numeric value specifying the absolute loading cutoff
#' for feature selection when `method = "threshold"`. Default is `0.3`.
#' @param action A character string indicating whether to return results
#' (`"get"`) or add them to metadata (`"add"`). Default is `"add"`.
#'
#' @details
#' The function extracts factor loadings from `metadata(exposomicset)`,
#' applies filtering based on
#' the selected method, and identifies top contributing features for
#' each specified factor.
#'
#' If `factors` is not provided, the function will automatically select
#' statistically significant factors from `metadata(exposomicset)$association$assoc_factors$results_df`
#' using the specified `pval_col` and `pval_thresh` as criteria.
#'
#' Features can be selected using:
#' - **Percentile-based filtering** (`method = "percentile"`): Selects
#' features with absolute loadings above a specified percentile.
#' - **Threshold-based filtering** (`method = "threshold"`): Selects
#' features with absolute loadings exceeding a fixed value.
#'
#' @return If `action = "add"`, returns the modified `exposomicset` with
#' selected features stored in metadata.
#' If `action = "get"`, returns a data frame containing:
#' \item{feature}{The selected feature contributing to the factor.}
#' \item{factor}{The factor to which the feature contributes.}
#' \item{loading}{The factor loading value of the feature.}
#' \item{exp_name}{The experiment from which the feature originated.}
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # perform multiomics integration
#' mae <- run_multiomics_integration(
#'     mae,
#'     method = "DIABLO",
#'     outcome = "smoker",
#'     n_factors = 3
#' )
#'
#' top_feats <- extract_top_factor_features(
#'     mae,
#'     factors = c("V1", "V2", "V3"),
#'     method = "percentile",
#'     percentile = 0.9,
#'     action = "get"
#' )
#'
#' @export
extract_top_factor_features <- function(
  exposomicset,
  factors = NULL,
  pval_col = "p_adjust",
  pval_thresh = 0.05,
  method = "percentile",
  percentile = 0.9,
  threshold = 0.3,
  action = "add"
) {
    message("Extracting top contributing features for specified factors.")

    # Get integration results
    integration_results <- MultiAssayExperiment::metadata(exposomicset)$multiomics_integration$integration_results
    method_used <- integration_results$method

    if (is.null(factors)) {
        factors <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck(
                "association",
                "assoc_factors",
                "results_df"
            ) |>
            dplyr::filter(!!dplyr::sym(pval_col) < pval_thresh) |>
            dplyr::pull(term)
    }

    # Extract factor loadings
    loadings <- switch(method_used,
        "MOFA" = {
            .check_suggested("MOFA2")
            message("Using MOFA+ factor loadings.")
            MOFA2::get_weights(integration_results$result)
        },
        "MCIA" = {
            .check_suggested("nipalsMCIA")
            message("Using MCIA block loadings.")
            # integration_results$result@block_loadings

            loadings <- integration_results$result@block_loadings
            # Rename columns within each blocks loading matrix
            loadings <- purrr::map(loadings, function(mat) {
                colnames(mat) <- paste0("Factor ", seq_len(ncol(mat)))
                mat
            })
            loadings
        },
        "DIABLO" = {
            message("Using DIABLO loadings.")
            integration_results$result$loadings
        },
        "RGCCA" = {
            message("Using RGCCA loadings.")
            integration_results$result$a
        },
        stop("Unsupported integration method: ", method_used)
    )

    # Convert to long format
    loadings_df <- loadings |>
        purrr::map2(names(loadings), function(df, exp_name) {
            df <- as.data.frame(df)
            df$exp_name <- exp_name
            df$feature <- rownames(df)
            rownames(df) <- NULL
            df
        }) |>
        dplyr::bind_rows() |>
        tidyr::pivot_longer(
            cols = -c(feature, exp_name),
            names_to = "factor",
            values_to = "loading"
        )

    if (method_used %in% c("DIABLO", "RGCCA")) {
        loadings_df <- loadings |>
            purrr::map2(names(loadings), function(df, exp_name) {
                df <- as.data.frame(df)
                df$feature <- rownames(df)
                df$exp_name <- exp_name
                rownames(df) <- NULL
                df
            }) |>
            dplyr::bind_rows() |>
            tidyr::pivot_longer(
                cols = -c(feature, exp_name),
                names_to = "component",
                values_to = "loading"
            ) |>
            dplyr::mutate(factor = paste(exp_name, component, sep = " "))
    }

    # Ensure factor names are character
    factors <- as.character(factors)
    loadings_df <- dplyr::filter(loadings_df, factor %in% factors)

    # Apply filtering
    filtered_features <- switch(method,
        "percentile" = {
            message(
                "Applying percentile-based filtering (>",
                percentile * 100, "%)."
            )
            loadings_df |>
                group_by(factor) |>
                mutate(rank = percent_rank(abs(loading))) |>
                filter(rank > percentile) |>
                ungroup()
        },
        "threshold" = {
            message("Applying raw threshold-based filtering (>|", threshold, "|).")
            dplyr::filter(loadings_df, abs(loading) > threshold)
        },
        stop("Invalid method. Choose 'percentile' or 'threshold'.")
    )

    message(
        "Selected ",
        nrow(filtered_features),
        " features contributing to specified factors."
    )

    # Store or return results
    if (action == "add") {
        MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck(
                "multiomics_integration",
                "top_factor_features"
            ) <- filtered_features

        step_record <- list(
            extract_top_factor_features = list(
                timestamp = Sys.time(),
                params = list(
                    factors = factors,
                    method = method,
                    percentile = percentile,
                    threshold = threshold
                ),
                notes = paste0(
                    "Selected ",
                    nrow(filtered_features),
                    " features contributing to specified factors."
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else if (action == "get") {
        return(filtered_features)
    } else {
        stop("Invalid action. Choose 'add' or 'get'.")
    }
}

# test_that("extract_top_factor_features works for MOFA", {
#     # MOFA2 has an issue running on Mac-arm64
#     # skipping on Bioconductor because of basilisk issues
#     skip_on_bioc()
#     skip_on_os("mac")
#     skip_if(grepl("arm64", R.version$platform) &&
#         Sys.info()[["sysname"]] == "Darwin")
#     skip_if_not_installed("MOFA2")
#
#     local({
#         dummy <- make_example_data(n_samples = 10)
#         mae <- create_exposomicset(
#             codebook = dummy$codebook,
#             exposure = dummy$exposure,
#             omics = dummy$omics,
#             row_data = dummy$row_data
#         )
#
#         # with this small of a dataset it needs to be 2 factors
#         mae <- run_multiomics_integration(mae, method = "MOFA", n_factors = 2)
#
#         get_factors <- get("get_factors", asNamespace("MOFA2"))
#         factors <- colnames(
#             get_factors(metadata(mae)$multiomics_integration$integration_results$result)[[1]]
#         )
#
#         top_feats <- extract_top_factor_features(
#             mae,
#             factors = factors,
#             method = "percentile",
#             percentile = 0.9,
#             action = "get"
#         )
#
#         # does this return a data frame
#         expect_s3_class(top_feats, "data.frame")
#
#         # do we get the expected column names
#         expect_true(all(c("feature", "factor", "loading", "exp_name") %in% colnames(top_feats)))
#
#         # do we have more than 0 rows for the top features
#         expect_gt(nrow(top_feats), 0)
#     })
# })

test_that("extract_top_factor_features works for MCIA", {
    # Skipping due to package RMD check failure
    skip_on_bioc()
    skip_if_not_installed("nipalsMCIA")

    dummy <- make_example_data(n_samples = 10)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    mae <- run_multiomics_integration(mae, method = "MCIA", n_factors = 3)

    # factors <- mae@metadata$multiomics_integration$integration_results$result@global_scores |>
    #     as.data.frame() |>
    #     colnames()
    factors <- c(
        "Factor 1",
        "Factor 2",
        "Factor 3"
    )

    top_feats <- extract_top_factor_features(
        mae,
        factors = factors,
        method = "percentile",
        percentile = 0.9,
        action = "get"
    )

    # does this return a data frame
    expect_s3_class(top_feats, "data.frame")

    # do we get the expected column names
    expect_true(all(c("feature", "factor", "loading", "exp_name") %in% colnames(top_feats)))

    # do we have more than 0 rows for the top features
    expect_gt(nrow(top_feats), 0)
})

test_that("extract_top_factor_features works for DIABLO", {
    skip_if_not_installed("mixOmics")

    dummy <- make_example_data(n_samples = 10)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    mae <- run_multiomics_integration(
        mae,
        method = "DIABLO",
        outcome = "smoker",
        n_factors = 3
    )

    # adding the assay name for multi-block components
    factors <- colnames(metadata(mae)$multiomics_integration$integration_results$result$variates[["mRNA"]])
    factors <- paste0("mRNA ", factors)

    top_feats <- extract_top_factor_features(
        mae,
        factors = factors,
        method = "percentile",
        percentile = 0.9,
        action = "get"
    )

    # does this return a data frame
    expect_s3_class(top_feats, "data.frame")

    # do we get the expected column names
    expect_true(all(c("feature", "factor", "loading", "exp_name") %in% colnames(top_feats)))

    # do we have more than 0 rows for the top features
    expect_gt(nrow(top_feats), 0)
})

test_that("extract_top_factor_features works for RGCCA", {
    skip_if_not_installed("RGCCA")

    dummy <- make_example_data(n_samples = 10)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    mae <- run_multiomics_integration(mae,
        method = "RGCCA",
        n_factors = 3
    )

    # adding the assay name for multi-block components
    factors <- colnames(metadata(mae)$multiomics_integration$integration_results$result$a[["mRNA"]] |>
        as.data.frame())
    factors <- paste0("mRNA ", factors)

    top_feats <- extract_top_factor_features(
        mae,
        factors = factors,
        method = "percentile",
        percentile = 0.9,
        action = "get"
    )

    # does this return a data frame
    expect_s3_class(top_feats, "data.frame")

    # do we get the expected column names
    expect_true(all(c("feature", "factor", "loading", "exp_name") %in% colnames(top_feats)))

    # do we have more than 0 rows for the top features
    expect_gt(nrow(top_feats), 0)
})

#' @keywords internal
#' @importFrom stats IQR as.dist as.formula coef cov cutree dist end
#' @importFrom stats glm hclust median na.omit prcomp qchisq qt quantile
#' @importFrom stats reorder rlnorm rnorm runif sd setNames start var
#' @importFrom utils head tail
#' @importFrom methods as

utils::globalVariables(c(
    ".exp_name", ".feature", ".sample", ":=", "binomial", "contrast",
    "Category", "Cluster", "FDR", "Freq", "GO_group", "PC1", "PC2", "Sample",
    "Var1", "abs_loading", "ancestors", "angle", "avg", "category",
    "centrality", "centrality_betweenness", "centrality_closeness",
    "centrality_degree", "centrality_eigen", "children", "cluster", "cols",
    "community", "component", "correlation", "degree", "direction", "edges",
    "edge_weight", "effect_consistency", "end", "estimate", "exp_label",
    "exp_name", "exp_name_1", "exp_name_2", "exp_name_clean",
    "exp_name_feature", "exp_name_plot", "exposure", "exposome_score_mean",
    "exposome_score_median", "exposome_score_quantile", "exposome_score_sum",
    "exposome_score_var", "feature", "from", "gene_col", "gene_count",
    "gene_list", "gene_node", "geom_edge_link", "geom_node_arc_bar",
    "geom_node_label", "geom_node_point", "ggraph", "go_group", "group",
    "id", "ids", "index", "jaccard", "label", "loading", "mean_betweenness",
    "mean_closeness", "mean_degree", "mean_eigen", "mean_log_p", "metric",
    "missingness", "n_above", "n_cat", "n_category", "n_exp", "n_exp_name",
    "n_exps", "n_omics", "n_terms", "n_values", "n_with_sel", "name",
    "node_index", "nodes", "num_shared", "omic", "overlap", "p.adjust",
    "p.value", "p_adjust", "p_value", "padj", "parents", "pct_miss",
    "presence_rate", "reorder", "sample_group", "scale_edge_width",
    "scaled_value", "score", "score_tmp", "se", "setNames", "shared",
    "source_std", "start", "std.error", "target", "target_std", "term",
    "term_name", "term_node", "thresh_met", "to", "total",
    "total_significant", "type", "value", "var", "var1", "var1_type", "var2",
    "var2_type", "var_exp", "var_label", "var_max", "var_min", "var_pc",
    "variable", "weight", "x", "y", "vertex_id","logFC", "P.Value", "adj.P.Val"
))

#' Internal - onLoad hook to register www assets
#'
#' Registers \code{inst/app/www} as a Shiny resource path \code{"www"} if present.
#'
#' @return Invisibly returns \code{NULL}
#' @keywords internal
#' @importFrom shiny addResourcePath
.onLoad <- function(libname, pkgname) {
    www <- system.file("app", "www", package = pkgname)
    if (dir.exists(www)) shiny::addResourcePath("www", www)
    invisible(NULL)
}

#' Plot Sensitivity Analysis Summary
#'
#' Generates a ridge plot and bar chart summarizing feature stability
#' scores across assays.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing sensitivity
#'  analysis results
#' in `metadata(exposomicset)$sensitivity_analysis`.
#' @param stability_score_thresh A numeric threshold for stability scores.
#' Default is `NULL`,
#' which uses the threshold stored in
#' `metadata(exposomicset)$sensitivity_analysis$score_thresh`.
#' @param stability_metric A character string specifying which
#'  stability metric to plot (e.g., "stability_score",
#'   "logp_weighted_score"). Default is "stability_score".
#' @param title A character string specifying the title of the ridge plot.
#'  Default is
#' "Distribution of Stability Scores".
#'
#' @details
#' This function:
#' - Extracts feature stability scores from `metadata(exposomicset)$sensitivity_analysis$feature_stability`.
#' - Displays a **ridge plot** of stability score distributions per assay.
#' - Displays a **bar chart** of the number of features per assay.
#' - Prints the number of features with stability scores above the threshold.
#'
#' @return A `patchwork` object combining a ridge plot and a bar chart.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#'
#' # Run differential abundance
#' mae <- run_differential_abundance(
#'     exposomicset = mae,
#'     formula = ~ smoker + sex,
#'     abundance_col = "counts",
#'     method = "limma_voom",
#'     action = "add"
#' )
#'
#' # Run the sensitivity analysis
#' mae <- run_sensitivity_analysis(
#'     exposomicset = mae,
#'     base_formula = ~ smoker + sex,
#'     methods = c("limma_voom"),
#'     scaling_methods = c("none"),
#'     covariates_to_remove = "sex",
#'     pval_col = "P.Value",
#'     logfc_col = "logFC",
#'     pval_threshold = 0.05,
#'     logFC_threshold = 0,
#'     bootstrap_n = 3,
#'     action = "add"
#' )
#'
#' # create the sensitivity summary plot
#' sens_sum_p <- mae |>
#'     plot_sensitivity_summary()
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom dplyr group_by summarise arrange left_join mutate n
#' @importFrom ggplot2 ggplot aes geom_bar geom_segment scale_fill_manual
#'   scale_color_manual theme labs geom_vline element_text
#' @importFrom ggpubr theme_pubr
#' @export
plot_sensitivity_summary <- function(
  exposomicset,
  stability_score_thresh = NULL,
  stability_metric = "stability_score",
  title = "Distribution of Stability Scores"
) {
    # require(ggplot2)
    # require(patchwork)
    .check_suggested(pkg = "forcats")
    .check_suggested(pkg = "ggridges")
    .check_suggested(pkg = "patchwork")

    if (!"sensitivity_analysis" %in% names(
        MultiAssayExperiment::metadata(exposomicset)$differential_analysis
    )) {
        stop("Please run `run_sensitivity_analysis()` first.")
    }

    feature_stability <- MultiAssayExperiment::metadata(exposomicset)$differential_analysis$sensitivity_analysis$feature_stability

    if (!stability_metric %in% colnames(feature_stability)) {
        stop(sprintf(
            "Invalid stability_metric: '%s'. Must be one of: %s",
            stability_metric,
            paste(colnames(feature_stability), collapse = ", ")
        ))
    }

    # sensitivity_sum <- feature_stability |>
    #   dplyr::group_by(exp_name) |>
    #   dplyr::summarise(n=dplyr::n()) |>
    #   dplyr::arrange(desc(n))

    sensitivity_sum <- feature_stability |>
        dplyr::group_by(exp_name) |>
        dplyr::summarise(
            n_above = sum(
                .data[[stability_metric]] > stability_score_thresh,
                na.rm = TRUE
            ),
            total = dplyr::n()
        ) |>
        dplyr::arrange(desc(total))

    sensitivity_bar <- sensitivity_sum |>
        ggplot(aes(
            x = n_above,
            y = forcats::fct_reorder(exp_name, n_above),
            fill = exp_name
        )) +
        geom_bar(stat = "identity", alpha = 0.7) +
        geom_segment(aes(
            x = n_above,
            xend = n_above,
            y = as.numeric(forcats::fct_reorder(exp_name, n_above)) - 0.45,
            yend = as.numeric(forcats::fct_reorder(exp_name, n_above)) + 0.45,
            color = exp_name,
        ), linewidth = 1) +
        scale_fill_tidy_exp() +
        scale_color_tidy_exp() +
        ggpubr::theme_pubr(legend = "none") +
        theme(
            plot.title = element_text(face = "bold.italic"),
            axis.text.y = element_blank(),
            text = element_text(size = 10)
        ) +
        labs(
            title = "",
            y = "",
            x = "No. of Features"
        )

    if (is.null(stability_score_thresh)) {
        stability_score_thresh <- MultiAssayExperiment::metadata(exposomicset)$differential_analysis$sensitivity_analysis$score_thresh
    }

    sensitivity_ridgeplot <- feature_stability |>
        dplyr::left_join(sensitivity_sum,
            by = "exp_name"
        ) |>
        mutate(exp_name = paste0(exp_name, ": (", n_above, "/", total, ")")) |>
        ggplot(aes(
            x = .data[[stability_metric]],
            y = forcats::fct_reorder(exp_name, n_above),
            fill = exp_name
        )) +
        ggridges::geom_density_ridges() +
        scale_fill_tidy_exp() +
        theme_minimal() +
        geom_vline(
            xintercept = stability_score_thresh,
            linetype = "dashed",
            color = "grey55"
        ) +
        theme(
            plot.title = element_text(face = "bold.italic"),
            legend.position = "none",
            axis.text.y = element_text(size = 10, color = "black")
        ) +
        labs(
            fill = "Assay",
            x = "Stability Score",
            y = "",
            title = title
        )


    message(
        "Number of Features with ",
        stability_metric,
        " > ",
        stability_score_thresh, ":"
    )
    for (i in seq_len(nrow(sensitivity_sum))) {
        message(
            sensitivity_sum$exp_name[i],
            ": ",
            sensitivity_sum$n_above[i],
            " / ",
            sensitivity_sum$total[i]
        )
    }

    return((sensitivity_ridgeplot | sensitivity_bar) +
        patchwork::plot_layout(
            widths = c(3, 1)
        ))
}

#' Plot Top Features by Factor from Integration Results
#'
#' Visualizes the top loading features for each factor from multi-omics
#' integration results (e.g., MOFA, MCIA, DIABLO, RGCCA).
#'
#' @param exposomicset A `MultiAssayExperiment` object containing
#' integration results in the `metadata` slot (must include `integration_results`).
#' @param feature_col A character string indicating the column name to use
#' for y-axis feature labels (e.g., `"feature"`, `"gene_symbol"`). This should
#' match a column in the output of `pivot_feature()`. Default is `"feature"`.
#' @param factors Character vector of factors to include
#' (e.g., "Factor1", "Factor2"). If `NULL`, all factors are plotted.
#' @param top_n Integer specifying the number of top features to show
#' per factor. Default is `5`.
#' @param facet_cols Optional color palette for facet strip backgrounds
#' (one per `exp_name`), used to distinguish factors.
#' @param exp_name_cols Optional color palette for experiment labels
#' in the plot (`exp_name`), passed to `scale_color_manual()`.
#' @param alpha Numeric value between 0 and 1 controlling the
#' transparency of facet strip background fill. Default is `0.5`.
#'
#' @details
#' This function supports the following integration methods:
#' - `"MOFA"`: Uses feature weights from MOFA2 (`get_weights()`).
#' - `"MCIA"`: Uses block loadings from MCIA (`@block_loadings`).
#' - `"DIABLO"`: Extracts block-specific loadings from `loadings`.
#' - `"RGCCA"`: Extracts block-specific loadings from `a`.
#'
#' For each factor, it:
#' - Selects the top `top_n` features by **absolute loading**.
#' - Merges with feature metadata using `pivot_feature()`.
#' - Creates a point-range plot showing the loading magnitude.
#' - Facets each factor with a customizable strip background.
#'
#' The `feature_col` argument allows you to control which feature-level
#' metadata column (e.g., gene symbols, metabolite names) is used for
#' labeling the y-axis.
#'
#' If palettes are not provided, defaults are chosen using
#' `ggpubr::get_palette()`.
#'
#' @return A `ggplot2` object with one facet per factor, showing the
#' top features and their loadings by experiment.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' mae <- run_multiomics_integration(
#'     mae,
#'     method = "DIABLO",
#'     outcome = "smoker",
#'     n_factors = 3
#' )
#'
#' # plot top features using default `feature` column
#' top_feature_p <- mae |>
#'     plot_top_factor_features()
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom purrr map2
#' @importFrom tibble rownames_to_column
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr mutate filter group_by arrange desc slice_head
#' bind_rows inner_join sym
#' @importFrom ggplot2 ggplot aes geom_point geom_segment theme_bw
#' theme labs scale_color_manual element_text
#' @importFrom scales alpha
#' @export
plot_top_factor_features <- function(
  exposomicset,
  feature_col = "feature",
  factors = NULL,
  top_n = 5,
  facet_cols = NULL,
  exp_name_cols = NULL,
  alpha = 0.5
) {
    # require(ggplot2)
    .check_suggested(pkg = "ggh4x")

    method <- MultiAssayExperiment::metadata(exposomicset)$multiomics_integration$integration_results$method
    result <- MultiAssayExperiment::metadata(exposomicset)$multiomics_integration$integration_results$result

    loadings_df <- switch(method,
        "MOFA" = {
            .check_suggested("MOFA2")
            MOFA2::get_weights(result) |>
                purrr::map2(
                    names(MOFA2::get_weights(result)),
                    ~ as.data.frame(.x) |>
                        tibble::rownames_to_column("feature") |>
                        tidyr::pivot_longer(-feature,
                            names_to = "factor",
                            values_to = "loading"
                        ) |>
                        dplyr::mutate(
                            abs_loading = abs(loading),
                            exp_name = .y
                        )
                ) |>
                dplyr::bind_rows()
        },
        "MCIA" = {
            result@block_loadings |>
                purrr::map2(
                    names(result@block_loadings),
                    ~ as.data.frame(.x) |>
                        tibble::rownames_to_column("feature") |>
                        tidyr::pivot_longer(-feature,
                            names_to = "factor",
                            values_to = "loading"
                        ) |>
                        dplyr::mutate(
                            abs_loading = abs(loading),
                            exp_name = .y
                        )
                ) |>
                dplyr::bind_rows()
        },
        "DIABLO" = {
            result$loadings |>
                purrr::map2(
                    names(result$loadings),
                    ~ as.data.frame(.x) |>
                        tibble::rownames_to_column("feature") |>
                        tidyr::pivot_longer(-feature,
                            names_to = "factor",
                            values_to = "loading"
                        ) |>
                        dplyr::mutate(
                            abs_loading = abs(loading),
                            exp_name = .y
                        )
                ) |>
                dplyr::bind_rows() |>
                dplyr::mutate(factor = paste(exp_name, factor, sep = " "))
        },
        "RGCCA" = {
            result$a |>
                purrr::map2(
                    names(result$a),
                    ~ as.data.frame(.x) |>
                        tibble::rownames_to_column("feature") |>
                        tidyr::pivot_longer(-feature,
                            names_to = "factor",
                            values_to = "loading"
                        ) |>
                        dplyr::mutate(
                            abs_loading = abs(loading),
                            exp_name = .y
                        )
                ) |>
                dplyr::bind_rows() |>
                dplyr::mutate(factor = paste(exp_name, factor, sep = " "))
        },
        stop("Method not supported.")
    )

    # Filter by factors if provided
    if (!is.null(factors)) {
        loadings_df <- loadings_df |>
            dplyr::filter(factor %in% factors)
    }

    # Select top features per factor
    df <- loadings_df |>
        dplyr::group_by(factor) |>
        dplyr::arrange(dplyr::desc(abs_loading)) |>
        dplyr::slice_head(n = top_n)

    # Map to the feature data
    df <- df |>
        inner_join(pivot_feature(exposomicset),
            by = c(
                "feature" = ".feature",
                "exp_name" = ".exp_name"
            )
        )

    # If no facet_cols provided, use default
    facet_cols <- facet_cols %||% tidy_exp_pal[
        seq_len(length(unique(df$factor)))
    ]

    # If no exp_name_cols provided, use default
    exp_name_cols <- exp_name_cols %||% rev(tidy_exp_pal)[
        seq_len(length(unique(df$exp_name)))
    ]


    # Create a plot of top features per factor
    features_per_factor_plot <- df |>
        ggplot(aes(
            x = abs_loading,
            y = reorder(!!dplyr::sym(feature_col), abs_loading),
            color = exp_name
        )) +
        geom_point(shape = 18, size = 5, alpha = 0.5) +
        geom_segment(
            aes(
                x = 0,
                xend = abs_loading,
                y = !!dplyr::sym(feature_col),
                yend = !!dplyr::sym(feature_col)
            ),
            color = "grey55"
        ) +
        theme_bw() +
        theme(
            strip.text.y = element_text(face = "bold.italic", angle = 0),
            axis.text.y = element_text(face = "italic")
        ) +
        ggh4x::facet_grid2(
            factor ~ .,
            scales = "free_y",
            space = "free_y",
            strip = ggh4x::strip_themed(
                background_y = ggh4x::elem_list_rect(
                    fill = scales::alpha(facet_cols, alpha)
                )
            )
        ) +
        scale_color_manual(values = exp_name_cols) +
        labs(
            x = "Absolute loading",
            y = "",
            color = "Experiment"
        )

    return(features_per_factor_plot)
}

## ----setup,include=FALSE------------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----pkg-overview, echo=FALSE,out.width="150%",fig.align='center',fig.cap="tidyexposomics pipeline overview. QC, association testing, integration, and enrichment steps on a MultiAssayExperiment."----
knitr::include_graphics("./overview.png")

## ----install, eval=FALSE------------------------------------------------------
# # install the package
# BiocManager::install("tidyexposomics")
# 
# # load the package
# library(tidyexposomics)

## ----command-str, echo=FALSE,fig.align='center',fig.cap= "Command naming conventions used throughout `tidyexposomics`. More complex pipelines begin with the `run_` prefix, visualizations with `plot_`, and data processing with `filter_`, `transform_`, `pivot_`, or `extract_` prefixes.",out.width="80%"----

knitr::include_graphics("./command_str.png")

## ----mae-overview,echo=FALSE,fig.align='center',fig.cap="Overview of the MultiAssayExperiment structure linking samples, assays, and metadata."----
knitr::include_graphics("./mae_rep.png")

## ----load-libraries,message=FALSE,warning=FALSE-------------------------------
# Load Libraries
library(tidyverse)
library(tidyexposomics)

## ----load-data----------------------------------------------------------------
# Load example data
data("tidyexposomics_example")

# Create exposomic set object
expom <- create_exposomicset(
    codebook = tidyexposomics_example$annotated_cb,
    exposure = tidyexposomics_example$meta,
    omics = list(
        "Gene Expression" = tidyexposomics_example$exp_filt,
        "Methylation" = tidyexposomics_example$methyl_filt
    ),
    row_data = list(
        "Gene Expression" = tidyexposomics_example$exp_fdata,
        "Methylation" = tidyexposomics_example$methyl_fdata
    )
)

## ----exp-vars-----------------------------------------------------------------
# Grab exposure variables
exp_vars <- tidyexposomics_example$annotated_cb |>
    filter(category %in% c(
        "aerosol",
        "main group molecular entity",
        "polyatomic entity"
    )) |>
    pull(variable) |>
    as.character()

## ----missing-bar,fig.height=3,fig.width=6,fig.cap="Count of features with missing data above a 0% missingness threshold by data layer. Exposure data have variables with missingness."----

# Plot the missingness summary
plot_missing(
    exposomicset = expom,
    plot_type = "summary",
    threshold = 0
)

## ----missing-bar-lollipop,fig.height=2,fig.width=4,fig.cap= "Percent missingness per exposure variable. Parity, `h_parity_None`, shows the highest missingness."----

# Plot missing variables withing exposure group
plot_missing(
    exposomicset = expom,
    plot_type = "lollipop",
    threshold = 0,
    layers = "Exposure"
)

## ----impute-missing-----------------------------------------------------------

# Impute missing values
expom <- run_impute_missing(
    exposomicset = expom,
    exposure_impute_method = "missforest",
    exposure_cols = exp_vars
)

## ----filter-omics-------------------------------------------------------------

# filter omics layers by variance and expression
# Methylation filtering
expom <- filter_omics(
    exposomicset = expom,
    method = "variance",
    assays = "Methylation",
    assay_name = 1,
    min_var = 0.05
)

# Gene expression filtering
expom <- filter_omics(
    exposomicset = expom,
    method = "expression",
    assays = "Gene Expression",
    assay_name = 1,
    min_value = 1,
    min_prop = 0.3
)

## ----normality-check----------------------------------------------------------

# Check variable normality
expom <- run_normality_check(
    exposomicset = expom,
    action = "add"
)

## ----trasform-vars------------------------------------------------------------

# Transform variables
expom <- transform_exposure(
    exposomicset = expom,
    transform_method = "boxcox_best",
    exposure_cols = exp_vars
)

## ----norm-plot,fig.width= 4,fig.height= 4, fig.cap= "Normality status of numeric exposure variables after Box-Cox transformation."----

# Examine normality summary
plot_normality_summary(
    exposomicset = expom,
    transformed = TRUE
)

## ----pca----------------------------------------------------------------------

# Perform principal component analysis
expom <- run_pca(
    exposomicset = expom,
    log_trans_exp = TRUE,
    log_trans_omics = TRUE,
    action = "add"
)

## ----pca-plot, fig.align='center', fig.width= 9, fig.height= 7,fig.cap= "PCA of sample and feature space with sample outlier detection."----

# Plot the PCA plot of sample and feature space
plot_pca(exposomicset = expom)

## ----filt-outliers------------------------------------------------------------

# Filter out sample outliers
expom <- filter_sample_outliers(
    exposomicset = expom,
    outliers = c("s1231")
)

## ----corr-pcs-----------------------------------------------------------------

# Run the correlation analysis
expom <- run_correlation(
    exposomicset = expom,
    feature_type = "pcs",
    exposure_cols = exp_vars,
    n_pcs = 20,
    action = "add",
    correlation_cutoff = 0,
    pval_cutoff = 1
)

## ----plot-pc-corr,fig.width= 7,fig.height= 2.2,fig.cap= "Correlation heatmap of exposures versus principal components. Child lead levels (`hs_pb_c_Log2`) and maternal BPA levels (`hs_bpa_madj_Log2`) are associated with the most principal components."----


# Plot the correlation tile plot
plot_correlation_tile(
    exposomicset = expom,
    feature_type = "pcs",
    pval_cutoff = 0.05
)

## ----exp-sum, fig.width=6, fig.height=4---------------------------------------

# Summarize exposure data
run_summarize_exposures(
    exposomicset = expom,
    action = "get",
    exposure_cols = exp_vars
) |>
    head()

## ----plot-aerosol, fig.width=4, fig.height=3.5, fig.cap="Distribution of aerosol exposures by sex."----

# Plot aerosol exposure distributions by sex
plot_exposures(
    exposomicset = expom,
    group_by = "e3_sex_None",
    exposure_cat = "aerosol",
    plot_type = "boxplot",
    ylab = "Values",
    title = "Aerosol Exposure by Sex"
)

## ----sample-clusters----------------------------------------------------------

# Sample clustering
expom <- run_cluster_samples(
    exposomicset = expom,
    exposure_cols = exp_vars,
    clustering_approach = "dynamic",
    action = "add"
)

## ----plot-sample-clusters, fig.height=4, fig.width=12, fig.cap="Sample clustering heatmap using exposure profiles (z-scored). Clusters appear mostly driven by aerosol exposure during pregnancy."----

# Plot the sample clusters
plot_sample_clusters(
    exposomicset = expom,
    exposure_cols = exp_vars
)

## ----correlate-exposures------------------------------------------------------

# Run correlation analysis
expom <- run_correlation(
    exposomicset = expom,
    feature_type = "exposures",
    action = "add",
    exposure_cols = exp_vars,
    correlation_cutoff = 0.3
)

## ----exposure-circos-corr, fig.height=10,fig.width=10,fig.align='center',fig.cap="Circos view of exposure-exposure correlations (threshold 0.3)."----

# Plot exposure correlation circos plot
plot_circos_correlation(
    exposomicset = expom,
    feature_type = "exposures",
    corr_threshold = 0.3,
    exposure_cols = exp_vars
)

## ----assoc-exposures----------------------------------------------------------

# Perform ExWAS
expom <- run_association(
    exposomicset = expom,
    source = "exposures",
    outcome = "hs_asthma",
    feature_set = exp_vars,
    action = "add",
    family = "binomial"
)

## ----plot-exposure-assoc,fig.height=3, fig.width=5, fig.align='center',fig.cap="ExWAS associations of exposures with asthma status. No exposures are significantly associated (P < 0.05) with asthma status."----

# Plot the association forest plot
plot_association(
    exposomicset = expom,
    source = "exposures",
    terms = exp_vars,
    filter_thresh = 0.05,
    filter_col = "p.value",
    r2_col = "r2"
)

## ----associate-top-omics,fig.height=3,fig.width=4.5,fig.align='center'--------

# Perform ExWAS
expom <- run_association(
    exposomicset = expom,
    outcome = "hs_asthma",
    source = "omics",
    top_n = 500,
    action = "add",
    family = "binomial"
)

## ----plot-manhattan,fig.height=7, fig.width=5, fig.align='center',fig.cap="Manhattan plot of omics-wide associations with asthma status."----

# Plot the manhattan plot
plot_manhattan(
    exposomicset = expom,
    min_per_cat = 0,
    feature_col = "feature_clean",
    vars_to_label = c(
        "TC19001180.hg.1",
        "TC01000565.hg.1",
        "cg01937701",
        "hs_mibp_cadj_Log2"
    ),
    panel_sizes = c(1, 3, 1, 3, 1, 1, 1),
    facet_angle = 0
)

## ----diff-abundance,results='hide'--------------------------------------------

# Run differential abundance analysis
expom <- run_differential_abundance(
    exposomicset = expom,
    formula = ~hs_asthma,
    method = "limma_trend",
    scaling_method = "none",
    action = "add"
)

## ----volcano-plot,fig.height=3.5, fig.width=6, fig.align='center',fig.cap="Volcano plot of differentially abundant features across omics layers."----

# Plot the volcano plot
plot_volcano(
    exposomicset = expom,
    top_n_label = 3,
    feature_col = "feature_clean",
    logFC_thresh = log2(1),
    pval_thresh = 0.05,
    pval_col = "P.Value",
    logFC_col = "logFC",
    nrow = 1
)

## ----exp-omics-assoc, fig.align='center'--------------------------------------
# Run association testing between every exposure and omics feature
expom <- run_exposure_omics_association(
    exposomicset = expom,
    exposures = exp_vars,
    action = "add"
)

## ----plot-exp-omics-assoc, fig.align='center',fig.width=7,fig.height=2.5,fig.cap="Barplot of the number of omics features associated with exposures."----

# Plot the number of exposure-omics associations
plot_exposure_omics_association(
  exposomicset = expom,
  plot_type = "category",
  pval_col = "p.value",
  pval_thresh = 0.05)

## ----var-map------------------------------------------------------------------
# Extract omics features associated with aerosols
var_map <- extract_results(
  exposomicset = expom,
  result = "association"
) |> 
  pluck("exposure_omics",
        "results_df") |> 
  filter(p.value<0.05) |>
  filter(category == "aerosol") |> 
  dplyr::select(
    exp_name = exp_name,
    variable = feature
    
  )

## ----enrichment---------------------------------------------------------------
# Run enrichment analysis on factor features correlated with exposures
expom <- run_enrichment(
    exposomicset = expom,
    variable_map = var_map,
    feature_type = "omics",
    feature_col = "feature_clean",
    db = c("GO"),
    species = "goa_human",
    fenr_col = "gene_symbol",
    padj_method = "none",
    pval_thresh = 0.1,
    min_set = 1,
    max_set = 800,
    clustering_approach = "diana",
    action = "add"
)

## ----enrichment-summary,fig.width=14,fig.height=5, fig.cap="Summary of enriched GO terms grouped by overlap and exposure category."----

# Plot the summary diagram
plot_enrichment(
    exposomicset = expom,
    feature_type = "omics",
    plot_type = "summary"
)

## ----enrichment-dotplot,fig.height=11, fig.width=11, fig.cap="Dotplot of top enriched GO terms by omics layer and exposure category."----

# Plot a dotplot of terms
plot_enrichment(
    exposomicset = expom,
    feature_type = "omics",
    plot_type = "dotplot",
    top_n = 15,
    add_top_genes = TRUE,
    top_n_genes = 5
)

## ----enrichment-term-network, fig.width=6, fig.height=5, fig.cap="Network of enriched GO terms connected by shared genes."----

# Plot the term network plot
# Setting a seed so that the plot layout is consistent
set.seed(42)
plot_enrichment(
    exposomicset = expom,
    feature_type = "omics",
    plot_type = "network",
    label_top_n = 3
)

## ----enrichment-heatmap, warning=FALSE,fig.height=3, fig.width=8, fig.cap="Heatmap of genes driving enriched GO terms (Group 2) with log2 fold-change overlay."----

# Plot a heatmap of genes and corresponding GO terms
plot_enrichment(
    exposomicset = expom,
    feature_type = "omics",
    go_groups = "Group 2",
    plot_type = "heatmap",
    heatmap_fill = TRUE,
    feature_col = "feature_clean"
)

## ----enrichment-cnetplot, fig.width=6, fig.height=6, fig.cap="Cnet plot linking enriched terms to contributing genes (Group 1)."----

# Plot the gene-term network
# Setting a seed so that the plot layout is consistent
set.seed(42)
plot_enrichment(
    exposomicset = expom,
    feature_type = "omics",
    go_groups = "Group 2",
    plot_type = "cnet",
    feature_col = "feature_clean"
)

## ----pipeline-summary---------------------------------------------------------

# Run the pipeline summary
expom |>
    run_pipeline_summary(console_print = TRUE, include_notes = TRUE)

## ----save-results-------------------------------------------------------------

# Save results
extract_results_excel(
    exposomicset = expom,
    file = tempfile(),
    result_types = "all"
)

## ----session_info-------------------------------------------------------------

sessionInfo()

#' Create an Exposomicset Object
#'
#' Constructs a `MultiAssayExperiment` object from exposure data and optionally
#' omics datasets, ensuring proper formatting and alignment of samples and
#' features. For epidemiology-only workflows, omics data can be omitted.
#'
#' @param codebook A data frame containing variable information metadata.
#' @param exposure A data frame containing exposure data,
#' with rows as samples and columns as variables.
#' @param omics An optional list of matrices or a single matrix representing
#' omics data. Each matrix should have samples as columns and features as rows.
#' If `NULL`, creates an exposure-only exposomicset. Default is `NULL`.
#' @param row_data An optional list of `DataFrame` objects providing
#' feature metadata for each omics dataset. If `NULL`,
#' row metadata is generated automatically. Default is `NULL`.
#'
#' @details
#' The function validates inputs and creates a `MultiAssayExperiment` object.
#' If omics data is provided, it converts matrices into `SummarizedExperiment`
#' objects with proper sample alignment. If omics is `NULL`, the function
#' creates an exposure-only object suitable for epidemiological analyses
#' using `run_association()` with `source = "exposures"`.
#'
#' @return A `MultiAssayExperiment` object containing the formatted exposure
#' and optionally omics datasets.
#'
#' @examples
#' # Epi user workflow
#' # so no omics data
#' epi_data <- data.frame(
#'     pm25 = rnorm(10),
#'     outcome = rbinom(10, 1, 0.5),
#'     age = rnorm(10, 45, 10),
#'     row.names = paste0("subj_", 1:10)
#' )
#'
#' codebook <- data.frame(
#'     variable = c("pm25", "outcome", "age"),
#'     category = c("exposure", "outcome", "covariate")
#' )
#'
#' mae <- create_exposomicset(
#'     codebook = codebook,
#'     exposure = epi_data
#' )
#'
#' # Multi-omics workflow
#' tmp <- make_example_data(n_samples = 10)
#'
#' mae <- create_exposomicset(
#'     codebook = tmp$codebook,
#'     exposure = tmp$exposure,
#'     omics = tmp$omics,
#'     row_data = tmp$row_data
#' )
#'
#' @export
create_exposomicset <- function(
  codebook,
  exposure,
  omics = NULL,
  row_data = NULL
) {
    # Validate exposure input
    if (!is.data.frame(exposure)) {
        stop("The 'exposure' argument must be a data frame.")
    }
    if (is.null(rownames(exposure))) {
        stop("Exposure data must have rownames.")
    }

    col_data <- S4Vectors::DataFrame(exposure, row.names = rownames(exposure))

    # Handle epi-only case (no omics)
    if (is.null(omics)) {
        message("No omics data provided. Creating exposure-only exposomicset.")

        # Create a minimal placeholder experiment with 0 features
        placeholder <- SummarizedExperiment::SummarizedExperiment(
            assays = S4Vectors::SimpleList(
                placeholder = matrix(
                    nrow = 0,
                    ncol = nrow(exposure),
                    dimnames = list(NULL, rownames(exposure))
                )
            )
        )

        mae <- MultiAssayExperiment::MultiAssayExperiment(
            experiments = list(.exposures = placeholder),
            colData = col_data,
            metadata = list(codebook = codebook)
        )
        message("MultiAssayExperiment created successfully.")
        return(mae)
    }

    # Validate omics-related inputs
    if (!is.null(row_data) && !is.list(row_data)) {
        stop("The 'row_data' argument must be a list if provided.")
    }

    # Convert a single matrix input into a named list
    if (is.matrix(omics)) {
        omics <- list(single_omic = omics)
    } else if (!is.list(omics)) {
        stop("The 'omics' argument must be a list or a single matrix.")
    }

    if (!is.null(row_data) && length(row_data) != length(omics)) {
        stop("Length of 'row_data' must match 'omics'.")
    }

    # Ensure all datasets in omics are matrices with column names
    message("Ensuring all omics datasets are matrices with column names.")
    omics <- lapply(omics, function(x) {
        if (!is.matrix(x)) x <- as.matrix(x)
        if (is.null(colnames(x))) {
            colnames(x) <- paste0("Sample_", seq_len(ncol(x)))
        }
        x
    })

    # Create row_data if not provided and ensure feature order matches assay rows
    row_data <- row_data %||% lapply(omics, function(df) {
        S4Vectors::DataFrame(row.names = rownames(df))
    })

    # Ensure rowData order matches assay rownames
    row_data <- mapply(function(row_meta, data) {
        row_meta <- row_meta[
            match(rownames(data), rownames(row_meta)), ,
            drop = FALSE
        ]
        row_meta
    }, row_meta = row_data, data = omics, SIMPLIFY = FALSE)

    # Create SummarizedExperiment objects with ordered samples and rowData
    message("Creating SummarizedExperiment objects.")
    experiments <- mapply(
        function(data, row_meta) {
            sample_order <- sort(colnames(data))
            data <- data[, sample_order, drop = FALSE]
            SummarizedExperiment::SummarizedExperiment(
                assays = S4Vectors::SimpleList(counts = data),
                rowData = row_meta
            )
        },
        data = omics,
        row_meta = row_data,
        SIMPLIFY = FALSE
    )

    # Create MultiAssayExperiment
    message("Creating MultiAssayExperiment object.")
    mae <- MultiAssayExperiment::MultiAssayExperiment(
        experiments = experiments,
        colData = col_data,
        metadata = list(codebook = codebook)
    )

    message("MultiAssayExperiment created successfully.")
    return(mae)
}

test_that("filter_non_normal removes non-normal exposures in exposomicset", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 100)

    # Create MultiAssayExperiment
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # run the normality check
    mae <- mae |>
        run_normality_check() |>
        transform_exposure()

    # Ensure normality metadata is present
    norm_df <- metadata(mae)$quality_control$transformation$norm_df
    expect_true(is.data.frame(norm_df))
    expect_true(all(c("exposure", "p.value") %in% colnames(norm_df)))

    # Identify non-normal exposures
    non_normal <- norm_df$exposure[norm_df$p.value < 0.05]

    # Apply filter
    filtered <- filter_non_normal(mae, p_thresh = 0.05)

    # Ensure non-normal exposures are removed from main colData
    expect_false(any(non_normal %in% colnames(colData(filtered))))

    # Ensure non-normal exposures are removed from all experiment colData
    for (exp_name in names(experiments(filtered))) {
        expect_false(any(non_normal %in% colnames(colData(experiments(filtered)[[exp_name]]))))
    }

    # Confirm metadata was updated
    steps <- metadata(filtered)$summary$steps
    expect_true("filter_non_normal" %in% names(steps))
    expect_match(steps$filter_non_normal$notes, "Filtered out non-normal exposure variables")
})

# test_that("run_multiomics_integration works with MOFA", {
#     # skipping on Bioconductor because of basilisk issues
#     skip_on_bioc()
#     # MOFA2 has an issue running on Mac-arm64
#     skip_on_os("mac")
#     skip_if(grepl("arm64", R.version$platform) &&
#         Sys.info()[["sysname"]] == "Darwin")
#     skip_if_not_installed("MOFA2")
#
#     local({
#         dummy <- make_example_data(n_samples = 20)
#         mae <- create_exposomicset(
#             codebook = dummy$codebook,
#             exposure = dummy$exposure,
#             omics = dummy$omics,
#             row_data = dummy$row_data
#         )
#
#         mae <- run_multiomics_integration(
#             mae,
#             method = "MOFA",
#             n_factors = 3,
#             scale = TRUE,
#             action = "add"
#         )
#
#         res <- metadata(mae)$multiomics_integration$integration_results
#
#         # check that the right method was recorded
#         expect_equal(res$method, "MOFA")
#
#         # safely grab MOFA2 functions
#         get_factors <- get("get_factors", asNamespace("MOFA2"))
#         get_weights <- get("get_weights", asNamespace("MOFA2"))
#
#         # ensure that sample level scores are captured
#         expect_equal(
#             rownames(get_factors(res$result)[[1]]) |> sort(),
#             rownames(mae@colData) |> sort()
#         )
#
#         # ensure that loading scores are captured
#         for (exp_name in names(mae@ExperimentList)) {
#             expect_equal(
#                 rownames(mae@ExperimentList[[exp_name]]),
#                 rownames(get_weights(res$result)[[exp_name]])
#             )
#         }
#
#         # ensure that the step was recorded
#         step_names <- names(metadata(mae)$summary$steps)
#         expect_true(any(grepl("run_multiomics_integration", step_names)))
#     })
# })


test_that("run_multiomics_integration works with MCIA", {
    # Skipping on Bioc due to RMD check failure
    skip_on_bioc()
    skip_if_not_installed("nipalsMCIA")

    # make the example data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # run multiomics integration
    mae <- run_multiomics_integration(
        mae,
        method = "MCIA",
        n_factors = 3,
        scale = TRUE,
        action = "add"
    )

    # grab the integration results
    res <- metadata(mae)$multiomics_integration$integration_results

    # check that the right method was recorded
    expect_equal(res$method, "MCIA")

    # ensure that sample level scores are captured
    # by seeing if the sample ids are present
    expect_equal(
        rownames(res$result@global_scores) |>
            sort(),
        rownames(mae@colData) |>
            sort()
    )

    # ensure that loading scores are captured
    for (exp_name in names(mae@ExperimentList)) {
        expect_equal(
            rownames(mae@ExperimentList[[exp_name]]),
            rownames(res$result@block_loadings[[exp_name]])
        )
    }

    # ensure that the step was recorded
    step_names <- names(metadata(mae)$summary$steps)
    expect_true(any(grepl("run_multiomics_integration", step_names)))
})


test_that("run_multiomics_integration works with RGCCA", {
    skip_if_not_installed("RGCCA")

    # make the example data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # run multiomics integration
    mae <- run_multiomics_integration(
        mae,
        method = "RGCCA",
        n_factors = 3,
        scale = TRUE,
        action = "add"
    )

    # grab the integration results
    res <- metadata(mae)$multiomics_integration$integration_results

    # check that the right method was recorded
    expect_equal(res$method, "RGCCA")


    # ensure that sample and loading scores are captured
    for (exp_name in names(mae@ExperimentList)) {
        # ensure that sample level scores are captured
        # by seeing if the sample ids are present
        # doing this inside the loop since these are calculated
        # on a block-wise basis
        expect_equal(
            rownames(res$result$Y[[exp_name]]) |>
                sort(),
            rownames(mae@colData) |>
                sort()
        )
        # ensure loading scores are present
        expect_equal(
            rownames(mae@ExperimentList[[exp_name]]),
            rownames(res$result$a[[exp_name]])
        )
    }

    # ensure that the step was recorded
    step_names <- names(metadata(mae)$summary$steps)
    expect_true(any(grepl("run_multiomics_integration", step_names)))
})


test_that("run_multiomics_integration works with DIABLO", {
    skip_if_not_installed("mixOmics")

    # make the example data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # run multiomics integration
    mae <- run_multiomics_integration(
        mae,
        method = "DIABLO",
        outcome = "smoker",
        n_factors = 3,
        scale = TRUE,
        action = "add"
    )

    # grab the integration results
    res <- metadata(mae)$multiomics_integration$integration_results

    # check that the right method was recorded
    expect_equal(res$method, "DIABLO")


    # ensure that sample and loading scores are captured
    for (exp_name in names(mae@ExperimentList)) {
        # ensure that sample level scores are captured
        # by seeing if the sample ids are present
        # doing this inside the loop since these are calculated
        # on a block-wise basis
        expect_equal(
            rownames(res$result$variates[[exp_name]]) |>
                sort(),
            rownames(mae@colData) |>
                sort()
        )
        # ensure loading scores are present
        expect_equal(
            rownames(mae@ExperimentList[[exp_name]]),
            rownames(res$result$loadings[[exp_name]])
        )
    }

    # ensure that the step was recorded
    step_names <- names(metadata(mae)$summary$steps)
    expect_true(any(grepl("run_multiomics_integration", step_names)))
})

test_that("extract_omics_exposure_df works with and without variable_map", {
    dummy <- make_example_data(n_samples = 10)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Without variable_map
    merged_df <- extract_omics_exposure_df(mae)

    # is it a df
    expect_s3_class(merged_df, "data.frame")

    # are the rownames of the returned df the same as the rownames of the
    # original meta data
    expect_true(all(rownames(merged_df) %in% rownames(dummy$exposure)))

    # ensure that the colnames have the right prefixes
    # should be features
    expect_true(any(grepl("mRNA_", colnames(merged_df))))
    expect_true(any(grepl("proteomics_", colnames(merged_df))))

    # ensure the number of columns is greater
    # than the number of exposures alone
    expect_gt(ncol(merged_df), 5)

    # With variable_map
    gene_subset <- rownames(SummarizedExperiment::assay(mae[["mRNA"]]))[1:5]
    prot_subset <- rownames(SummarizedExperiment::assay(mae[["proteomics"]]))[1:5]
    var_map <- rbind(
        data.frame(variable = gene_subset, exp_name = "mRNA"),
        data.frame(variable = prot_subset, exp_name = "proteomics"),
        data.frame(variable = c("age", "bmi"), exp_name = "exposure")
    )

    merged_df_map <- extract_omics_exposure_df(mae, variable_map = var_map)

    # ensure we have the right prefixes - again features
    expect_true(any(grepl("^mRNA_", colnames(merged_df_map))))
    expect_true(any(grepl("^proteomics_", colnames(merged_df_map))))

    # make sure that we have our metadata variables in the returned
    # df
    expect_true(all(c("age", "bmi") %in% colnames(merged_df_map)))
})

test_that("extract_omics_exposure_df handles log2 transformation correctly", {
    dummy <- make_example_data(n_samples = 10)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    df_log2 <- extract_omics_exposure_df(mae, log2_trans = TRUE)
    df_raw <- extract_omics_exposure_df(mae, log2_trans = FALSE)

    m_col <- grep("^mRNA_", colnames(df_log2), value = TRUE)[1]

    # ensure the log2 transformed values are less than the raw values
    expect_lt(mean(df_log2[[m_col]]), mean(df_raw[[m_col]]))
})

test_that("filter_missing removes features and generates QC plots", {
    skip_if_not_installed("naniar")

    # Create dummy data with injected missingness
    dummy <- make_example_data(n_samples = 10)
    dummy$exposure$age[1:3] <- NA
    dummy$omics$mRNA[1:2, 1:5] <- NA
    dummy$omics$proteomics[1:4, 1:3] <- NA

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # filter it so that variables with more than 20% NAs are filtered out
    mae_filtered <- filter_missing(mae, na_thresh = 20)

    # Test filtered exposure columns
    # get the missing summary for the coldata
    col_missing <- naniar::miss_var_summary(as.data.frame(colData(mae)))

    # what do we expect to be removed
    expected_removed <- col_missing$variable[col_missing$pct_miss > 20]

    # we want this to be false becuase the removed variables should not be
    # in the coldata of the filtered object
    expect_false(any(expected_removed %in% colnames(colData(mae_filtered))))

    # Test filtered omics features
    for (omic in names(experiments(mae))) {
        # check the rownames of the original data and filtered
        raw_feats <- rownames(dummy$omics[[omic]])
        filtered_feats <- rownames(mae_filtered[[omic]])

        # transpose the omics data
        assay_data <- t(dummy$omics[[omic]])

        # generate the missinness summary
        miss_summary <- naniar::miss_var_summary(as.data.frame(assay_data))

        # what do we expect to be removed
        expected_removed <- miss_summary$variable[miss_summary$pct_miss > 20]

        # ensure that the features with too many NAs are out of the object
        expect_false(any(expected_removed %in% filtered_feats))
    }
})

test_that("run_normality_check works as expected", {
    # make the example data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # run the normality check
    mae_checked <- run_normality_check(mae,
        action = "add"
    )

    # grab the results from metadata
    norm_meta <- metadata(mae_checked)$quality_control$normality

    # check that the norm_df exists
    expect_s3_class(norm_meta$norm_df, "data.frame")

    # are the expected column names present
    expect_true(all(c("statistic", "p.value", "method", "exposure") %in% colnames(norm_meta$norm_df)))

    # check that the summary is a data frame with correct structure
    expect_s3_class(norm_meta$norm_summary, "data.frame")
    expect_true(all(c("var", "value") %in% colnames(norm_meta$norm_summary)))

    # check that the step was recorded
    step_names <- names(metadata(mae_checked)$summary$steps)
    expect_true("run_normality_check" %in% step_names)
})

test_that("run_cluster_samples performs clustering and records output", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Choose two numeric exposures to cluster on
    available_numeric <- mae |>
        MultiAssayExperiment::colData() |>
        as.data.frame() |>
        dplyr::select(where(is.numeric)) |>
        colnames()

    expect_true(length(available_numeric) >= 2)

    selected_exposures <- available_numeric[1:2]

    # Run clustering
    clustered <- run_cluster_samples(
        exposomicset = mae,
        exposure_cols = selected_exposures,
        dist_method = "euclidean",
        clustering_approach = "diana",
        cluster_method = "ward.D",
        action = "add"
    )

    # Check that sample_group column is added to colData
    expect_true("sample_group" %in% colnames(MultiAssayExperiment::colData(clustered)))

    # Check that metadata contains clustering output
    qc_meta <- MultiAssayExperiment::metadata(clustered)$quality_control$sample_clustering
    expect_true("sample_cluster" %in% names(qc_meta))
    expect_true("sample_groups" %in% names(qc_meta))
    expect_s3_class(qc_meta$sample_cluster, "hclust")

    # Check that step was recorded
    steps <- MultiAssayExperiment::metadata(clustered)$summary$steps
    expect_true("run_cluster_samples" %in% names(steps))
    expect_match(steps$run_cluster_samples$notes, "Optimal number of clusters")
})

test_that("extract_results correctly retrieves results from metadata", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 5)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Inject mock results into metadata
    MultiAssayExperiment::metadata(mae)$association$omics <- data.frame(
        feature = "B2M",
        exposure = "age",
        p = 0.01
    )

    MultiAssayExperiment::metadata(mae)$enrichment$omics <- data.frame(
        term = "antigen presentation",
        p_value = 0.01,
        ids = "HLA-DRB1,TAP1,B2M"
    )

    # Extract and check results
    assoc_result <- extract_results(mae, result = "association")
    enrich_result <- extract_results(mae, result = "enrichment")

    # ensure it is pulling a df and that the expected column names are present
    expect_s3_class(assoc_result$omics, "data.frame")
    expect_named(assoc_result$omics, c("feature", "exposure", "p"))
    expect_s3_class(enrich_result$omics, "data.frame")
    expect_true("term" %in% names(enrich_result$omics))
})


test_that("extract_results returns NULL for missing metadata entry", {
    dummy <- make_example_data(n_samples = 5)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Should return NULL for result not present
    missing_result <- extract_results(mae, result = "network")
    expect_null(missing_result)
})

test_that("create_exposomicset returns a valid MultiAssayExperiment", {
    dummy <- make_example_data(n_samples = 10, n_proteins = 50)

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # is it a MAE
    expect_s4_class(mae, "MultiAssayExperiment")

    # do we have the expected assays
    expect_named(experiments(mae), c("mRNA", "proteomics"))

    # do we have the same samples in coldata and exposure
    expect_equal(nrow(colData(mae)), nrow(dummy$exposure))

    # do the features line up
    expect_equal(nrow(mae[["mRNA"]]), nrow(dummy$omics$mRNA))
    expect_equal(nrow(mae[["proteomics"]]), nrow(dummy$omics$proteomics))

    # do the rownames line up
    expect_equal(rownames(rowData(mae[["mRNA"]])), rownames(dummy$omics$mRNA))

    # does the feature data line up
    expect_equal(rowData(mae[["mRNA"]])$symbol[1], dummy$row_data$mRNA$symbol[1])

    # do we have the same samples as in the exposure df
    expect_true(all((colnames(mae) |>
        unlist() |>
        unique()) %in% rownames(dummy$exposure)))

    # did we add the codebook
    expect_true("codebook" %in% names(metadata(mae)))
})

test_that("create_exposomicset works with single omics input", {
    dummy <- make_example_data(n_samples = 8)
    single_matrix <- dummy$omics$mRNA
    single_row_data <- dummy$row_data$mRNA

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = list("Gene Expression" = single_matrix),
        row_data = list(single_row_data)
    )

    # does it work with just one experiment
    expect_s4_class(mae, "MultiAssayExperiment")
    expect_equal(length(experiments(mae)), 1)
    expect_true("Gene Expression" %in% names(experiments(mae)))
})

test_that("create_exposomicset generates row_data if not provided", {
    dummy <- make_example_data(n_samples = 6)

    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics
        # no row_data provided
    )

    # ensure that some DataFrame is created when none is provided
    expect_s4_class(mae, "MultiAssayExperiment")
    expect_true(all(names(experiments(mae)) %in% c("mRNA", "proteomics")))
    expect_true(is(rowData(mae[["mRNA"]]), "DataFrame"))
})

test_that("create_exposomicset throws error on invalid input types", {
    dummy <- make_example_data()

    # error out if exposure is not a df
    expect_error(
        create_exposomicset(codebook = dummy$codebook, exposure = as.matrix(dummy$exposure), omics = dummy$omics),
        "must be a data frame"
    )

    # error out if omics is not a list or matrix
    expect_error(
        create_exposomicset(codebook = dummy$codebook, exposure = dummy$exposure, omics = 42),
        "must be a list or a single matrix"
    )

    # error out if the row data is not a list
    expect_error(
        create_exposomicset(codebook = dummy$codebook, exposure = dummy$exposure, omics = dummy$omics, row_data = "not_a_list"),
        "must be a list"
    )

    # error out if the rowdata does not match omics length
    expect_error(
        create_exposomicset(codebook = dummy$codebook, exposure = dummy$exposure, omics = dummy$omics, row_data = list(S4Vectors::DataFrame())),
        "Length of 'row_data' must match 'omics'"
    )
})

test_that("run_impute_missing() performs exposure and omics imputation correctly", {
    skip_if_not_installed("naniar")
    skip_if_not_installed("impute")

    # Create dummy data with missing values injected
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Inject NA values into exposure and omics
    colData(mae)$exposure_pm25[1:3] <- NA
    assay(mae[["mRNA"]])[1:2, 1:2] <- NA

    # Add minimal NA QC metadata to trigger exposure imputation
    metadata(mae)$quality_control <- list(
        na_qc = list(exposure = TRUE)
    )

    # Run imputation
    imputed <- run_impute_missing(
        mae,
        exposure_impute_method = "median",
        exposure_cols = c("exposure_pm25"),
        omics_impute_method = "knn",
        omics_to_impute = c("mRNA")
    )

    # Exposure imputation, no NAs should remain in selected column
    expect_false(any(is.na(colData(imputed)$exposure_pm25)))

    # Omics imputation, no NAs should remain in imputed assay
    imputed_assay <- assay(experiments(imputed)[["mRNA"]])
    expect_false(any(is.na(imputed_assay)))

    # Step metadata should be updated
    step_log <- metadata(imputed)$summary$steps
    expect_true(any(names(step_log) == "run_impute_missing"))
})

test_that("run_pca works as expected", {
    # make the example data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # run PCA
    mae_pca <- run_pca(mae,
        log_trans_exp = TRUE,
        log_trans_omics = TRUE,
        action = "add"
    )

    # grab the PCA results from metadata
    pca_meta <- metadata(mae_pca)$quality_control$pca

    # check that PCA results exist and are structured correctly
    expect_s3_class(pca_meta$pca_df, "tbl_df")
    expect_s3_class(pca_meta$pca_sample, "prcomp")
    expect_s3_class(pca_meta$pca_feature, "prcomp")
    expect_type(pca_meta$outliers, "character")

    # grab the colData
    col_data <- colData(mae_pca) |>
        as.data.frame()

    # ensure that PCs are added to colData
    expect_true(any(grepl("^PC[0-9]+$", colnames(col_data))))

    # ensure that the step was recorded
    step_names <- names(metadata(mae_pca)$summary$steps)
    expect_true("run_pca" %in% step_names)
})

test_that("transform_exposure applies boxcox_best correctly", {
    skip_if_not_installed("MASS")

    # Create dummy data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Select a few exposures to transform
    exposures <- colnames(SummarizedExperiment::colData(mae))[1:3]

    # Apply transformation
    transformed <- transform_exposure(
        exposomicset = mae,
        exposure_cols = exposures,
        transform_method = "boxcox_best"
    )

    # Check that colData still contains the transformed columns
    transformed_data <- as.data.frame(SummarizedExperiment::colData(transformed))
    expect_true(all(exposures %in% colnames(transformed_data)))

    # Check that metadata has transformation results
    trans_meta <- MultiAssayExperiment::metadata(transformed)$quality_control$transformation
    expect_s3_class(trans_meta$norm_df, "data.frame")
    expect_true("p.value" %in% colnames(trans_meta$norm_df))
    expect_true("exposure" %in% colnames(trans_meta$norm_df))

    # Check codebook is updated
    codebook <- MultiAssayExperiment::metadata(transformed)$codebook
    expect_s3_class(codebook, "data.frame")
    expect_true(all(exposures %in% codebook$variable))
    expect_true("transformation_applied" %in% colnames(codebook))

    # Check step summary
    steps <- MultiAssayExperiment::metadata(transformed)$summary$steps
    expect_true("transform_exposure" %in% names(steps))
    expect_match(steps$transform_exposure$notes, "Applied 'boxcox_best' transformation")
})


test_that("transform_exposure applies log2 transformation", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Select a few exposures to transform
    exposures <- c("age", "bmi", "exposure_pm25")

    # Apply transformation
    transformed <- transform_exposure(
        exposomicset = mae,
        exposure_cols = exposures,
        transform_method = "log2"
    )

    # Check that colData still contains the transformed columns
    transformed_data <- as.data.frame(SummarizedExperiment::colData(transformed))
    expect_true(all(exposures %in% colnames(transformed_data)))

    # Check transformation metadata is present
    trans_meta <- MultiAssayExperiment::metadata(transformed)$quality_control$transformation
    expect_s3_class(trans_meta$norm_df, "data.frame")
    expect_true("p.value" %in% colnames(trans_meta$norm_df))
    expect_true("exposure" %in% colnames(trans_meta$norm_df))

    # Check transformation label is "log2"
    codebook <- MultiAssayExperiment::metadata(transformed)$codebook
    expect_true(all(codebook[codebook$variable %in% exposures, ]$transformation_applied == "log2"))

    # Check step summary
    steps <- MultiAssayExperiment::metadata(transformed)$summary$steps
    expect_true("transform_exposure" %in% names(steps))
    expect_match(steps$transform_exposure$notes, "Applied 'log2' transformation")
})

test_that("filter_sample_outliers removes PCA-identified outliers", {
    # make the example data
    dummy <- make_example_data(n_samples = 20)

    # artificially make a sample an outlier
    dummy$omics$mRNA[1:3, 1] <- c(1e5, -1e5, 1e5)

    # create the multiassayexperiment
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # run PCA to detect outliers
    mae_pca <- run_pca(mae,
        action = "add"
    )

    # get initial sample count
    initial_n <- nrow(colData(mae_pca))

    # run outlier filter
    mae_filtered <- filter_sample_outliers(mae_pca)

    # check that sample count decreased (or stayed the same)
    expect_lt(nrow(colData(mae_filtered)), initial_n)

    # check that PCA outliers are no longer in colData
    outliers <- metadata(mae_pca)$quality_control$pca$outliers
    expect_false(any(outliers %in% rownames(colData(mae_filtered))))

    # ensure that the step was recorded
    step_names <- names(metadata(mae_filtered)$summary$steps)
    expect_true("filter_sample_outliers" %in% step_names)
})

test_that("filter_omics removes low-quality features using variance method", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Get initial feature counts
    initial_nrow <- nrow(SummarizedExperiment::assay(MultiAssayExperiment::experiments(mae)[["proteomics"]]))

    # Artificially lower expression in proteomics
    SummarizedExperiment::assay(MultiAssayExperiment::experiments(mae)[["proteomics"]])[1:15, ] <- 0

    # Apply variance filter
    filtered <- filter_omics(
        exposomicset = mae,
        method = "variance",
        assays = "proteomics",
        assay_name = 1,
        min_var = 0.8,
        verbose = FALSE
    )

    # Check that features were removed
    final_nrow <- nrow(SummarizedExperiment::assay(MultiAssayExperiment::experiments(filtered)[["proteomics"]]))
    expect_lt(final_nrow, initial_nrow)

    # Check metadata logging
    step_names <- names(metadata(filtered)$summary$steps)
    expect_true(any(grepl("filter_omics_proteomics", step_names)))
})

test_that("filter_omics removes low-expressed features using expression method", {
    # Create dummy data
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # Artificially lower expression in mRNA
    SummarizedExperiment::assay(MultiAssayExperiment::experiments(mae)[["mRNA"]])[1:5, ] <- 0

    # Get initial feature counts
    initial_nrow <- nrow(SummarizedExperiment::assay(MultiAssayExperiment::experiments(mae)[["mRNA"]]))

    # Apply expression filter
    filtered <- filter_omics(
        exposomicset = mae,
        method = "expression",
        assays = "mRNA",
        assay_name = 1,
        min_value = 1,
        min_prop = 0.5,
        verbose = FALSE
    )

    # Check that features were removed
    final_nrow <- nrow(SummarizedExperiment::assay(MultiAssayExperiment::experiments(filtered)[["mRNA"]]))
    expect_lt(final_nrow, initial_nrow)

    # Check metadata logging
    step_names <- names(metadata(filtered)$summary$steps)
    expect_true(any(grepl("filter_omics_mRNA", step_names)))
})

test_that("run_exposome_score computes scores with all supported methods", {
    skip_if_not_installed("matrixStats")

    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # try all the methods
    methods <- c("mean", "sum", "median", "pca", "quantile", "var")

    for (method in methods) {
        result <- run_exposome_score(
            exposomicset = mae,
            score_type = method,
            scale = TRUE
        )
        # what is the score column
        score_col <- grep("exposome_score", colnames(colData(result)), value = TRUE)

        # is the score column there?
        expect_true(length(score_col) == 1, label = paste("Missing score column for", method))

        # esnure that it is the right type
        expect_type(colData(result)[[score_col]], "double")

        # make sure it is equal to the number of samples
        expect_equal(nrow(colData(result)), 20)
    }
})

test_that("run_exposome_score computes score for user-defined columns", {
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    exposure_vars <- c("age", "bmi", "exposure_pm25")

    result <- run_exposome_score(
        exposomicset = mae,
        score_type = "sum",
        exposure_cols = exposure_vars,
        scale = FALSE,
        score_column_name = "custom_score"
    )

    # ensure the column name is there
    expect_true("custom_score" %in% colnames(colData(result)))

    # ensure it is the right type
    expect_type(colData(result)$custom_score, "double")
})

test_that("run_exposome_score with IRT works if mirt is available", {
    skip_if_not_installed("mirt")

    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    exposure_vars <- c("age", "bmi", "exposure_pm25")

    result <- run_exposome_score(
        exposomicset = mae,
        score_type = "irt",
        exposure_cols = exposure_vars,
        scale = TRUE
    )

    # ensure column is present
    expect_true("exposome_score_irt" %in% colnames(colData(result)))

    # ensure column is the right type
    expect_type(colData(result)$exposome_score_irt, "double")
})

test_that("run_exposome_score throws error for invalid method", {
    dummy <- make_example_data(n_samples = 20)
    mae <- create_exposomicset(
        codebook = dummy$codebook,
        exposure = dummy$exposure,
        omics = dummy$omics,
        row_data = dummy$row_data
    )

    # ensure it throws an error with an unsupported score_type
    expect_error(
        run_exposome_score(mae, score_type = "invalid"),
        "Choose either 'sum', mean', 'median', 'pca', 'irt', or 'quantile'."
    )
})

#' Transform Exposure Variables for Normality
#'
#' Applies a transformation to selected numeric exposure variables in the
#' `colData` of a `MultiAssayExperiment` to improve their normality
#' (e.g., log, Box-Cox, sqrt). Transformation results and normality
#' statistics are stored in metadata for tracking.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing exposure
#' variables in `colData`.
#' @param exposure_cols Optional character vector of exposure variable
#' names to transform.
#'   If `NULL`, uses exposures found in `metadata(exposomicset)$quality_control$normality$norm_df$exposure`.
#' @param transform_method Character. Transformation method to apply. Options:
#'   \itemize{
#'     \item `"none"`: no transformation
#'     \item `"log2"`: log base 2 transformation
#'     \item `"x_1_3"`: cube-root transformation
#'     \item `"sqrt"`: square-root transformation
#'     \item `"boxcox_best"`: data-driven Box-Cox approximation with
#'     heuristic labeling
#'   }
#'
#' @return A `MultiAssayExperiment` object with transformed exposures
#' in `colData`, and transformation details stored in:
#' \itemize{
#'   \item `metadata(exposomicset)$quality_control$transformation$norm_df`:
#'    Shapiro-Wilk test results
#'   \item `metadata(exposomicset)$quality_control$transformation$norm_summary`:
#'   Summary of normality
#'   \item `metadata(exposomicset)$codebook`: Updated with transformation info
#'    per variable
#'   \item `metadata(exposomicset)$summary$steps`: Updated with step record
#' }
#'
#' @details
#' For `transform_method = "boxcox_best"`, the function automatically shifts
#' values to be strictly positive and chooses from a discrete set of
#' transformations (e.g., `1/x`, `log(x)`, `x^2`)
#' based on estimated Box-Cox lambda. Each variable may receive a
#' different transformation.
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Test for normality
#' mae <- mae |>
#'     run_normality_check() |>
#'     transform_exposure(
#'         exposure_cols = c("age", "bmi", "exposure_pm25"),
#'         transform_method = "boxcox_best"
#'     )
#'
#' @seealso \code{\link[MASS]{boxcox}}, \code{\link[stats]{shapiro.test}}
#'
#' @export
transform_exposure <- function(
  exposomicset,
  exposure_cols = NULL,
  transform_method = "boxcox_best"
) {
    col_data <- as.data.frame(MultiAssayExperiment::colData(exposomicset))
    # numeric_cols <- names(col_data)[sapply(col_data, is.numeric)]
    numeric_cols <- names(col_data)[
        vapply(col_data, is.numeric, FUN.VALUE = logical(1))
    ]
    variables_to_transform <- if (!is.null(exposure_cols)) {
        exposure_cols
    } else {
        MultiAssayExperiment::metadata(
            exposomicset
        )$quality_control$normality$norm_df$exposure
    }
    variables_to_transform <- intersect(variables_to_transform, numeric_cols)

    if (length(variables_to_transform) == 0) {
        stop("No numeric exposure variables to transform.")
    }

    message("Applying the ", transform_method, " transformation.")
    numeric_data <- col_data[, variables_to_transform, drop = FALSE]
    non_numeric_data <- col_data[
        , setdiff(names(col_data), variables_to_transform)
    ]

    transformed_numeric <- .apply_transformation(
        numeric_data,
        variables_to_transform,
        transform_method
    )

    if (transform_method == "boxcox_best") {
        transformation_info <- data.frame(
            variable = colnames(transformed_numeric),
            transformation_applied = attr(transformed_numeric, "labels"),
            stringsAsFactors = FALSE
        )
    } else {
        transformation_info <- data.frame(
            variable = variables_to_transform,
            transformation_applied = transform_method,
            stringsAsFactors = FALSE
        )
    }

    updated_col_data <- dplyr::bind_cols(
        non_numeric_data,
        transformed_numeric
    )
    MultiAssayExperiment::colData(exposomicset) <- S4Vectors::DataFrame(
        updated_col_data
    )

    norm_results <- transformed_numeric |>
        dplyr::select_if(~ length(unique(na.omit(.))) >= 3) |>
        apply(2, function(x) shapiro.test(x) |> broom::tidy()) |>
        (\(x) do.call(rbind, x))() |>
        dplyr::mutate(exposure = colnames(transformed_numeric))

    norm_summary <- norm_results |>
        dplyr::summarise(
            "Normal" = sum(p.value > 0.05),
            "Not Normal" = sum(p.value <= 0.05)
        ) |>
        t() |>
        as.data.frame() |>
        tibble::rownames_to_column("var") |>
        setNames(c("var", "value"))

    all_metadata <- MultiAssayExperiment::metadata(exposomicset)
    all_metadata$quality_control$transformation <- list(
        norm_df = norm_results,
        norm_summary = norm_summary
    )
    MultiAssayExperiment::metadata(exposomicset) <- all_metadata

    # Add step record
    n_transformed <- length(variables_to_transform)

    # Pull normality results
    all_metadata <- MultiAssayExperiment::metadata(exposomicset)
    norm_df <- all_metadata$quality_control$transformation$norm_df

    n_normal <- sum(norm_df$p.value > 0.05, na.rm = TRUE)
    p_normal <- round(n_normal / n_transformed * 100, 1)

    step_record <- list(
        transform_exposure = list(
            timestamp = Sys.time(),
            params = list(
                transform_method = transform_method,
                n_transformed = n_transformed,
                exposures = variables_to_transform
            ),
            notes = paste0(
                "Applied '",
                transform_method,
                "' transformation to ",
                n_transformed,
                " exposure variables. ",
                n_normal,
                " passed normality (Shapiro-Wilk p > 0.05, ",
                p_normal,
                "%)."
            )
        )
    )

    MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
        MultiAssayExperiment::metadata(exposomicset)$summary$steps,
        step_record
    )

    # Merge with existing codebook or initialize
    existing_codebook <- MultiAssayExperiment::metadata(exposomicset)$codebook

    # Create a new data frame for the updated variable information
    updated_codebook <- existing_codebook |>
        full_join(
            transformation_info,
            by = "variable"
        )

    # Update the codebook in the metadata
    MultiAssayExperiment::metadata(exposomicset)$codebook <- updated_codebook

    return(exposomicset)
}

# --- Box-Cox Best Function ------
#' Box-Cox based transformation selector
#'
#' This internal function applies a Box-Cox analysis to determine the most
#' appropriate transformation for a numeric vector. Depending on the estimated
#' Box-Cox lambda, it applies a power, inverse, log, or identity transform.
#'
#' @param var A numeric vector to transform.
#'
#' @return A list with two elements:
#'   - `values`: the transformed vector
#'   - `label`: the transformation applied as a string
#' @importFrom MASS boxcox
#' @keywords internal
#' @noRd
.boxcox_best <- function(var) {
    # Shift to make values strictly positive
    # code credit: https://academic.oup.com/bioinformatics/article/35/24/5344/5523848#supplementary-data
    shift_val <- 0
    if (any(var <= 0, na.rm = TRUE)) {
        shift_val <- abs(min(var, na.rm = TRUE)) + 0.00001
        var <- var + shift_val
    }

    bc <- MASS::boxcox(var ~ 1, plotit = FALSE)
    lambda <- bc$x[which.max(bc$y)]

    if (lambda < -1.5) {
        transformed <- 1 / (var^2)
        label <- "1/x^2"
    } else if (lambda < -0.75) {
        transformed <- 1 / var
        label <- "1/x"
    } else if (lambda < -0.25) {
        transformed <- 1 / sqrt(var)
        label <- "1/sqrt(x)"
    } else if (lambda < 0.25) {
        transformed <- log(var)
        label <- "log(x)"
    } else if (lambda < 0.75) {
        transformed <- sqrt(var)
        label <- "sqrt(x)"
    } else if (lambda < 1.5) {
        transformed <- var
        label <- "identity"
    } else {
        transformed <- var^2
        label <- "x^2"
    }

    return(list(values = transformed, label = label))
}

# --- Apply Transformation Function -----
#' Apply a transformation to selected columns
#'
#' This internal function applies a variety of transformations (log, sqrt,
#' Box-Cox, etc.) to specific columns in a data frame.
#'
#' @param data A data frame or matrix containing numeric columns.
#' @param columns Character vector of column names to transform.
#' @param method Transformation method. Supported options:
#'   - `"none"`: no transformation
#'   - `"log2"`: log2(x + |min(x)| + 1)
#'   - `"x_1_3"`: cube root transform (x + |min(x)|)^(1/3)
#'   - `"sqrt"`: square root transform (x + |min(x)|)
#'   - `"boxcox_best"`: adaptive Box-Cox transformation using
#'   \code{.boxcox_best()}
#'
#' @return A data frame with transformed columns. If `method = "boxcox_best"`,
#'   an attribute `"labels"` is attached containing the transformation labels
#'   used for each column.
#'
#' @keywords internal
#' @importFrom dplyr mutate across all_of
#' @noRd
.apply_transformation <- function(data, columns, method) {
    data <- as.data.frame(data)
    if (method == "none") {
        return(data)
    } else if (method == "log2") {
        return(dplyr::mutate(
            data,
            dplyr::across(
                dplyr::all_of(columns),
                ~ log2(. + abs(min(.)) + 1)
            )
        ))
    } else if (method == "x_1_3") {
        return(dplyr::mutate(
            data,
            dplyr::across(
                dplyr::all_of(columns),
                ~ (. + abs(min(.)))^(1 / 3)
            )
        ))
    } else if (method == "sqrt") {
        return(dplyr::mutate(
            data,
            dplyr::across(
                dplyr::all_of(columns),
                ~ sqrt(. + abs(min(.)))
            )
        ))
    } else if (method == "boxcox_best") {
        result <- lapply(columns, function(col) .boxcox_best(data[[col]]))
        transformed <- lapply(result, `[[`, "values")

        labels <- vapply(result, `[[`, "label", FUN.VALUE = character(1))

        names(transformed) <- columns
        transformed_df <- as.data.frame(transformed)

        # attach labels to the data.frame AFTER creation
        attr(transformed_df, "labels") <- labels
        return(transformed_df)
    } else {
        stop("Unsupported transformation method: ", method)
    }
}

#' Filter Sample Outliers
#'
#' Removes sample outliers from a `MultiAssayExperiment` object
#' based on PCA analysis.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing
#' omics and exposure data.
#' @param outliers An optional character vector specifying
#' sample names to be removed.
#' If `NULL`, the function uses outliers identified in
#' `metadata(exposomicset)$pca$outliers`. Default is `NULL`.
#'
#' @details
#' The function checks for the presence of PCA results in
#' `metadata(exposomicset)`. If `outliers` is not provided,
#' it retrieves precomputed outliers from `metadata(exposomicset)$pca$outliers`.
#' The identified samples are removed
#' from the dataset.
#'
#' @return A `MultiAssayExperiment` object with the specified outliers removed.
#'
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run PCA
#' mae <- mae |>
#'     run_pca()
#'
#' # filter outliers if present
#' mae <- mae |>
#'     filter_sample_outliers()
#'
#' @export
filter_sample_outliers <- function(
  exposomicset,
  outliers = NULL
) {
    # Check if the input is a MultiAssayExperiment object
    if (!"pca" %in% names(
        MultiAssayExperiment::metadata(exposomicset)$quality_control
    )) {
        stop("PCA not performed on the data. Please run 'pca_analysis' first.")
    }

    # Check if outliers are provided
    if (is.null(outliers)) {
        outliers <- exposomicset |>
            MultiAssayExperiment::metadata() |>
            purrr::pluck("quality_control") |>
            purrr::pluck("pca") |>
            purrr::pluck("outliers")
    } else {
        outliers <- outliers
    }

    message("Removing outliers: ", paste(outliers, collapse = ", "))

    # Subset by ColData to properly filter outliers
    exposomicset <- MultiAssayExperiment::subsetByColData(
        exposomicset,
        !(rownames(SummarizedExperiment::colData(exposomicset)) %in% outliers)
    )


    # Add analysis steps taken to metadata
    step_record <- list(filter_sample_outliers = list(
        timestamp = Sys.time(),
        params = list(),
        notes = paste("Outliers: ", paste(outliers, collapse = ", "), collapse = "")
    ))

    MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
        MultiAssayExperiment::metadata(exposomicset)$summary$steps,
        step_record
    )

    # exposomicset <- exposomicset[,!colnames(exposomicset) %in% outliers]
    return(exposomicset)
}

#' Internal - builds ontology annotation UI
#'
#' @keywords internal
#' @return A \code{shiny.tag} UI definition.
#' @importFrom shiny fluidPage tags HTML div img titlePanel fileInput uiOutput
.ont_annot_build_ui <- function() {
    logo_path <- system.file("app", "www", "logo.png", package = "tidyexposomics")
    shiny::fluidPage(
        shiny::tags$head(shiny::tags$style(shiny::HTML(
            "body { background:#fff; color:#1e0c1f; font-family:Helvetica,Arial,sans-serif; }
       h1,h4 { color:#1e0c1f; }
       .btn { background:#71255A; border-color:#71255A; color:#fff; }
       .btn:hover { background:#5a1e47; border-color:#5a1e47; }
       .selectize-input, .form-control { background:#fff; color:#1e0c1f; }
       .dataTables_wrapper .paginate_button { color:#71255A!important; }
       .shiny-input-container { margin-bottom:20px; }"
        ))),
        shiny::div(
            style = "position:absolute;top:10px;right:10px;z-index:1000;",
            if (file.exists(logo_path)) shiny::img(src = "www/logo.png", height = "150px")
        ),
        shiny::titlePanel(shiny::div("Ontology Annotation Tool", style = "color:#1e0c1f")),
        shiny::fileInput("user_df", "Upload Your Exposure Metadata (CSV)", accept = ".csv"),
        shiny::uiOutput("main_ui")
    )
}

#' Internal - builds ontology annotation server
#'
#' @keywords internal
#' @param ontologies A list with \code{hpo}, \code{ecto}, \code{chebi} data.frames.
#' @return A Shiny server function.
#' @importFrom shiny renderUI column fluidRow h4 selectizeInput actionButton hr
#' @importFrom shiny selectInput numericInput downloadButton observeEvent
#' @importFrom shiny reactiveValues showModal modalDialog updateSelectizeInput
#' @importFrom shiny req HTML
#' @importFrom DT datatable renderDT DTOutput dataTableProxy replaceData formatStyle styleEqual
#' @importFrom dplyr mutate distinct bind_rows case_when filter any_of
.ont_annot_build_server <- function(ontologies) {
    force(ontologies)

    preprocess_ont <- function(ont) {
        fix_rel <- function(x) if (!is.list(x)) strsplit(x, ";\\s*") else x
        dplyr::mutate(
            ont,
            ancestors = fix_rel(ancestors),
            parents   = fix_rel(parents),
            children  = fix_rel(children),
            depth     = lengths(ancestors)
        )
    }

    # Prepare once
    ont_list <- lapply(ontologies[c("hpo", "ecto", "chebi")], preprocess_ont)

    ontology_df_all <-
        dplyr::bind_rows(
            dplyr::mutate(ont_list$hpo[, c("id", "name")], source = "hpo"),
            dplyr::mutate(ont_list$ecto[, c("id", "name")], source = "ecto"),
            dplyr::mutate(ont_list$chebi[, c("id", "name")], source = "chebi")
        ) |>
        dplyr::distinct()

    function(input, output, session) {
        rv <- shiny::reactiveValues(data = NULL)

        shiny::observeEvent(input$user_df, {
            shiny::req(input$user_df)
            df <- utils::read.csv(input$user_df$datapath, stringsAsFactors = FALSE)

            if (!"variable" %in% names(df)) {
                shiny::showModal(shiny::modalDialog("Your file needs a 'variable' column.", easyClose = TRUE))
                return()
            }

            expected <- c(
                "selected_ontology_label", "selected_ontology_id",
                "root_id", "root_label", "category", "category_source"
            )
            for (col in expected) if (!col %in% names(df)) df[[col]] <- NA_character_

            rv$data <- df
        })

        output$main_ui <- shiny::renderUI({
            shiny::req(rv$data)
            shiny::fluidRow(
                shiny::column(
                    4,
                    shiny::h4("Step 1: Annotate Variable"),
                    shiny::uiOutput("selected_var"),
                    shiny::selectizeInput("ontology_choice", "Choose Term:",
                        choices = NULL,
                        options = list(placeholder = "Type to search")
                    ),
                    shiny::actionButton("apply_annotation", "Apply Annotation"),
                    shiny::uiOutput("ontology_description"),
                    shiny::hr(),
                    shiny::h4("Step 2: Categorize Variables"),
                    shiny::selectInput("cat_ontology", "Ontology:", choices = c("hpo", "ecto", "chebi"), selected = "hpo"),
                    shiny::numericInput("cat_root_depth", "Root depth level:", value = 2, min = 0),
                    shiny::actionButton("apply_categorization", "Apply Categorization"),
                    shiny::hr(),
                    shiny::h4("Step 3: Download Annotated Data"),
                    shiny::hr(),
                    shiny::h4("Legend"),
                    shiny::div(
                        style = "padding-left:10px;",
                        shiny::tags$div(
                            style = "background:#fde0dc;padding:4px;border-radius:3px;margin-bottom:4px;",
                            "Missing ontology annotation"
                        ),
                        shiny::tags$div(
                            style = "background:#fff2cc;padding:4px;border-radius:3px;margin-bottom:4px;",
                            "Missing category"
                        ),
                        shiny::tags$div(
                            style = "background:#fff7e6;padding:4px;border-radius:3px;",
                            "Manually categorized"
                        )
                    ),
                    shiny::downloadButton("save_csv", "Download Annotated CSV")
                ),
                shiny::column(8, DT::DTOutput("table"))
            )
        })

        shiny::observe({
            shiny::req(rv$data)
            shiny::updateSelectizeInput(session, "ontology_choice",
                choices = ontology_df_all$name, server = TRUE
            )
        })

        output$selected_var <- shiny::renderUI({
            sel <- input$table_rows_selected
            if (length(sel)) {
                shiny::HTML(paste0("<strong>Selected:</strong> ", rv$data$variable[sel]))
            } else {
                shiny::HTML("<em>No row selected</em>")
            }
        })

        output$table <- DT::renderDT(
            {
                shiny::req(rv$data)
                display_data <- rv$data
                display_data$row_status <- dplyr::case_when(
                    is.na(display_data$selected_ontology_id) ~ "Missing Annotation",
                    is.na(display_data$category) ~ "Missing Category",
                    display_data$category_source == "manual" ~ "Manual",
                    TRUE ~ "OK"
                )

                DT::datatable(
                    display_data,
                    selection = "multiple",
                    rownames = FALSE,
                    editable = list(
                        target = "cell",
                        disable = list(columns = which(names(display_data) != "category") - 1)
                    ),
                    options = list(
                        pageLength = 2000,
                        scrollY    = "400px",
                        scrollX    = TRUE,
                        autoWidth  = TRUE,
                        stateSave  = TRUE,
                        columnDefs = list(list(visible = FALSE, targets = which(names(display_data) == "row_status") - 1))
                    ),
                    class = "stripe nowrap"
                ) |>
                    DT::formatStyle("row_status",
                        target = "row",
                        backgroundColor = DT::styleEqual(
                            c("Missing Annotation", "Missing Category", "Manual"),
                            c("#fde0dc", "#fff2cc", "#fff7e6")
                        )
                    )
            },
            server = TRUE
        )

        proxy <- DT::dataTableProxy("table")

        shiny::observeEvent(input$apply_annotation, {
            shiny::req(rv$data, input$ontology_choice)
            rows <- input$table_rows_selected
            if (length(rows)) {
                sel <- dplyr::filter(ontology_df_all, name == input$ontology_choice)
                if (nrow(sel) == 1) {
                    rv$data$selected_ontology_label[rows] <- sel$name
                    rv$data$selected_ontology_id[rows] <- sel$id
                }
            }
            DT::replaceData(proxy, rv$data, resetPaging = FALSE, rownames = FALSE)
        })

        output$ontology_description <- shiny::renderUI({
            shiny::req(input$ontology_choice)
            term <- dplyr::filter(ontology_df_all, name == input$ontology_choice)
            if (nrow(term) == 1) {
                desc <- .get_ols_description(term$id, sub(":.*$", "", term$id))
                if (!is.na(desc)) {
                    shiny::div(
                        style = "border-left:4px solid #71255A;padding-left:10px;margin:10px 0;",
                        shiny::strong("Description:"), shiny::br(), desc
                    )
                }
            }
        })

        shiny::observeEvent(input$apply_categorization, {
            shiny::req(rv$data)
            if (all(is.na(rv$data$selected_ontology_id))) {
                shiny::showModal(shiny::modalDialog("Annotate at least one ID before categorizing.", easyClose = TRUE))
                return()
            }
            rows <- input$table_rows_selected
            if (!length(rows)) {
                shiny::showModal(shiny::modalDialog("Please select one or more rows to categorize.", easyClose = TRUE))
                return()
            }

            onto_name <- input$cat_ontology
            root_lvl <- as.numeric(input$cat_root_depth)
            target_df <- ont_list[[onto_name]]

            updated <- .run_categorize_ontology(
                data         = rv$data[rows, , drop = FALSE],
                id_col       = "selected_ontology_id",
                ontologyDF   = target_df,
                root_level   = root_lvl,
                assign_label = TRUE
            )

            rv$data$root_id[rows] <- updated$root_id
            rv$data$root_label[rows] <- updated$root_label
            rv$data$category[rows] <- updated$category
            rv$data$category_source[rows] <- updated$category_source

            DT::replaceData(proxy, rv$data, resetPaging = FALSE, rownames = FALSE)
        })

        shiny::observeEvent(input$table_cell_edit, {
            info <- input$table_cell_edit
            i <- info$row
            j <- info$col
            v <- info$value
            colname <- names(rv$data)[j + 1] # DT is 0-based

            if (identical(colname, "category") &&
                (is.na(rv$data$root_id[i]) || nchar(rv$data$root_id[i]) <= 1)) {
                rv$data$category[i] <- v
                rv$data$category_source[i] <- "manual"
                DT::replaceData(proxy, rv$data, resetPaging = FALSE, rownames = FALSE)
            } else {
                shiny::showModal(shiny::modalDialog(
                    title = "Editing not allowed",
                    "Only 'Unmapped' categories can be changed manually.",
                    easyClose = TRUE
                ))
            }
        })

        output$save_csv <- shiny::downloadHandler(
            filename = function() paste0("annotated_", Sys.Date(), ".csv"),
            content  = function(file) utils::write.csv(rv$data, file, row.names = FALSE)
        )
    }
}

#' Plot Missing Data Across Exposure and Omic Layers
#'
#' Visualizes missing data patterns in a `MultiAssayExperiment` object using
#' summary bar plots or feature-level lollipop plots.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing exposure
#' and omics assays. Missing data is inferred directly from the assays.
#' @param threshold Numeric. The percentage threshold (0-100) above which
#' features are counted as missing in the summary plot. Default is `5`.
#' @param plot_type Character. Type of plot to generate. Either `"summary"`
#' for a bar plot showing number of features above the missing threshold,
#'  or `"lollipop"` for a per-feature lollipop plot with layer annotations.
#'   Default is `"summary"`.
#' @param layers Optional character vector. If specified, filters the plot
#' to include only selected layers (e.g., `"Exposure"`, `"Transcriptome"`).
#'
#' @details
#' The function calculates missing data per feature (or variable) across all
#'  assays (including exposure variables) and generates:
#'
#' - **Summary plot (`plot_type = "summary`)**: A bar plot showing the number
#' of variables in each assay exceeding the specified missingness threshold.
#' - **Lollipop plot (`plot_type = "lollipop`)**: A feature-level plot where
#' each feature's percent missingness is shown, along with a color-coded tile
#' on the side indicating its layer of origin.
#'
#' The tile colors in the lollipop plot match the experiment colors used in
#' other visualizations (e.g., via `scale_color_tidy_exp()`).
#'
#' @return A `ggplot` or `patchwork` object depending on the selected
#' `plot_type`.
#'
#' @import ggplot2
#'
#' @examples
#'
#' #' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Introduce some missingness
#' MultiAssayExperiment::colData(mae)$exposure_pm25[sample(1:20, 5)] <- NA
#'
#' # Summary bar plot of missing data
#' summary_p <- plot_missing(
#'     mae,
#'     threshold = 10,
#'     plot_type = "summary"
#' )
#'
#' # Lollipop plot for all features with any missingness
#' lollipop_p <- plot_missing(
#'     mae,
#'     plot_type = "lollipop"
#' )
#'
#' @importFrom ggplot2 ggplot aes geom_bar geom_segment geom_point geom_tile
#'   labs theme_bw theme element_text theme_void
#' @importFrom MultiAssayExperiment colData experiments
#' @importFrom SummarizedExperiment assay
#' @importFrom dplyr bind_rows filter group_by reframe mutate arrange
#'   reframe select ungroup
#' @importFrom purrr map
#' @importFrom naniar miss_var_summary
#' @importFrom ggpubr theme_pubr
#'
#' @export
plot_missing <- function(
  exposomicset,
  threshold = 5,
  plot_type = c("summary", "lollipop"),
  layers = NULL
) {
    # require(ggplot2)
    # require(patchwork)
    .check_suggested(pkg = "forcats")
    .check_suggested(pkg = "patchwork")

    plot_type <- match.arg(plot_type)
    # grab exposure data
    exposure <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame()

    # grab omics assay data
    omics <- lapply(
        names(MultiAssayExperiment::experiments(exposomicset)),
        function(exp_name) {
            exp <- exposomicset[[exp_name]]
            assay <- SummarizedExperiment::assay(exp) |>
                t() |>
                as.data.frame()
        }
    ) |>
        setNames(names(MultiAssayExperiment::experiments(exposomicset)))

    # get the number of NA values per exposure variable and omic feature
    na_df <- c(
        list(Exposure = exposure),
        omics
    ) |>
        purrr::map(~ .x |>
            naniar::miss_var_summary()) |>
        dplyr::bind_rows(.id = "exp_name")

    if (!is.null(layers)) {
        na_df <- na_df |>
            dplyr::filter(exp_name %in% layers)
    }

    if (plot_type == "summary") {
        plot_data <- na_df |>
            dplyr::group_by(exp_name) |>
            dplyr::reframe(
                missingness = sum(pct_miss > threshold),
                .groups = "drop"
            ) |>
            dplyr::mutate(exp_label = paste0(exp_name, " (", missingness, ")"))

        p <- plot_data |>
            ggplot(aes(
                y = forcats::fct_reorder(exp_label, missingness),
                x = missingness,
                fill = exp_name
            )) +
            geom_bar(stat = "identity", alpha = 0.7) +
            geom_segment(aes(
                x = missingness,
                xend = missingness,
                y = as.numeric(forcats::fct_reorder(exp_label, missingness)) - 0.45,
                yend = as.numeric(forcats::fct_reorder(exp_label, missingness)) + 0.45,
                color = exp_name
            ), linewidth = 1) +
            scale_fill_tidy_exp() +
            scale_color_tidy_exp() +
            labs(
                title = paste("No. of Features Over ", threshold, "% Threshold"),
                y = NULL,
                x = "No. of Features"
            ) +
            ggpubr::theme_pubr(legend = "none") +
            theme(plot.title = element_text(face = "bold.italic"))
    } else if (plot_type == "lollipop") {
        na_df_filtered <- na_df |>
            dplyr::filter(pct_miss > 0) |>
            dplyr::arrange(pct_miss) |>
            dplyr::mutate(var_label = factor(variable, levels = unique(variable)))

        # main lollipop plot
        p_lollipop <- na_df_filtered |>
            ggplot(aes(
                x = pct_miss,
                y = var_label
            )) +
            geom_segment(
                aes(
                    x = 0,
                    xend = pct_miss,
                    yend = var_label
                ),
                color = "midnightblue"
            ) +
            geom_point(color = "midnightblue") +
            theme_bw(base_size = 10) +
            labs(
                title = "Missingness per Feature",
                x = "% Missing",
                y = NULL
            ) +
            theme(
                axis.text.y = element_text(),
                plot.title = element_text(face = "bold.italic"),
                legend.position = "right"
            )

        # side tile plot (just layer colors)
        p_tile <- na_df_filtered |>
            ggplot(aes(
                x = 1,
                y = var_label,
                fill = exp_name
            )) +
            geom_tile() +
            scale_fill_tidy_exp() +
            theme_void() +
            theme(legend.position = "right") +
            labs(fill = "Experiment")

        # combine plots with patchwork
        p <- p_lollipop + p_tile + patchwork::plot_layout(
            widths = c(1, 0.05),
            guides = "collect"
        )
    }

    p
}

#' Summarize Exposure Variables
#'
#' Computes summary statistics for numeric exposure variables and
#' optionally stores the results in the `MultiAssayExperiment` metadata.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing exposure
#' data in the sample metadata.
#' @param exposure_cols A character vector of exposure variable names
#' to summarize. If `NULL`, all numeric exposure variables are included.
#' @param action A string specifying the action to take. Use `"add"`
#' to attach the summary table to `metadata(exposomicset)` or `"get"`
#' to return the summary table directly. Default is `"add"`.
#'
#' @details
#' This function:
#' - Extracts sample-level exposure data using `pivot_sample()`.
#' - Filters to user-specified exposures (`exposure_cols`) if provided.
#' - Computes descriptive statistics for each numeric variable:
#'   - Number of values (`n_values`)
#'   - Number of NAs (`n_na`)
#'   - Minimum, maximum, and range
#'   - Sum, median, mean
#'   - Standard error of the mean
#'   - 95% confidence interval of the mean
#'   - Variance, standard deviation
#'   - Coefficient of variation (`sd / mean`)
#' - Merges the result with variable metadata stored in
#' `metadata(exposomicset)$codebook`.
#'
#' @return
#' A modified `MultiAssayExperiment` object (if `action = "add"`),
#' or a data frame of summary statistics (if `action = "get"`).
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Summarize exposure data
#' exp_sum <- mae |>
#'     run_summarize_exposures(
#'         exposure_cols = c("age", "bmi", "exposure_pm25"),
#'         action = "get"
#'     )
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom dplyr select all_of where group_by summarise ungroup
#' inner_join mutate_if everything
#' @importFrom tidyr pivot_longer
#' @importFrom purrr pluck
#' @export
run_summarize_exposures <- function(
  exposomicset,
  exposure_cols = NULL,
  action = "add"
) {
    # library(dplyr)
    # library(tidyr)

    # Extract exposure data
    exposure_df <- exposomicset |>
        pivot_sample()

    # Subset to selected columns if specified
    if (!is.null(exposure_cols)) {
        exposure_df <- exposure_df |>
            dplyr::select(dplyr::all_of(exposure_cols))
    }

    # Keep only numeric columns
    exposure_df <- exposure_df |>
        dplyr::select(dplyr::where(is.numeric))

    # Pivot to long format for summary
    long_df <- exposure_df |>
        tidyr::pivot_longer(
            cols = dplyr::everything(),
            names_to = "variable",
            values_to = "value"
        )

    # Group by variable and compute stats
    exposure_summary_df <- long_df |>
        dplyr::group_by(variable) |>
        dplyr::summarise(
            n_values = sum(!is.na(value)),
            n_na = sum(is.na(value)),
            min = min(value, na.rm = TRUE),
            max = max(value, na.rm = TRUE),
            range = max - min,
            sum = sum(value, na.rm = TRUE),
            median = median(value, na.rm = TRUE),
            mean = mean(value, na.rm = TRUE),
            se = sd(value, na.rm = TRUE) / sqrt(n_values),
            ci_lower = mean - qt(0.975, df = n_values - 1) * se,
            ci_upper = mean + qt(0.975, df = n_values - 1) * se,
            variance = var(value, na.rm = TRUE),
            sd = sd(value, na.rm = TRUE),
            coef_var = sd / mean
        ) |>
        dplyr::ungroup()

    # Merge with variable info
    exposure_summary_df <- exposure_summary_df |>
        dplyr::inner_join(
            exposomicset |>
                MultiAssayExperiment::metadata() |>
                purrr::pluck("codebook"),
            by = "variable"
        ) |>
        dplyr::mutate_if(is.numeric, ~ round(., digits = 2))

    if (action == "add") {
        # Store results
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$quality_control$exposure_summary_df <- exposure_summary_df
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        # Add step record
        step_record <- list(
            run_summarize_exposures = list(
                timestamp = Sys.time(),
                params = list(
                    exposure_cols = exposure_cols,
                    n_vars_summarized = length(unique(exposure_summary_df$variable))
                ),
                notes = paste0(
                    "Summarized ", length(unique(exposure_summary_df$variable)),
                    " numeric exposure variables. ",
                    if (is.null(exposure_cols)) {
                        "Included all numeric exposures from colData."
                    } else {
                        paste0(
                            "Subset to user-specified exposure columns (n = ",
                            length(exposure_cols),
                            ")."
                        )
                    }
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else if (action == "get") {
        # Return the summary data frame
        return(exposure_summary_df)
    } else {
        stop("Invalid action. Use 'add' or 'get'.")
    }
}

#' Filter Non-Normal Exposure Variables
#'
#' Removes exposure variables that deviate significantly from a
#' normal distribution based on
#' normality test results stored in metadata.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing
#' exposure and omics data.
#' @param p_thresh A numeric value specifying the p-value
#' threshold for normality. Variables with `p.value < p_thresh` are removed.
#'  Default is `0.05`.
#'
#' @details
#' The function identifies exposure variables that fail a normality test
#'  using `metadata(exposomicset)$transformation$norm_df`.
#' - Exposure variables with `p.value < p_thresh` are removed from
#' `colData(exposomicset)`.
#' - The same filtering is applied to `colData` in each experiment
#' within `experiments(exposomicset)`.
#'
#' @return A `MultiAssayExperiment` object with non-normal
#' exposure variables removed.
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Test for normality
#' mae <- mae |>
#'     run_normality_check() |>
#'     transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))
#'
#' # Remove non-normal variables
#' mae_filtered <- mae |>
#'     filter_non_normal()
#'
#' @export
filter_non_normal <- function(
  exposomicset,
  p_thresh = 0.05
) {
    if ("transformation" %in% names(exposomicset |>
        MultiAssayExperiment::metadata() |>
        purrr::pluck("quality_control"))) {
        norm_df <- exposomicset |>
            MultiAssayExperiment::metadata() |>
            purrr::pluck(
                "quality_control",
                "transformation",
                "norm_df"
            )
    } else {
        norm_df <- exposomicset |>
            MultiAssayExperiment::metadata() |>
            purrr::pluck(
                "quality_control",
                "normality",
                "norm_df"
            )
    }
    norm_df <- exposomicset |>
        MultiAssayExperiment::metadata() |>
        purrr::pluck(
            "quality_control",
            "transformation",
            "norm_df"
        )

    if (!all(c("exposure", "p.value") %in% colnames(norm_df))) {
        stop("norm_df must contain 'exposure' and 'p.value' columns.")
    }

    # Get exposures to drop
    non_normal_vars <- norm_df |>
        dplyr::filter(p.value < p_thresh) |>
        dplyr::pull(exposure)

    message(
        "Filtering out ",
        length(non_normal_vars),
        " non-normal exposure variables."
    )

    # Filter colData in the main object
    MultiAssayExperiment::colData(exposomicset) <- MultiAssayExperiment::colData(
        exposomicset
    )[
        , !colnames(MultiAssayExperiment::colData(exposomicset)) %in% non_normal_vars,
        drop = FALSE
    ]

    # Filter colData in each experiment
    for (omics_name in names(MultiAssayExperiment::experiments(exposomicset))) {
        current_colData <- MultiAssayExperiment::colData(
            MultiAssayExperiment::experiments(exposomicset)[[omics_name]]
        )

        current_colData <- current_colData[
            , !colnames(current_colData) %in% non_normal_vars,
            drop = FALSE
        ]

        MultiAssayExperiment::colData(
            MultiAssayExperiment::experiments(
                exposomicset
            )[[omics_name]]
        ) <- current_colData
    }

    # Log metadata
    step_record <- list(
        filter_non_normal = list(
            timestamp = Sys.time(),
            params = list(p_thresh = p_thresh),
            notes = paste0(
                "Filtered out non-normal exposure variables,
        Based on p-value threshold of ",
                p_thresh, ". ",
                length(non_normal_vars),
                " variables removed."
            )
        )
    )

    MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
        MultiAssayExperiment::metadata(exposomicset)$summary$steps,
        step_record
    )

    return(exposomicset)
}

#' Plot PCA Results for Features and Samples
#'
#' Generates PCA plots for both feature space and sample space,
#' including scatter plots and scree plots.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing PCA results
#' in `metadata(exposomicset)$pca`.
#' @param feature_col A character string specifying the color for the
#'  feature scree plot.
#' Default is `"#00a9b2"`.
#' @param sample_col A character string specifying the color for the
#'  sample scree plot.
#' Default is `"#8a4f77"`.
#' @param sample_outlier_col A character string specifying the color
#' for sample outlier labels.
#' Default is `"firebrick"`.
#'
#' @details
#' This function creates four PCA visualizations:
#' - **Feature Space PCA Plot**: Colored by category (e.g., omics, exposure).
#' - **Feature Scree Plot**: Displays the variance explained by each
#' principal component.
#' - **Sample Space PCA Plot**: Highlights outlier samples.
#' - **Sample Scree Plot**: Displays variance explained in the sample PCA.
#'
#' Outliers are labeled based on `metadata(exposomicset)$pca$outliers`.
#'
#' @return A combined `ggplot` object containing the four PCA plots.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run pca
#' mae <- mae |>
#'     run_pca()
#'
#' # create the pca plot
#' pca_p <- mae |>
#'     plot_pca()
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom dplyr mutate case_when
#' @import ggplot2
#' @importFrom ggpubr theme_pubr
#' @importFrom factoextra fviz_eig
#' @importFrom ggrepel geom_text_repel
#'
#' @export
plot_pca <- function(
  exposomicset,
  feature_col = "#00a9b2",
  sample_col = "#8a4f77",
  sample_outlier_col = "firebrick"
) {
    .check_suggested(pkg = "patchwork")

    # Check if the required metadata is present
    if (is.null(MultiAssayExperiment::metadata(exposomicset)$quality_control$pca)) {
        stop("Please run `run_pca` first.")
    }

    # grab data
    dat <- MultiAssayExperiment::metadata(exposomicset)$quality_control$pca$pca_df |>
        as.data.frame()
    pca_feature <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "quality_control",
            "pca",
            "pca_feature"
        )
    pca_sample <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "quality_control",
            "pca",
            "pca_sample"
        )

    # capitalize exposure for plot
    dat <- dat |>
        dplyr::mutate(category = dplyr::case_when(
            category == "exposure" ~ "Exposure",
            .default = category
        ))


    # PCA Feature Scatter Plot
    pca_plot_feature <- pca_feature |>
        purrr::pluck("x") |>
        as.data.frame() |>
        mutate(category = dat$category) |>
        ggplot(aes(
            x = PC1,
            y = PC2,
            color = category
        )) +
        geom_point() +
        ggpubr::theme_pubr(legend = "right") +
        scale_color_tidy_exp() +
        labs(
            title = "PCA of Feature Space",
            color = "",
            x = sprintf(
                "PC1 (%s%%)",
                round(
                    (summary(pca_feature)$importance[2, seq_len(2)] * 100)[1],
                    digits = 1
                )
            ),
            y = sprintf(
                "PC2 (%s%%)",
                round(
                    (summary(pca_feature)$importance[2, seq_len(2)] * 100)[2],
                    digits = 1
                )
            )
        ) +
        theme(
            plot.title = element_text(face = "bold.italic"),
            plot.subtitle = element_text(face = "italic")
        )
    # pca_plot_feature <- autoplot(pca_feature,
    #     data = dat,
    #     colour = "category"
    # ) +
    #     ggpubr::theme_pubr(legend = "right") +
    #     ggsci::scale_color_aaas() +
    #     labs(title = "PCA of Feature Space", color = "") +
    #     theme(
    #         plot.title = element_text(face = "bold.italic"),
    #         plot.subtitle = element_text(face = "italic")
    #     )

    # PCA Feature Scree Plot
    scree_feature <- factoextra::fviz_eig(
        pca_feature,
        barfill = feature_col,
        barcolor = feature_col,
        main = "Scree Plot of Feature Space"
    ) +
        theme(
            plot.title = element_text(face = "bold.italic"),
            plot.subtitle = element_text(face = "italic")
        )

    # Define outliers
    outlier_samples <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "quality_control",
            "pca",
            "outliers"
        )

    # PCA sample scatter plot
    pca_plot_sample <- pca_sample |>
        purrr::pluck("x") |>
        as.data.frame() |>
        (\(df){
            df$id <- rownames(df)
            df
        })() |>
        mutate(label = ifelse(id %in% outlier_samples, id, NA)) |>
        ggplot(aes(
            x = PC1,
            y = PC2
        )) +
        geom_point(color = sample_col) +
        ggrepel::geom_text_repel(
            aes(
                x = PC1,
                y = PC2,
                label = label
            ),
            color = sample_outlier_col,
            na.rm = TRUE
        ) +
        ggpubr::theme_pubr() +
        labs(
            title = "PCA of Sample Space",
            x = sprintf(
                "PC1 (%s%%)",
                round(
                    (summary(pca_sample)$importance[2, seq_len(2)] * 100)[1],
                    digits = 1
                )
            ),
            y = sprintf(
                "PC2 (%s%%)",
                round(
                    (summary(pca_sample)$importance[2, seq_len(2)] * 100)[2],
                    digits = 1
                )
            )
        ) +
        theme(
            plot.title = element_text(face = "bold.italic"),
            plot.subtitle = element_text(face = "italic")
        )

    # Sample Scree Plot
    scree_sample <- factoextra::fviz_eig(
        pca_sample,
        barfill = sample_col,
        barcolor = sample_col,
        main = "Scree Plot of Sample Space"
    ) +
        theme(
            plot.title = element_text(face = "bold.italic"),
            plot.subtitle = element_text(face = "italic")
        )

    # Combine plots
    combined_plot <- patchwork::wrap_plots(
        pca_plot_feature, scree_feature,
        pca_plot_sample, scree_sample
    )

    return(combined_plot)
}

#' Plot Circular Network of Exposure Relationships
#'
#' Generates a circular network plot to visualize relationships between
#' exposures, either based on correlation ("exposures") or
#' shared features ("degs", "factors").
#'
#' @param exposomicset A MultiAssayExperiment object.
#' @param feature_type One of "exposures", "degs", or "factors".
#' @param exposure_cols Character vector of exposures to include
#' (only for "exposures").
#' @param corr_threshold Minimum |correlation| (only for "exposures").
#' @param shared_cutoff Minimum number of shared features
#' (only for "degs" or "factors"). Default = 10.
#' @param annotation_colors Optional named vector of colors for categories.
#' @param low low value color for edges.
#' @param mid middle value color for edges.
#' @param high high value color for edges.
#' @param midpoint Midpoint for edge color gradient. Defaults to 0
#' (for correlations) or mean shared features.
#'
#' @return A ggplot object (ggraph circular plot).
#'
#' @import ggplot2
#' @importFrom dplyr filter rename arrange
#' @importFrom igraph graph_from_data_frame
#'
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run correlation analysis
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "exposures",
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # create the circos plot
#' circos_plot <- mae |>
#'     plot_circos_correlation(
#'         feature_type = "exposures"
#'     )
#'
#' @export
plot_circos_correlation <- function(
  exposomicset,
  feature_type = c(
      "degs",
      "omics",
      "factors",
      "factor_features",
      "exposures", "pcs"
  ),
  exposure_cols = NULL,
  corr_threshold = NULL,
  shared_cutoff = 10,
  annotation_colors = NULL,
  low = "#006666",
  mid = "white",
  high = "#8E0152",
  midpoint = NULL
) {
    # require(ggplot2)
    # require(ggraph)
    # require(igraph)
    .check_suggested(pkg = "ggraph")
    .check_suggested(pkg = "scales")

    feature_type <- match.arg(feature_type)

    # Load correlation or shared feature data
    if (feature_type == "exposures") {
        correlation_df <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck("correlation", "exposures")

        if (is.null(correlation_df)) {
            stop("Run `run_correlation(..., feature_type = 'exposures')` first.")
        }

        # Optional exposure filtering
        if (!is.null(exposure_cols)) {
            correlation_df <- dplyr::filter(
                correlation_df,
                var1 %in% exposure_cols & var2 %in% exposure_cols
            )
        }

        if (!is.null(corr_threshold)) {
            correlation_df <- dplyr::filter(
                correlation_df, abs(correlation) >= corr_threshold
            )
        }

        correlation_df$edge_weight <- abs(correlation_df$correlation)
        correlation_df <- dplyr::rename(correlation_df,
            source = var1,
            target = var2
        )
        edge_color_var <- correlation_df$correlation
        midpoint_val <- ifelse(is.null(midpoint), 0, midpoint)
    } else {
        # degs or factors
        tag <- if (feature_type == "degs") {
            "omics_exposure_deg_correlation"
        } else {
            "omics_exposure_factor_correlation"
        }

        correlation_list <- MultiAssayExperiment::metadata(exposomicset)$correlation
        correlation_df <- correlation_list[[feature_type]]

        # if (is.null(correlation_df)){
        #   stop(paste0(
        #     "No correlation found for feature_type = '",
        #     feature_type,
        #     "'. Run `run_correlation(..., feature_type = '",
        #     feature_type,
        #     "')` first."))
        # }
        if (is.null(correlation_df)) {
            stop(sprintf(
                "No correlation found for feature_type = '%s'.
        Run `run_correlation(..., feature_type = '%s')` first.",
                feature_type, feature_type
            ))
        }

        feature_list <- split(
            correlation_df,
            correlation_df$exposure
        ) |>
            purrr::map(~ unique(.x$feature))

        overlap_df <- .get_pairwise_overlaps(feature_list)
        overlap_df <- overlap_df |>
            dplyr::filter(num_shared > shared_cutoff)

        correlation_df <- overlap_df |>
            dplyr::rename(
                source = source,
                target = target,
                edge_weight = num_shared
            )

        edge_color_var <- correlation_df$edge_weight
        midpoint_val <- ifelse(is.null(midpoint),
            mean(correlation_df$edge_weight),
            midpoint
        )
    }

    # Get node metadata
    codebook <- MultiAssayExperiment::metadata(exposomicset)$codebook
    node_vars <- unique(c(correlation_df$source, correlation_df$target))
    vertex_df <- dplyr::filter(codebook, variable %in% node_vars)

    # Sort Correlation Df
    correlation_df <- correlation_df |>
        arrange(abs(edge_weight))

    # Ensure igraph object
    graph <- igraph::graph_from_data_frame(correlation_df,
        vertices = vertex_df,
        directed = FALSE
    )

    # Colors
    all_categories <- na.omit(unique(vertex_df$category))
    if (!is.null(annotation_colors)) {
        if (!all(all_categories %in% names(annotation_colors))) {
            stop("All node categories must be included in annotation_colors.")
        }
        cat_colors <- annotation_colors
    } else {
        cat_colors <- setNames(
            tidy_exp_pal[seq_along(all_categories)],
            all_categories
        )
    }

    # Plot
    ggraph::ggraph(graph,
        layout = "linear",
        circular = TRUE
    ) +
        ggraph::geom_edge_arc(aes(
            color = edge_weight,
            width = scales::rescale(edge_weight, to = c(1, 5))
        )) +
        ggraph::geom_node_point(aes(fill = category),
            shape = 21,
            size = 4,
            alpha = 0.8
        ) +
        ggraph::geom_node_text(
            aes(
                label = name,
                x = 1.1 * x,
                y = 1.1 * y,
                angle = ggraph::node_angle(x, y)
            ),
            hjust = "outward",
            fontface = "bold.italic",
            check_overlap = TRUE
        ) +
        ggraph::theme_graph(base_family = "sans") +
        ggplot2::scale_fill_manual(values = cat_colors) +
        ggraph::scale_edge_color_gradient2(
            low = low,
            mid = mid,
            high = high,
            midpoint = midpoint_val,
            guide = ggraph::guide_edge_colorbar()
        ) +
        ggplot2::coord_fixed(xlim = c(-2, 2), ylim = c(-2, 2)) +
        ggplot2::guides(edge_width = "none") +
        ggplot2::labs(
            edge_color = ifelse(feature_type == "exposures",
                "Correlation",
                "Shared Features"
            ),
            fill = "Category"
        )
}

#' Plot Network Graph of Features or Exposures
#'
#' Visualizes network structures created by `run_create_network()` from
#' the metadata of a `MultiAssayExperiment` object.
#' Nodes can represent features (e.g., genes or factors) or exposures,
#' and edges represent correlations or shared connections.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing network results
#' in metadata.
#' @param network Character string specifying the network type.
#' One of `"degs"`, `"omics"`, `"factors"`,
#'   `"factor_features"`, `"exposures"`, `"degs_feature_cor"`,
#'   `"omics_feature_cor"`, `"factor_features_feature_cor"`.
#' @param include_stats Logical; if `TRUE`, include edge weights and
#' node centrality metrics in the plot aesthetics. Default is `TRUE`.
#' @param nodes_to_include Optional character vector of node names to
#' include (subset of `name`).
#' @param centrality_thresh Optional numeric threshold to filter nodes
#' by centrality degree.
#' @param top_n_nodes Optional integer to keep only the top N nodes by
#' centrality.
#' @param cor_thresh Optional numeric threshold to filter edges by
#' minimum absolute correlation.
#' @param label Logical; whether to label nodes.
#' If `TRUE`, top nodes will be labeled.
#' @param label_top_n Integer; number of top-centrality nodes to
#' label if `label = TRUE`. Default is `5`.
#' @param nodes_to_label Optional character vector of specific nodes to label.
#' @param facet_var Optional node metadata column to facet the
#' network layout by.
#' @param foreground Color for node outlines and edge borders.
#' Default is `"steelblue"`.
#' @param fg_text_colour Color of node label text. Default is `"grey25"`.
#' @param node_colors Optional named vector of colors for node groups.
#' @param node_color_var Optional node attribute used for node color mapping.
#' @param alpha Alpha transparency for nodes and edges. Default is `0.5`.
#' @param size_lab Legend title for node size (typically centrality).
#'  Default is `"Centrality"`.
#' @param color_lab Legend title for node color group. Default is `"Group"`.
#'
#' @return A `ggraph` plot object.
#'
#' @details
#' This function retrieves the stored graph object and optionally filters or
#'  labels nodes based on:
#' centrality, correlation, user input, or group-specific attributes.
#'  It supports layout faceting,
#' custom color mappings, and highlights highly central nodes.
#'
#' Large graphs (> 500 nodes) will prompt the user before plotting.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # perform correlation analyses
#' # correlate with exposures
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # create the networks
#' mae <- mae |>
#'     run_create_network(
#'         feature_type = "omics",
#'         action = "add"
#'     )
#'
#' # plot the network
#' network_p <- mae |>
#'     plot_network(
#'         network = "omics"
#'     )
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom igraph gorder
#' @export
plot_network <- function(
  exposomicset,
  network = c(
      "degs",
      "omics",
      "factors",
      "factor_features",
      "exposures",
      "degs_feature_cor",
      "omics_feature_cor",
      "factor_features_feature_cor"
  ),
  include_stats = TRUE,
  nodes_to_include = NULL,
  centrality_thresh = NULL,
  top_n_nodes = NULL,
  cor_thresh = NULL,
  label = FALSE,
  label_top_n = 5,
  nodes_to_label = NULL,
  facet_var = NULL,
  foreground = "steelblue",
  fg_text_colour = "grey25",
  node_colors = NULL,
  node_color_var = NULL,
  alpha = 0.5,
  size_lab = "Centrality",
  color_lab = "Group"
) {
    # require(ggraph)
    # require(tidygraph)
    .check_suggested(pkg = "tidygraph")
    .check_suggested(pkg = "ggraph")
    # tidygraph as_tbl_graph activate filter mutate centrality_degree
    # arrange slice_head select as_tibble
    network <- match.arg(network)
    net_key <- paste0("network_", network)
    net_obj <- MultiAssayExperiment::metadata(exposomicset)$network[[net_key]]

    if (is.null(net_obj)) {
        stop(
            "No network found for `feature_type = '",
            network,
            "'. Please run `run_create_network()` first."
        )
    }

    message("Extracting graph.")
    g <- tidygraph::as_tbl_graph(net_obj$graph)

    if (!is.null(nodes_to_include)) {
        g <- g |>
            tidygraph::activate(nodes) |>
            tidygraph::filter(name %in% nodes_to_include)
    }

    if (!is.null(cor_thresh)) {
        g <- g |>
            tidygraph::activate(edges) |>
            tidygraph::filter(abs(correlation) > cor_thresh)
    }

    if (!is.null(centrality_thresh)) {
        g <- g |>
            tidygraph::activate(nodes) |>
            mutate(centrality = tidygraph::centrality_degree()) |>
            tidygraph::filter(centrality > centrality_thresh)
    }

    if (!is.null(top_n_nodes)) {
        g <- g |>
            tidygraph::activate(nodes) |>
            tidygraph::mutate(centrality = tidygraph::centrality_degree()) |>
            tidygraph::arrange(desc(centrality)) |>
            tidygraph::slice_head(n = top_n_nodes)
    }

    if (!is.null(nodes_to_label) && isTRUE(label)) {
        g <- g |>
            tidygraph::activate(nodes) |>
            tidygraph::mutate(label = ifelse(name %in% nodes_to_label, name, NA))
    } else if (isTRUE(label)) {
        g <- g |>
            tidygraph::activate(nodes) |>
            tidygraph::mutate(centrality = tidygraph::centrality_degree()) |>
            tidygraph::arrange(desc(centrality)) |>
            tidygraph::mutate(label = ifelse(row_number() <= label_top_n, name, NA))
    }

    # Prune nodes without edges
    used_node_indices <- g |>
        tidygraph::activate(edges) |>
        tidygraph::as_tibble() |>
        tidygraph::select(from, to) |>
        unlist() |>
        unique()

    g <- g |>
        tidygraph::activate(nodes) |>
        tidygraph::mutate(node_index = row_number()) |>
        tidygraph::filter(node_index %in% used_node_indices) |>
        tidygraph::select(-node_index)

    # Warn if plotting very large networks
    if (igraph::gorder(g) > 500) {
        user_input <- readline("The network has more than 500 nodes.
        Plot anyway? (y/n): ")
        if (tolower(user_input) != "y") stop("Exiting.")
    }

    # Build plot using internal helper
    g_plot <- .build_ggraph_plot(
        g = g,
        node_color_var = node_color_var,
        label = label,
        facet_var = facet_var,
        include_stats = include_stats,
        fg_text_colour = fg_text_colour,
        foreground = foreground,
        alpha = alpha,
        color_lab = color_lab,
        node_colors = node_colors,
        size_lab = size_lab
    )

    return(g_plot)
}

#' Summarize and Visualize Analysis Pipeline Steps
#'
#' This function prints and visualizes the analysis steps stored in the
#' metadata of a \code{MultiAssayExperiment} object. The steps are optionally
#' printed to the console as a numbered list and can be rendered as a
#' left-to-right Mermaid flowchart.
#' The flowchart connects steps with arrows and includes step notes
#' if requested.
#'
#' @param exposomicset A \code{MultiAssayExperiment} object that contains a
#' "summary"
#' entry in its metadata, which includes a list named \code{steps}.
#' @param show_index Logical, default \code{TRUE}. If \code{TRUE},
#' prefixes each step with its index.
#' @param console_print Logical, default \code{TRUE}. If \code{TRUE},
#' prints the step list to the console.
#' @param diagram_print Logical, default \code{FALSE}. If \code{TRUE},
#'  renders a Mermaid diagram of the steps.
#' @param include_notes Logical, default \code{TRUE}. If \code{TRUE},
#' appends any "notes" associated with each step to the label.
#'
#' @return No return value. This function is called for its side effects:
#' console output and/or diagram rendering.
#'
#' @details The Mermaid flowchart is rendered left-to-right and connects
#' each step in sequence. Each node is labeled using the step name and,
#'  optionally,
#' any attached notes.
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Test for normality
#' mae <- mae |>
#'     run_normality_check() |>
#'     transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))
#'
#' # Run the pipeline summary
#' run_pipeline_summary(mae)
#'
#' @export
run_pipeline_summary <- function(
  exposomicset,
  show_index = TRUE,
  console_print = TRUE,
  diagram_print = FALSE,
  include_notes = TRUE
) {
    # Validate metadata
    summary_md <- MultiAssayExperiment::metadata(exposomicset)$summary
    if (!("steps" %in% names(summary_md))) {
        stop("Please run analysis steps before running the pipeline summary.")
    }

    # Get step names
    step_names <- names(summary_md$steps)

    # Optionally include notes in step text
    step_labels <- purrr::map_chr(step_names, function(step) {
        note <- if (include_notes && !is.null(summary_md$steps[[step]]$notes)) {
            paste0(" - ", summary_md$steps[[step]]$notes)
        } else {
            ""
        }
        paste0(step, note)
    })

    # Add index if needed
    labeled_steps <- if (show_index) {
        sprintf("%d. %s", seq_along(step_labels), step_labels)
    } else {
        step_labels
    }

    # Optional console output
    if (console_print) {
        message(paste(labeled_steps, collapse = "\n"))
    }

    if (diagram_print) {
        .check_suggested("DiagrammeR")
        # Build Mermaid diagram
        mermaid_lines <- c("graph TD")
        for (i in seq_along(labeled_steps)[-length(labeled_steps)]) {
            from <- sprintf("step%d", i)
            to <- sprintf("step%d", i + 1)
            from_label <- labeled_steps[i]
            to_label <- labeled_steps[i + 1]

            mermaid_lines <- c(
                mermaid_lines,
                sprintf('%s["%s"] --> %s["%s"]', from, from_label, to, to_label)
            )
        }

        DiagrammeR::mermaid(paste(mermaid_lines, collapse = "\n"))
    }
}

#' Run Multi-Omics Integration
#'
#' Performs multi-omics integration using one of several available methods:
#'  MOFA, MCIA, RGCCA, or DIABLO. This function takes a `MultiAssayExperiment`
#'  object with two or more assays and computes shared latent factors
#'  across omics layers.
#'
#' @param exposomicset A `MultiAssayExperiment` object with at least two assays.
#' @param method Character. Integration method to use. Options are
#' `"MOFA"`, `"MCIA"`, `"RGCCA"`, or `"DIABLO"`.
#' @param n_factors Integer. Number of latent factors/components to compute.
#' Default is 10.
#' @param scale Logical. Whether to scale each assay before integration.
#'  Default is `TRUE`.
#' @param outcome Character. Required if `method = "DIABLO"`.
#' Name of outcome variable in `colData` used for supervised integration.
#' @param max.iter numeric. Option to increase the number of iterations for
#'  `mixOmics::block.splsda` the default is 500.
#' @param near.zero.var Logical. Option to remove variables with near zero
#'  variance for `mixOmics::block.splsda`, default is `TRUE` .
#' @param action Character. Whether to `"add"` results to the metadata or
#' `"get"` them as a list. Default is `"add"`.
#'
#' @return If `action = "add"`, returns a `MultiAssayExperiment` with
#'  integration results
#' stored in `metadata(exposomicset)$multiomics_integration$integration_results`.
#'  If `action = "get"`, returns a list with integration `method` and `result`.
#'
#' @details
#' - `"MOFA"` runs Multi-Omics Factor Analysis using the `MOFA2` package and
#' returns a trained model.
#' - `"MCIA"` runs multi-co-inertia analysis using the `nipalsMCIA` package.
#' - `"RGCCA"` runs Regularized Generalized Canonical Correlation Analysis
#'  using the `RGCCA` package.
#' - `"DIABLO"` performs supervised integration using the `mixOmics` package
#' and a specified outcome.
#'
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # perform multiomics integration
#' mae <- run_multiomics_integration(
#'     mae,
#'     method = "DIABLO",
#'     outcome = "smoker",
#'     n_factors = 3
#' )
#'
#' @export
run_multiomics_integration <- function(
  exposomicset,
  method = "MCIA",
  n_factors = 10,
  scale = TRUE,
  outcome = NULL,
  max.iter = 500,
  near.zero.var = TRUE,
  action = "add"
) {
    if (length(MultiAssayExperiment::experiments(exposomicset)) < 2) {
        stop("Multi-Omics Integration requires at least two assays.")
    }

    if (scale) {
        exposomicset_mo <- .scale_multiassay(exposomicset)
    } else {
        exposomicset_mo <- exposomicset
    }

    message("Running multi-omics integration using ", method, "...")

    result <- switch(method,
        "MOFA" = .run_mofa2(
            exposomicset_mo = exposomicset_mo,
            n_factors = n_factors
        ),
        "MCIA" = .run_mcia(
            exposomicset_mo = exposomicset_mo,
            n_factors = n_factors
        ),
        "RGCCA" = .run_rgcca(
            exposomicset_mo = exposomicset_mo,
            n_factors = n_factors
        ),
        "DIABLO" = .run_diablo(
            exposomicset_mo = exposomicset_mo,
            n_factors = n_factors,
            outcome = outcome,
            max.iter = max.iter,
            near.zero.var = near.zero.var
        )
    )

    if (action == "add") {
        # Store results in MultiAssayExperiment metadata
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$multiomics_integration$integration_results <- list(
            method = method,
            result = result
        )
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        # Add analysis step record
        step_record <- list(
            run_multiomics_integration = list(
                timestamp = Sys.time(),
                params = list(
                    method = method,
                    n_factors = n_factors,
                    scale = scale
                ),
                notes = paste0(
                    "Performed multi-omics integration using ", method,
                    " with ", n_factors, " latent factors. Scaling was ",
                    ifelse(scale, "enabled.", "disabled.")
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )


        return(exposomicset)
    } else if (action == "get") {
        return(list(
            method = method,
            result = result
        ))
    } else {
        stop("Invalid action. Choose from 'add' or 'get'.")
    }
}

# --- MOFA2 Integration Function ---------------
#' Decide which backend to use (basilisk vs reticulate)
#'
#' @return "basilisk" or "reticulate"
#' @keywords internal
#' @noRd
.mofa_backend <- function() {
    # user override wins
    backend <- getOption("tidyexposomics.mofa.backend", NULL)
    if (!is.null(backend)) {
        return(backend)
    }

    # auto-detect: Apple Silicon, reticulate, otherwise basilisk
    if (Sys.info()[["sysname"]] == "Darwin" &&
        grepl("arm64", R.version$platform)) {
        return("reticulate")
    } else {
        return("basilisk")
    }
}

#' Check that the chosen backend is safe to use
#' @keywords internal
#' @noRd
.check_mofa_safe <- function(backend = .mofa_backend()) {
    if (identical(backend, "reticulate")) {
        if (!requireNamespace("reticulate", quietly = TRUE)) {
            stop("reticulate backend requested, but 'reticulate' is not installed.", call. = FALSE)
        }
        ret <- asNamespace("reticulate")
        if (!isTRUE(ret$py_available(initialize = FALSE))) {
            stop("reticulate backend requested, but no Python is available.", call. = FALSE)
        }
        if (!isTRUE(ret$py_module_available("mofapy2"))) {
            stop("reticulate backend requested, but Python module 'mofapy2' is missing.", call. = FALSE)
        }
    } else if (identical(backend, "basilisk")) {
        if (!requireNamespace("MOFA2", quietly = TRUE)) {
            stop("basilisk backend requested, but 'MOFA2' is not installed.", call. = FALSE)
        }
    } else {
        stop("Unknown backend: '", backend, "'.", call. = FALSE)
    }
    invisible(TRUE)
}

#' Run MOFA2 safely with backend switching
#'
#' @param exposomicset_mo A MultiAssayExperiment object
#' @param n_factors Number of latent factors to infer
#' @return A trained MOFA model object
#' @keywords internal
#' @noRd
.run_mofa2 <- function(exposomicset_mo, n_factors) {
    backend <- .mofa_backend()
    message("Using MOFA backend: ", backend)
    .check_mofa_safe(backend)


    # Create MOFA object
    mofa <- MOFA2::create_mofa(exposomicset_mo)

    # Options
    model_opts <- MOFA2::get_default_model_options(mofa)
    model_opts$num_factors <- n_factors
    data_opts <- MOFA2::get_default_data_options(mofa)
    train_opts <- MOFA2::get_default_training_options(mofa)

    # Prepare model
    mofa <- MOFA2::prepare_mofa(
        object = mofa,
        data_options = data_opts,
        model_options = model_opts,
        training_options = train_opts
    )

    outfile <- file.path(tempdir(), "mofa_model.hdf5")

    # Switch runner based on backend
    if (backend == "reticulate") {
        mofa_trained <- MOFA2::run_mofa(mofa, outfile, use_basilisk = FALSE)
    } else {
        mofa_trained <- MOFA2::run_mofa(mofa, outfile, use_basilisk = TRUE)
    }

    # Reload trained model
    MOFA2::load_model(outfile)
}

# --- MCIA Integration Function ----------------
#' @title Run MCIA Integration
#' @description Applies MCIA using `nipalsMCIA` on a
#' MultiAssayExperiment.
#'
#' @param exposomicset_mo A MultiAssayExperiment object containing omics data.
#' @param n_factors Integer. Number of components (PCs) to compute.
#'
#' @return An MCIA result object from `nipalsMCIA::nipals_multiblock()`.
#' @keywords internal
#' @noRd
.run_mcia <- function(
  exposomicset_mo,
  n_factors
) {
    message("Applying MCIA with `nipalsMCIA`")

    # Run NIPALS MCIA on the MultiAssayExperiment
    .check_suggested("nipalsMCIA")
    result <- nipalsMCIA::nipals_multiblock(
        exposomicset_mo,
        col_preproc_method = "colprofile",
        num_PCs = n_factors,
        tol = 1e-12,
        plots = "none"
    )
}
# --- RGCCA Integration Function ---------------
#' @title Run RGCCA Integration
#' @description Applies unsupervised RGCCA to integrate multiple
#' omics blocks.
#'
#' @param exposomicset_mo A MultiAssayExperiment object containing omics data.
#' @param n_factors Integer. Number of components to extract.
#'
#' @return An RGCCA result object from `RGCCA::rgcca()`.
#' @keywords internal
#' @noRd
#' @importFrom RGCCA rgcca
#' @importFrom SummarizedExperiment assay
#' @importFrom MultiAssayExperiment experiments
.run_rgcca <- function(
  exposomicset_mo,
  n_factors
) {
    message("Applying RGCCA integration.")

    # Prepare data: list of assays (samples x features)
    x <- purrr::map(
        MultiAssayExperiment::experiments(exposomicset_mo),
        ~ t(SummarizedExperiment::assay(.x)) # transpose to samples x features
    )

    # Use default shrinkage or manual choice (e.g., 1 for PCA-like)
    tau <- rep(1, length(x)) # no regularization

    # Run RGCCA
    rgcca_result <- RGCCA::rgcca(
        blocks = x,
        connection = 1 - diag(length(x)),
        tau = tau,
        ncomp = n_factors,
        scheme = "centroid",
        scale = FALSE,
        verbose = FALSE
    )

    result <- rgcca_result
}
# --- DIABLO Integration Function --------------
#' @title Run DIABLO Supervised Integration
#' @description Applies supervised integration using DIABLO (block.splsda)
#' from mixOmics.
#'
#' @param exposomicset_mo A MultiAssayExperiment object containing omics data.
#' @param n_factors Integer. Number of components per block.
#' @param outcome Character. Column name in colData to use as the outcome.
#'
#' @return A DIABLO result object from `mixOmics::block.splsda()`.
#' @keywords internal
#' @noRd
#' @importFrom mixOmics block.splsda
#' @importFrom SummarizedExperiment assay
#' @importFrom MultiAssayExperiment experiments colData
.run_diablo <- function(
  exposomicset_mo,
  n_factors,
  outcome,
  max.iter,
  near.zero.var
) {
    message("Applying DIABLO supervised integration.")

    if (is.null(outcome)) {
        stop("DIABLO requires an outcome variable for supervision.")
    }
    # Ensure that only the common samples are taken
    common <- purrr::map(
        MultiAssayExperiment::experiments(exposomicset_mo),
        ~ colnames(.x)
    ) |>
        (\(lst) Reduce(intersect, lst))()

    # Subset the samples
    exposomicset_mo <- exposomicset_mo[, common]

    # Extract colData and outcome
    y <- MultiAssayExperiment::colData(exposomicset_mo)[[outcome]]
    if (!is.factor(y)) y <- as.factor(y)

    # Prepare blocks (assays) as list of matrices (samples x features)
    blocks <- purrr::map(
        MultiAssayExperiment::experiments(exposomicset_mo),
        ~ t(SummarizedExperiment::assay(.x))
    )

    # Use 1 component by default for each block
    ncomp <- rep(n_factors, length(blocks))
    names(ncomp) <- names(blocks)

    # Run DIABLO
    design <- matrix(1, ncol = length(blocks), nrow = length(blocks))
    diag(design) <- 0
    rownames(design) <- colnames(design) <- names(blocks)

    result <- mixOmics::block.splsda(
        X = blocks,
        Y = y,
        ncomp = n_factors,
        design = design,
        max.iter = max.iter,
        near.zero.var = near.zero.var
    )
}

#' Build the Ontology Annotation Shiny app
#'
#' Returns a Shiny app object for ontology annotation. This function does not
#' launch the app. Please call \code{shiny::runApp()} yourself (see examples).
#'
#' @param use_demo Logical, if TRUE, load packaged lightweight demo ontologies.
#' @param ... Optional named overrides passed as data.frames with columns
#'   \code{id}, \code{name}, \code{ancestors}, \code{parents}, \code{children}:
#'   \code{hpo}, \code{ecto}, \code{chebi}.
#'
#' @return A \code{shiny.appobj}.
#' @examples
#' if (interactive()) {
#'     app <- build_ont_annot_app()
#'     shiny::runApp(app)
#' }
#' @export
#' @importFrom shiny shinyApp
build_ont_annot_app <- function(use_demo = TRUE, ...) {
    ont <- .load_ontologies(use_demo = use_demo, ...)
    ui <- .ont_annot_build_ui()
    srv <- .ont_annot_build_server(ontologies = ont)
    shiny::shinyApp(ui = ui, server = srv)
}

#' Volcano Plot of Differential Abundance
#'
#' Generates a **volcano plot** to visualize differential abundance results
#' across one or more omics layers.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing differential
#' abundance results in `metadata(exposomicset)$differential_abundance`.
#' @param pval_col A character string specifying the column containing p-values.
#'  Default is `"adj.P.Val"`.
#' @param pval_thresh A numeric threshold for significance. Features with
#'  p-values below this are considered significant. Default is `0.05`.
#' @param logFC_col A character string specifying the column for
#' log fold changes. Default is `"logFC"`.
#' @param logFC_thresh A numeric threshold for absolute
#' log fold change significance. Default is `log2(1.5)`.
#' @param plot_n_sig Logical; if `TRUE`, appends the number of
#'  significant features to facet titles. Default is `TRUE`.
#' @param top_n_label Optional integer. If provided, the top `n`
#' most significant features per assay will be labeled on the plot.
#' @param features_to_label Optional character vector.
#' Specific features to label regardless of significance.
#' @param feature_col A character string naming the feature ID
#' column to use for labeling. Default is `"feature"`.
#' @param xlab Label for the x-axis. Default is `expression(Log[2]*"FC")`.
#' @param ylab Label for the y-axis. Default is `expression(-Log[10]*"P")`.
#' @param title Plot title. Default is
#' `"Volcano Plot of Differential Abundance"`.
#' @param nrow Number of rows in the `facet_wrap()` layout.
#' Default is `2`.
#'
#' @details
#' The function:
#' - Extracts differential abundance results from
#' `metadata(exposomicset)$differential_abundance`.
#' - Assigns each feature a direction of change:
#' **Upregulated**, **Downregulated**, or **Not-Significant**.
#' - Uses `logFC_thresh` and `pval_thresh` to define thresholds.
#' - Adds dashed lines to indicate cutoffs for fold change and significance.
#' - Uses `facet_wrap()` to display each assay (`exp_name`) separately.
#' - Optionally labels the most significant features or user-defined ones.
#'
#' @return A `ggplot2` object representing the volcano plot.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # perform differential abundance analysis
#' mae <- run_differential_abundance(
#'     exposomicset = mae,
#'     formula = ~ smoker + sex,
#'     abundance_col = "counts",
#'     method = "limma_voom",
#'     action = "add"
#' )
#'
#' # create the volcano plot
#' volcano_p <- mae |>
#'     plot_volcano()
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom dplyr group_by summarise mutate inner_join arrange
#' desc case_when filter slice_head sym
#' @importFrom ggplot2 ggplot aes geom_point geom_vline geom_hline
#' facet_wrap labs theme element_text scale_color_manual
#' @importFrom ggrepel geom_label_repel
#' @importFrom ggpubr theme_pubr
#' @export
plot_volcano <- function(
  exposomicset,
  pval_col = "adj.P.Val",
  pval_thresh = 0.05,
  logFC_col = "logFC",
  logFC_thresh = log2(1.5),
  plot_n_sig = TRUE,
  top_n_label = NULL,
  features_to_label = NULL,
  feature_col = "feature",
  xlab = expression(Log[2] * "FC"),
  ylab = expression(-Log[10] * "P"),
  title = "Volcano Plot of Differential Abundance",
  nrow = 2
) {
    # require(ggplot2)

    # Check to see if Differential Abundance Results are available
    if (!"differential_abundance" %in% names(
        MultiAssayExperiment::metadata(exposomicset)$differential_analysis
    )) {
        stop("Please run `run_differential_abundance()` first.")
    }

    if (plot_n_sig) {
        # Grab the significant features per experiment
        exp_sum <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck(
                "differential_analysis",
                "differential_abundance"
            ) |>
            dplyr::group_by(exp_name) |>
            dplyr::summarise(
                total = dplyr::n(),
                total_significant = sum(
                    !!sym(pval_col) < pval_thresh &
                        abs(!!sym(logFC_col)) > logFC_thresh
                )
            ) |>
            dplyr::mutate(exp_name_plot = paste(
                exp_name,
                "\n",
                " (",
                total_significant,
                "/",
                total,
                ")",
                sep = ""
            ))
    } else {
        exp_sum <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck(
                "differential_analysis",
                "differential_abundance"
            ) |>
            dplyr::group_by(exp_name) |>
            dplyr::summarise(
                total = dplyr::n(),
                total_significant = sum(
                    !!sym(pval_col) < pval_thresh &
                        abs(!!sym(logFC_col)) > logFC_thresh
                )
            ) |>
            dplyr::mutate(exp_name_plot = exp_name)
    }


    plot_df <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "differential_analysis",
            "differential_abundance"
        ) |>
        dplyr::inner_join(exp_sum,
            by = "exp_name"
        ) |>
        dplyr::arrange(dplyr::desc(total)) |>
        dplyr::mutate(exp_name_plot = factor(
            exp_name_plot,
            levels = unique(exp_name_plot)
        )) |>
        dplyr::mutate(direction = dplyr::case_when(
            !!sym(logFC_col) > logFC_thresh &
                !!sym(pval_col) < pval_thresh ~ "Upregulated",
            !!sym(logFC_col) < -logFC_thresh &
                !!sym(pval_col) < pval_thresh ~ "Downregulated",
            .default = "Not-Significant"
        ))

    volcano <- plot_df |>
        ggplot(aes(
            x = !!sym(logFC_col),
            y = -log10(!!sym(pval_col)),
            color = direction
        )) +
        geom_point(alpha = 0.5) +
        geom_vline(
            xintercept = c(-logFC_thresh, logFC_thresh),
            linetype = "dashed",
            color = "grey55"
        ) +
        geom_hline(
            yintercept = -log10(pval_thresh),
            linetype = "dashed",
            color = "grey55"
        ) +
        ggpubr::theme_pubr(legend = "bottom") +
        scale_color_manual(values = c(
            "Upregulated" = "#8E0152",
            "Downregulated" = "#006666",
            "Not-Significant" = "grey55"
        )) +
        facet_wrap(. ~ exp_name_plot, nrow = nrow) +
        theme(
            strip.text = element_text(face = "bold.italic"),
            plot.title = element_text(face = "bold.italic")
        ) +
        labs(
            x = xlab,
            y = ylab,
            title = title,
            color = "Direction"
        )

    if (!is.null(top_n_label)) {
        # Label the top n points
        volcano <- volcano +
            ggrepel::geom_label_repel(
                data = plot_df |>
                    group_by(exp_name) |>
                    dplyr::arrange(!!sym(pval_col)) |>
                    dplyr::slice_head(n = top_n_label),
                aes(label = paste0("italic('", !!sym(feature_col), "')")),
                parse = TRUE,
                size = 3,
                max.overlaps = Inf,
                show.legend = FALSE
            )
    }

    if (!is.null(features_to_label)) {
        # Label specific features
        volcano <- volcano +
            ggrepel::geom_label_repel(
                data = plot_df |>
                    dplyr::filter(!!sym(feature_col) %in% features_to_label),
                aes(label = paste0("italic('", !!sym(feature_col), "')")),
                parse = TRUE,
                size = 3,
                max.overlaps = Inf,
                show.legend = FALSE
            )
    }

    volcano
}

#' Plot Exposure Distributions by Category or Group
#'
#' Visualizes exposure variable distributions using **boxplots**
#' or **ridge plots**.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing exposure data.
#' @param exposure_cat A character string or vector specifying exposure
#' category names (from `codebook$category`) to include.
#' Use `"all"` to include all exposures.
#' @param exposure_cols Optional character vector specifying exact
#' exposure variables to plot.
#' @param group_by A string specifying the column in `colData(exposomicset)`
#' used to fill the plot (e.g., `"sex"`). Defaults to `NULL`, in which case
#' exposures are colored by `category`.
#' @param plot_type Type of plot: `"boxplot"` (default) or `"ridge"`.
#' @param alpha Transparency level for background facet color strips.
#' Default is `0.5`.
#' @param panel_sizes A numeric vector passed to `ggh4x::force_panelsizes()`
#' for controlling facet widths or heights.
#' @param title Plot title. Default is `"Exposure Levels by Category"`.
#' @param xlab X-axis label. Default is an empty string.
#' @param ylab Y-axis label. Default is an empty string.
#' @param facet_cols Optional vector of colors to use as background for
#' facet categories. If `NULL`, a default palette is used.
#' @param group_cols Optional named vector of colors for `group_by` levels.
#' If `NULL`, a default palette is used.
#' @param box_width A numeric value specifying the width of the boxplots.
#' Only used when `plot_type = "boxplot"`. Default is `0.1`.
#' @param fill_lab Legend title for the fill aesthetic
#' (e.g., `"Sex"` or `"Exposure Group"`). Default is `""`.
#'
#' @details
#' This function:
#' - Filters exposure data based on category or selected columns.
#' - Merges variable metadata from `metadata(exposomicset)$codebook`.
#' - Supports either **boxplot** (vertical distributions per variable)
#' or **ridgeplot** (horizontal density plots per variable).
#' - If `group_by` is specified, that variable defines the plot fill color;
#' otherwise, the fill is based on exposure `category`.
#' - Facets by `category` using `ggh4x::facet_grid2()`
#' with color-coded strip backgrounds.
#' - The `box_width` argument controls the width of the boxplots when
#' `plot_type = "boxplot"`.
#'
#' @return A `ggplot2` object showing exposure distributions,
#' optionally grouped.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # plot exposure data
#' exposure_plot <- mae |>
#'     plot_exposures(
#'         exposure_cols = c("exposure_pm25", "exposure_no2"),
#'         box_width = 0.2
#'     )
#'
#' @importFrom ggplot2 ggplot aes geom_boxplot geom_jitter
#' geom_tile scale_color_manual scale_fill_manual scale_y_log10
#' scale_x_log10 labs theme_minimal theme element_text guides guide_legend
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr filter select pull inner_join all_of where distinct mutate
#' @importFrom purrr pluck
#' @importFrom ggpubr theme_pubclean
#'
#' @export
plot_exposures <- function(
  exposomicset,
  exposure_cat = "all",
  exposure_cols = NULL,
  group_by = NULL,
  plot_type = "boxplot",
  alpha = 0.3,
  panel_sizes = rep(1, 100),
  title = "Exposure Levels by Category",
  xlab = "",
  ylab = "",
  facet_cols = NULL,
  group_cols = NULL,
  box_width = 0.1,
  fill_lab = ""
) {
    # require(ggplot2)
    .check_suggested(pkg = "ggh4x")
    .check_suggested(pkg = "ggridges")

    # Extract exposure data
    exposure_data <- exposomicset |>
        pivot_sample()

    # Extract variable description file
    des <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck("codebook")

    # Filter by exposure category if specified
    if (exposure_cat == "all") {
        exposure_data <- exposure_data
    } else {
        vars_to_keep <- des |>
            dplyr::filter(category %in% exposure_cat) |>
            dplyr::pull(variable)
        cols_to_keep <- c(".sample", vars_to_keep)
        if (!is.null(group_by)) cols_to_keep <- c(cols_to_keep, group_by)
        cols_to_keep <- intersect(cols_to_keep, colnames(exposure_data))
        exposure_data <- exposure_data[, cols_to_keep]
    }

    # If specific columns are provided, filter to those
    if (!is.null(exposure_cols)) {
        exposure_data <- exposure_data[, c(".sample", group_by, exposure_cols)]
    }

    # Ensure only numeric exposure variables + .sample are retained
    numeric_cols <- exposure_data |>
        dplyr::select(dplyr::where(is.numeric)) |>
        colnames()

    exposure_data <- exposure_data |>
        dplyr::select(dplyr::all_of(c(".sample", group_by, numeric_cols)))


    # Pivot to long format and join with variable metadata
    sample_metadata <- exposure_data |>
        tidyr::pivot_longer(
            cols = -c(.sample, group_by),
            names_to = "variable",
            values_to = "value"
        ) |>
        dplyr::inner_join(des, by = "variable")

    # set facet colors
    if (!is.null(facet_cols)) {
        facet_cols <- facet_cols
    } else {
        facet_cols <- tidy_exp_pal[
            seq_len(length(unique(sample_metadata$category)))
        ]
    }

    if (!is.null(group_by)) {
        group_var <- group_by
    } else {
        group_var <- "category"
    }

    if (!is.null(group_cols)) {
        group_cols <- group_cols
    } else {
        group_cols <- tidy_exp_pal[
            seq_len(length(unique(sample_metadata[[group_var]])))
        ]
    }

    if (plot_type == "boxplot") {
        legend_logic <- ifelse(!is.null(group_by), "right", "none")

        # Create boxplot
        sample_metadata |>
            dplyr::filter(value > 0) |>
            ggplot(aes(
                x = variable,
                y = value,
                fill = !!sym(group_var),
                color = !!sym(group_var)
            )) +
            # geom_violin()+
            geom_boxplot(alpha = 0.4) +
            geom_jitter(alpha = 0.03) +
            ggh4x::facet_grid2(
                ~category,
                scales = "free_x",
                space = "free_x",
                strip = ggh4x::strip_themed(
                    background_x = ggh4x::elem_list_rect(
                        fill = scales::alpha(
                            facet_cols,
                            alpha
                        )
                    )
                )
            ) +
            scale_color_manual(values = group_cols) +
            scale_fill_manual(values = group_cols) +
            ggh4x::force_panelsizes(cols = panel_sizes) +
            labs(
                title = title,
                x = xlab,
                y = ylab,
                fill = fill_lab
            ) +
            # ggpubr::theme_pubclean() +
            theme_minimal() +
            theme(
                axis.text.x = element_text(angle = 55, hjust = 1),
                legend.position = legend_logic,
                strip.text.x = element_text(face = "bold.italic"),
                plot.title = element_text(face = "bold.italic")
            ) +
            guides(color = "none") +
            scale_y_log10()
    } else if (plot_type == "ridge") {
        # Create boxplot
        sample_metadata |>
            filter(value > 0) |>
            ggplot(aes(
                x = value,
                y = variable,
                fill = !!sym(group_var)
            )) +
            ggridges::geom_density_ridges() +
            ggh4x::facet_grid2(
                category ~ .,
                scales = "free_y",
                space = "free_y",
                strip = ggh4x::strip_themed(
                    background_y = ggh4x::elem_list_rect(
                        fill = scales::alpha(
                            facet_cols,
                            alpha
                        )
                    )
                )
            ) +
            scale_fill_manual(values = group_cols) +
            ggh4x::force_panelsizes(cols = panel_sizes) +
            labs(
                title = title,
                x = xlab,
                y = ylab,
                fill = fill_lab
            ) +
            ggpubr::theme_pubclean() +
            theme(
                axis.text.x = element_text(angle = 45, hjust = 1),
                legend.position = ifelse(!is.null(group_var), "right", "none"),
                strip.text.y = element_text(angle = 0, face = "bold.italic"),
                plot.title = element_text(face = "bold.italic")
            ) +
            scale_x_log10()
    } else {
        stop("Unsupported plot type.")
    }
}

#' Example exposome multi-omics dataset
#'
#' A downsampled version of the ISGlobal Exposome Data Challenge 2021
#' dataset (Maitre *et al.*, *Environment International*, 2022; DOI: 10.1016/j.envint.2022.107422).
#' This example dataset is included with **tidyexposomics** for vignette
#' evaluation. This subset represents samples from children with
#' low socioeconomic status in the first cohort of the original dataset.
#'
#' @details
#' The data have been filtered and transformed to illustrate
#' the `tidyexposomics` workflow. Only a small subset of variables, and
#' the top 500 most variable features per omic layer are retained.
#'
#' **Contents**
#' \describe{
#'   \item{meta}{A data frame of selected exposure, demographic, and
#'   outcome variables for a subset of participants.}
#'   \item{annotated_cb}{A data frame providing ontology-linked annotation
#'   for the exposures in `meta`.}
#'   \item{exp_filt}{A numeric matrix of gene expression values
#'   (500 features, 48 samples) representing the top-variance transcripts.}
#'   \item{exp_fdata}{Feature-level metadata for `exp_filt`, including
#'   cleaned gene symbols.}
#'   \item{methyl_filt}{A numeric matrix of DNA methylation M-values
#'                      (500 CpG sites, 48 samples).}
#'   \item{methyl_fdata}{Feature-level metadata for `methyl_filt`.}
#' }
#'
#' **Source**
#'
#' Derived from the ISGlobal Exposome Data Challenge 2021
#' (Maitre *et al.*, *Environment International*, 2022;
#' DOI: 10.1016/j.envint.2022.107422),licensed under CC-BY 4.0.
#' The original data are available on Figshare (Project 98813)
#' and GitHub (`isglobal-exposomeHub/ExposomeDataChallenge2021`).
#' This example dataset was processed and downsampled by the
#' tidyexposomics authors and is not a replacement for the full dataset.
#'
#' @docType data
#' @usage data("tidyexposomics_example")
#'
#' @examples
#' data("tidyexposomics_example")
#'
"tidyexposomics_example"

#' Calculate Exposure Impact from Feature-Exposure Correlation Networks
#'
#' Generalized centrality-based exposure impact analysis using DEG, omics,
#' or factor features.
#'
#' @param exposomicset A `MultiAssayExperiment` object with correlation
#' and network metadata.
#' @param feature_type One of `"degs"`, `"omics"`, or `"factor_features"`.
#' @param pval_col Column in differential abundance results to filter DEGs.
#' Default = `"adj.P.Val"`.
#' @param pval_thresh DEG p-value threshold. Ignored unless
#' `feature_type == "degs"`.
#' @param action Either `"add"` (store in metadata) or `"get"` (return list).
#'
#' @return Either an updated MultiAssayExperiment (if action = "add") or a list.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # perform correlation analyses
#' # correlate with exposures
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     ) |>
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         feature_cors = TRUE,
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # create the networks
#' mae <- mae |>
#'     run_create_network(
#'         feature_type = "omics_feature_cor",
#'         action = "add"
#'     ) |>
#'     run_create_network(
#'         feature_type = "omics",
#'         action = "add"
#'     )
#'
#' # perform impact analysis
#' mae <- mae |>
#'     run_exposure_impact(
#'         feature_type = "omics"
#'     )
#'
#' @export
run_exposure_impact <- function(
  exposomicset,
  feature_type = c(
      "degs",
      "omics",
      "factor_features"
  ),
  pval_col = "adj.P.Val",
  pval_thresh = 0.1,
  action = c("add", "get")
) {
    .check_suggested(pkg = "tidygraph")

    feature_type <- match.arg(feature_type)
    action <- match.arg(action)

    md <- MultiAssayExperiment::metadata(exposomicset)

    # --- checks
    if (is.null(md$correlation[[feature_type]])) {
        stop(
            "Correlation results missing for feature_type = ", feature_type,
            ". Please run run_correlation(..., feature_type = '", feature_type, "')."
        )
    }

    # prefer "..._feature_cor" but fall back to plain "network_<feature_type>"
    graph_key <- paste0("network_", feature_type, "_feature_cor")
    if (is.null(md$network[[graph_key]])) {
        fallback_key <- paste0("network_", feature_type)
        if (!is.null(md$network[[fallback_key]])) {
            graph_key <- fallback_key
        } else {
            stop(
                "Network missing for feature_type = ", feature_type,
                ". Please run run_create_network(..., feature_type = '", feature_type, "')."
            )
        }
    }

    correlation_df <- md$correlation[[feature_type]]
    net_graph <- md$network[[graph_key]]$graph

    # Optional p-val filter if column exists
    if (!is.null(pval_col) && pval_col %in% names(correlation_df)) {
        correlation_df <- dplyr::filter(correlation_df, .data[[pval_col]] <= pval_thresh)
    }

    # --- centralities from graph
    node_centrality <- tidygraph::as_tbl_graph(net_graph) |>
        tidygraph::activate(nodes) |>
        dplyr::mutate(
            centrality_degree      = tidygraph::centrality_degree(),
            centrality_eigen       = tidygraph::centrality_eigen(),
            centrality_closeness   = tidygraph::centrality_closeness(),
            centrality_betweenness = tidygraph::centrality_betweenness()
        ) |>
        tibble::as_tibble()

    # --- construct join keys that mirror create_network()
    # Case A: exposure-feature schema: feature, exposure, exp_name, category
    if (all(c("feature", "exposure", "exp_name", "category") %in% names(correlation_df))) {
        df <- correlation_df |>
            dplyr::mutate(
                feature_vertex_id  = paste(.data$feature, .data$exp_name, sep = "_"),
                exposure_vertex_id = paste(.data$exposure, .data$category, sep = "_")
            )

        exposure_col <- "exposure"

        # generic var1/var2 with explicit types (expect var1 == exposure)
    } else if (all(c("var1", "var2", "var1_type", "var2_type") %in% names(correlation_df))) {
        df <- correlation_df |>
            dplyr::filter(.data$var1_type == "exposure", .data$var2_type != "exposure") |>
            dplyr::mutate(
                exposure            = .data$var1,
                feature             = .data$var2,
                feature_vertex_id   = paste(.data$var2, .data$var2_type, sep = "_"),
                exposure_vertex_id  = paste(.data$var1, .data$var1_type, sep = "_")
            )

        exposure_col <- "exposure"

        # Case C: cross-experiment pairings (use exp_name_1/exp_name_2 as groups)
    } else if (all(c("var1", "var2", "exp_name_1", "exp_name_2") %in% names(correlation_df))) {
        df <- correlation_df |>
            dplyr::mutate(
                # Naming here keeps "exposure" just as a grouping label for summary;
                # adapt as needed for your schema.
                exposure            = .data$var1,
                feature             = .data$var2,
                feature_vertex_id   = paste(.data$var2, .data$exp_name_2, sep = "_"),
                exposure_vertex_id  = paste(.data$var1, .data$exp_name_1, sep = "_")
            )

        exposure_col <- "exposure"
    } else {
        stop("Unrecognized correlation format: cannot construct vertex ids to match the graph.")
    }

    # --- join feature nodes to their centralities via vertex_id
    df2 <- df |>
        dplyr::left_join(
            node_centrality,
            by = c("feature_vertex_id" = "name") # name is vertex_id in the graph
        )

    # --- aggregate to exposure-level impact
    impact_df <- df2 |>
        dplyr::filter(!is.na(.data$centrality_degree)) |>
        dplyr::group_by(.data[[exposure_col]]) |>
        dplyr::summarise(
            mean_degree = mean(centrality_degree, na.rm = TRUE),
            mean_eigen = mean(centrality_eigen, na.rm = TRUE),
            mean_closeness = mean(centrality_closeness, na.rm = TRUE),
            mean_betweenness = mean(centrality_betweenness, na.rm = TRUE),
            n_features = dplyr::n(),
            .groups = "drop"
        ) |>
        left_join(
            exposomicset@metadata$codebook,
            by = c("exposure" = "variable")
        )

    result <- list(exposure_impact = impact_df)

    if (action == "add") {
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$network$exposure_impact[[feature_type]] <- result
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        step_record <- list(run_exposure_impact = list(
            timestamp = Sys.time(),
            params    = list(feature_type = feature_type, pval_col = pval_col, pval_thresh = pval_thresh),
            notes     = paste("Computed exposure impact using", feature_type, "correlation network.")
        ))
        MultiAssayExperiment::metadata(exposomicset)$summary$steps <-
            c(MultiAssayExperiment::metadata(exposomicset)$summary$steps, step_record)

        return(exposomicset)
    } else {
        return(result)
    }
}

#' Pivot a selected omics dataset from a MultiAssayExperiment into tidybulk format
#'
#' Extracts a specified omics dataset from a \code{MultiAssayExperiment},
#' optionally filters by feature (row) names,
#' and returns a tidy tibble. The output includes assay values along with
#' sample metadata and feature metadata.
#'
#' @param exposomicset A \code{MultiAssayExperiment} object containing
#'   one or more omics assays.
#' @param exp_name A character string. The name of the omics dataset
#'   to extract (e.g., "Proteomics").
#' @param features Optional character vector of row (feature) names to retain.
#'   If \code{NULL}, all features are included.
#'
#' @return A tibble in tidy format with one row per feature/sample pair,
#'   including all metadata and a new column \code{exp_name}
#'   indicating the assay source. Assay values are provided in separate columns
#'   named after the assay slot(s).
#'
#' @importFrom dplyr mutate left_join
#' @importFrom tidyr pivot_longer
#' @importFrom tibble as_tibble rownames_to_column
#' @importFrom SummarizedExperiment assays rowData colData
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # pivot experiment
#' exp_data <- pivot_exp(
#'     exposomicset = mae,
#'     exp_name = "mRNA",
#'     features = c("feat_42")
#' )
#'
#' @export
pivot_exp <- function(exposomicset,
                      exp_name,
                      features = NULL) {
    # Extract the SummarizedExperiment
    se <- .update_assay_colData(exposomicset, exp_name = exp_name)

    # Optionally subset features
    if (!is.null(features)) {
        se <- se[rownames(se) %in% features, ]
    }

    # Row and column data
    rd <- as.data.frame(rowData(se))
    cd <- as.data.frame(colData(se))

    # Iterate over assays and pivot to long format
    assay_list <- lapply(names(assays(se)), function(aname) {
        a <- assays(se)[[aname]]
        tibble::as_tibble(a, rownames = ".feature") |>
            tidyr::pivot_longer(
                cols = -.feature,
                names_to = ".sample",
                values_to = aname
            )
    })

    # Merge multiple assays by feature/sample pair
    assay_df <- Reduce(
        function(x, y) dplyr::left_join(x, y, by = c(".feature", ".sample")),
        assay_list
    )

    # Add rowData and colData
    assay_df <- assay_df |>
        dplyr::left_join(
            tibble::rownames_to_column(rd, ".feature"),
            by = ".feature"
        ) |>
        dplyr::left_join(
            tibble::rownames_to_column(cd, ".sample"),
            by = ".sample"
        ) |>
        dplyr::mutate(
            exp_name = exp_name,
            .before = .feature
        )

    assay_df
}

#' Extract Results from `MultiAssayExperiment` Metadata
#'
#' Retrieves a specific analysis result
#' from the metadata slot of a `MultiAssayExperiment` object.
#'
#' @param exposomicset A `MultiAssayExperiment` object.
#' @param result A character string indicating which result to
#' extract from metadata. Must be one of:
#' `"codebook"`, `"quality_control"`, `"correlation"`, `"association"`,
#' `"mixture_analysis"`, `"differential_analysis"`, `"multiomics_integration"`,
#'  `"network"`, `"enrichment"`.
#'
#' @return The corresponding result object stored in `metadata(exposomicset)`,
#'  or `NULL` if not present.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # extract results
#' res <- extract_results(
#'     exposomicset = mae,
#'     result = "codebook"
#' )
#'
#' @export
extract_results <- function(
  exposomicset,
  result = c(
      "codebook",
      "quality_control",
      "correlation",
      "association",
      "differential_analysis",
      "multiomics_integration",
      "network",
      "enrichment"
  )
) {
    result <- exposomicset |>
        MultiAssayExperiment::metadata() |>
        purrr::pluck(result)

    return(result)
}

# --- Define Custom Color Pallette  ---

tidy_exp_pal <- c(
    "#1E0C1F",
    "#00c9c1",
    "#cf68d9",
    "#6d7ea9",
    "#e8d5b5",
    "#944470",
    "#67abff",
    "#b5a7b6",
    "#002f4f",
    "#f2b459",
    "#4f4350",
    "#51aabb",
    "#994a4f",
    "#00a3cd",
    "#240d00",
    "#48817d"
)

divergent_cols <- c("#0000FF", "#FF0000", "#00FF00", "#000033", "#FF00B6", "#005300", "#FFD300", "#009FFF", "#9A4D42", "#00FFBE", "#783FC1", "#1F9698", "#FFACFD", "#B1CC71", "#F1085C", "#FE8F42", "#DD00FF", "#201A01", "#720055", "#766C95", "#02AD24", "#C8FF00", "#886C00", "#FFB79F", "#858567", "#A10300", "#14F9FF", "#00479E", "#DC5E93", "#93D4FF", "#004CFF")

scale_fill_tidy_exp <- function(..., rev = FALSE) {
    if (rev) {
        tidy_exp_pal <- rev(tidy_exp_pal)
    }
    ggplot2::scale_fill_manual(values = tidy_exp_pal, ...)
}

scale_color_tidy_exp <- function(..., rev = FALSE) {
    if (rev) {
        tidy_exp_pal <- rev(tidy_exp_pal)
    }
    ggplot2::scale_color_manual(values = tidy_exp_pal, ...)
}


# --- Update Assay ColData ------

#' Update Assay colData in a MultiAssayExperiment
#'
#' Synchronizes the sample metadata (`colData`) of a specific assay
#' with the global `colData`
#' from a `MultiAssayExperiment` object.
#'
#' @keywords internal
#' @noRd
.update_assay_colData <- function(
  exposomicset,
  exp_name
) {
    # Retrieve the assay
    assay <- MultiAssayExperiment::experiments(exposomicset)[[exp_name]]

    # Extract colData for the assay's samples
    assay_samples <- colnames(assay)
    global_coldata <- as.data.frame(MultiAssayExperiment::colData(exposomicset))
    coldata <- global_coldata[
        rownames(global_coldata) %in% assay_samples, ,
        drop = FALSE
    ]

    # Ensure the sample order matches
    coldata <- coldata[match(assay_samples, rownames(coldata)), , drop = FALSE]

    # Add a check to ensure the order is correct
    if (!identical(rownames(coldata), assay_samples)) {
        stop(
            "Sample order mismatch detected in assay: ", exp_name,
            "\nEnsure the samples in colData are aligned with the assay samples."
        )
    }

    # Update colData in the assay
    MultiAssayExperiment::colData(assay) <- S4Vectors::DataFrame(coldata)

    return(assay)
}
# --- Log2 MultiAssayExperiment Assays --------
#' Log2 Transform Assays in a MultiAssayExperiment
#'
#' Applies log2 transformation to each assay in a `MultiAssayExperiment` object.
#'
#' @keywords internal
#' @noRd
.log2_multiassay <- function(exposomicset) {
    message("Log2-Transforming each assay in MultiAssayExperiment.")

    # Apply log2 transformation to each assay
    log2_experiments <- lapply(
        MultiAssayExperiment::experiments(exposomicset),
        function(assay_obj) {
            if (inherits(assay_obj, "SummarizedExperiment")) {
                assay_mat <- SummarizedExperiment::assay(assay_obj)
                log2_mat <- log2(assay_mat + 1) # Log2 transformation
                SummarizedExperiment::assay(assay_obj) <- log2_mat
                return(assay_obj)
            } else if (is.matrix(assay_obj)) {
                return(log2(assay_obj + 1)) # Directly log2 transform matrices
            } else {
                stop("Unsupported assay type.")
            }
        }
    )

    log2_exposomicset <- MultiAssayExperiment::MultiAssayExperiment(
        experiments = log2_experiments,
        colData = MultiAssayExperiment::colData(exposomicset),
        metadata = MultiAssayExperiment::metadata(exposomicset)
    )


    return(log2_exposomicset)
}
# --- Scale MultiAssayExperiment Assays --------
#' Scale Assays in a MultiAssayExperiment
#'
#' Standardizes each assay in a `MultiAssayExperiment` object using
#' Z-score scaling.
#'
#' @keywords internal
#' @noRd
.scale_multiassay <- function(
  exposomicset,
  log2 = FALSE
) {
    message("Scaling each assay in MultiAssayExperiment.")

    # Check if log2 transformation is needed
    if (log2) {
        exposomicset <- .log2_multiassay(exposomicset)
    }

    # Apply scaling to each assay
    scaled_experiments <- lapply(
        MultiAssayExperiment::experiments(exposomicset),
        function(assay_obj) {
            if (inherits(assay_obj, "SummarizedExperiment")) {
                assay_mat <- SummarizedExperiment::assay(assay_obj)
                scaled_mat <- scale(assay_mat) # Standardize (Z-score)
                SummarizedExperiment::assay(assay_obj) <- scaled_mat
                return(assay_obj)
            } else if (is.matrix(assay_obj)) {
                return(scale(assay_obj)) # Directly scale matrices
            } else {
                stop("Unsupported assay type.")
            }
        }
    )

    # Create a new MultiAssayExperiment with scaled data
    scaled_exposomicset <- MultiAssayExperiment::MultiAssayExperiment(
        experiments = scaled_experiments,
        colData = MultiAssayExperiment::colData(exposomicset),
        metadata = MultiAssayExperiment::metadata(exposomicset)
    )

    return(scaled_exposomicset)
}
# --- Get Top Variant Features ---------------
#' Get Top Variant Features
#'
#' Extracts the top features based on variance from a
#' `MultiAssayExperiment` object.
#'
#' @keywords internal
#' @noRd
.top_var_multiassay <- function(
  exposomicset,
  n = 1000,
  assay_name = NULL
) {
    # exposomicset <- .log2_multiassay(exposomicset)

    # Grab the top n features based on variance
    top_var_experiments <- lapply(
        MultiAssayExperiment::experiments(exposomicset),
        function(assay_obj) {
            if (inherits(assay_obj, "SummarizedExperiment")) {
                assay_mat <- SummarizedExperiment::assay(assay_obj)
                feature_variance <- apply(assay_mat, 1, var) |>
                    sort() |>
                    tail(n = n) |>
                    names()


                return(feature_variance)
            } else if (is.matrix(assay_obj)) {
                return(apply(assay_obj, 1, var) |>
                    sort() |>
                    tail(n = n) |>
                    names()) # Directly scale matrices
            } else {
                stop("Unsupported assay type.")
            }
        }
    )


    return(top_var_experiments)
}
# --- Run Differential Abundance Analysis ------
#' Run Differential Abundance Analysis on a SummarizedExperiment
#'
#' Performs differential abundance analysis using `tidybulk`,
#'  with optional contrast testing.
#'
#' @importMethodsFrom tidybulk test_differential_abundance identify_abundant
#' @keywords internal
#' @noRd
.run_se_differential_abundance <- function(
  se,
  formula,
  abundance_col = "counts",
  method = "limma_trend",
  scaling_method = "none",
  contrasts = NULL
) {
    # Confirm there are no negative values
    if (min(SummarizedExperiment::assay(se, abundance_col)) < 0) {
        message("Negative values detected. Adding pseudocount to all values..")
        min_value <- abs(min(SummarizedExperiment::assay(se, abundance_col))) + 1
        new_assay <- SummarizedExperiment::assay(se, abundance_col) + min_value
        SummarizedExperiment::assay(se, abundance_col) <- new_assay
    }

    # if using limma_trend, skip tidybulk
    if (method == "limma_trend") {
        return(.run_limma_trend(
            se = se,
            formula = formula,
            abundance_col = abundance_col,
            contrasts = contrasts,
            scaling_method = scaling_method
        ))
    }

    # Otherwise, use tidybulk path
    if (!is.null(contrasts)) {
        res_list <- list()
        for (contrast in contrasts) {
            contrast_results <- se |>
                identify_abundant(
                    minimum_counts = 0,
                    minimum_proportion = 0
                ) |>
                test_differential_abundance(
                    formula,
                    .abundance = !!sym(abundance_col),
                    method = method,
                    contrasts = contrast,
                    scaling_method = scaling_method
                )

            res <- as.data.frame(S4Vectors::elementMetadata(contrast_results))
            colnames(res) <- gsub("__.*", "", colnames(res))
            res <- res |>
                dplyr::mutate(
                    feature = rownames(contrast_results),
                    contrast = contrast,
                    method = method,
                    scaling = scaling_method
                )
            res_list[[contrast]] <- res
        }
        return(dplyr::bind_rows(res_list))
    } else {
        contrast_results <- se |>
            identify_abundant(
                minimum_counts = 0,
                minimum_proportion = 0
            ) |>
            test_differential_abundance(
                formula,
                .abundance = !!sym(abundance_col),
                method = method,
                scaling_method = scaling_method
            )

        res <- as.data.frame(S4Vectors::elementMetadata(contrast_results))
        colnames(res) <- gsub("__.*", "", colnames(res))
        res <- res |>
            dplyr::mutate(
                feature = rownames(contrast_results),
                contrast = all.vars(formula)[1],
                method = method,
                scaling = scaling_method
            )
        return(res)
    }
}


# --- Calculate Feature Stability -------
#' Calculate Feature Stability Across Sensitivity Conditions
#'
#' Computes a stability score for features based on their significance across multiple sensitivity tests.
#'
#' @keywords internal
#' @noRd
.calculate_feature_stability <- function(
  sensitivity_df,
  pval_col = "adj.P.Val",
  logfc_col = "logFC",
  pval_threshold = 0.05
) {
    # Extract sensitivity analysis results
    sensitivity_df <- sensitivity_df

    message("Computing feature stability across sensitivity conditions.")

    feature_stability_df <- sensitivity_df |>
        group_by(feature, exp_name) |>
        summarise(
            # selection frequency (fraction of significant p-values)
            presence_rate = mean(!!sym(pval_col) < pval_threshold, na.rm = TRUE),

            # effect size consistency (inverse of coefficient of variation)
            effect_consistency = 1 / (1 + (sd(
                !!sym(logfc_col),
                na.rm = TRUE
            ) /
                mean(abs(!!sym(logfc_col)), na.rm = TRUE))),

            # main hybrid score combining presence and effect stability
            stability_score = presence_rate * effect_consistency,

            # statistical signal strength based on p-values
            mean_log_p = mean(-log10(!!sym(pval_col) + 1e-10), na.rm = TRUE),

            # hybrid score using log p-values
            logp_weighted_score = mean_log_p * effect_consistency,

            # effect size variability
            sd_logFC = sd(!!sym(logfc_col), na.rm = TRUE),
            iqr_logFC = IQR(!!sym(logfc_col), na.rm = TRUE),
            cv_logFC = sd(!!sym(logfc_col), na.rm = TRUE) /
                abs(mean(!!sym(logfc_col), na.rm = TRUE)),

            # sign flip frequency of logfc across runs
            sign_flip_freq = mean(
                sign(!!sym(logfc_col)) != sign(
                    mean(!!sym(logfc_col),
                        na.rm = TRUE
                    )
                ),
                na.rm = TRUE
            ),

            # variability of p-values on the log scale
            sd_log_p = sd(log10(!!sym(pval_col) + 1e-10), na.rm = TRUE)
        )

    message("Feature stability analysis completed.")
    return(feature_stability_df)
}


# --- Pairwise Overlaps -------------
#' Compute Pairwise Overlaps Between Sets
#'
#' Calculates pairwise overlaps, Jaccard indices,
#' and shared elements for a list of unique sets.
#'
#' @keywords internal
#' @noRd
.get_pairwise_overlaps <- function(sets) {
    # credit for most of the code:
    # https://blog.jdblischak.com/posts/pairwise-overlaps/
    # Ensure that all sets are unique character vectors
    sets_are_vectors <- vapply(sets, is.vector, logical(1))
    if (any(!sets_are_vectors)) {
        stop("Sets must be vectors")
    }
    sets_are_atomic <- vapply(sets, is.atomic, logical(1))
    if (any(!sets_are_atomic)) {
        stop("Sets must be atomic vectors, i.e. not lists")
    }
    sets <- lapply(sets, as.character)
    is_unique <- function(x) length(unique(x)) == length(x)
    sets_are_unique <- vapply(sets, is_unique, logical(1))
    if (any(!sets_are_unique)) {
        stop("Sets must be unique, i.e. no duplicated elements")
    }

    n_sets <- length(sets)
    set_names <- names(sets)
    n_overlaps <- choose(n = n_sets, k = 2)

    vec_name1 <- character(length = n_overlaps)
    vec_name2 <- character(length = n_overlaps)
    vec_num_shared <- integer(length = n_overlaps)
    vec_overlap <- numeric(length = n_overlaps)
    vec_jaccard <- numeric(length = n_overlaps)
    vec_shared_terms <- character(length = n_overlaps)
    overlaps_index <- 1

    for (i in seq_len(n_sets - 1)) {
        name1 <- set_names[i]
        set1 <- sets[[i]]
        for (j in seq(i + 1, n_sets)) {
            name2 <- set_names[j]
            set2 <- sets[[j]]

            shared_terms <- paste(Reduce(intersect, list(set1, set2)), collapse = ",")

            set_intersect <- set1[match(set2, set1, 0L)]
            set_union <- unique(c(set1, set2))
            num_shared <- length(set_intersect)
            overlap <- num_shared / min(length(set1), length(set2))
            jaccard <- num_shared / length(set_union)

            vec_name1[overlaps_index] <- name1
            vec_name2[overlaps_index] <- name2
            vec_num_shared[overlaps_index] <- num_shared
            vec_overlap[overlaps_index] <- overlap
            vec_jaccard[overlaps_index] <- jaccard
            vec_shared_terms[overlaps_index] <- shared_terms

            overlaps_index <- overlaps_index + 1
        }
    }

    result <- data.frame(
        source = vec_name1,
        target = vec_name2,
        num_shared = vec_num_shared,
        overlap = vec_overlap,
        jaccard = vec_jaccard,
        shared_terms = vec_shared_terms,
        stringsAsFactors = FALSE
    )
    return(result)
}
# --- Cluster Matrix ------------------
#' Perform Hierarchical Clustering on a Data Matrix
#'
#' Clusters samples using various distance metrics and clustering approaches to determine optimal cluster numbers.
#'
#' @keywords internal
#' @noRd
.cluster_mat <- function(
  data_matrix,
  dist_method = NULL,
  cluster_method = "ward.D",
  clustering_approach = "gap",
  gap_stat_k_max = 20,
  gap_stat_B = 50,
  density_quantile = 0.90
) {
    # # Determine appropriate distance metric
    # if (is.null(dist_method)) {
    #   if (all(sapply(data_matrix, is.numeric))) {
    #     dist_method <- "euclidean"
    #   } else {
    #     dist_method <- "gower"
    #   }
    # }
    # Determine appropriate distance metric
    if (is.null(dist_method)) {
        if (all(vapply(data_matrix, is.numeric, FUN.VALUE = logical(1)))) {
            dist_method <- "euclidean"
        } else {
            dist_method <- "gower"
        }
    }

    # Compute distance matrix
    sample_dist <- if (dist_method == "gower") {
        cluster::daisy(data_matrix, metric = "gower")
    } else {
        dist(data_matrix, method = dist_method)
    }

    # Function to determine optimal k based on the selected clustering approach
    determine_k <- function(dist_matrix,
                            cluster_method,
                            clustering_approach) {
        if (clustering_approach == "diana") {
            # Determine optimal k using the height difference method
            sample_cluster <- cluster::diana(as.dist(dist_matrix))
            height_diffs <- diff(sample_cluster$height)
            cutoff_index <- which.max(height_diffs)
            return(length(sample_cluster$height) - cutoff_index)
        } else if (clustering_approach == "gap") {
            # Determine optimal k using the gap statistic
            gap_stat <- cluster::clusGap(as.matrix(dist_matrix),
                FUN = hcut,
                K.max = gap_stat_k_max,
                B = gap_stat_B
            )
            return(cluster::maxSE(gap_stat$Tab[, "gap"], gap_stat$Tab[, "SE.sim"]))
        } else if (clustering_approach == "elbow") {
            # Determine optimal k using the elbow method
            wss_plot <- factoextra::fviz_nbclust(
                as.matrix(dist_matrix),
                FUN = hcut,
                method = "wss"
            )

            # Identify the first significant drop and ensure it's a number we can use
            k_optimal <- which.min(diff(diff(wss_plot$data$y))) + 1
            if (is.na(k_optimal) || k_optimal < 2) k_optimal <- 3
            return(k_optimal)
        } else if (clustering_approach == "dynamic") {
            .check_suggested("dynamicTreeCut")
            # Determine optimal k using dynamic tree cut
            sample_cluster <- hclust(as.dist(dist_matrix), method = cluster_method)
            cut_clusters <- dynamicTreeCut::cutreeDynamic(
                dendro = sample_cluster,
                distM = as.matrix(as.dist(dist_matrix)),
                deepSplit = 2
            )
            return(length(unique(cut_clusters)))
        } else if (clustering_approach == "density") {
            .check_suggested("densityClust")
            # Determine optimal k using density-based clustering
            dclust <- densityClust::densityClust(
                as.dist(dist_matrix),
                gaussian = TRUE
            )
            dclust <- densityClust::findClusters(
                dclust,
                rho = quantile(dclust$rho, density_quantile),
                delta = quantile(dclust$delta, density_quantile)
            )
            return(length(unique(dclust$clusters)))
        } else {
            stop("Invalid clustering approach selected.")
        }
    }

    k_samples <- determine_k(
        sample_dist,
        cluster_method,
        clustering_approach
    )

    # Perform hierarchical clustering only if needed
    sample_cluster <- hclust(as.dist(sample_dist), method = cluster_method)

    # Cut dendrograms using optimal k values
    sample_groups <- cutree(sample_cluster, k = k_samples)

    message("Optimal number of clusters for samples: ", k_samples)

    return(sample_groups)
}
# --- Check Suggested Packages ------------
#' Check that required suggested package is available
#'
#' This helper ensures that a suggested package is installed before plotting.
#' It can be reused in any function that depends on optional packages.
#'
#' @param pkg Name of the suggested package (character).
#' @param reason Optional string explaining why the package is needed (e.g., "for Venn diagrams").
#' @param call_stop Logical; if TRUE, throw error, otherwise just warn.
#'
#' @return Invisible TRUE if package is available; otherwise stops or warns.
#' @keywords internal
#' @noRd
.check_suggested <- function(
  pkg, reason = NULL, call_stop = TRUE
) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
        msg <- paste0(
            "Please install '", pkg, "'",
            if (!is.null(reason)) paste0(" (", reason, ")"),
            " to use this function."
        )
        if (call_stop) {
            stop(msg, call. = FALSE)
        } else {
            warning(msg, call. = FALSE)
        }
        return(FALSE)
    }
    invisible(TRUE)
}

# --- TidyGraph to Data Frame -------------
#' Convert TidyGraph Object to Data Frame
#'
#' Converts a TidyGraph object to a data frame format for
#' easier manipulation and analysis.
#'
#' @keywords internal
#' @noRd
.tidygraph_to_df <- function(tg) {
    # require(tidygraph)
    # require(dplyr)
    .check_suggested(pkg = "tidygraph")
    merged_df <- tg |>
        # Grab edge attributes
        tidygraph::activate(edges) |>
        tidygraph::as_tibble() |>
        # Join with node attributes
        dplyr::left_join(
            tg |>
                tidygraph::activate(nodes) |>
                tidygraph::as_tibble() |>
                dplyr::mutate(from = dplyr::row_number()),
            by = "from"
        ) |>
        # Rename columns for clarity
        dplyr::rename(from_name = name) |>
        # Join with node attributes again for the target node
        dplyr::left_join(
            tg |>
                tidygraph::activate(nodes) |>
                tidygraph::as_tibble() |>
                dplyr::mutate(to = dplyr::row_number()),
            by = "to"
        ) |>
        # Rename columns for clarity
        dplyr::rename(to_name = name)

    return(merged_df)
}
# --- Summarize Graph ----------
#' Summarize Graph Attributes
#'
#' Generates a summary of graph attributes including number of nodes, edges, components, diameter, mean distance, and modularity.
#'
#' @keywords internal
#' @noRd
.summarize_graph <- function(g) {
    .check_suggested(pkg = "tidygraph")
    tibble::tibble(
        "No. of Nodes" = g |> igraph::gorder(),
        "No. of Edges" = g |> igraph::gsize(),
        "No. of Components" = g |> igraph::count_components(),
        "Diameter" = g |> igraph::diameter(),
        "Mean Distance" = g |> igraph::mean_distance(),
        "Modularity" = g |>
            tidygraph::as_tbl_graph() |>
            tidygraph::activate(nodes) |>
            dplyr::mutate(community = tidygraph::group_louvain()) |>
            tidygraph::with_graph(tidygraph::graph_modularity(community))
    )
}
# --- Build Network Plot -------------------
#' Build Network Plot
#'
#' Generates a network plot using ggraph, with options for node color,
#' labels, and facetting.
#'
#' @keywords internal
#' @importFrom ggplot2 labs scale_color_manual theme element_text
#' @importFrom dplyr pull mutate
#' @importFrom scales alpha
#' @noRd
.build_ggraph_plot <- function(
  g,
  node_color_var,
  node_colors,
  label,
  facet_var,
  include_stats,
  fg_text_colour,
  foreground,
  alpha,
  size_lab,
  color_lab
) {
    # require(ggplot2)
    # require(tidygraph)
    .check_suggested(pkg = "ggh4x")
    .check_suggested(pkg = "ggraph")

    if (!is.null(node_color_var)) {
        p <- g |>
            mutate(centrality = tidygraph::centrality_degree()) |>
            ggraph::ggraph(layout = "kk") +
            ggraph::geom_edge_fan(aes(alpha = after_stat(index)), show.legend = FALSE) +
            ggraph::geom_node_point(aes(size = centrality, color = !!sym(node_color_var))) +
            ggraph::theme_graph(
                fg_text_colour = fg_text_colour,
                base_family = "sans"
            ) +
            labs(
                size = size_lab,
                color = color_lab
            )
    } else {
        p <- g |>
            tidygraph::mutate(centrality = tidygraph::centrality_degree()) |>
            ggraph::ggraph(layout = "kk") +
            ggraph::geom_edge_fan(aes(alpha = after_stat(index)), show.legend = FALSE) +
            ggraph::geom_node_point(aes(size = centrality)) +
            ggraph::theme_graph(
                fg_text_colour = fg_text_colour,
                base_family = "sans"
            ) +
            labs(size = size_lab)
    }

    if (!is.null(node_colors)) {
        p <- p + scale_color_manual(values = node_colors)
    } else {
        n_colors <- g |>
            tidygraph::activate(nodes) |>
            tidygraph::as_tibble() |>
            dplyr::pull(group) |>
            unique() |>
            length()
        p <- p + scale_color_tidy_exp()
    }

    if (!is.null(facet_var)) {
        p <- p +
            ggh4x::facet_grid2(
                as.formula(paste("~", facet_var)),
                scales = "free_x",
                space = "free_x",
                strip = ggh4x::strip_themed(
                    background_x = ggh4x::elem_list_rect(
                        fill = scales::alpha(foreground, alpha = alpha)
                    )
                )
            ) +
            theme(strip.text.x = element_text(face = "bold.italic"))
    }

    if (label) {
        p <- p +
            ggraph::geom_node_label(aes(label = label),
                fontface = "italic",
                repel = TRUE,
                na.rm = TRUE
            )
    }

    if (include_stats) {
        summary_info <- .summarize_graph(g)
        label_text <- paste0(
            "Nodes: ", summary_info$`No. of Nodes`,
            " | Edges: ", summary_info$`No. of Edges`,
            " | Components: ", summary_info$`No. of Components`,
            " | Diameter: ", summary_info$Diameter,
            " | Mean Distance: ", round(summary_info$`Mean Distance`, 2),
            " | Modularity: ", round(summary_info$Modularity, 3)
        )

        p <- p +
            labs(caption = label_text) +
            theme(plot.caption = element_text(face = "italic"))
    }

    return(p)
}
# --- Ontology Root --------------
#' Collapse Ontology Root Nodes
#'
#' Collapses a set of ontology terms to either user specified root nodes
#' or root level depending on the level of detail they want
#'
#' @keywords internal
#' @noRd
.collapse_ont_terms <- function(
  ontology_df,
  term_ids,
  root_level = "top",
  assign_label = TRUE
) {
    # Sanitize input: fix character to list columns
    fix_rel_cols <- function(x) {
        if (!is.list(x)) strsplit(x, ";\\s*") else x
    }

    ontology_df <- ontology_df |>
        dplyr::mutate(
            ancestors = fix_rel_cols(ancestors),
            parents = fix_rel_cols(parents),
            children = fix_rel_cols(children)
        )

    get_ancestors <- function(term_id) {
        idx <- match(term_id, ontology_df$id)
        if (!is.na(idx)) ontology_df$ancestors[[idx]] else character(0)
    }

    get_name <- function(term_id) {
        idx <- match(term_id, ontology_df$id)
        if (!is.na(idx)) ontology_df$name[[idx]] else NA_character_
    }

    get_depth <- function(term_id) {
        length(get_ancestors(term_id))
    }

    # Root nodes depend on input
    if (identical(root_level, "top")) {
        root_nodes <- ontology_df$id[
            vapply(
                ontology_df$parents,
                function(p) length(p) == 0,
                FUN.VALUE = logical(1)
            )
        ]

        assign_to_root <- function(term_id) {
            anc <- c(term_id, get_ancestors(term_id))
            matched <- intersect(anc, root_nodes)
            if (length(matched)) matched[1] else NA_character_
        }
    } else if (is.numeric(root_level)) {
        all_depths <- vapply(ontology_df$id, get_depth, FUN.VALUE = integer(1))
        id_to_depth <- setNames(all_depths, ontology_df$id)

        assign_to_root <- function(term_id) {
            anc <- c(term_id, get_ancestors(term_id))
            if (length(anc) == 0) {
                return(NA_character_)
            }
            anc_depths <- id_to_depth[anc]
            anc_depths <- anc_depths[!is.na(anc_depths)]
            if (length(anc_depths) == 0) {
                return(NA_character_)
            }

            # Find the deepest matching node less than or greater than root_level
            eligible <- anc_depths[anc_depths <= root_level]
            if (length(eligible)) {
                return(names(eligible)[which.max(eligible)]) # closest from below
            } else {
                return(NA_character_)
            }
        }
    } else if (is.character(root_level)) {
        root_nodes <- root_level
        assign_to_root <- function(term_id) {
            anc <- c(term_id, get_ancestors(term_id))
            matched <- intersect(anc, root_nodes)
            if (length(matched)) matched[1] else NA_character_
        }
    } else {
        stop("`root_level` must be 'top', a depth integer, or vector of term IDs.")
    }

    assigned_root_ids <- vapply(term_ids,
        assign_to_root,
        FUN.VALUE = character(1)
    )

    if (assign_label) {
        tibble::tibble(
            term_id = term_ids,
            term_label = vapply(term_ids, get_name, FUN.VALUE = character(1)),
            root_id = assigned_root_ids,
            root_label = vapply(assigned_root_ids,
                get_name,
                FUN.VALUE = character(1)
            )
        )
    } else {
        assigned_root_ids
    }
}

#' Run Sensitivity Analysis for Differential Abundance
#'
#' Performs sensitivity analysis by systematically varying statistical methods,
#' scaling strategies, and model formulas (with optional bootstrap sampling) to
#' assess the stability of differentially abundant features.
#'
#' @param exposomicset A `MultiAssayExperiment` containing the assays to analyze.
#' @param base_formula The base model formula used for differential analysis.
#' @param abundance_col Character. Name of the column in the assays representing
#'  abundance. Default is `"counts"`.
#' @param methods Character vector of differential expression methods.
#' Options include `"limma_trend"` ,`"limma_voom"`, `"DESeq2"`, and `"edgeR_quasi_likelihood"`.
#' @param scaling_methods Character vector of normalization methods to try.
#'  Options include `"none"`, `"TMM"`, and `"quantile"`.
#' @param contrasts Optional list of contrasts to apply for differential testing.
#' @param covariates_to_remove Optional character vector of covariates
#' to remove from the base formula to generate model variants.
#' @param pval_col Name of the column containing p-values or adjusted
#' p-values used to define significance.
#' @param logfc_col Name of the column containing log fold changes.
#' @param pval_threshold Numeric threshold for significance. Default is 0.05.
#' @param logFC_threshold Numeric threshold for absolute log fold change.
#' Default is `log2(1)` (i.e., 0).
#' @param score_thresh Optional threshold for the selected stability metric.
#' If not provided, calculated using `score_quantile`.
#' @param score_quantile Quantile used to define the threshold
#'  if `score_thresh` is not provided. Default is 0.9.
#' @param stability_metric Character. Name of the column in
#' `feature_stability` to use as the scoring metric. Default is `"stability_score"`.
#' @param action Whether to `"add"` results to `metadata()` or
#' `"get"` them as a list. Default is `"add"`.
#' @param bootstrap_n Integer. Number of bootstrap iterations.
#' If 0, no resampling is performed. Default is 1.
#'
#' @return If `action = "add"`, returns a `MultiAssayExperiment`
#' with results stored in
#'   `metadata(exposomicset)$differential_analysis$sensitivity_analysis`.
#'   If `action = "get"`,
#'   returns a list with three elements:
#'   \describe{
#'     \item{\code{sensitivity_df}}{Data frame of all differential results
#'     across model/method combinations.}
#'     \item{\code{feature_stability}}{Data frame summarizing feature
#'     stability scores.}
#'     \item{\code{score_thresh}}{The threshold used to define stable features.}
#'   }
#'
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#'
#' # Run differential abundance
#' mae <- run_differential_abundance(
#'     exposomicset = mae,
#'     formula = ~ smoker + sex,
#'     abundance_col = "counts",
#'     method = "limma_voom",
#'     action = "add"
#' )
#'
#' # Run the sensitivity analysis
#' mae <- run_sensitivity_analysis(
#'     exposomicset = mae,
#'     base_formula = ~ smoker + sex,
#'     methods = c("limma_voom"),
#'     scaling_methods = c("none"),
#'     covariates_to_remove = "sex",
#'     pval_col = "P.Value",
#'     logfc_col = "logFC",
#'     pval_threshold = 0.05,
#'     logFC_threshold = 0,
#'     bootstrap_n = 3,
#'     action = "add"
#' )
#'
#' @export
run_sensitivity_analysis <- function(
  exposomicset,
  base_formula,
  abundance_col = "counts",
  methods = c("limma_trend", "limma_voom", "DESeq2", "edgeR_quasi_likelihood"),
  scaling_methods = c("none", "TMM", "quantile"),
  contrasts = NULL,
  covariates_to_remove = NULL,
  pval_col = "adj.P.Val",
  logfc_col = "logFC",
  pval_threshold = 0.05,
  logFC_threshold = log2(1),
  score_thresh = NULL,
  score_quantile = 0.9,
  stability_metric = "stability_score",
  action = "add",
  bootstrap_n = 1
) {
    model_list <- .build_model_list(base_formula, covariates_to_remove)

    if (bootstrap_n > 0) {
        # Combine all raw differential results from bootstrap runs
        sensitivity_df <- purrr::map_dfr(seq_len(bootstrap_n), function(b) {
            # message(paste("Running bootstrap iteration", b, "of", bootstrap_n))
            message(sprintf("Running bootstrap iteration %d of %d", b, bootstrap_n))

            mae_b <- .resample_MAE(exposomicset)

            sensitivity_df <- .run_sensitivity_grid(
                mae_b,
                model_list,
                methods,
                scaling_methods,
                abundance_col,
                contrasts
            ) |>
                dplyr::mutate(bootstrap_id = b)
        })
    } else {
        # Run sensitivity analysis without bootstrapping
        sensitivity_df <- .run_sensitivity_grid(
            exposomicset,
            model_list,
            methods,
            scaling_methods,
            abundance_col,
            contrasts
        )
    }

    # Calculate feature stability scores
    feature_stability_df <- .calculate_feature_stability(
        sensitivity_df,
        pval_col,
        logfc_col,
        pval_threshold
    )

    # Check if stability_metric is valid
    if (!stability_metric %in% colnames(feature_stability_df)) {
        stop(sprintf("Invalid stability_metric: '%s'.", stability_metric))
    }


    # Determine score threshold if not provided
    if (is.null(score_thresh)) {
        stability_metric_col <- feature_stability_df[[stability_metric]]
        score_thresh <- quantile(
            stability_metric_col[feature_stability_df[[stability_metric]] > 0],
            score_quantile,
            na.rm = TRUE
        )
    }

    # Summarize stable features
    .summarize_stable_features(
        feature_stability_df,
        score_thresh,
        stability_metric
    )

    # Add results to MultiAssayExperiment metadata or return as list
    if (action == "add") {
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$differential_analysis$sensitivity_analysis <- list(
            sensitivity_df = sensitivity_df,
            feature_stability = feature_stability_df,
            score_thresh = score_thresh
        )
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        step_record <- list(
            run_sensitivity_analysis = list(
                timestamp = Sys.time(),
                params = list(
                    formula = format(base_formula),
                    methods = methods,
                    scaling_methods = scaling_methods,
                    abundance_col = abundance_col,
                    contrasts = contrasts,
                    covariates_to_remove = covariates_to_remove,
                    pval_threshold = pval_threshold,
                    logFC_threshold = logFC_threshold,
                    score_quantile = score_quantile,
                    stability_metric = stability_metric,
                    bootstrap_n = bootstrap_n
                ),
                notes = paste(
                    "Ran sensitivity analysis across",
                    bootstrap_n, "bootstrap iterations and",
                    length(methods), "methods and",
                    length(scaling_methods), "scaling strategies.",
                    if (!is.null(covariates_to_remove)) {
                        paste(
                            "Covariates removed in model variations:",
                            paste(covariates_to_remove, collapse = ", ")
                        )
                    } else {
                        "No covariates were removed from the base model."
                    }
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else if (action == "get") {
        return(list(
            sensitivity_df = sensitivity_df,
            feature_stability = feature_stability_df,
            score_thresh = score_thresh
        ))
    } else {
        stop("Invalid action. Use 'add' or 'get'.")
    }
}

.run_sensitivity_grid <- function(
  exposomicset,
  model_list,
  methods,
  scalings,
  abundance_col,
  contrasts
) {
    grid <- expand.grid(
        model_name = names(model_list),
        method = methods,
        scaling = scalings,
        exp_name = names(MultiAssayExperiment::experiments(exposomicset)),
        stringsAsFactors = FALSE
    )

    results <- purrr::pmap_dfr(grid, function(model_name,
                                              method,
                                              scaling,
                                              exp_name) {
        formula <- model_list[[model_name]]
        exp <- .update_assay_colData(exposomicset, exp_name)

        if (method == "DESeq2") {
            SummarizedExperiment::assay(exp, abundance_col) <- round(
                SummarizedExperiment::assay(exp, abundance_col), 0
            )
        }

        res <- .run_da_pipeline(
            exp,
            formula,
            method,
            scaling,
            abundance_col,
            contrasts
        )

        if (is.null(res)) {
            return(NULL)
        }
        res$model <- model_name
        res$exp_name <- exp_name
        res
    })

    results <- results |>
        tibble::as_tibble()

    return(results)
}

.run_da_pipeline <- function(
  se,
  formula,
  method,
  scaling,
  abundance_col,
  contrasts
) {
    invisible(.run_se_differential_abundance(
        se = se,
        formula = formula,
        abundance_col = abundance_col,
        method = method,
        scaling_method = scaling,
        contrasts = contrasts
    ))
}

.summarize_stable_features <- function(
  feature_stability_df,
  score_thresh,
  stability_metric
) {
    sum <- feature_stability_df |>
        dplyr::group_by(exp_name) |>
        dplyr::reframe(
            n_above = sum(!!rlang::sym(stability_metric) > score_thresh),
            n = dplyr::n()
        )

    message(
        "Number Of Features Above Threshold Of ",
        round(score_thresh, 2), ":"
    )
    message("----------------------------------------")
    for (cur_exp_name in unique(feature_stability_df$exp_name)) {
        n_above <- sum |>
            dplyr::filter(exp_name == !!cur_exp_name) |>
            dplyr::pull(n_above)
        n <- sum |>
            dplyr::filter(exp_name == !!cur_exp_name) |>
            dplyr::pull(n)
        message(cur_exp_name, ": ", n_above, "/", n)
    }
}

# Resample samples in the MultiAssayExperiment object
.resample_MAE <- function(mae) {
    all_ids <- colnames(mae) |>
        unlist() |>
        unique()
    sample_ids <- sample(all_ids, replace = TRUE)
    return(mae[, sample_ids])
}

# build list of model formulas by removing covariates from base model
.build_model_list <- function(
  base_formula,
  covariates_to_remove
) {
    base_terms <- all.vars(base_formula)
    model_list <- list("full model" = base_formula)
    if (!is.null(covariates_to_remove)) {
        for (covar in covariates_to_remove) {
            reduced_terms <- setdiff(base_terms, covar)
            if (length(reduced_terms) > 1) {
                reduced_formula <- as.formula(
                    paste("~", paste(reduced_terms, collapse = " + "))
                )
                model_list[[paste("without", covar)]] <- reduced_formula
            }
        }
    }
    return(model_list)
}

#' Extract Sample Metadata from MultiAssayExperiment or SummarizedExperiment
#'
#' Extracts and formats sample-level metadata (`colData`) from a
#' `MultiAssayExperiment` or `SummarizedExperiment` object.
#'
#' @param x A `MultiAssayExperiment` or `SummarizedExperiment` object.
#' @param ... Additional arguments passed to `pivot_sample()` from `tidybulk`
#' for `SummarizedExperiment` objects.
#'
#' @details
#' This function:
#' - Extracts **sample metadata** from `MultiAssayExperiment` using `colData()`,
#'  converting it to a tibble.
#' - Calls `pivot_sample()` from `tidybulk` when applied to a
#' `SummarizedExperiment` object.
#' - **Error Handling**: Returns an error if `x` is not a
#'  `MultiAssayExperiment` or `SummarizedExperiment`.
#'
#' @return A tibble containing sample metadata with an added `.sample` column.
#'
#' @importMethodsFrom tidybulk pivot_sample
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' sample_data <- mae |>
#'     pivot_sample()
#'
#' @export
pivot_sample <- function(x, ...) {
    if (inherits(x, "MultiAssayExperiment")) {
        # Call your custom function
        MultiAssayExperiment::colData(x) |>
            as.data.frame() |>
            tibble::rownames_to_column(".sample") |>
            dplyr::as_tibble()
    } else if (inherits(x, "SummarizedExperiment")) {
        # Call the pivot_sample() from the other package
        pivot_sample(x, ...)
    } else {
        stop("Needs a MultiAssayExperiment/SummarizedExperiment object.")
    }
}

#' Filter Features and Variables with High Missingness
#'
#' Removes exposure variables and omics features with
#'  missing values above a specified threshold.
#' Generates missing data summaries and quality control (QC) plots.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing
#' exposure and omics data.
#' @param na_thresh A numeric value specifying the percentage of
#' missing data allowed before a variable or feature is removed. Default is `20`.
#' @param na_plot_thresh A numeric value specifying the
#' minimum missing percentage for inclusion in QC plots. Default is `5`.
#'
#' @details
#' The function assesses missingness in both `colData(exposomicset)`
#' (exposure data) and `experiments(exposomicset)` (omics data).
#' - Exposure variables with more than `na_thresh`% missing values are removed.
#' - Omics features (rows in assay matrices) exceeding `na_thresh`%
#' missing values are filtered.
#' - Missingness summaries and QC plots are generated using
#' `naniar::gg_miss_var()` and stored in metadata.
#'
#' @return A `MultiAssayExperiment` object with filtered exposure
#' variables and omics features.
#' QC results, including missingness summaries and plots,
#'  are stored in `metadata(exposomicset)$na_qc`.
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Introduce some missingness
#' MultiAssayExperiment::colData(mae)$exposure_pm25[sample(1:20, 5)] <- NA
#'
#' # Filter features and exposures with high missingness
#' mae_filtered <- filter_missing(
#'     exposomicset = mae,
#'     na_thresh = 20,
#'     na_plot_thresh = 5
#' )
#'
#' @export
filter_missing <- function(
  exposomicset,
  na_thresh = 20,
  na_plot_thresh = 5
) {
    # Helper function to identify variables/features to exclude
    get_vars_to_exclude <- function(data, thresh) {
        miss_summary <- naniar::miss_var_summary(data)
        list(
            to_exclude = miss_summary |>
                dplyr::filter(pct_miss > thresh) |>
                dplyr::pull(variable),
            summary = miss_summary
        )
    }

    # Handle metadata (colData)
    exposure <- as.data.frame(MultiAssayExperiment::colData(exposomicset))
    col_exclude <- get_vars_to_exclude(exposure, na_thresh)
    MultiAssayExperiment::colData(exposomicset) <- as(
        MultiAssayExperiment::colData(exposomicset)[
            , !colnames(MultiAssayExperiment::colData(exposomicset)) %in%
                col_exclude$to_exclude,
            drop = FALSE
        ],
        "DataFrame"
    )

    # QC plot for metadata
    MultiAssayExperiment::metadata(exposomicset)$quality_control$na_qc <- list(
        exposure = list(
            vars_to_exclude_exposure_sum = col_exclude$summary |>
                dplyr::filter(pct_miss > na_thresh),
            all_var_sum = col_exclude$summary |>
                dplyr::filter(pct_miss > 0),
            na_exposure_qc_plot = naniar::gg_miss_var(
                exposure |>
                    dplyr::select(
                        dplyr::all_of(col_exclude$to_exclude)
                    )
            )
        )
    )

    message("Missing Data Filter threshold: ", na_thresh, "%")
    message(
        "Filtered metadata variables: ",
        paste(col_exclude$to_exclude,
            collapse = ", "
        )
    )

    # Handle omics experiments
    all_experiments <- MultiAssayExperiment::experiments(exposomicset)
    metadata_list <- MultiAssayExperiment::metadata(exposomicset)

    for (omics_name in names(all_experiments)) {
        experiment <- all_experiments[[omics_name]]
        original_assay <- SummarizedExperiment::assays(experiment)[[1]]

        # Transpose and convert assay to data.frame for processing
        assay_data <- as.data.frame(t(original_assay))

        # Identify features (rows) to exclude based on missingness
        row_exclude <- get_vars_to_exclude(assay_data, na_thresh)

        # Filter rows with high missingness
        experiment <- experiment[
            !rownames(experiment) %in% row_exclude$to_exclude,
        ]

        # Store the filtered experiment
        all_experiments[[omics_name]] <- experiment

        # QC plot for omics data
        metadata_list$quality_control$na_qc[[omics_name]] <- list(
            vars_to_exclude_omics_sum = row_exclude$summary |>
                dplyr::filter(pct_miss > na_thresh),
            all_var_sum = row_exclude$summary |>
                dplyr::filter(pct_miss > 0),
            na_omics_qc_plot = naniar::gg_miss_var(
                assay_data |>
                    dplyr::select(dplyr::all_of(row_exclude$to_exclude))
            )
        )

        message(
            "Filtered rows with high missingness in ",
            omics_name, ": ", length(row_exclude$to_exclude)
        )
    }

    # Save modified experiments and metadata back to exposomicset
    MultiAssayExperiment::experiments(exposomicset) <- all_experiments
    MultiAssayExperiment::metadata(exposomicset) <- metadata_list

    # Add analysis steps taken to metadata
    step_record <- list(
        filter_missing =
            list(
                timestamp = Sys.time(),
                params = list(na_thresh = na_thresh),
                notes = paste0(
                    "Filtered exposure variables and omics features with more than ",
                    na_thresh,
                    "% missing values. QC plots generated for exposure and omics data."
                )
            )
    )

    MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
        MultiAssayExperiment::metadata(exposomicset)$summary$steps,
        step_record
    )


    return(exposomicset)
}

## ----setup,include=FALSE------------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----load-libraries,message=FALSE,warning=FALSE-------------------------------
# Load Libraries
library(tidyverse)
library(tidyexposomics)

## ----load-data----------------------------------------------------------------
# Load example data
data("tidyexposomics_example")

# Create exposomic set object
expom <- create_exposomicset(
    codebook = tidyexposomics_example$annotated_cb,
    exposure = tidyexposomics_example$meta,
    omics = list(
        "Gene Expression" = tidyexposomics_example$exp_filt,
        "Methylation" = tidyexposomics_example$methyl_filt
    ),
    row_data = list(
        "Gene Expression" = tidyexposomics_example$exp_fdata,
        "Methylation" = tidyexposomics_example$methyl_fdata
    )
)

## ----exp-vars-----------------------------------------------------------------
# Grab exposure variables
exp_vars <- tidyexposomics_example$annotated_cb |>
    filter(category %in% c(
        "aerosol",
        "main group molecular entity",
        "polyatomic entity"
    )) |>
    pull(variable) |>
    as.character()

## ----impute-missing-----------------------------------------------------------

# Impute missing values
expom <- run_impute_missing(
    exposomicset = expom,
    exposure_impute_method = "missforest",
    exposure_cols = exp_vars
)

## ----trasform-vars------------------------------------------------------------

# Transform variables
expom <- transform_exposure(
    exposomicset = expom,
    transform_method = "boxcox_best",
    exposure_cols = exp_vars
)

## ----calc-exposome, results='hide'--------------------------------------------

# determine which aerosol variables to use
aerosols <- c("h_pm25_ratio_preg_None", "h_pm10_ratio_preg_None")

# Create exposome scores
expom <- expom |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "median",
        score_column_name = "exposome_median_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "pca",
        score_column_name = "exposome_pca_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "irt",
        score_column_name = "exposome_irt_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "quantile",
        score_column_name = "exposome_quantile_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "var",
        score_column_name = "exposome_var_score"
    )

## ----associate-exposome-score-------------------------------------------------

# Associate exposome scores with outcome
expom <- run_association(
    exposomicset = expom,
    outcome = "hs_asthma",
    source = "exposures",
    feature_set = c(
        "exposome_median_score",
        "exposome_pca_score",
        "exposome_irt_score",
        "exposome_quantile_score",
        "exposome_var_score"
    ),
    action = "add",
    family = "binomial"
)

## ----plot-exposome-scores, fig.height=2.5, fig.width=5,fig.cap="Associations of aerosol exposome scores with asthma status. The variance-based score has the strongest association with asthma status."----

# Plot the association forest plot
plot_association(
    exposomicset = expom,
    source = "exposures",
    terms = c(
        "exposome_median_score",
        "exposome_pca_score",
        "exposome_irt_score",
        "exposome_quantile_score",
        "exposome_var_score"
    ),
    filter_col = "p.value",
    filter_thresh = 0.05,
    r2_col = "r2"
)

## ----session_info-------------------------------------------------------------

sessionInfo()

#' Identify and Annotate Shared Top Features Across Integration Factors
#'
#' Identifies top features shared across factors based on integration method.
#' For MOFA/MCIA, takes intersection across factors. For DIABLO/RGCCA,
#' takes features recurring in more than 2 block-specific components.
#'
#' @param exposomicset A `MultiAssayExperiment` with integration results
#' and top factor features.
#' @param robust Logical; if `TRUE`, uses sensitivity score. Otherwise,
#'  uses DEG thresholds.
#' @param stability_score Optional numeric threshold
#' (overrides default from metadata).
#' @param score_col Column name for sensitivity score.
#' Default is `"stability_score"`.
#' @param pval_thresh DEG p-value threshold (if `robust = FALSE`).
#' Default is `0.05`.
#' @param logfc_thresh DEG logFC threshold (if `robust = FALSE`).
#'  Default is `log2(1.5)`.
#' @param pval_col Column name for p-value. Default is `"padj"`.
#' @param logfc_col Column name for logFC. Default is `"logFC"`.
#' @param action `"add"` to return modified object, `"get"` to
#' return data.frame.
#'
#' @return Modified `MultiAssayExperiment` or `data.frame` of
#' shared top features.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # perform multiomics integration
#' mae <- run_multiomics_integration(
#'     mae,
#'     method = "DIABLO",
#'     outcome = "smoker",
#'     n_factors = 3
#' )
#'
#'
#' # identify the features that contribute most to the factors
#' mae <- extract_top_factor_features(
#'     mae,
#'     factors = c("V1", "V2", "V3"),
#'     method = "percentile",
#'     percentile = 0.5,
#'     action = "add"
#' )
#'
#' # perform differential abundance analysis
#' mae <- run_differential_abundance(
#'     exposomicset = mae,
#'     formula = ~ smoker + sex,
#'     abundance_col = "counts",
#'     method = "limma_voom",
#'     action = "add"
#' )
#'
#' # determine the overlap in features
#' mae <- mae |>
#'     run_factor_overlap(
#'         robust = FALSE,
#'         pval_col = "adj.P.Val"
#'     )
#'
#' @export
run_factor_overlap <- function(
  exposomicset,
  robust = TRUE,
  stability_score = NULL,
  score_col = "stability_score",
  pval_thresh = 0.05,
  logfc_thresh = log2(1.5),
  pval_col = "padj",
  logfc_col = "logFC",
  action = "add"
) {
    # Ensure top features exist
    if (!"top_factor_features" %in% names(MultiAssayExperiment::metadata(
        exposomicset
    )$multiomics_integration)) {
        stop("Please run 'extract_top_factor_features()' first.")
    }

    method <- MultiAssayExperiment::metadata(exposomicset)$multiomics_integration$integration_results$method

    top_factor_features <- MultiAssayExperiment::metadata(exposomicset)$multiomics_integration$top_factor_features |>
        dplyr::mutate(exp_name_feature = paste(exp_name, feature, sep = "_"))

    # Identify common features across factors
    common_features <- if (method %in% c("DIABLO", "RGCCA")) {
        top_factor_features |>
            dplyr::group_by(exp_name, feature) |>
            dplyr::summarise(n = dplyr::n(), .groups = "drop") |>
            dplyr::filter(n > 1) |>
            dplyr::mutate(exp_name_feature = paste(exp_name, feature, sep = "_")) |>
            dplyr::pull(exp_name_feature)
    } else {
        top_factor_features |>
            (\(df) split(df, df$factor))() |>
            purrr::map(~ dplyr::pull(.x, exp_name_feature)) |>
            purrr::reduce(intersect)
    }

    top_factor_features <- top_factor_features |>
        dplyr::filter(exp_name_feature %in% common_features)

    # Annotate with DEG/robust marker
    da_res <- if (robust) {
        score <- stability_score %||%
            purrr::pluck(
                MultiAssayExperiment::metadata(exposomicset),
                "differential_analysis",
                "sensitivity_analysis",
                "score_thresh"
            )

        purrr::pluck(
            MultiAssayExperiment::metadata(exposomicset),
            "differential_analysis",
            "sensitivity_analysis",
            "feature_stability"
        ) |>
            dplyr::filter(!!rlang::sym(score_col) > score) |>
            dplyr::mutate(exp_name_feature = paste(exp_name, feature, sep = "_"))
    } else {
        purrr::pluck(
            MultiAssayExperiment::metadata(exposomicset),
            "differential_analysis",
            "differential_abundance"
        ) |>
            dplyr::filter(
                !!rlang::sym(pval_col) < pval_thresh,
                abs(!!rlang::sym(logfc_col)) > logfc_thresh
            ) |>
            dplyr::mutate(exp_name_feature = paste(exp_name, feature, sep = "_"))
    }

    top_factor_features <- top_factor_features |>
        dplyr::mutate(is_deg = exp_name_feature %in% da_res$exp_name_feature)

    # Optional: Add annotations using pivot_feature()
    top_factor_features <- top_factor_features |>
        dplyr::left_join(
            pivot_feature(exposomicset),
            by = c(
                "feature" = ".feature",
                "exp_name" = ".exp_name"
            )
        )

    sprintf(
        "Found %s common features across factors",
        length(unique(top_factor_features$exp_name_feature))
    )

    if (action == "add") {
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$multiomics_integration$common_top_factor_features <-
            top_factor_features
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        step_record <- list(
            run_factor_overlap = list(
                timestamp = Sys.time(),
                params = list(
                    robust = robust,
                    stability_score = stability_score,
                    score_col = score_col,
                    pval_thresh = pval_thresh,
                    logfc_thresh = logfc_thresh
                ),
                notes = if (robust) {
                    paste0(
                        "Annotated based on '",
                        score_col,
                        "' score > ",
                        round(score, 3), "."
                    )
                } else {
                    paste0(
                        "Annotated using p<",
                        pval_thresh,
                        " and |logFC|>",
                        round(logfc_thresh, 3), "."
                    )
                }
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else {
        return(top_factor_features)
    }
}

#' Extract Feature Metadata from a MultiAssayExperiment
#'
#' Extracts feature-level metadata across all assays in a
#' `MultiAssayExperiment` and returns a combined tibble.
#'
#' @param exposomicset A `MultiAssayExperiment` object.
#'
#' @details
#' This function:
#' - Iterates over all assays in the `MultiAssayExperiment`.
#' - Updates each assay's sample metadata (`colData`) using
#' `.update_assay_colData()`.
#' - Extracts feature-level metadata using `pivot_transcript()` from `tidybulk`.
#' - Combines results across assays into a single tibble,
#' adding a `.exp_name` column.
#'
#' @return A tibble with feature metadata from all assays,
#' with an added `.exp_name` column.
#'
#'
#' @examples
#'
#' #' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # pivot experiment
#' feature_data <- mae |>
#'     pivot_feature()
#' @importMethodsFrom tidybulk pivot_transcript
#'
#' @export
pivot_feature <- function(exposomicset) {
    res <- lapply(
        names(MultiAssayExperiment::experiments(exposomicset)), function(exp_name) {
            exp <- .update_assay_colData(exposomicset, exp_name) |>
                pivot_transcript()
        }
    )

    names(res) <- names(MultiAssayExperiment::experiments(exposomicset))

    res <- res |>
        dplyr::bind_rows(.id = ".exp_name")
    return(res)
}

#' Plot Sample Clusters
#'
#' Generates a heatmap of sample clustering results and
#' summarizes sample group assignments.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing sample
#'  clustering results
#' in `metadata(exposomicset)$sample_clustering`.
#' @param exposure_cols A character vector specifying columns from `colData`
#' to include
#' in the summary. Default is `NULL`, which includes all available columns.
#'
#' @details
#' This function:
#' - Extracts sample cluster assignments from
#' `metadata(exposomicset)$sample_clustering`.
#' - Merges cluster labels with `colData(exposomicset)`.
#' - Plots the heatmap stored in
#' `metadata(exposomicset)$sample_clustering$heatmap`.
#'
#' @return A `ComplexHeatmap` plot displaying sample clustering results.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 30,
#'     return_mae = TRUE
#' )
#'
#' # determine sample clusters
#' mae <- run_cluster_samples(
#'     exposomicset = mae,
#'     exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi"),
#'     clustering_approach = "diana"
#' )
#'
#' # plot sample clusters
#' sample_cluster_p <- mae |>
#'     plot_sample_clusters(
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' @importFrom dplyr select mutate_all inner_join rename group_by mutate
#' @importFrom tidyr pivot_longer
#' @importFrom tibble rownames_to_column
#' @importFrom purrr pluck
#' @export
plot_sample_clusters <- function(
  exposomicset,
  exposure_cols = NULL
) {
    # check for suggested packages
    .check_suggested(pkg = "tidyHeatmap")
    .check_suggested(pkg = "circlize")
    if (!"sample_clustering" %in% names(
        MultiAssayExperiment::metadata(exposomicset)$quality_control
    )) {
        stop("Please run `run_cluster_samples()` first")
    }

    if (is.null(exposure_cols)) {
        exposure_cols <- colnames(exposomicset)
    } else {
        exposure_cols <- exposure_cols[exposure_cols %in% colnames(
            MultiAssayExperiment::colData(exposomicset)
        )]
    }

    df <- exposomicset |>
        MultiAssayExperiment::colData() |>
        as.data.frame() |>
        dplyr::select(exposure_cols) |>
        dplyr::mutate_all(~ as.numeric(.)) |>
        tibble::rownames_to_column("Sample") |>
        dplyr::inner_join(
            data.frame(
                Sample = names(
                    MultiAssayExperiment::metadata(exposomicset) |>
                        purrr::pluck(
                            "quality_control",
                            "sample_clustering",
                            "sample_groups"
                        )
                ),
                sample_group = paste0(
                    "Group_",
                    as.character(
                        MultiAssayExperiment::metadata(exposomicset) |>
                            purrr::pluck(
                                "quality_control",
                                "sample_clustering",
                                "sample_groups"
                            )
                    )
                )
            ),
            by = c("Sample" = "Sample")
        ) |>
        tidyr::pivot_longer(-c(Sample, sample_group),
            names_to = "variable",
            values_to = "value"
        ) |>
        dplyr::inner_join(
            MultiAssayExperiment::metadata(exposomicset)$codebook,
            by = "variable"
        ) |>
        dplyr::rename(Category = category)

    df |>
        mutate(Category = as.character(Category)) |>
        dplyr::group_by(sample_group) |>
        tidyHeatmap::heatmap(
            variable,
            Sample,
            value,
            scale = "row",
            palette_value = circlize::colorRamp2(
                c(-2, 0, 2),
                c("#006666", "white", "#8E0152")
            )
        ) |>
        tidyHeatmap::annotation_tile(
            Category,
            palette = rev(tidy_exp_pal)[
                seq_len(length(unique(df$Category)))
            ]
        ) |>
        tidyHeatmap::annotation_tile(
            sample_group,
            palette = tidy_exp_pal[
                seq_len(length(unique(df$sample_group)))
            ]
        )
}

#' @title Export tidyexposomics Results to Excel
#'
#' @description
#' Exports selected results stored in a `MultiAssayExperiment` object created
#' by the `tidyexposomics` pipeline to an Excel workbook.
#' Users can select which result types to include,
#' and optionally add placeholder sheets for missing data.
#'
#' @param exposomicset A `MultiAssayExperiment` object with results stored
#' in `@metadata`, typically created by the `tidyexposomics` pipeline.
#' @param file Character. Path to the output Excel file.
#' @param result_types Character vector specifying which result categories
#'  to export. Options include:
#'   - `"correlation"`: Correlation results.
#'   - `"association"`: Association results.
#'   - `"mixture_analysis"`: Mixture Analysis results.
#'   - `"differential_analysis"`: Differential abundance results,
#'    including sensitivity analysis if available.
#'   - `"multiomics_integration"`: Common top features contributing to
#'    latent factors from multi-omics integration.
#'   - `"network"`: Exposure impact metrics from network analyses.
#'   - `"enrichment"`: Enrichment results by omic and exposure category.
#'   - `"exposure_summary"`: Summary statistics for exposure variables.
#'   - `"pipeline"`: Overview of steps completed in the pipeline.
#'
#'   Use `"all"` to export all of the above categories.
#'
#' @param include_empty_tabs Logical. If `TRUE`, adds placeholder sheets for
#' any missing result types. Default is `FALSE`.
#'
#' @return An Excel file is written to the specified path.
#' A message is printed with the file location.
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # run correlation analysis
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "exposures",
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # file path of the output file
#' tmp <- tempfile(fileext = ".xlsx")
#'
#' # extract the correlation results
#' extract_results_excel(
#'     exposomicset = mae,
#'     result_types = "correlation",
#'     file = tmp
#' )
#'
#' @importFrom purrr iwalk
#' @importFrom tibble enframe
#' @export
extract_results_excel <- function(
  exposomicset,
  file = "tidyexposomics_results.xlsx",
  result_types = c(
      "correlation",
      "association",
      "mixture_analysis",
      "differential_analysis",
      "multiomics_integration",
      "network",
      "enrichment",
      "exposure_summary",
      "pipeline"
  ),
  include_empty_tabs = FALSE
) {
    .check_suggested(pkg = "openxlsx")
    stopifnot("MultiAssayExperiment" %in% class(exposomicset))

    # Handle "all"
    all_types <- c(
        "correlation",
        "association",
        "mixture_analysis",
        "differential_analysis",
        "multiomics_integration",
        "network",
        "enrichment",
        "exposure_summary",
        "pipeline"
    )
    if ("all" %in% result_types) {
        result_types <- all_types
    }

    wb <- openxlsx::createWorkbook()

    safe_add_sheet <- function(name, data) {
        if (!is.null(data) && is.data.frame(data)) {
            openxlsx::addWorksheet(wb, name)
            openxlsx::writeData(wb, name, data, withFilter = TRUE)

            # Bold headers + border around header row
            header_style <- openxlsx::createStyle(
                textDecoration = "bold",
                border = "TopBottomLeftRight",
                borderStyle = "thin"
            )

            openxlsx::addStyle(
                wb,
                sheet = name, style = header_style,
                rows = 1, cols = seq_len(ncol(data)), gridExpand = TRUE
            )

            openxlsx::setColWidths(
                wb,
                sheet = name,
                cols = seq_len(ncol(data)), widths = "auto"
            )
        } else if (include_empty_tabs) {
            openxlsx::addWorksheet(wb, name)
            openxlsx::writeData(wb, name, data.frame(Note = "No data available."))
        }
    }

    # --- Adding Correlation Results ---------
    if ("correlation" %in% result_types) {
        message("Writing Correlation Results.")
        enr <- MultiAssayExperiment::metadata(exposomicset)$correlation
        if (!is.null(enr) && is.list(enr)) {
            purrr::iwalk(enr, function(obj, groupname) {
                df <- obj
                safe_add_sheet(paste0("Correlation_", groupname), df)
            })
        }
    }

    # --- Adding Association Results -----

    if ("association" %in% result_types) {
        message("Writing Association Results.")
        enr <- MultiAssayExperiment::metadata(exposomicset)$association
        if (!is.null(enr) && is.list(enr)) {
            purrr::iwalk(enr, function(obj, groupname) {
                df <- obj
                safe_add_sheet(paste0("association_", groupname), df)
            })
        }
    }

    # --- Adding Mixture Analysis Results ---------
    if ("mixture_analysis" %in% result_types) {
        message("Writing Mixture Analysis Results.")
        mix <- MultiAssayExperiment::metadata(exposomicset)$mixture_analysis

        if (!is.null(mix) && is.list(mix)) {
            purrr::iwalk(mix, function(res, method_name) {
                # Weights (all methods have this)
                if (!is.null(res$weights)) {
                    safe_add_sheet(
                        paste0("Mixture_Weights_", method_name),
                        res$weights
                    )
                }

                # Mixture effect (all methods have this)
                if (!is.null(res$mixture_effect)) {
                    safe_add_sheet(
                        paste0("Mixture_Effect_", method_name),
                        res$mixture_effect
                    )
                }

                # Method-specific outputs
                # qgcomp: partial_effects
                if (!is.null(res$partial_effects)) {
                    safe_add_sheet(
                        paste0("Mixture_Partial_", method_name),
                        res$partial_effects
                    )
                }

                # wqs: model_summary
                if (!is.null(res$model_summary)) {
                    safe_add_sheet(
                        paste0("Mixture_Model_", method_name),
                        res$model_summary
                    )
                }

                # Metadata summary sheet
                meta_df <- tibble::tibble(
                    field = c("method", "outcome", "family", "n_samples", "exposures", "covariates"),
                    value = c(
                        res$method %||% NA_character_,
                        res$outcome %||% NA_character_,
                        res$family %||% NA_character_,
                        as.character(res$n_samples %||% NA),
                        paste(res$exposures, collapse = ", "),
                        paste(res$covariates %||% "none", collapse = ", ")
                    )
                )
                safe_add_sheet(
                    paste0("Mixture_Info_", method_name),
                    meta_df
                )
            })
        }
    }

    # --- Adding Differential Abundance Results --------
    if ("differential_analysis" %in% result_types) {
        message("Writing Differential Abundance Results.")
        da <- MultiAssayExperiment::metadata(exposomicset)$differential_analysis$differential_abundance
        if (!is.null(da)) {
            safe_add_sheet("Differential Abundance", da)
        }

        if ("sensitivity_analysis" %in% names(
            MultiAssayExperiment::metadata(exposomicset)$differential_analysis
        )) {
            message("Writing Sensitivity Analysis Results.")
            sens <- MultiAssayExperiment::metadata(exposomicset)$differential_analysis$sensitivity_analysis$feature_stability
            if (!is.null(sens)) {
                safe_add_sheet("Sensitivity Analysis", sens)
            }
        }
    }

    # --- Adding Multiomics Integration Results ----------
    if ("multiomics_integration" %in% result_types) {
        message("Writing Multiomics Integration Results.")
        factors <- MultiAssayExperiment::metadata(exposomicset)$multiomics_integration$common_top_factors
        if (!is.null(factors)) {
            safe_add_sheet("Common Factor Features", factors)
        }
    }

    # --- Adding Network Results --------------
    if ("network" %in% result_types) {
        message("Writing Network Impact Results.")
        exposure_impact <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck(
                "network",
                "exposure_impact"
            )
        if (!is.null(exposure_impact) && is.list(exposure_impact)) {
            purrr::iwalk(exposure_impact, function(obj, groupname) {
                df <- obj
                safe_add_sheet(paste0("Network_Impact_", groupname), df)
            })
        }
    }

    # --- Adding Enrichment Results -------------
    if ("enrichment" %in% result_types) {
        message("Writing Enrichment Results.")
        enr <- MultiAssayExperiment::metadata(exposomicset)$enrichment
        if (!is.null(enr) && is.list(enr)) {
            purrr::iwalk(enr, function(obj, groupname) {
                df <- obj
                safe_add_sheet(paste0("Enrichment_", groupname), df)
            })
        }
    }

    # --- Adding Pipeline summary -------
    if ("pipeline" %in% result_types) {
        message("Writing Pipeline Summary.")
        pipeline_info <- MultiAssayExperiment::metadata(exposomicset)$summary$steps
        if (is.list(pipeline_info)) {
            steps_df <- tibble::enframe(pipeline_info,
                name = "Step",
                value = "Details"
            )
            safe_add_sheet("Pipeline_Steps", steps_df)
        }
    }

    # --- Adding Exposure summaries -------
    if ("exposure_summary" %in% result_types) {
        message("Writing Exposure Summary Results.")
        safe_add_sheet(
            "Exposure_Summary",
            tryCatch(
                {
                    run_summarize_exposures(exposomicset, action = "get")
                },
                error = function(e) NULL
            )
        )
    }

    # Save
    openxlsx::saveWorkbook(wb, file, overwrite = TRUE)
    message("Results written to: ", normalizePath(file))
}

#' Create Correlation Network from Feature Data
#'
#' Constructs an undirected feature-feature or feature-exposure
#' correlation network from correlation results stored in a
#' `MultiAssayExperiment` object. The function
#' supports multiple correlation formats depending on `feature_type`,
#' and stores or returns an `igraph` object with associated node
#' and edge metadata.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing correlation
#' results in metadata.
#' @param feature_type Type of correlation result to convert to a network.
#'  One of:
#'   `"degs"`, `"omics"`, `"factors"`, `"factor_features"`, `"exposures"`,
#'   `"degs_feature_cor"`, `"omics_feature_cor"`,
#'    or `"factor_features_feature_cor"`.
#' @param action Whether to `"add"` the network to the object or
#' `"get"` it as a list.
#'
#' @return If `action = "add"`, returns the updated `MultiAssayExperiment`
#' with a new `network` entry in metadata.
#'         If `action = "get"`, returns a list with `graph`
#'         (an `igraph` object) and `summary` (a tibble).
#'
#' @details The function detects the appropriate edge and node structure
#' based on column names in the correlation results.
#' Edge weights are based on correlation coefficients and include FDR values.
#'
#'
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # perform correlation analyses
#' # correlate with exposures
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     ) |>
#'     # correlate omics features with themselves
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         feature_cors = TRUE,
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # create the networks
#' mae <- mae |>
#'     run_create_network(
#'         feature_type = "omics_feature_cor",
#'         action = "add"
#'     ) |>
#'     run_create_network(
#'         feature_type = "omics",
#'         action = "add"
#'     )
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom dplyr bind_rows select distinct
#' @importFrom igraph graph_from_data_frame
#' @export
run_create_network <- function(
  exposomicset,
  feature_type = c(
      "degs",
      "omics",
      "factors",
      "factor_features",
      "exposures",
      "degs_feature_cor",
      "omics_feature_cor",
      "factor_features_feature_cor"
  ),
  action = c(
      "add",
      "get"
  )
) {
    feature_type <- match.arg(feature_type)
    action <- match.arg(action)

    all_metadata <- MultiAssayExperiment::metadata(exposomicset)
    cor_data <- all_metadata$correlation[[feature_type]]

    if (is.null(cor_data)) {
        stop(sprintf("No correlation data found for feature_type = '%s'", feature_type))
    }

    message("Creating network from correlation results.")

    # --- exposure-feature correlations
    if (all(c("feature", "exposure", "exp_name", "category") %in% colnames(cor_data))) {
        node_tbl <- dplyr::distinct(
            dplyr::bind_rows(
                dplyr::select(cor_data, name = feature, group = exp_name),
                dplyr::select(cor_data, name = exposure, group = category)
            )
        ) |>
            dplyr::mutate(vertex_id = paste(name, group, sep = "_"))

        edge_df <- cor_data |>
            dplyr::mutate(
                from = paste(exposure, category, sep = "_"),
                to   = paste(feature, exp_name, sep = "_")
            ) |>
            dplyr::select(from, to, correlation, FDR)

        # --- generic pairwise with var1/var2
    } else if (all(c("var1", "var2", "var1_type", "var2_type") %in% colnames(cor_data))) {
        node_tbl <- dplyr::distinct(
            dplyr::bind_rows(
                dplyr::select(cor_data, name = var1, group = var1_type),
                dplyr::select(cor_data, name = var2, group = var2_type)
            )
        ) |>
            dplyr::mutate(vertex_id = paste(name, group, sep = "_"))

        edge_df <- cor_data |>
            dplyr::mutate(
                from = paste(var1, var1_type, sep = "_"),
                to   = paste(var2, var2_type, sep = "_")
            ) |>
            dplyr::select(from, to, correlation, FDR)

        # --- correlations across experiments
    } else if (all(c("var1", "var2", "exp_name_1", "exp_name_2") %in% colnames(cor_data))) {
        node_tbl <- dplyr::distinct(
            dplyr::bind_rows(
                dplyr::select(cor_data, name = var1, group = exp_name_1),
                dplyr::select(cor_data, name = var2, group = exp_name_2)
            )
        ) |>
            dplyr::mutate(vertex_id = paste(name, group, sep = "_"))

        edge_df <- cor_data |>
            dplyr::mutate(
                from = paste(var1, exp_name_1, sep = "_"),
                to   = paste(var2, exp_name_2, sep = "_")
            ) |>
            dplyr::select(from, to, correlation, FDR)
    } else {
        stop("Unrecognized correlation format.")
    }

    # --- build graph safely with unique vertex_id ---
    vertices <- node_tbl |>
        dplyr::transmute(
            name = vertex_id, # unique ID for igraph
            label = name, # original biological name
            group = group # grouping info
        )

    g <- igraph::graph_from_data_frame(
        edge_df,
        directed = FALSE,
        vertices = vertices
    )

    net_summary <- .summarize_graph(g)
    net_name <- paste0("network_", feature_type)

    if (action == "add") {
        MultiAssayExperiment::metadata(exposomicset)[["network"]][[net_name]] <- list(
            graph = g,
            summary = net_summary
        )
        step_record <- list(run_create_network = list(
            timestamp = Sys.time(),
            params = list(feature_type = feature_type),
            notes = paste0(
                "Created undirected network from correlation results for '",
                feature_type, "'."
            )
        ))
        MultiAssayExperiment::metadata(exposomicset)$summary$steps <-
            c(MultiAssayExperiment::metadata(exposomicset)$summary$steps, step_record)

        message("Network added to metadata as: ", net_name)
        return(exposomicset)
    } else {
        return(list(graph = g, summary = net_summary))
    }
}

#' Filter low-quality features in omics assays
#'
#' This function applies variance- or expression-based filtering
#' across one or more assays within a `MultiAssayExperiment` object.
#' It is useful for removing low-quality or uninformative features
#'  before downstream analysis.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing omics assays.
#' @param method Filtering method: either `"variance"` or `"expression"`.
#' @param assays Character vector of assay names to filter.
#' If `NULL`, all assays are filtered.
#' @param assay_name Name or index of the assay within each
#' `SummarizedExperiment` to use.
#' @param min_var Minimum variance threshold (used if `method = "variance"`).
#' @param min_value Minimum expression value (used if `method = "expression"`).
#' @param min_prop Minimum proportion of samples exceeding `min_value`
#' (used if `method = "expression"`).
#' @param verbose Whether to print messages for each assay being filtered.
#'
#' @return A filtered `MultiAssayExperiment` object with
#' updated assays and step record.
#' @examples
#' # Filter the proteomics assay by variance
#' filtered_mae <- filter_omics(
#'     exposomicset = make_example_data(return_mae = TRUE),
#'     method = c("variance"),
#'     assays = "proteomics",
#'     assay_name = 1,
#'     min_var = 0.01,
#'     verbose = TRUE
#' )
#'
#' @export
filter_omics <- function(
  exposomicset,
  method = c("variance", "expression"),
  assays = NULL,
  assay_name = 1,
  min_var = 1e-5,
  min_value = 5,
  min_prop = 0.7,
  verbose = TRUE
) {
    method <- match.arg(method)
    if (!inherits(exposomicset, "MultiAssayExperiment")) {
        stop("Input must be a MultiAssayExperiment object.")
    }

    exp_list <- MultiAssayExperiment::experiments(exposomicset)
    assay_names <- names(exp_list)
    if (is.null(assays)) assays <- assay_names
    assays <- intersect(assays, assay_names)

    removed_features <- list()

    # Now filter_single RETURNS a list: list(se = ..., stats = ...)
    filter_single <- function(se, name) {
        mat <- SummarizedExperiment::assay(se, assay_name)
        total <- nrow(mat)

        keep <- switch(method,
            variance = {
                row_vars <- apply(mat, 1, stats::var, na.rm = TRUE)
                row_vars > min_var
            },
            expression = {
                prop_expressed <- rowMeans(mat > min_value, na.rm = TRUE)
                prop_expressed >= min_prop
            }
        )

        removed <- sum(!keep)
        stats <- list(
            removed = removed,
            total = total,
            kept = total - removed
        )

        if (verbose) {
            message(sprintf(
                "Filtered %d of %d features from '%s' using method '%s'",
                removed, total, name, method
            ))
        }

        list(se = se[keep, , drop = FALSE], stats = stats)
    }

    filtered_exps <- list()
    for (name in assay_names) {
        se <- exp_list[[name]]
        if (name %in% assays) {
            if (verbose) message(sprintf("Filtering assay: %s", name))
            result <- filter_single(se, name)
            filtered_exps[[name]] <- result$se
            removed_features[[name]] <- result$stats
        } else {
            filtered_exps[[name]] <- se
        }
    }

    filtered_exps_list <- MultiAssayExperiment::ExperimentList(filtered_exps)
    MultiAssayExperiment::experiments(exposomicset) <- filtered_exps_list

    # Update metadata with step records
    for (assay in names(removed_features)) {
        info <- removed_features[[assay]]
        step_name <- paste0("filter_omics_", assay)
        step_record <- list()
        step_record[[step_name]] <- list(
            timestamp = Sys.time(),
            params = list(
                method = method,
                assay = assay,
                assay_name = assay_name,
                min_var = min_var,
                min_value = min_value,
                min_prop = min_prop
            ),
            notes = sprintf(
                "Filtered omics features from '%s'
        Using method = '%s': %d removed of %d (%.1f%%).",
                assay,
                method,
                info$removed,
                info$total,
                100 * info$removed / info$total
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )
    }

    return(exposomicset)
}

# filter_omics <- function(exposomicset,
#                          method = c("variance", "expression"),
#                          assays = NULL,
#                          assay_name = 1,
#                          min_var = 1e-5,
#                          min_value = 5,
#                          min_prop = 0.7,
#                          verbose = TRUE) {
#   method <- match.arg(method)
#
#   if (!inherits(exposomicset, "MultiAssayExperiment")) {
#     stop("Input must be a MultiAssayExperiment object.")
#   }
#
#   exp_list <- MultiAssayExperiment::experiments(exposomicset)
#   assay_names <- names(exp_list)
#
#   if (is.null(assays)) assays <- assay_names
#   assays <- intersect(assays, assay_names)
#
#   removed_features <- list()
#
#   filter_single <- function(se, name) {
#     mat <- SummarizedExperiment::assay(se, assay_name)
#     total <- nrow(mat)
#
#     keep <- switch(method,
#                    variance = {
#                      row_vars <- apply(mat, 1, stats::var, na.rm = TRUE)
#                      row_vars > min_var
#                    },
#                    expression = {
#                      prop_expressed <- rowMeans(mat > min_value, na.rm = TRUE)
#                      prop_expressed >= min_prop
#                    })
#
#     removed <- sum(!keep)
#     removed_features[[name]] <<- list(
#       removed = removed,
#       total = total,
#       kept = total - removed
#     )
#
#     if (verbose) {
#       message(sprintf("Filtered %d of %d features from '%s' using method '%s'",
#                       removed, total, name, method))
#     }
#
#     se[keep, , drop = FALSE]
#   }
#
#   filtered_exps <- lapply(assay_names, function(name) {
#     se <- exp_list[[name]]
#     if (name %in% assays) {
#       if (verbose) message(sprintf("Filtering assay: %s", name))
#       filter_single(se, name)
#     } else {
#       se
#     }
#   })
#
#   names(filtered_exps) <- assay_names
#   MultiAssayExperiment::experiments(exposomicset) <- MultiAssayExperiment::ExperimentList(filtered_exps)
#
#   # Add step records per assay
#   for (assay in names(removed_features)) {
#     info <- removed_features[[assay]]
#     step_name <- paste0("filter_omics_", assay)
#     step_record <- list()
#     step_record[[step_name]] <- list(
#       timestamp = Sys.time(),
#       params = list(
#         method = method,
#         assay = assay,
#         assay_name = assay_name,
#         min_var = min_var,
#         min_value = min_value,
#         min_prop = min_prop
#       ),
#       notes = sprintf(
#         "Filtered omics features from '%s' using method = '%s': %d removed of %d (%.1f%%).",
#         assay, method, info$removed, info$total, 100 * info$removed / info$total
#       )
#     )
#
#     MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
#       MultiAssayExperiment::metadata(exposomicset)$summary$steps,
#       step_record
#     )
#   }
#
#   return(exposomicset)
# }
#

## ----setup,include=FALSE------------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----load-libraries,message=FALSE,warning=FALSE-------------------------------
# Load Libraries
library(tidyverse)
library(tidyexposomics)

## ----load-data----------------------------------------------------------------
# Load example data
data("tidyexposomics_example")

# Create exposomic set object
expom <- create_exposomicset(
    codebook = tidyexposomics_example$annotated_cb,
    exposure = tidyexposomics_example$meta,
    omics = list(
        "Gene Expression" = tidyexposomics_example$exp_filt,
        "Methylation" = tidyexposomics_example$methyl_filt
    ),
    row_data = list(
        "Gene Expression" = tidyexposomics_example$exp_fdata,
        "Methylation" = tidyexposomics_example$methyl_fdata
    )
)

## ----pivot-sample-------------------------------------------------------------

# Pivot sample data to a tibble
expom |>
    pivot_sample() |>
    head()

## ----pivot-sample-example-----------------------------------------------------

# Count the number of asthma cases by sex status
expom |>
    pivot_sample() |>
    group_by(hs_asthma, e3_sex_None) |>
    reframe(n = n())

## ----pivot-feature------------------------------------------------------------

# Pivot feature data to a tibble
expom |>
    pivot_feature() |>
    head()

## ----pivot-feature-example----------------------------------------------------

# Count the number of features per omic layer
expom |>
    pivot_feature() |>
    group_by(.exp_name) |>
    summarise(n = n())

## ----pivot-exp----------------------------------------------------------------

# Pivot experiment data to a tibble
expom |>
    pivot_exp(
        exp_name = "Gene Expression",
        features = "TC01004453.hg.1"
    ) |>
    head()

## ----pivot-exp-example, fig.height=4, fig.width=4, fig.align='center',fig.cap="IL23R gene expression levels (probe TC01004453.hg.1) by asthma status."----

# Plot expression of IL23R
expom |>
    pivot_exp(
        exp_name = "Gene Expression",
        features = "TC01004453.hg.1"
    ) |>
    ggplot(aes(
        x = as.character(hs_asthma),
        y = log2(counts + 1),
        color = as.character(hs_asthma),
        fill = as.character(hs_asthma)
    )) +
    geom_boxplot(alpha = 0.5) +
    geom_jitter(alpha = 0.1) +
    ggpubr::geom_pwc(label = "{p.adj.format}{p.adj.signif}") +
    theme_minimal() +
    ggpubr::rotate_x_text(angle = 45) +
    ggsci::scale_color_cosmic() +
    ggsci::scale_fill_cosmic() +
    labs(
        x = "",
        y = expression(Log[2] * "Abd."),
        fill = "Asthma Status",
        color = "Asthma Status"
    )

## ----session_info-------------------------------------------------------------

sessionInfo()

#' Run Exposure-Omics Association
#'
#' Test associations between each exposure and each omics feature using
#' limma's linear modeling framework.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing exposome
#'   and omics data.
#' @param exposures Character vector of exposure variable names to test.
#'   If `NULL`, uses all variables from the codebook.
#' @param exp_name Name(s) of the omics assay(s) to test against. If `NULL`,
#'   uses all assays.
#' @param covariates Optional character vector of covariate names to include
#'   in the model.
#' @param scaling_method Character. Scaling method to apply before modeling.
#'   Options include `"none"` (default).
#' @param correction_method Method for p-value adjustment. Default is `"fdr"`.
#' @param top_pct Top X% of features to retain using either mean or variance
#'  which is specified by `filter_by`. If `NULL`, no features will be filtered.
#' @param filter_by Determination of how to filter omics features either by
#'   mean or variance.
#' @param action If `"add"` (default), saves results to metadata,
#'   if `"get"`, returns results as a data frame.
#'
#' @return If `action = "add"`, returns updated `MultiAssayExperiment`.
#'   Otherwise, returns a tibble with association results.
#'
#' @details
#' This function uses limma to test associations between multiple exposures
#' and omics features. For each exposure, a linear model is fit with the
#' exposure as the predictor and each omics feature as the outcome,
#' adjusting for covariates.
#'
#' `omics_feature ~ exposure + covariate1 + covariate2 + ...`
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run exposure-omics association
#' mae <- mae |>
#'     run_exposure_omics_association(
#'         exposures = c("exposure_pm25", "exposure_no2"),
#'         covariates = c("age", "sex")
#'     )
#'
#' @importFrom limma lmFit eBayes topTable
#' @importFrom stats model.matrix as.formula
#' @export
run_exposure_omics_association <- function(
  exposomicset,
  exposures = NULL,
  exp_name = NULL,
  covariates = NULL,
  scaling_method = "none",
  correction_method = "fdr",
  top_pct = NULL,
  filter_by = c("variance", "mean"),
  action = "add"
) {
    .check_suggested("limma")
    filter_by <- match.arg(filter_by)

    # Get exposure names from codebook if not provided
    if (is.null(exposures)) {
        exposures <- MultiAssayExperiment::metadata(exposomicset)$codebook$variable
    }

    # Validate exposures exist
    col_data <- exposomicset |>
        MultiAssayExperiment::colData() |>
        as.data.frame()

    missing_exp <- setdiff(exposures, colnames(col_data))
    if (length(missing_exp) > 0) {
        stop("Exposures not found in colData: ", paste(missing_exp, collapse = ", "))
    }

    # Remove covariates from exposures if present
    exposures <- setdiff(exposures, covariates)

    # Get assay names
    if (is.null(exp_name)) {
        exp_name <- names(MultiAssayExperiment::experiments(exposomicset))
    }

    message(sprintf(
        "Testing %d exposures across %d assays",
        length(exposures), length(exp_name)
    ))

    # Run analysis across all assays
    results <- exp_name |>
        lapply(function(assay_name) {
            message("Processing assay: ", assay_name)

            exposomicset |>
                .update_assay_colData(assay_name) |>
                .filter_top_features(
                    top_pct = top_pct,
                    filter_by = filter_by
                ) |>
                .run_exposure_omics_limma(
                    exposures = exposures,
                    covariates = covariates,
                    scaling_method = scaling_method
                )
        }) |>
        stats::setNames(exp_name) |>
        dplyr::bind_rows(.id = "exp_name") |>
        dplyr::filter(contrast == ".exposure") |>
        dplyr::mutate(p_adjust = stats::p.adjust(p.value, method = correction_method)) |>
        dplyr::select(-contrast) |>
        tibble::as_tibble()

    # Add exposure annotations from codebook
    codebook <- MultiAssayExperiment::metadata(exposomicset)$codebook
    if (!is.null(codebook) && "variable" %in% colnames(codebook)) {
        results <- results |>
            dplyr::left_join(codebook, by = c("exposure" = "variable"))
    }

    if (action == "add") {
        MultiAssayExperiment::metadata(exposomicset)$association$exposure_omics <- list(
            results_df = results,
            covariates = covariates,
            exposures = exposures,
            exp_name = exp_name,
            method = "limma_trend"
        )

        step_record <- list(
            run_exposure_omics_association = list(
                timestamp = Sys.time(),
                params = list(
                    exposures = exposures,
                    exp_name = exp_name,
                    covariates = covariates,
                    scaling_method = scaling_method,
                    correction_method = correction_method
                ),
                notes = sprintf(
                    "Tested %d exposures against %d assays using limma-trend",
                    length(exposures), length(exp_name)
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else {
        return(results)
    }
}

#' Run limma-trend for exposure-omics associations
#' @keywords internal
#' @noRd
.run_exposure_omics_limma <- function(
  se,
  exposures,
  covariates,
  scaling_method
) {
    exposures |>
        lapply(function(exp_var) {
            # Scale numeric exposures, keep categorical as is
            if (is.numeric(SummarizedExperiment::colData(se)[[exp_var]])) {
                SummarizedExperiment::colData(se)$.exposure <- as.numeric(
                    scale(SummarizedExperiment::colData(se)[[exp_var]])
                )
            } else {
                SummarizedExperiment::colData(se)$.exposure <-
                    SummarizedExperiment::colData(se)[[exp_var]]
            }

            formula <- if (!is.null(covariates) && length(covariates) > 0) {
                stats::as.formula(paste("~ .exposure +", paste(covariates, collapse = " + ")))
            } else {
                ~.exposure
            }

            .run_limma_trend(
                se = se,
                formula = formula,
                abundance_col = SummarizedExperiment::assayNames(se)[1],
                scaling_method = scaling_method
            ) |>
                dplyr::mutate(exposure = exp_var) |>
                dplyr::rename(
                    feature = feature,
                    estimate = logFC,
                    t_statistic = t,
                    p.value = P.Value,
                    p_adjust = adj.P.Val
                )
        }) |>
        dplyr::bind_rows()
}

#' Filter to top features by variance or mean
#' @keywords internal
#' @noRd
.filter_top_features <- function(
  se,
  top_pct = NULL,
  filter_by = "variance"
) {
    if (is.null(top_pct)) {
        return(se)
    }

    mat <- SummarizedExperiment::assay(se)

    scores <- switch(filter_by,
        variance = apply(mat, 1, var, na.rm = TRUE),
        mean = apply(mat, 1, mean, na.rm = TRUE)
    )

    n_features <- ceiling(nrow(se) * top_pct / 100)
    top_features <- names(sort(scores, decreasing = TRUE))[seq_len(n_features)]

    message(sprintf(
        "Filtered to top %d%% (%d features) by %s",
        top_pct, n_features, filter_by
    ))

    se[top_features, ]
}

#' Compute Composite Exposome Scores
#'
#' Calculates a summary exposome score per sample using one of several methods
#' including mean, sum, median, PCA (first principal component),
#' IRT (Item Response Theory), quantile binning, or row-wise variance.
#' The resulting score is added to the `colData` of the
#' `MultiAssayExperiment` object.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing exposure
#' data in its `colData`.
#' @param score_type Character. The method used to compute the score.
#' Options are:
#'   `"mean"`, `"sum"`, `"median"`, `"pca"`, `"irt"`, `"quantile"`, `"var"`.
#' @param exposure_cols Optional character vector. Specific exposure
#'  column names to include. If `NULL`, all numeric columns are used.
#' @param scale Logical. Whether to scale the exposures before computing
#' the score. Default is `TRUE`.
#' @param score_column_name Optional name for the resulting score column.
#' If `NULL`, an automatic name is used (e.g., `"exposome_score_pca"`).
#'
#' @return A `MultiAssayExperiment` object with the exposome score added
#'  to `colData()`.
#'
#' @details
#' - `"pca"` uses the first principal component from `prcomp()`.
#' - `"irt"` uses the `mirt` package to fit a graded response model to
#' discretized exposures.
#' - `"quantile"` assigns decile bins (1-10) to each variable and sums
#' them row-wise.
#' - `"var"` computes the row-wise variance across exposures.
#'
#' @examples
#' # create the example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # create the air pollution score
#' mae <- run_exposome_score(
#'     mae,
#'     score_type = "pca",
#'     exposure_cols = c("exposure_pm25", "exposure_no2"),
#'     scale = TRUE,
#'     score_column_name = "air_pollution_score"
#' )
#'
#' @export
run_exposome_score <- function(
  exposomicset,
  score_type,
  exposure_cols = NULL,
  scale = TRUE,
  score_column_name = NULL
) {
    # Extract and preprocess colData
    message("Extracting exposure data...")
    data <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame() |>
        dplyr::select_if(is.numeric)

    # Unselect PCs
    data <- data |>
        dplyr::select(-dplyr::starts_with("PC"))

    # Check if exposure_cols is provided
    if (!is.null(exposure_cols)) {
        exposure_cols <- intersect(exposure_cols, colnames(data))

        data <- data |>
            dplyr::select(all_of(exposure_cols))
    }

    # Scale the data if user says so
    if (scale) {
        data <- data |>
            dplyr::mutate_all(~ scale(.))
    }

    if (score_type == "mean") {
        # Calculate mean exposome score
        message("Calculating mean exposure scores...")

        scores <- data |>
            dplyr::mutate(exposome_score_mean = rowMeans(
                dplyr::across(dplyr::everything()),
                na.rm = TRUE
            )) |>
            dplyr::select(exposome_score_mean)
    } else if (score_type == "sum") {
        # Calculate sum exposome score
        message("Calculating sum exposure scores...")

        scores <- data |>
            dplyr::mutate(exposome_score_sum = rowSums(
                dplyr::across(dplyr::everything()),
                na.rm = TRUE
            )) |>
            dplyr::select(exposome_score_sum)
    } else if (score_type == "median") {
        # Calculate median exposome score
        message("Calculating median exposure scores...")
        .check_suggested("matrixStats")
        scores <- data |>
            dplyr::mutate(exposome_score_median = matrixStats::rowMedians(
                as.matrix(
                    dplyr::pick(
                        dplyr::everything()
                    )
                ),
                na.rm = TRUE
            )) |>
            dplyr::select(exposome_score_median)
    } else if (score_type == "pca") {
        # Calculate PCA exposome score
        message("Calculating PCA exposure scores...")

        pca_result <- prcomp(data, center = TRUE)
        scores <- data.frame(pca_result$x[, 1])
        colnames(scores) <- c("exposome_score_pca")
    } else if (score_type == "irt") {
        # Calculate IRT exposome score
        message("Calculating IRT exposure scores...")
        .check_suggested("mirt")

        exposures_ordinal <- apply(data, 2, function(x) dplyr::ntile(x, 10))
        model <- mirt::mirt(exposures_ordinal, 1, itemtype = "graded")
        scores <- mirt::fscores(model, full.scores.SE = FALSE)
        scores <- scores |>
            as.data.frame() |>
            setNames("exposome_score_irt")
        rownames(scores) <- rownames(data)
    } else if (score_type == "quantile") {
        # Calculate quantile exposome score
        message("Calculating quantile exposure scores...")

        quantile_data <- data |>
            dplyr::mutate(
                dplyr::across(
                    dplyr::everything(),
                    ~ dplyr::ntile(., 10)
                )
            )

        scores <- quantile_data |>
            dplyr::mutate(exposome_score_quantile = rowSums(
                dplyr::across(dplyr::everything()),
                na.rm = TRUE
            )) |>
            dplyr::select(exposome_score_quantile)
    } else if (score_type == "var") {
        # Calculate variance exposome score
        message("Calculating variance exposure scores...")
        .check_suggested("matrixStats")
        scores <- data |>
            dplyr::mutate(exposome_score_var = matrixStats::rowVars(
                as.matrix(
                    dplyr::pick(
                        dplyr::everything()
                    )
                ),
                na.rm = TRUE
            )) |>
            dplyr::select(exposome_score_var)
    } else {
        stop("Choose either 'sum', mean', 'median', 'pca', 'irt', or 'quantile'.")
    }

    if (!is.null(score_column_name)) {
        colnames(scores) <- score_column_name
    }

    # Add scores to the MultiAssayExperiment object
    updated_col_data <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame() |>
        # if name exists in df - overwrite the score column name
        dplyr::select(-dplyr::any_of(colnames(scores))) |>
        tibble::rownames_to_column("id") |>
        dplyr::left_join(
            scores |>
                tibble::rownames_to_column("id"),
            by = "id"
        ) |>
        tibble::column_to_rownames("id")

    # Update the colData of the MultiAssayExperiment object
    MultiAssayExperiment::colData(exposomicset) <- S4Vectors::DataFrame(
        updated_col_data
    )

    # Add summary step record
    # Create step content
    step_content <- list(
        timestamp = Sys.time(),
        params = list(
            score_type = score_type,
            scaled = scale,
            exposure_cols = exposure_cols,
            score_column_name = score_column_name
        ),
        notes = paste0(
            "Exposome score computed using method: '",
            score_type,
            "'"
        )
    )

    # Assign dynamic name to the step
    step_record <- setNames(
        list(step_content),
        paste0(
            "run_exposome_score_",
            score_column_name
        )
    )

    MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
        MultiAssayExperiment::metadata(exposomicset)$summary$steps,
        step_record
    )

    return(exposomicset)
}

#' Generate Example Data for Testing
#'
#' This helper function generates a reproducible dummy dataset
#' containing exposures, mRNA data, and proteomics data. It can
#' optionally return the data as a \code{MultiAssayExperiment} using
#' \code{\link{create_exposomicset}}.
#'
#' @param n_samples Integer. Number of samples to simulate (default: 12).
#' @param n_proteins Integer. Number of proteins to simulate (default: 80).
#' @param use_batch Logical. If \code{TRUE},
#' include a "batch" variable in the exposure data (default: FALSE).
#' @param return_mae Logical. If \code{TRUE},
#' return a \code{MultiAssayExperiment} created using
#' \code{create_exposomicset()} (default: FALSE).
#'
#' @return Either:
#' \itemize{
#'   \item A named list containing \code{codebook},
#'    \code{exposure}, \code{omics}, and \code{row_data},
#'    if \code{return_mae = FALSE}.
#'   \item A \code{MultiAssayExperiment}, if \code{return_mae = TRUE}.
#' }
#'
#' @examples
#' # Return as a list
#' dummy <- make_example_data()
#'
#' # Return as a MultiAssayExperiment
#' mae <- make_example_data(return_mae = TRUE)
#'
#' @export
make_example_data <- function(
  n_samples = 12,
  n_proteins = 80,
  use_batch = FALSE,
  return_mae = FALSE
) {
    sample_ids <- paste0("S", seq_len(n_samples))

    # Gene sets with some known function
    xeno_genes <- c(
        "AADAC", "BAK1", "SLCO1B1", "GATA3", "POR", "HTR2B",
        "CYP26A1", "SLC28A3", "EMX2", "KCNH2",
        "RORC", "RBM22", "CCND1", "TLR3", "LCN2", "CRKL",
        "NUDT15", "XPO1", "ROGDI", "FZD1",
        "BCL2", "NTRK1", "PTPRM", "MIR34B",
        "FUT1", "CDH1", "ABCD3", "FMO4", "SLCO1B3", "CYP2U1"
    )

    antigen_genes <- c(
        "ERAP1", "PSMB8", "IFI30", "CD1D", "IGHE", "MARCHF8",
        "THBS1", "FCER2", "FCGR2B", "LILRB2",
        "CTSH", "HLA-DRB5", "LGMN", "CD209",
        "HLA-DRB1", "HLA-E", "TAPBP", "CST7", "CD1E", "RAB10",
        "HLA-G", "MARCHF1", "HLA-DQB2", "HLA-DRA", "TRAF6",
        "PIKFYVE", "CCL21", "HLA-DPB1", "RAB35", "TAPBPL"
    )

    # Randomly sampled gene sets from msig
    other_genes <- c(
        "MYCT1", "NCDN", "MGAT4C", "ATF1", "REEP6", "NEURL4", "HOATZ",
        "RBM39", "HLA-DRB5", "EFNA1",
        "MRPS18A", "MTO1", "MIRLET7A1", "MCTS2", "ZMAT3", "CHRM1",
        "TRAV22", "PBLD", "NOTCH2", "RHBDL1",
        "KIF27", "CXCL10", "ASB8", "CDKN1C", "ZPLD1", "MTHFD2", "ANKIB1",
        "POLD2", "TOM1L2", "SYVN1",
        "PPP3CA", "PRAMEF27", "ARHGEF3", "TBC1D24", "EMG1", "CCL21",
        "TUSC3", "SNORD111", "CEBPA", "TMEM63C",
        "SLC7A8", "ZNF738", "NEU3", "FSCN2", "BUB1", "MYEF2", "GMIP",
        "BAIAP2L1", "SEPSECS", "EPB41L1",
        "RIMS3", "IFT70A", "GLIS2", "IGLV3-12", "CGRRF1", "MESP2",
        "TCEA2", "TIMP2", "HECTD2", "CRTAC1",
        "C1R", "BRMS1L", "IZUMO1", "FDX1", "RPL14", "SLC35B2", "KRT16",
        "DPH2", "OCSTAMP", "NME5"
    )

    gene_symbols <- unique(c(antigen_genes, xeno_genes, other_genes))
    n_genes <- length(gene_symbols)
    mrna_ids <- paste0("feat_", seq_len(n_genes))

    # Simulate exposures
    exposure <- data.frame(
        age = sample(30:70, n_samples, replace = TRUE),
        sex = sample(c("M", "F"), n_samples, replace = TRUE),
        bmi = round(rnorm(n_samples, 25, 3), 1),
        smoker = sample(c("yes", "no"), n_samples, replace = TRUE),
        alcohol = sample(c("none", "low", "moderate", "high"),
            n_samples,
            replace = TRUE
        ),
        exposure_pm25 = runif(n_samples, 5, 30),
        exposure_no2 = runif(n_samples, 10, 60),
        row.names = sample_ids
    )

    if (use_batch) {
        exposure$batch <- sample(c("A", "B"), n_samples, replace = TRUE)
    }

    # Codebook with categories
    codebook <- data.frame(
        variable = colnames(exposure),
        description = c(
            "Age of subject", "Sex at birth", "Body mass index",
            "Current smoking status", "Alcohol intake",
            "Air pollution: PM2.5", "Air pollution: NO2",
            if (use_batch) "Batch label" else NULL
        ),
        category = c(
            "Demographics", "Demographics", "Demographics",
            "Lifestyle", "Lifestyle",
            "Environment", "Environment",
            if (use_batch) "Technical" else NULL
        ),
        stringsAsFactors = FALSE
    )

    # Simulate latent structure
    lv1 <- scale(rnorm(n_samples)) # antigen signal
    lv2 <- scale(rnorm(n_samples)) # xeno signal

    # Simulate mRNA matrix - continuous, positive values
    mrna_mat <- matrix(
        rlnorm(n_genes * n_samples,
            meanlog = 7.5,
            sdlog = 1
        ),
        nrow = n_genes,
        dimnames = list(
            mrna_ids,
            sample_ids
        )
    )

    # Add latent structure and exposure signal
    mrna_mat[mrna_ids[gene_symbols %in% antigen_genes], ] <- mrna_mat[
        mrna_ids[gene_symbols %in% antigen_genes],
    ] +
        matrix(
            rep(lv1,
                each = sum(gene_symbols %in% antigen_genes)
            ),
            nrow = sum(gene_symbols %in% antigen_genes)
        )

    mrna_mat[mrna_ids[gene_symbols %in% xeno_genes], ] <- mrna_mat[
        mrna_ids[gene_symbols %in% xeno_genes],
    ] +
        matrix(
            rep(lv2,
                each = sum(gene_symbols %in% xeno_genes)
            ),
            nrow = sum(gene_symbols %in% xeno_genes)
        )

    mrna_mat[mrna_ids[gene_symbols %in% antigen_genes], ] <- mrna_mat[
        mrna_ids[gene_symbols %in% antigen_genes],
    ] +
        matrix(
            rep(scale(exposure$exposure_pm25),
                each = sum(gene_symbols %in% antigen_genes)
            ),
            nrow = sum(gene_symbols %in% antigen_genes)
        )

    # Create rowData for mRNA
    row_data_mrna <- S4Vectors::DataFrame(
        symbol = gene_symbols,
        row.names = mrna_ids
    )

    # Simulate proteomics with real gene names
    matched_genes <- sample(gene_symbols, n_proteins, replace = TRUE)
    prot_ids <- paste0("prot_", seq_len(n_proteins))

    prot_mat <- mrna_mat[mrna_ids[match(matched_genes, gene_symbols)], ] +
        matrix(
            rlnorm(n_proteins * n_samples,
                meanlog = 7.5,
                sdlog = 0.5
            ),
            nrow = n_proteins
        )

    rownames(prot_mat) <- prot_ids
    colnames(prot_mat) <- sample_ids

    # Create rowData for proteomics
    row_data_prot <- S4Vectors::DataFrame(
        symbol = matched_genes,
        row.names = prot_ids
    )

    # Combine everything
    row_data <- list(
        mRNA = row_data_mrna,
        proteomics = row_data_prot
    )

    omics <- list(
        mRNA = mrna_mat,
        proteomics = prot_mat
    )

    if (return_mae) {
        res_list <- list(
            codebook = codebook,
            exposure = exposure,
            omics = omics,
            row_data = row_data
        )
        res <- create_exposomicset(
            codebook = res_list$codebook,
            exposure = res_list$exposure,
            omics = res_list$omics,
            row_data = res_list$row_data
        )
    } else {
        res <- list(
            codebook = codebook,
            exposure = exposure,
            omics = omics,
            row_data = row_data
        )
    }

    return(res)
}

#' Assess Normality of Exposure Variables
#'
#' Performs Shapiro-Wilk tests to check the normality of numeric exposure
#' variables in `colData` of a `MultiAssayExperiment` object.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing exposure
#' data in `colData`.
#' @param action A character string specifying whether to store (`"add"`)
#' or return (`"get"`) the results. Default is `"add"`.
#'
#' @details
#' This function:
#' - Extracts **numeric, non-constant** exposure variables from `colData`.
#' - Runs **Shapiro-Wilk tests** to assess normality.
#' - Summarizes the number of normally and non-normally distributed exposures.
#' - Generates a bar plot visualizing the normality results.
#' - **Output Handling**:
#'   - `"add"`: Stores results in `metadata(exposomicset)$normality`.
#'   - `"get"`: Returns a list containing the normality test results and plot.
#'
#' @return A `MultiAssayExperiment` object with normality results added to
#' metadata (if `action = "add"`) or a list with:
#' \item{norm_df}{A data frame of Shapiro-Wilk test results for each
#' exposure variable.}
#' \item{norm_plot}{A ggplot object showing the distribution of normal
#'  and non-normal exposures.}
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Test for normality
#' mae <- mae |>
#'     run_normality_check() |>
#'     transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))
#'
#' @importFrom MultiAssayExperiment colData metadata
#' @importFrom dplyr select_if mutate summarise case_when
#' @importFrom broom tidy
#' @importFrom tibble rownames_to_column
#' @importFrom ggplot2 ggplot aes geom_bar geom_segment labs
#' @importFrom stats shapiro.test
#' @importFrom ggpubr theme_pubr
#' @export
run_normality_check <- function(exposomicset,
                                action = "add") {
    # require(ggplot2)

    message("Checking Normality Using Shapiro-Wilk Test")

    # Extract numeric exposure data from colData
    exposure_data <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame() |>
        dplyr::select_if(is.numeric) |>
        dplyr::select_if(function(x) !all(x == x[1]))

    if (ncol(exposure_data) == 0) {
        stop("No numeric or non-constant exposure variables found.")
    }

    # Perform Shapiro-Wilk test for normality
    norm_df <- exposure_data |>
        apply(2, function(x) {
            shapiro.test(x) |>
                broom::tidy()
        }) |>
        (\(x) do.call(rbind, x))() |>
        dplyr::mutate(exposure = colnames(exposure_data))

    # Summarize normality results
    norm_summary <- norm_df |>
        dplyr::summarise(
            "Normal" = sum(p.value > 0.05),
            "Not Normal" = sum(p.value <= 0.05)
        ) |>
        t() |>
        as.data.frame() |>
        tibble::rownames_to_column("var") |>
        setNames(c("var", "value"))

    # Create normality plot
    norm_plot <- table(norm_df$p.value > 0.05) |>
        as.data.frame() |>
        dplyr::mutate(Var1 = dplyr::case_when(
            Var1 == "FALSE" ~ "Not Normal",
            Var1 == "TRUE" ~ "Normal"
        )) |>
        ggplot(aes(
            x = Var1,
            y = Freq,
            fill = Var1
        )) +
        geom_bar(stat = "identity", alpha = 0.5) +
        geom_segment(
            aes(
                x = as.numeric(as.factor(Var1)) - 0.45,
                xend = as.numeric(as.factor(Var1)) + 0.45,
                y = Freq,
                yend = Freq,
                color = Var1
            ),
            linewidth = 1
        ) +
        ggpubr::theme_pubr(legend = "right") +
        # ggsci::scale_fill_lancet() +
        # ggsci::scale_color_lancet(guide = FALSE) +
        scale_fill_tidy_exp() +
        scale_color_tidy_exp(guide = "none") +
        labs(
            x = "",
            y = "No. of Exposures",
            fill = "",
            title = "Normality of Exposure Variables",
            subtitle = "Shapiro-Wilk Test"
        )

    # Print normality summary
    num_normal <- table(norm_df$p.value > 0.05)["TRUE"] |> as.numeric()
    num_not_normal <- table(norm_df$p.value < 0.05)["TRUE"] |> as.numeric()

    message(
        ifelse(is.na(num_normal), 0, num_normal),
        " Exposure Variables are Normally Distributed"
    )

    message(
        ifelse(is.na(num_not_normal), 0, num_not_normal),
        " Exposure Variables are NOT Normally Distributed"
    )

    if (action == "add") {
        # Add normality results to exposomicset metadata
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$quality_control$normality <- list(
            norm_df = norm_df,
            norm_summary = norm_summary
        )
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        # Add step record
        step_record <- list(
            run_normality_check = list(
                timestamp = Sys.time(),
                params = list(
                    method = "Shapiro-Wilk",
                    alpha = 0.05
                ),
                notes = paste0(
                    "Assessed normality of ",
                    nrow(norm_df),
                    " numeric exposure variables. ",
                    sum(norm_df$p.value > 0.05),
                    " were normally distributed (p > 0.05), ",
                    sum(norm_df$p.value <= 0.05),
                    " were not."
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else if (action == "get") {
        # Return normality results
        return(list(norm_df = norm_df, norm_plot = norm_plot))
    } else {
        stop("Use 'add' to add results to exposomicset or 'get' to return results.")
    }
}

#' Plot Exposure Impact on Network Centrality Metrics
#'
#' Visualizes the impact of exposures on network centrality measures of
#' associated features (e.g., genes or latent factors) as a heatmap.
#'  Each exposure is scored by four centrality metrics,
#' scaled within metric, and grouped by exposure category.
#'
#' @param exposomicset A `MultiAssayExperiment` object with results
#' from `run_exposure_impact()`.
#' @param feature_type Character string specifying the feature type.
#'  One of `"degs"`, `"omics"`, or `"factors"`.
#' @param min_per_group Minimum number of features per exposure for
#' inclusion (not currently used). Default is `5`.
#' @param facet_cols Optional named vector of colors for exposure categories.
#' @param bar_cols Optional vector of colors for bar plots (if enabled).
#' @param alpha Transparency level for category strips (if enabled).
#' Default is `0.3`.
#' @param ncol,nrow Layout for optional patchwork combination
#' (currently unused). Default: `ncol = 2`, `nrow = 1`.
#' @param heights Relative heights and widths for combined plots
#' (currently unused). Default: `c(1,1)`.
#' @param widths Relative widths for combined plots
#' (currently unused). Default: `c(2,1)`.
#'
#' @return A `ggplot`/`patchwork` object showing a heatmap of
#' scaled network centrality scores per exposure, annotated by category.
#'
#' @details
#' This function uses the output of `run_exposure_impact()` to
#' calculate and visualize the mean centrality
#' values for each exposure across its associated features.
#' The following network centrality metrics are shown:
#' \itemize{
#'   \item Degree centrality
#'   \item Eigenvector centrality
#'   \item Closeness centrality
#'   \item Betweenness centrality
#' }
#' All values are scaled within metric across exposures.
#' A side bar indicates the category of each exposure.
#'
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # perform correlation analyses
#' # correlate with exposures
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     ) |>
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         feature_cors = TRUE,
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # create the networks
#' mae <- mae |>
#'     run_create_network(
#'         feature_type = "omics_feature_cor",
#'         action = "add"
#'     ) |>
#'     run_create_network(
#'         feature_type = "omics",
#'         action = "add"
#'     )
#'
#' # perform impact analysis
#' mae <- mae |>
#'     run_exposure_impact(
#'         feature_type = "omics"
#'     )
#'
#' # create the exposure impact plot
#' exposure_impact_p <- mae |>
#'     plot_exposure_impact(
#'         feature_type = "omics"
#'     )
#'
#' @importFrom ggplot2 ggplot aes geom_tile scale_fill_manual
#' scale_fill_gradient2 labs theme_minimal theme element_text
#' element_blank margin
#' @importFrom dplyr select distinct pull group_by summarise mutate
#' arrange ungroup case_when
#' @importFrom ggpubr get_palette
#' @importFrom tidyr pivot_longer
#' @importFrom purrr pluck
#' @export
plot_exposure_impact <- function(
  exposomicset,
  feature_type = c("degs", "omics", "factors"),
  min_per_group = 5,
  facet_cols = NULL,
  bar_cols = NULL,
  alpha = 0.3,
  ncol = 2,
  nrow = 1,
  heights = c(1, 1),
  widths = c(2, 1)
) {
    # require(ggplot2)
    # require(patchwork)
    .check_suggested(pkg = "patchwork")

    # Check that exposure impact analysis has been run
    if (!("exposure_impact" %in% names(
        MultiAssayExperiment::metadata(exposomicset)$network
    ))) {
        stop("Please run `run_exposure_impact()` first.")
    }

    # Extract exposure impact degree df
    exposure_impact_degree <-
        exposomicset |>
        MultiAssayExperiment::metadata() |>
        purrr::pluck("network") |>
        purrr::pluck("exposure_impact") |>
        purrr::pluck(feature_type) |>
        purrr::pluck("exposure_impact")

    if (!is.null(facet_cols)) {
        facet_cols <- facet_cols
    } else {
        facet_cols <- tidy_exp_pal[
            seq_len(
                length(
                    unique(
                        exposure_impact_degree |>
                            dplyr::pull(category)
                    )
                )
            )
        ]
    }


    # set bar colors
    if (!is.null(bar_cols)) {
        bar_cols <- bar_cols
    } else {
        bar_cols <- ggpubr::get_palette(
            palette = "uchicago",
            k = length(
                unique(
                    exposure_impact_degree |>
                        dplyr::pull(category)
                )
            )
        )
    }

    # Scale centrality metrics
    centrality_heatmap_df <- exposure_impact_degree |>
        dplyr::select(
            exposure,
            category,
            mean_degree,
            mean_eigen,
            mean_closeness,
            mean_betweenness
        ) |>
        dplyr::distinct() |>
        tidyr::pivot_longer(
            cols = -c(exposure, category),
            names_to = "metric",
            values_to = "value"
        ) |>
        dplyr::group_by(metric) |>
        dplyr::mutate(scaled_value = scale(value)[, 1]) |>
        dplyr::ungroup()

    # Create exposure order based on centrality_heatmap_df
    exposure_order <- centrality_heatmap_df |>
        dplyr::group_by(exposure) |>
        dplyr::summarise(avg = mean(scaled_value, na.rm = TRUE)) |>
        dplyr::arrange(avg) |>
        dplyr::pull(exposure)

    # apply this order in both plots
    centrality_heatmap_df <- centrality_heatmap_df |>
        dplyr::mutate(exposure = factor(exposure, levels = exposure_order))

    # Create the bar plot on the side
    category_bar <- centrality_heatmap_df |>
        dplyr::select(exposure, category) |>
        dplyr::distinct() |>
        ggplot(aes(y = exposure, x = 1, fill = category)) +
        geom_tile() +
        scale_fill_manual(values = facet_cols, name = "Category") +
        theme_void() +
        theme(
            legend.position = "right",
            axis.text.y = element_text(size = 10)
        )

    # Create the main heatmap
    heatmap_plot <- centrality_heatmap_df |>
        mutate(metric = dplyr::case_when(
            metric == "mean_degree" ~ "Mean Degree",
            metric == "mean_eigen" ~ "Mean Eigenvector Centrality",
            metric == "mean_closeness" ~ "Mean Closeness Centrality",
            metric == "mean_betweenness" ~ "Mean Betweenness Centrality"
        )) |>
        ggplot(aes(
            x = metric,
            y = exposure,
            fill = scaled_value
        )) +
        geom_tile(color = "white") +
        scale_fill_gradient2(
            low = "blue4",
            mid = "white",
            high = "red4",
            midpoint = 0,
            name = "Scaled Value"
        ) +
        labs(x = "", y = "", fill = "Scaled\nValue") +
        theme_minimal() +
        theme(
            axis.text.x = element_text(angle = 45, hjust = 1),
            panel.grid = element_blank(),
            axis.text.y = element_blank(),
            axis.ticks = element_blank(),
            plot.margin = margin(2, 0, 2, 2)
        )
    heatmap_plot <- category_bar + heatmap_plot +
        patchwork::plot_layout(ncol = 2, widths = c(0.15, 1), guides = "collect")


    return(heatmap_plot)
}

## ----setup,include=FALSE------------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----ontology-app,eval=FALSE--------------------------------------------------
# # Launch the shiny app to annotate exposure variables
# shiny::runApp(build_ont_annot_app())

## ----ontology-app-img, echo=FALSE,out.width="150%",fig.align='center',fig.cap="Screenshot of the ontology annotation app. The sidebar has an upload button to load your exposure metadata file. The main panel displays the uploaded exposure metadata where users can select variables to annotate and categorize. The sidebar also contains buttons to apply annotations/categorizations and download the annotated file."----
knitr::include_graphics("./ont_annot.png")

## ----session_info-------------------------------------------------------------

sessionInfo()

#' Plot Enrichment Results from exposomicset
#'
#' Visualize enrichment results stored in a `MultiAssayExperiment` object.
#' Supports dotplots, heatmaps, cnetplots, networks,
#' and multi-panel summary plots.
#'
#' @param exposomicset A `MultiAssayExperiment` object with enrichment results
#' added via `run_enrichment()`.
#' @param feature_type Character; one of `"degs"`, `"degs_robust"`,
#' `"omics"`, `"factor_features"`,
#'   `"degs_cor"`, `"omics_cor"`, or `"factor_features_cor"`.
#'   Defines which enrichment results to use.
#' @param plot_type Type of plot to generate. One of `"dotplot"`,
#'  `"cnet"`, `"network"`, `"heatmap"`, or `"summary"`.
#'
#' @param top_n Integer; number of top `go_group`s to include
#' (used in `"dotplot"`). Default is `5`.
#' @param n_per_group Integer; number of terms per group to plot
#' (used in `"dotplot"`). Default is `5`.
#' @param add_top_genes Logical; if `TRUE`, appends top shared genes to
#'  dotplot facets. Default is `TRUE`.
#' @param top_n_genes Integer; number of top genes to show in each group
#' (used in `"dotplot"`). Default is `5`.
#'
#' @param heatmap_fill Logical; whether to fill tiles by logFC in the heatmap.
#' Default is `TRUE`.
#' @param logfc_thresh Numeric; log2 fold change threshold for filtering
#' (heatmap only). Default is `log2(1)`.
#' @param pval_col Column name of the p-value used for filtering in `"degs"`
#'  heatmap. Default is `"P.Value"`.
#' @param pval_thresh Threshold for `pval_col` (heatmap only).
#' Default is `0.05`.
#' @param score_metric Column for stability score
#' (used in `"degs_robust"` heatmap). Default is `"stability_score"`.
#' @param score_thresh Numeric; threshold for `score_metric`
#' (heatmap only). Default is `NULL`.
#'
#' @param overlap_thresh Numeric; Jaccard threshold for edges in
#' the network plot. Default is `0.2`.
#' @param node_radius Numeric; node size in network plot. Default is `0.2`.
#' @param pie_colors Optional named vector of colors for pie charts
#'  (network and cnet).
#' @param label_top_n Integer; number of top nodes to label in network.
#'  Default is `NULL`.
#' @param label_colour Color of node labels in network. Default is `"black"`.
#' @param net_facet_by Column used to facet the network plot
#' (e.g., `"category"`). Default is `NULL`.
#'
#' @param max_terms Integer; max number of terms to include in the cnet plot.
#'  Default is `30`.
#' @param node_size Numeric; base node size for cnet plot. Default is `1`.
#' @param term_node_correction Scaling factor for term nodes in cnet plot.
#' Default is `0.2`.
#' @param gene_node_correction Scaling factor for gene nodes in cnet plot.
#' Default is `3`.
#'
#' @param go_groups Optional character vector of GO group names to subset
#' enrichment results (all plots).
#' @param layout_algo Graph layout algorithm to use in `"network"` and `"cnet"`
#'  plots. Default is `"fr"`.
#' @param edge_alpha Transparency of network/cnet plot edges.
#'  Default is `0.3`.
#' @param label_size Font size for labels in network and cnet plots.
#'  Default is `3`.
#' @param feature_col Column name used to join gene-level metadata.
#' Default is `"feature"`.
#' @param logfc_col Column name used for log2 fold change values.
#' Default is `"logFC"`.
#'
#' @return A `ggplot` or `patchwork` object corresponding to the
#' requested plot type.
#'
#' @details
#' This function visualizes results from `run_enrichment()`
#'  using one of several plot types:
#' \itemize{
#'   \item `"dotplot"`: Enrichment terms grouped by GO group, colored by
#'   significance.
#'   \item `"heatmap"`: Term - gene matrix with optional logFC fill and
#'   shared gene highlighting.
#'   \item `"network"`: Graph of term overlap based on shared genes,
#'   faceted by metadata if desired.
#'   \item `"cnet"`: Gene - term bipartite graph with gene logFC values
#'   and term pie slices.
#'   \item `"summary"`: Multi-panel figure with GO group ridgeplots,
#'   gene counts, and Venn diagram.
#' }
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 30,
#'     return_mae = TRUE
#' )
#'
#' # perform differential abundance analysis
#' mae <- run_differential_abundance(
#'     exposomicset = mae,
#'     formula = ~ smoker + sex,
#'     abundance_col = "counts",
#'     method = "limma_voom",
#'     action = "add"
#' )
#'
#' # perform enrichment analysis
#' mae <- run_enrichment(
#'     exposomicset = mae,
#'     feature_type = "degs",
#'     feature_col = "symbol",
#'     species = "goa_human",
#'     deg_logfc_threshold = log2(1),
#'     deg_pval_col = "P.Value",
#'     deg_pval_threshold = 0.2,
#'     action = "add"
#' )
#'
#' # create an enrichment plot
#' enr_plot <- plot_enrichment(
#'     exposomicset = mae,
#'     feature_type = "degs",
#'     plot_type = "network"
#' )
#'
#' @export
plot_enrichment <- function(
  exposomicset,
  feature_type = c(
      "degs",
      "degs_robust",
      "omics",
      "factor_features",
      "degs_cor",
      "omics_cor",
      "factor_features_cor"
  ),
  plot_type = c(
      "dotplot",
      "cnet",
      "network",
      "heatmap",
      "summary"
  ),
  # Dotplot arguments
  top_n = 5,
  n_per_group = 5,
  add_top_genes = TRUE,
  top_n_genes = 5,
  # Heatmap arguments
  heatmap_fill = TRUE,
  logfc_thresh = log2(1),
  pval_col = "P.Value",
  pval_thresh = 0.05,
  score_metric = "stability_score",
  score_thresh = NULL,
  # Network arguments
  overlap_thresh = 0.2,
  node_radius = 0.2,
  pie_colors = NULL,
  label_top_n = NULL,
  label_colour = "black",
  net_facet_by = NULL,
  # Cnet arguments
  max_terms = 30,
  node_size = 1,
  term_node_correction = 0.2,
  gene_node_correction = 3,
  # Shared arguments
  go_groups = NULL,
  layout_algo = "fr",
  edge_alpha = 0.3,
  label_size = 3,
  feature_col = "feature",
  logfc_col = "logFC"
) {
    # Check that the user ran the enrichment analysis
    if (!"enrichment" %in% names(MultiAssayExperiment::metadata(exposomicset))) {
        stop("Please run `run_enrichment` first.")
    }

    # match arguments
    feature_type <- match.arg(feature_type)
    plot_type <- match.arg(plot_type)

    enr_res <- exposomicset@metadata |>
        purrr::pluck("enrichment") |>
        purrr::pluck(feature_type)

    plot <- switch(plot_type,
        "dotplot" = {
            .plot_dotplot_enrichment(
                enr_res,
                top_n = top_n,
                n_per_group = n_per_group,
                add_top_genes = add_top_genes,
                top_n_genes = top_n_genes,
                go_groups = go_groups
            )
        },
        "cnet" = {
            .plot_cnet_enrichment(
                enr_res,
                exposomicset = exposomicset,
                feature_type = feature_type,
                layout_algo = layout_algo,
                go_groups = go_groups,
                max_terms = max_terms,
                edge_alpha = edge_alpha,
                node_size = node_size,
                label_size = label_size,
                term_node_correction = term_node_correction,
                gene_node_correction = gene_node_correction,
                feature_col = feature_col,
                logfc_col = logfc_col
            )
        },
        "heatmap" = {
            .plot_heatmap_enrichment(
                enr_res = enr_res,
                exposomicset = exposomicset,
                feature_type = feature_type,
                score_metric = score_metric,
                score_thresh = score_thresh,
                go_groups = go_groups,
                heatmap_fill = heatmap_fill,
                feature_col = feature_col,
                logfc_col = logfc_col,
                logfc_thresh = logfc_thresh,
                pval_col = pval_col,
                pval_thresh = pval_thresh
            )
        },
        "network" = {
            .plot_network_enrichment(
                enr_res = enr_res,
                feature_type = feature_type,
                go_groups = go_groups,
                overlap_thresh = overlap_thresh,
                layout_algo = layout_algo,
                node_radius = node_radius,
                edge_alpha = edge_alpha,
                pie_colors = pie_colors,
                label_top_n = label_top_n,
                label_size = label_size,
                label_colour = label_colour,
                net_facet_by = net_facet_by
            )
        },
        "summary" = {
            .plot_summary_enrichment(
                enr_res
            )
        }
    )

    return(plot)
}

# --- Enrichment DotPlot --------
#' @noRd
#'
#' @title Internal: Dotplot of Enrichment Terms by GO Group
#'
#' @description
#' Generates a dotplot of enriched terms colored by -log10(p) and
#' sized by the number of genes, optionally grouped by `go_group` and
#'  annotated with top genes.
#'
#' @param enr_res A data frame of enrichment results.
#' @param top_n Number of top GO groups to plot based on a scoring heuristic.
#' @param n_per_group Number of terms per group to show.
#' @param add_top_genes Logical; whether to append top genes to GO group labels.
#' @param top_n_genes Number of top genes to show in label.
#' @param go_groups Optional subset of GO groups to display.
#'
#' @return A `ggplot` dotplot object.
#'
#' @importFrom ggplot2 ggplot aes geom_point labs scale_color_gradient
#' facet_grid theme element_text
#' @importFrom dplyr filter group_by mutate reframe arrange slice_head
#' ungroup inner_join pull bind_cols
#' @importFrom tibble rownames_to_column
#' @importFrom purrr map
#' @importFrom ggpubr theme_pubr rotate_x_text
#'
#' @keywords internal
.plot_dotplot_enrichment <- function(
  enr_res,
  top_n,
  n_per_group,
  add_top_genes,
  top_n_genes,
  go_groups
) {
    # require(ggplot2)
    .check_suggested(pkg = "forcats")

    if (nrow(enr_res) < 1) {
        return(NULL)
    }

    if (!is.null(go_groups)) {
        enr_res <- enr_res |>
            dplyr::filter(go_group %in% go_groups)
    } else {
        enr_res <- enr_res |>
            dplyr::filter(go_group %in% c(
                enr_res |>
                    dplyr::group_by(go_group) |>
                    dplyr::mutate(score_tmp = (-log10(p_adjust) * n_with_sel)) |>
                    dplyr::reframe(score = mean(score_tmp)) |>
                    dplyr::arrange(desc(score)) |>
                    dplyr::slice_head(n = top_n) |>
                    dplyr::pull(go_group)
            ))
    }

    go_group_genes_df <- enr_res |>
        (\(df) split(df, df$go_group))() |>
        purrr::map(~ .x |>
            dplyr::pull(ids) |>
            (\(x) strsplit(x, ","))() |>
            unlist() |>
            table() |>
            sort() |>
            tail(n = top_n_genes) |>
            names() |>
            (\(genes) {
                # Split into groups of 5 and add line breaks
                gene_chunks <- split(genes, ceiling(seq_along(genes) / 5))
                paste(
                    vapply(
                        gene_chunks,
                        function(chunk) paste(chunk, collapse = ", "),
                        FUN.VALUE = character(1)
                    ),
                    collapse = "\n"
                )
            })()) |>
        bind_cols() |>
        t() |>
        as.data.frame() |>
        setNames("gene_col") |>
        tibble::rownames_to_column("go_group")

    if (add_top_genes) {
        enr_res <- enr_res |>
            dplyr::inner_join(
                go_group_genes_df,
                by = "go_group"
            ) |>
            dplyr::mutate(go_group = gsub("_", " ", go_group))
    } else {
        enr_res <- enr_res |>
            dplyr::mutate(go_group = gsub("_", " ", go_group))
    }

    if ("category" %in% colnames(enr_res)) {
        enr_res |>
            dplyr::mutate(go_group = paste(go_group, gene_col, sep = "\n")) |>
            dplyr::group_by(category, exp_name, go_group) |>
            dplyr::arrange(p_adjust) |>
            dplyr::slice_head(n = n_per_group) |>
            dplyr::ungroup() |>
            ggplot(aes(
                x = forcats::fct_reorder(exp_name, -n_with_sel),
                y = forcats::fct_reorder(term_name, -log10(p_adjust)),
                size = n_with_sel,
                color = -log10(p_adjust)
            )) +
            geom_point() +
            ggpubr::theme_pubr(
                legend = "right",
                base_size = 10
            ) +
            facet_grid(go_group ~ category,
                space = "free",
                scales = "free"
            ) +
            ggpubr::rotate_x_text(angle = 45) +
            scale_color_gradient(low = "thistle1", high = "#8E0152") +
            theme(
                strip.text.x = element_text(
                    face = "bold.italic",
                    angle = 90
                ),
                strip.text.y = element_text(
                    face = "bold.italic",
                    angle = 0
                )
            ) +
            labs(
                x = "",
                y = "",
                title = "",
                size = "Number of Genes",
                color = expression(-Log[10](P)),
            )
    } else {
        enr_res |>
            dplyr::mutate(go_group = paste(go_group, gene_col, sep = "\n")) |>
            dplyr::group_by(exp_name, go_group) |>
            dplyr::arrange(p_adjust) |>
            dplyr::slice_head(n = n_per_group) |>
            dplyr::ungroup() |>
            ggplot(aes(
                x = forcats::fct_reorder(exp_name, -n_with_sel),
                y = forcats::fct_reorder(term_name, -log10(p_adjust)),
                size = n_with_sel,
                color = -log10(p_adjust)
            )) +
            geom_point() +
            ggpubr::theme_pubr(
                legend = "right",
                base_size = 10
            ) +
            facet_grid(go_group ~ ., space = "free", scales = "free") +
            ggpubr::rotate_x_text(angle = 45) +
            scale_color_gradient(low = "thistle1", high = "#8E0152") +
            theme(
                strip.text.x = element_text(
                    face = "bold.italic",
                    angle = 90
                ),
                strip.text.y = element_text(
                    face = "bold.italic",
                    angle = 0
                )
            ) +
            labs(
                x = "",
                y = "",
                title = "",
                size = "Number of Genes",
                color = expression(-Log[10](P)),
            )
    }
}

# --- Enrichment Heatmap -----
#' @noRd
#'
#' @title Internal: Heatmap of Enrichment Term by Gene LogFC Matrix
#'
#' @description
#' Generates a tile-based heatmap of enrichment terms by associated genes,
#'  optionally colored by log2 fold change.
#'
#' @param enr_res Enrichment results with exploded gene-term relationships.
#' @param exposomicset The original `MultiAssayExperiment` object.
#' @param feature_type Feature source (e.g., "degs", "degs_robust").
#' @param score_metric Column name for stability filtering (for `degs_robust`).
#' @param score_thresh Numeric threshold for stability score filtering.
#' @param go_groups GO groups to include in the heatmap.
#' @param heatmap_fill Logical; whether to fill tiles by logFC.
#' @param feature_col Column for matching features.
#' @param logfc_col Column for log2 fold change values.
#' @param logfc_thresh Threshold for absolute logFC.
#' @param pval_col Column for p-values.
#' @param pval_thresh Significance threshold.
#'
#' @return A `ggplot` heatmap object.
#'
#' @importFrom ggplot2 ggplot aes theme_bw facet_grid theme
#' element_text labs geom_tile scale_fill_gradient2
#' @importFrom dplyr filter distinct count mutate left_join inner_join select
#' @importFrom purrr pluck
#' @importFrom tidyr separate_rows
#'
#' @keywords internal
.plot_heatmap_enrichment <- function(
  enr_res = enr_res,
  exposomicset = exposomicset,
  feature_type = feature_type,
  score_metric = score_metric,
  score_thresh = score_thresh,
  go_groups = go_groups,
  heatmap_fill = heatmap_fill,
  feature_col = feature_col,
  logfc_col = logfc_col,
  logfc_thresh = logfc_thresh,
  pval_col = pval_col,
  pval_thresh = pval_thresh
) {
    .check_suggested("forcats")
    if (nrow(enr_res) < 1) {
        return(NULL)
    }

    enr_res <- enr_res |>
        tidyr::separate_rows("ids", sep = ", ")

    if (is.null(go_groups)) {
        stop("Please specify a `go_group`")
    } else {
        enr_res <- enr_res |>
            dplyr::filter(go_group %in% go_groups)
    }

    shared_genes <- enr_res |>
        tidyr::separate_rows(ids, sep = ", ") |>
        dplyr::distinct(exp_name, ids) |>
        dplyr::count(ids, name = "n_exp") |>
        dplyr::mutate(shared = n_exp > 1)

    if (heatmap_fill) {
        if (feature_type == "degs_robust") {
            if (is.null(score_thresh)) {
                score_thresh <- exposomicset@metadata |>
                    purrr::pluck("differential_analysis") |>
                    purrr::pluck("sensitivity_analysis") |>
                    purrr::pluck("score_thresh")
            } else {
                score_thresh <- score_thresh
            }

            # grab feature information from sensitivity analysis
            f_stable_df <- exposomicset@metadata |>
                purrr::pluck("differential_analysis") |>
                purrr::pluck("sensitivity_analysis") |>
                purrr::pluck("feature_stability") |>
                dplyr::filter(!!dplyr::sym(score_metric) > score_thresh) |>
                dplyr::select(c(feature, exp_name))

            enr_res <- enr_res |>
                inner_join(
                    exposomicset@metadata |>
                        purrr::pluck("differential_analysis") |>
                        purrr::pluck("differential_abundance") |>
                        dplyr::select(c(
                            exp_name,
                            feature,
                            feature_col,
                            logfc_col
                        )),
                    by = c(
                        "exp_name" = "exp_name",
                        "ids" = feature_col
                    ),
                    relationship = "many-to-many"
                ) |>
                inner_join(
                    f_stable_df,
                    by = c(
                        "exp_name" = "exp_name",
                        "feature" = "feature"
                    )
                )
        } else if (feature_type == "degs") {
            enr_res <- enr_res |>
                inner_join(
                    exposomicset@metadata |>
                        purrr::pluck("differential_analysis") |>
                        purrr::pluck("differential_abundance") |>
                        dplyr::filter(
                            !!dplyr::sym(pval_col) < pval_thresh,
                            abs(!!dplyr::sym(logfc_col)) > logfc_thresh
                        ) |>
                        dplyr::select(c(
                            exp_name,
                            feature,
                            feature_col,
                            logfc_col
                        )),
                    by = c(
                        "exp_name" = "exp_name",
                        "ids" = feature_col
                    ),
                    relationship = "many-to-many"
                )
        } else {
            enr_res <- enr_res |>
                inner_join(
                    exposomicset@metadata |>
                        purrr::pluck("differential_analysis") |>
                        purrr::pluck("differential_abundance") |>
                        dplyr::select(c(
                            exp_name,
                            feature,
                            feature_col,
                            logfc_col
                        )),
                    by = c(
                        "exp_name" = "exp_name",
                        "ids" = feature_col
                    ),
                    relationship = "many-to-many"
                )
        }
    }

    # Join shared status
    enr_res <- enr_res |>
        dplyr::left_join(shared_genes, by = "ids") |>
        dplyr::mutate(
            ids = ifelse(shared, paste0("*", ids), ids)
        )

    go_groups_clean <- gsub("_", " ", go_groups)

    if ("category" %in% colnames(enr_res)) {
        p <- enr_res |>
            ggplot(aes(
                x = forcats::fct_reorder2(ids, term_name, category),
                y = forcats::fct_reorder2(term_name, exp_name, category)
            )) +
            theme_bw() +
            facet_grid(category ~ exp_name,
                scales = "free",
                space = "free"
            ) +
            theme(
                axis.text.x = element_text(angle = 90, face = "italic"),
                axis.ticks.x = element_blank(),
                strip.text.x = element_text(face = "bold.italic"),
                strip.text.y = element_text(face = "bold.italic"),
                plot.caption = element_text(face = "italic"),
                plot.title = element_text(face = "bold.italic")
            ) +
            labs(
                caption = "* Appears in more than one omic",
                x = "",
                y = "",
                title = paste(
                    "Enrichment Term Heatmap for ",
                    paste(go_groups_clean, collapse = ",")
                )
            )
    } else {
        p <- enr_res |>
            ggplot(aes(
                x = forcats::fct_reorder(ids, term_name),
                y = forcats::fct_reorder(term_name, exp_name)
            )) +
            theme_bw() +
            facet_grid(~exp_name,
                scales = "free",
                space = "free"
            ) +
            theme(
                axis.text.x = element_text(angle = 90, face = "italic"),
                axis.ticks.x = element_blank(),
                strip.text.x = element_text(face = "bold.italic"),
                plot.caption = element_text(face = "italic"),
                plot.title = element_text(face = "bold.italic")
            ) +
            labs(
                caption = "* Appears in more than one omic",
                x = "",
                y = "",
                title = paste(
                    "Enrichment Term Heatmap for ",
                    paste(go_groups_clean, collapse = ",")
                )
            )
    }

    if (heatmap_fill) {
        p <- p +
            geom_tile(aes(fill = !!sym(logfc_col))) +
            scale_fill_gradient2(
                name = expression("Log"[2] * "FC"),
                low = "blue4",
                mid = "white",
                high = "red4",
                midpoint = 0
            )
    } else {
        p <- p +
            geom_tile(fill = "grey30")
    }
}

# --- Enrichment Network Plot -------
#' @noRd
#'
#' @title Internal: Network Plot of Enrichment Term Overlaps
#'
#' @description
#' Builds a graph from overlapping genes between terms and renders a
#'  circular or force-directed layout colored by experiment.
#'
#' @param enr_res Enrichment results.
#' @param feature_type Type of feature used.
#' @param go_groups Optional GO groups to subset.
#' @param overlap_thresh Minimum Jaccard overlap to draw edges.
#' @param layout_algo Graph layout algorithm (e.g., "fr").
#' @param node_radius Radius of node pie arcs.
#' @param edge_alpha Edge transparency.
#' @param pie_colors Optional named vector of colors for experiments.
#' @param label_top_n Number of top nodes to label by degree centrality.
#' @param label_size Font size of node labels.
#' @param label_colour Color of node labels.
#' @param net_facet_by Optional column to facet the network by.
#'
#' @return A `ggraph` object.
#'
#' @importFrom igraph graph_from_data_frame cluster_louvain
#' @importFrom ggplot2 facet_wrap theme element_text labs scale_fill_manual
#'  theme_void
#' @importFrom ggrepel geom_label_repel
#' @importFrom dplyr filter mutate select distinct group_by ungroup
#' left_join top_n pull
#' @importFrom purrr map
#' @importFrom tidyr separate_rows
#' @importFrom stringr str_trim
#' @importFrom tibble as_tibble
#'
#' @keywords internal
.plot_network_enrichment <- function(
  enr_res = enr_res,
  feature_type = feature_type,
  go_groups = go_groups,
  overlap_thresh = overlap_thresh,
  layout_algo = layout_algo,
  node_radius = node_radius,
  edge_alpha = edge_alpha,
  pie_colors = pie_colors,
  label_top_n = label_top_n,
  label_size = label_size,
  label_colour = label_colour,
  net_facet_by = net_facet_by
) {
    # require(igraph)
    # require(tidygraph)
    # require(ggraph)
    .check_suggested(pkg = "ggraph")
    .check_suggested(pkg = "tidygraph")

    if (nrow(enr_res) < 1) {
        return(NULL)
    }

    # Get enrichment table
    enr <- enr_res

    if (!is.null(go_groups)) {
        enr <- enr |>
            dplyr::filter(go_group %in% go_groups)
    }

    # Build pairwise overlaps by term_name
    term_overlaps <- split(enr, enr$term_name) |>
        purrr::map(~ .x |>
            tidyr::separate_rows("ids", sep = ",") |>
            dplyr::pull(ids) |>
            stringr::str_trim() |>
            unique()) |>
        .get_pairwise_overlaps()

    # Build deduplicated edge list
    edge_df <- term_overlaps |>
        dplyr::mutate(
            source_std = pmin(source, target),
            target_std = pmax(source, target)
        ) |>
        dplyr::select(
            source = source_std,
            target = target_std,
            overlap
        ) |>
        dplyr::filter(overlap > overlap_thresh) |>
        dplyr::distinct()

    # Ensure unique node list by term_name
    node_tbl <- enr |>
        dplyr::select(term_name) |>
        dplyr::distinct() |>
        dplyr::mutate(name = term_name) # igraph expects "name"

    # Only keep nodes involved in edges
    keep_nodes <- unique(c(edge_df$source, edge_df$target))
    node_tbl <- node_tbl |> dplyr::filter(name %in% keep_nodes)

    # Build igraph object
    g <- igraph::graph_from_data_frame(
        edge_df,
        directed = FALSE,
        vertices = node_tbl
    ) |>
        tidygraph::as_tbl_graph()

    # Layout
    layout <- ggraph::create_layout(g, layout = layout_algo)

    # Compute pie slice positions for omics types
    node_omics <- enr |>
        dplyr::select(term_name, exp_name) |>
        dplyr::distinct() |>
        dplyr::group_by(term_name) |>
        dplyr::mutate(
            n = dplyr::n(),
            angle = 2 * pi / n,
            start = (row_number() - 1) * angle,
            end = row_number() * angle
        ) |>
        dplyr::ungroup()

    # Join layout with slice data
    arc_data <- layout |>
        as_tibble() |>
        dplyr::left_join(node_omics, by = c("name" = "term_name"))

    # Stop if join failed
    if (!"exp_name" %in% names(arc_data)) {
        stop("Join failed: 'exp_name' not found in arc_data.")
    }

    # Default pie colors
    if (is.null(pie_colors)) {
        omics_types <- unique(arc_data$exp_name)
        n_colors <- max(3, length(omics_types))
        col_options <- c(
            "#66C2A5", "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854",
            "#FFD92F", "#E5C494", "#B3B3B3"
        )
        # Repeat colors if necessary
        pie_colors <- setNames(
            rep(col_options, length.out = length(omics_types)),
            omics_types
        )
    }

    # Final plot
    p <- ggraph::ggraph(layout) +
        ggraph::geom_edge_link(aes(width = overlap),
            alpha = edge_alpha,
            color = "gray50"
        ) +
        ggraph::scale_edge_width(range = c(0.2, 2), guide = "none") +
        ggraph::geom_node_arc_bar(
            data = arc_data,
            aes(
                x0 = x, y0 = y, r0 = 0, r = node_radius,
                start = start, end = end,
                fill = exp_name
            ),
            color = "transparent"
        ) +
        scale_fill_manual(values = tidy_exp_pal) +
        theme_void() +
        labs(fill = "Experiment")

    if (!is.null(net_facet_by)) {
        # Add category info to node positions (layout)
        category_map <- enr |>
            dplyr::select(term_name, !!rlang::sym(net_facet_by)) |>
            dplyr::distinct()

        layout <- layout |>
            dplyr::left_join(category_map, by = c("name" = "term_name"))

        arc_data <- arc_data |>
            dplyr::left_join(category_map, by = c("name" = "term_name"))

        if (!net_facet_by %in% names(layout)) {
            stop(sprintf("Could not join `net_facet_by` by %s.", net_facet_by))
        }

        p <- p +
            facet_wrap(as.formula(paste("~", net_facet_by))) +
            theme(strip.text.x = element_text(
                face = "bold.italic",
                size = 12
            ))
    }

    if (!is.null(label_top_n)) {
        community <- igraph::cluster_louvain(g)

        # Compute layout + node metrics
        node_metrics <- tidygraph::activate(g, nodes) |>
            dplyr::mutate(
                degree = tidygraph::centrality_degree(),
                cluster = community$membership
            ) |>
            dplyr::as_tibble()

        #
        top_nodes <- node_metrics |>
            dplyr::group_by(cluster) |>
            dplyr::top_n(label_top_n, degree) |>
            dplyr::ungroup() |>
            dplyr::pull(name)

        layout_labels <- layout |>
            dplyr::filter(name %in% top_nodes)

        p <- p +
            ggrepel::geom_label_repel(
                data = layout_labels,
                aes(x = x, y = y, label = name),
                size = label_size,
                color = label_colour,
                force = 2, # stronger repel
                max.overlaps = Inf,
                segment.color = "grey30",
                segment.size = 0.2,
                box.padding = 0.3,
                point.padding = 0.2
            )
    }

    p
}

# --- Enrichment Cnetplot ------
#' @noRd
#'
#' @title Internal: Gene - Term Network (Cnetplot)
#'
#' @description
#' Builds a bipartite graph between genes and enriched terms,
#' with logFC values for genes and pie slices for multi-omic terms.
#'
#' @param enr_res Enrichment results.
#' @param exposomicset A `MultiAssayExperiment` object.
#' @param feature_type Source of features ("degs", "omics", etc.).
#' @param layout_algo Graph layout algorithm. Default is `"fr"`.
#' @param go_groups Optional subset of GO groups.
#' @param max_terms Maximum number of terms to include.
#' @param edge_alpha Transparency of edges.
#' @param node_size Base size for graph nodes.
#' @param label_size Size of text labels.
#' @param term_node_correction Scaling factor for term nodes.
#' @param gene_node_correction Scaling factor for gene nodes.
#' @param feature_col Feature column used for matching.
#' @param logfc_col Column name with log fold change values.
#'
#' @return A `ggraph` plot object.
#'
#' @importFrom igraph graph_from_data_frame
#' @importFrom ggplot2 scale_color_gradient2 theme_void labs element_text
#' @importFrom dplyr filter mutate select distinct left_join
#' count slice_head pull bind_rows group_by ungroup add_count if_else
#' @importFrom tidyr separate_rows
#' @importFrom tibble as_tibble
#' @importFrom purrr pluck
#'
#' @keywords internal
.plot_cnet_enrichment <- function(
  enr_res,
  exposomicset,
  feature_type,
  layout_algo = "fr",
  go_groups = NULL,
  max_terms = 30,
  edge_alpha = 0.5,
  node_size = 1,
  label_size = 3,
  term_node_correction = 0.2,
  gene_node_correction = 3,
  feature_col = "feature_clean",
  logfc_col = "logFC"
) {
    if (nrow(enr_res) < 1) {
        return(NULL)
    }
    .check_suggested(pkg = "ggnewscale")
    .check_suggested(pkg = "ggraph")
    .check_suggested(pkg = "tidygraph")

    # Explode to gene by term by exp rows
    long_enr <- enr_res |>
        tidyr::separate_rows(ids,
            sep = ",\\s*"
        ) |>
        dplyr::select(
            term_name,
            go_group,
            ids,
            exp_name
        ) |>
        dplyr::distinct()

    # Filter GO groups or select top terms
    if (!is.null(go_groups)) {
        long_enr <- long_enr |>
            dplyr::filter(go_group %in% go_groups)
    } else {
        top_terms <- long_enr |>
            dplyr::count(term_name, sort = TRUE) |>
            dplyr::slice_head(n = max_terms) |>
            dplyr::pull(term_name)
        long_enr <- long_enr |>
            dplyr::filter(term_name %in% top_terms)
    }

    # Join logFC from DA results
    da_df <- exposomicset@metadata |>
        purrr::pluck(
            "differential_analysis",
            "differential_abundance"
        ) |>
        dplyr::select(
            exp_name,
            feature,
            !!sym(feature_col),
            !!sym(logfc_col)
        ) |>
        dplyr::distinct()

    long_enr <- long_enr |>
        dplyr::left_join(
            da_df,
            by = c("exp_name",
                "ids" = feature_col
            ),
            relationship = "many-to-many"
        )

    # Assign unique node names
    long_enr <- long_enr |>
        dplyr::mutate(
            gene_node = paste0("gene:", ids, "::", exp_name),
            term_node = paste0("term:", term_name)
        )

    # Build edges
    edges <- long_enr |>
        dplyr::select(from = gene_node, to = term_node)

    # Node metadata
    gene_meta <- long_enr |>
        dplyr::select(
            name = gene_node,
            !!sym(logfc_col)
        ) |>
        dplyr::distinct() |>
        dplyr::mutate(type = "gene")

    term_meta <- long_enr |>
        dplyr::select(
            name = term_node,
            exp_name
        ) |>
        dplyr::distinct() |>
        dplyr::mutate(type = "term")

    nodes <- dplyr::bind_rows(
        term_meta,
        gene_meta
    ) |>
        dplyr::distinct(name, .keep_all = TRUE)

    # Graph layout
    g <- igraph::graph_from_data_frame(
        edges,
        vertices = nodes,
        directed = FALSE
    )

    layout <- ggraph::create_layout(g,
        layout = layout_algo
    )

    # Pie slices for term nodes
    term_arc_data <- long_enr |>
        dplyr::mutate(name = paste0("term:", term_name)) |>
        dplyr::select(name, exp_name) |>
        dplyr::distinct() |>
        dplyr::group_by(name) |>
        dplyr::mutate(
            n = n(),
            angle = 2 * pi / n,
            start = (dplyr::row_number() - 1) * angle,
            end = dplyr::row_number() * angle
        ) |>
        dplyr::ungroup() |>
        dplyr::left_join(
            layout |>
                tibble::as_tibble() |>
                dplyr::select(name, x, y),
            by = "name"
        )

    # Gene labels with conditional exp_name
    gene_labels <- long_enr |>
        dplyr::distinct(ids, exp_name) |>
        dplyr::add_count(ids, name = "n_exps") |>
        dplyr::mutate(
            label = if_else(n_exps > 1, paste0(ids, " (", exp_name, ")"), ids),
            name = paste0("gene:", ids, "::", exp_name)
        ) |>
        dplyr::select(name, label)

    # Merge labels into layout
    layout <- layout |>
        dplyr::left_join(gene_labels,
            by = "name"
        ) |>
        dplyr::mutate(
            label = ifelse(is.na(label),
                gsub("^.*?:", "", name),
                label
            )
        )

    # Build plot
    p <- ggraph::ggraph(layout) +
        ggraph::geom_edge_link(alpha = edge_alpha, color = "grey70")

    # Term nodes as pie chart by exp_name
    p <- p +
        ggraph::geom_node_arc_bar(
            data = term_arc_data,
            aes(
                x0 = x, y0 = y, r0 = 0, r = node_size * term_node_correction,
                start = start, end = end, fill = exp_name
            ),
            color = "transparent"
        ) +
        scale_fill_tidy_exp(name = "Experiment")

    # Reset scale before gene coloring
    p <- p +
        ggnewscale::new_scale_color()

    # Gene nodes colored by logFC
    p <- p +
        ggraph::geom_node_point(
            data = layout |>
                dplyr::filter(type == "gene"),
            aes(
                x = x,
                y = y,
                color = !!sym(logfc_col)
            ),
            size = node_size * gene_node_correction
        ) +
        scale_color_gradient2(
            name = expression(Log[2] * "FC"),
            low = "blue4",
            mid = "white",
            high = "red4",
            midpoint = 0,
            na.value = "grey60"
        )

    # Add labels
    p <- p +
        ggraph::geom_node_label(
            data = layout,
            aes(
                x = x, y = y,
                label = label,
                fontface = ifelse(type == "gene", "italic", "bold")
            ),
            size = label_size,
            repel = TRUE
        ) +
        theme_void()

    p <- p +
        labs(
            title = "Gene-Term Network"
        ) +
        theme(plot.title = element_text(face = "bold.italic"))

    return(p)
}


# --- Enrichment Summary -------
#' @noRd
#'
#' @title Internal: Multi-panel Summary of Enrichment Results
#'
#' @description
#' Assembles barplots, ridgeplots, and a Venn diagram to summarize enrichment
#' results across GO groups and experiments.
#'
#' @param enr_res Enrichment result data frame.
#'
#' @return A `patchwork` object combining multiple ggplots.
#'
#' @importFrom ggplot2 ggplot aes geom_col coord_flip labs theme_minimal
#' theme element_blank element_text element_line element_rect
#' element_text scale_y_continuous scale_fill_gradient
#'  scale_size_continuous scale_fill_manual
#' @importFrom dplyr mutate distinct count filter left_join
#' arrange pull select group_by ungroup
#' @importFrom tidyr separate_rows
#' @importFrom purrr map
#'
#' @keywords internal
.plot_summary_enrichment <- function(
  enr_res
) {
    if (nrow(enr_res) < 1) {
        return(NULL)
    }
    .check_suggested("ggridges")
    .check_suggested("ggvenn")
    .check_suggested("patchwork")

    # Clean up enr_res so that go_groups have no underscore
    enr_res <- enr_res |>
        dplyr::mutate(go_group = gsub("_", " ", go_group))

    # --- Barplot: # of enriched terms per exp_name
    bar_exp <- enr_res |>
        dplyr::distinct(
            term_name,
            exp_name
        ) |>
        dplyr::count(exp_name) |>
        ggplot(aes(
            x = reorder(exp_name, n),
            y = n,
            fill = exp_name
        )) +
        geom_col() +
        coord_flip() +
        scale_fill_tidy_exp() +
        scale_y_continuous(breaks = scales::pretty_breaks(n = 3)) +
        labs(
            y = "No. Enriched Terms",
            x = ""
        ) +
        theme_minimal() +
        theme(legend.position = "none")

    # --- Barplot: # of enriched terms per category
    if ("category" %in% colnames(enr_res)) {
        bar_cat <- enr_res |>
            dplyr::distinct(term_name, category) |>
            dplyr::count(category) |>
            ggplot(aes(
                x = reorder(category, n),
                y = n,
                fill = category
            )) +
            geom_col() +
            coord_flip() +
            scale_fill_tidy_exp(rev = TRUE) +
            labs(
                y = "No. Enriched Terms",
                x = "Category"
            ) +
            theme_minimal() +
            theme(legend.position = "none")
    }
    # --- Combination Plot ----
    if ("go_group" %in% colnames(enr_res)) {
        # Count how many terms per group
        group_term_counts <- enr_res |>
            dplyr::filter(!is.na(go_group)) |>
            dplyr::count(go_group, name = "n_terms") |>
            dplyr::filter(n_terms > 0)

        valid_groups <- group_term_counts$go_group

        # Order go_groups by number of terms
        group_levels <- group_term_counts |>
            dplyr::arrange(dplyr::desc(n_terms)) |>
            dplyr::pull(go_group)

        # Count genes per valid group (for dot size + ridge fill)
        go_counts <- enr_res |>
            dplyr::filter(go_group %in% valid_groups) |>
            tidyr::separate_rows(ids, sep = ",\\s*") |>
            dplyr::distinct(go_group, ids) |>
            dplyr::count(go_group, name = "gene_count")

        # Prepare enrichment table with metadata
        enr_go <- enr_res |>
            dplyr::filter(go_group %in% valid_groups) |>
            dplyr::left_join(go_counts, by = "go_group") |>
            dplyr::mutate(go_group = factor(go_group, levels = group_levels))

        # --- (Optional) Barplot: number of unique categories per group
        if ("category" %in% colnames(enr_res)) {
            bar_cat_group <- enr_res |>
                dplyr::filter(go_group %in% valid_groups) |>
                dplyr::distinct(go_group, category) |>
                dplyr::count(go_group, category, name = "n_cat") |>
                dplyr::mutate(go_group = factor(go_group, levels = group_levels))

            bar_category <- bar_cat_group |>
                ggplot(aes(x = n_cat, y = go_group, fill = category)) +
                geom_col(position = "stack") +
                scale_fill_tidy_exp(rev = TRUE) +
                scale_x_continuous(breaks = scales::pretty_breaks(n = 3)) +
                labs(
                    x = "No. Categories",
                    y = NULL,
                    fill = "Category"
                ) +
                theme_minimal() +
                theme(
                    axis.text.y = element_blank(),
                    axis.ticks.y = element_blank(),
                    panel.grid = element_blank(),
                    legend.position = "right"
                )
        }


        # --- Dotplot of total gene count per GO group
        dot_gene <- go_counts |>
            dplyr::mutate(go_group = factor(go_group, levels = group_levels)) |>
            ggplot(aes(x = 1, y = go_group, size = gene_count)) +
            geom_point(color = "midnightblue", stroke = 0.3) +
            scale_size_continuous(range = c(2, 8)) +
            labs(x = NULL, y = NULL, size = "No. Genes") +
            coord_cartesian(clip = "off") +
            theme_minimal() +
            theme(
                axis.text.x = element_blank(),
                axis.text.y = element_blank(),
                axis.ticks = element_blank(),
                axis.title = element_blank(),
                panel.grid = element_blank(),
                legend.position = "right"
            )

        # --- Ridgeplot of -log10(padj), filled by gene count
        ridge_p <- enr_go |>
            ggplot(aes(x = -log10(padj), y = go_group, fill = gene_count)) +
            ggridges::geom_density_ridges(scale = 1.2, alpha = 0.6) +
            scale_fill_gradient(low = "thistle", high = "midnightblue") +
            labs(x = expression("-Log"[10] * "P"), y = NULL, fill = "No. Genes") +
            ggridges::theme_ridges() +
            theme(
                legend.position = "none",
                axis.title.x = element_text(size = 12)
            )

        # --- Barplot: number of terms per group by exp_name
        group_exp_terms <- enr_res |>
            dplyr::filter(go_group %in% valid_groups) |>
            dplyr::distinct(go_group, exp_name, term_name) |>
            dplyr::count(go_group, exp_name, name = "n_terms") |>
            dplyr::mutate(go_group = factor(go_group, levels = group_levels))

        bar_expname <- group_exp_terms |>
            ggplot(aes(x = n_terms, y = go_group, fill = exp_name)) +
            geom_col(position = "stack") +
            scale_fill_tidy_exp() +
            scale_x_continuous(breaks = scales::pretty_breaks(n = 3)) +
            labs(x = "No. Terms", y = NULL, fill = "Experiment") +
            theme_minimal() +
            theme(
                axis.text.y = element_blank(),
                axis.ticks.y = element_blank(),
                legend.position = "right"
            )

        # --- Combine panels
        if (exists("bar_category")) {
            ridge <- bar_category + dot_gene + ridge_p + bar_expname +
                patchwork::plot_layout(widths = c(1.5, 1, 3.5, 2), guides = "collect")
        } else {
            ridge <- dot_gene + ridge_p + bar_expname +
                patchwork::plot_layout(widths = c(1, 3.5, 2), guides = "collect")
        }
    }


    # --- Venn diagram of term overlap
    venn_data <- enr_res |>
        dplyr::select(exp_name, term_name) |>
        dplyr::distinct() |>
        split(~exp_name) |>
        purrr::map(~ .x$term_name)

    venn <- ggvenn::ggvenn(
        venn_data,
        fill_color = tidy_exp_pal,
        stroke_color = "transparent",
        set_name_size = 4,
        show_percentage = TRUE
    ) +
        theme(
            plot.margin = margin(0, 0, 0, 0),
            plot.background = element_blank(),
            panel.background = element_blank()
        )

    # --- Layout assembly
    layout_left <- ridge
    layout_right <- (bar_exp / venn) +
        patchwork::plot_layout(heights = c(1, 2.5))

    final_plot <- (layout_left | layout_right) +
        patchwork::plot_annotation(title = "Enrichment Summary") &
        theme(plot.title = element_text(face = "bold.italic"))

    return(final_plot)
}

#' Plot Exposure-Omics Associations
#'
#' @description
#' Plots the number of significant exposure-omics associations, grouped
#' either by exposure or the exposure category.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing association results.
#' @param plot_type Character. One of `"exposures"` or `"category"`. Controls
#'   whether associations are summarized per exposure or per exposure category.
#'   Defaults to `"exposures"`.
#' @param pval_col Character. Name of the column used for p-value filtering.
#'   Defaults to `"p_adjust"`.
#' @param pval_thresh Numeric. Significance threshold applied to `pval_col`.
#'   Rows with values below this threshold are retained. Defaults to `0.05`.
#'
#' @return A `ggplot` object.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # run exposure-omics association
#' mae <- mae |>
#'     run_exposure_omics_association(
#'         exposures = c("exposure_pm25", "exposure_no2"),
#'         covariates = c("age", "sex")
#'     )
#'
#' plot_exposure_omics_association(
#'     exposomicset = mae,
#'     plot_type = "exposures"
#' )
#'
#' @export
plot_exposure_omics_association <- function(
  exposomicset,
  plot_type = c("exposures", "category"),
  pval_col = "p_adjust",
  pval_thresh = 0.05
) {
    plot <- switch(plot_type,
        "exposures" = .plot_exp_omic_exposure(
            exposomicset,
            pval_col = pval_col,
            pval_thresh = pval_thresh
        ),
        "category" = .plot_exp_omic_category(
            exposomicset,
            pval_col = pval_col,
            pval_thresh = pval_thresh
        )
    )
    return(plot)
}

#' Plot Significant Omics Associations per Exposure
#'
#' @description
#' Internal helper that counts and plots the number of significant omics
#' associations for each individual exposure.
#'
#' @inheritParams plot_exposure_omics_association
#'
#' @return A `ggplot` object.
#'
#' @keywords internal
.plot_exp_omic_exposure <- function(
  exposomicset,
  pval_col = "p_adjust",
  pval_thresh = 0.05
) {
    extract_results(
        exposomicset = exposomicset,
        result = "association"
    ) |>
        purrr::pluck(
            "exposure_omics",
            "results_df"
        ) |>
        filter(!!sym(pval_col) < pval_thresh) |>
        group_by(exposure) |>
        dplyr::count() |>
        ggplot(aes(
            x = n,
            y = reorder(exposure, n),
            fill = n
        )) +
        geom_col(linewidth = 0.005, color = "black") +
        scale_fill_gradient(
            low = "#fdeff9",
            high = "#03001e"
        ) +
        ggpubr::theme_pubr(legend = "right") +
        labs(
            x = "No. of Omics Associations",
            y = "",
            fill = "No. of Omics Associations",
            title = "No. of Omics Associations per Exposure"
        ) +
        theme(
            plot.title = element_text(face = "bold.italic")
        )
}

#' Plot Significant Omics Associations per Exposure Category
#'
#' @description
#' Internal helper that counts and plots the number of significant omics
#' associations for each exposure category.
#'
#' @inheritParams plot_exposure_omics_association
#'
#' @return A `ggplot` object.
#'
#' @keywords internal
.plot_exp_omic_category <- function(
  exposomicset,
  pval_col = "p_adjust",
  pval_thresh = 0.05
) {
    extract_results(
        exposomicset = exposomicset,
        result = "association"
    ) |>
        purrr::pluck(
            "exposure_omics",
            "results_df"
        ) |>
        filter(!!sym(pval_col) < pval_thresh) |>
        group_by(category) |>
        dplyr::count() |>
        ggplot(aes(
            x = n,
            y = reorder(category, n),
            fill = n
        )) +
        geom_col(linewidth = 0.005, color = "black") +
        scale_fill_gradient(
            low = "#fdeff9",
            high = "#03001e"
        ) +
        ggpubr::theme_pubr(legend = "right") +
        labs(
            x = "No. of Omics Associations",
            y = "",
            fill = "No. of Omics Associations",
            title = "No. of Omics Associations per Category"
        ) +
        theme(
            plot.title = element_text(face = "bold.italic")
        )
}

#' Extract Merged Omics and Exposure Data Frame
#'
#' This function extracts and merges exposure variables from `colData`
#'  with selected features from omics datasets
#' in a `MultiAssayExperiment` object. Optionally applies log2 transformation
#' to omics data and restricts features based on a variable map.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing
#' omics and exposure data.
#' @param variable_map A data frame with columns `"variable"` and `"exp_name"`,
#'  indicating which variables belong to each omics or exposure domain.
#' @param log2_trans Logical; whether to log2-transform omics data.
#' Default is `TRUE`.
#'
#' @return A data frame where rows correspond to samples,
#' and columns contain exposure variables and log2-transformed omics features.
#' Columns from different omics types are disambiguated using prefixes.
#'
#' @details
#' If `variable_map` is provided, it is used to select variables from
#' both exposures and omics. If not provided, all numeric `colData` variables
#' are used as exposures (excluding variables matching `^PC`),
#' and all omics features are included.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#' # export the omics exposure df
#' merged_df <- extract_omics_exposure_df(
#'     mae,
#'     log2_trans = TRUE
#' )
#'
#' @export
extract_omics_exposure_df <- function(
  exposomicset,
  variable_map = NULL,
  log2_trans = TRUE
) {
    # Validate variable_map
    if (!is.null(variable_map)) {
        stopifnot(all(c("variable", "exp_name") %in% colnames(variable_map)))
    }

    # Extract and preprocess colData
    col_df <- exposomicset |>
        MultiAssayExperiment::colData() |>
        as.data.frame()

    # Select exposure variables
    if (!is.null(variable_map)) {
        exposure_vars <- variable_map$variable[variable_map$exp_name == "exposure"]
        exposure_vars <- intersect(exposure_vars, colnames(col_df))
        exposure_data <- col_df[, exposure_vars]
    } else {
        exposure_data <- dplyr::select_if(col_df, is.numeric)
        # Remove PCs
        if (any(grepl("^PC", colnames(exposure_data)))) {
            exposure_data <- exposure_data |>
                dplyr::select(-dplyr::matches("^PC"))
        }
    }

    if (log2_trans) {
        # Log2-transform omics
        omics_data <- .log2_multiassay(exposomicset)
    } else {
        omics_data <- exposomicset
    }

    # Determine selected features per omics dataset
    if (!is.null(variable_map)) {
        omics_map <- variable_map[variable_map$exp_name != "exposure", ]
        selected_features <- split(omics_map$variable, omics_map$exp_name)
    } else {
        # Use all features if no map provided
        selected_features <- lapply(
            MultiAssayExperiment::experiments(omics_data),
            function(se) SummarizedExperiment::rownames(se)
        )
    }

    # Extract and filter omics
    omics_list <- lapply(
        names(MultiAssayExperiment::experiments(omics_data)),
        function(name) {
            # Extract omics data for each dataset
            se <- MultiAssayExperiment::experiments(omics_data)[[name]]

            # Filter features based on selected features
            feats <- selected_features[[name]]

            # If no features are selected, return NULL
            if (is.null(feats)) {
                return(NULL)
            }

            # Filter features based on selected features
            feats <- intersect(feats, rownames(se))

            # If no features are selected or found, return NULL
            if (length(feats) == 0) {
                return(NULL)
            }

            # Subset assay
            se <- se[feats, , drop = FALSE]

            df <- se |>
                SummarizedExperiment::assay() |>
                t() |>
                as.data.frame()

            # Set column names with prefix
            colnames(df) <- paste0(name, "_", colnames(df))

            ###
            colnames(df) <- gsub(" |-", "_", colnames(df))
            df <- df |> tibble::rownames_to_column("id")
            return(df)
        }
    ) |>
        # Remove NULL entries
        purrr::compact()

    # Check if any omics were retained
    if (length(omics_list) == 0) {
        stop("No omics features matched the variable_map.")
    }

    omics_df <- purrr::reduce(omics_list, dplyr::full_join, by = "id")

    # Merge with exposures
    merged_data <- exposure_data |>
        tibble::rownames_to_column("id") |>
        dplyr::left_join(omics_df, by = "id") |>
        na.omit() |>
        `rownames<-`(NULL) |>
        tibble::column_to_rownames("id")

    return(merged_data)
}

#' Plot Association Results (Unified Forest Plot)
#'
#' Generates a forest plot for association results from any source stored
#' in the metadata of a `MultiAssayExperiment` object.
#' Supports faceting and visual augmentation with R^2 tiles when available.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing association
#' results in metadata.
#' @param source Character string indicating the association source.
#' One of `"omics"`, `"exposures"`, `"factors"`, or `"go_pcs"`.
#' @param terms Optional character vector of term names to subset the plot to.
#'  Default is `NULL` (include all).
#' @param filter_col Column used to assess statistical significance
#' (default: `"p.value"`).
#' @param filter_thresh Numeric threshold applied to `filter_col`
#' (default: `0.05`).
#' @param direction_filter Direction of associations to retain.
#' One of `"all"` (default), `"up"`, or `"down"`.
#' @param add_r2_tile Logical; if `TRUE`, includes a tile plot for `r2_col`
#'  (default: `TRUE`).
#' @param r2_col Column used for coloring the tile plot (default: `"adj_r2"`).
#' @param facet Logical; if `TRUE` and `source == "go_pcs"`,
#'  apply nested faceting by experiment and GO cluster (default: `FALSE`).
#' @param nrow Integer; number of rows for facet layout if enabled (default: `1`).
#' @param subtitle Optional subtitle for the plot. If `NULL`,
#' automatically generated from covariates used in the model.
#'
#' @return A `ggplot2` object: either a single forest plot or a composite plot
#'  with an R^2 tile strip.
#'
#' @details
#' This function visualizes effect size estimates and confidence intervals from
#' association analyses. It allows filtering by
#' direction (`"up"` for positive, `"down"` for negative) and by significance.
#'  For `source = "go_pcs"`, it supports special
#' formatting by splitting term labels into nested facets.
#'
#' The R^2 tile (if enabled) adds a side heatmap indicating model fit for
#' each association. This can be useful for model diagnostics.
#'
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run association tests
#' mae <- mae |>
#'     run_association(
#'         source = "exposures",
#'         feature_set = c("exposure_pm25", "exposure_no2"),
#'         outcome = "smoker",
#'         covariates = c("age"),
#'         family = "binomial"
#'     )
#'
#' assoc_plot <- mae |>
#'     plot_association(
#'         source = "exposures"
#'     )
#'
#' @export
plot_association <- function(
  exposomicset,
  source = c("omics", "exposures", "factors", "go_pcs"),
  terms = NULL,
  filter_col = "p.value",
  filter_thresh = 0.05,
  direction_filter = "all",
  add_r2_tile = TRUE,
  r2_col = "adj_r2",
  facet = FALSE,
  nrow = 1,
  subtitle = NULL
) {
    requireNamespace("ggplot2")
    requireNamespace("dplyr")
    requireNamespace("tidyr")
    .check_suggested(pkg = "patchwork")

    source <- match.arg(source)
    assoc_key <- paste0("assoc_", source)

    # Check metadata
    assoc_metadata <- MultiAssayExperiment::metadata(exposomicset)$association
    if (is.null(assoc_metadata) || !assoc_key %in% names(assoc_metadata)) {
        stop(
            "Association results for source '",
            source,
            "' not found in metadata. Run `run_association()` first."
        )
    }

    assoc_result <- assoc_metadata[[assoc_key]]
    results_df <- assoc_result$results_df
    covariates <- assoc_result$covariates

    if (!is.null(terms)) {
        df <- dplyr::filter(results_df, term %in% terms)
    } else {
        df <- results_df
    }

    # Mark significance
    df <- df |>
        dplyr::mutate(
            is_significant = !!rlang::sym(filter_col) < filter_thresh,
            direction = dplyr::case_when(
                (estimate > 0 & (is_significant == TRUE)) ~ "up",
                (estimate < 0 & (is_significant == TRUE)) ~ "down",
                .default = "ns"
            )
        )

    # Apply direction filter
    if (direction_filter == "up") {
        df <- dplyr::filter(df, direction == "up")
    } else if (direction_filter == "down") {
        df <- dplyr::filter(df, direction == "down")
    }

    # Extract y-axis variable and handle GO PC formatting
    if (source == "go_pcs") {
        df <- tidyr::separate(
            df,
            col = "term",
            into = c("PC", "exp_name", "Cluster", "go_group"),
            sep = "/",
            remove = FALSE
        )
        df$label <- df$go_group
        facet_formula <- exp_name + Cluster ~ .
    } else {
        df$label <- df$term
        facet_formula <- NULL
    }

    # Only proceed if add_r2_tile is TRUE and column exists
    if (add_r2_tile && r2_col %in% names(df)) {
        leg_label <- ifelse(r2_col == "r2",
            expression("R"^2),
            expression("Adj. R"^2)
        )
        tile_plot <- df |>
            ggplot2::ggplot(ggplot2::aes(
                x = 1,
                y = reorder(label, estimate),
                fill = !!rlang::sym(r2_col)
            )) +
            ggplot2::geom_tile() +
            scale_fill_gradient(
                low = "#e8e1eb",
                high = "#3f1b4f"
            ) +
            ggplot2::labs(x = NULL, y = NULL, fill = r2_col) +
            ggplot2::theme_minimal() +
            ggplot2::theme(
                axis.text.x = ggplot2::element_blank(),
                axis.ticks.x = ggplot2::element_blank(),
                axis.title.y = ggplot2::element_blank(),
                panel.grid = ggplot2::element_blank(),
                axis.text.y = element_blank()
            ) +
            labs(fill = leg_label)
    }


    # Auto-subtitle
    if (is.null(subtitle)) {
        if (!is.null(covariates) && length(covariates) > 0) {
            subtitle <- paste("Covariates:", paste(covariates, collapse = ", "))
        }
    }

    # Plot
    p <- ggplot2::ggplot(df, ggplot2::aes(
        x = estimate,
        y = reorder(label, estimate),
        color = direction
    )) +
        ggplot2::geom_vline(xintercept = 0, linetype = "dashed") +
        ggplot2::geom_errorbarh(
            ggplot2::aes(
                xmin = estimate - std.error,
                xmax = estimate + std.error
            ),
            color = "grey55",
            width = 0.2
        ) +
        ggplot2::geom_point(
            shape = 18,
            size = 5,
            alpha = 0.5
        ) +
        ggplot2::scale_color_manual(values = c(
            up = "#8E0152",
            down = "#006666",
            ns = "grey55"
        )) +
        ggplot2::theme_bw() +
        ggplot2::theme(
            legend.position = "none",
            plot.subtitle = ggplot2::element_text(
                face = "italic"
            ),
            plot.title = ggplot2::element_text(
                face = "bold.italic"
            ),
        ) +
        ggplot2::labs(
            x = "Effect size",
            y = "",
            title = paste0(tools::toTitleCase(source), "-Outcome Associations"),
            subtitle = subtitle
        )

    if (facet && source == "go_pcs") {
        if (!requireNamespace("ggh4x", quietly = TRUE)) {
            warning("Faceting requires the `ggh4x` package.")
        } else {
            p <- p +
                ggh4x::facet_nested_wrap(
                    facet_formula,
                    nrow = nrow,
                    scales = "free_x"
                ) +
                ggplot2::theme(strip.text = ggplot2::element_text(face = "bold.italic"))
        }
    }

    if (add_r2_tile && exists("tile_plot")) {
        return(patchwork::wrap_plots(
            p + ggplot2::theme(axis.text.y = element_text()),
            tile_plot,
            widths = c(3, 0.5)
        ))
    } else {
        return(p)
    }

    return(p)
}

#' Cluster Samples Based on Exposure Data
#'
#' Performs hierarchical clustering of samples using exposure data from
#' `colData(exposomicset)`.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing omics
#' and exposure data.
#' @param exposure_cols A character vector of column names in
#' `colData(exposomicset)` to use for clustering.
#' @param dist_method A character string specifying the distance metric
#' (`"euclidean"`, `"gower"`, etc.). If `NULL`, it is automatically determined.
#' @param user_k An integer specifying the number of clusters.
#' If `NULL`, an optimal `k` is determined.
#' @param cluster_method A character string specifying the hierarchical
#' clustering method. Default is `"ward.D"`.
#' @param clustering_approach A character string specifying the method
#'  for determining `k` (`"diana"`, `"gap"`, `"elbow"`, `"dynamic"`,
#'  or `"density"`). Default is `"diana"`.
#' @param action A character string specifying `"add"`
#' (store results in metadata) or `"get"` (return clustering results).
#'  Default is `"add"`.
#'
#' @details
#' This function:
#' - Extracts **numeric exposure data** from `colData(exposomicset)`.
#' - Computes a **distance matrix** (`"gower"` for mixed data,
#' `"euclidean"` for numeric).
#' - Determines the **optimal number of clusters (`k`)** using the
#' specified method.
#' - Performs **hierarchical clustering** (`hclust`) and assigns
#' samples to clusters.
#' - Generates a **heatmap** of scaled exposure values.
#' - Stores results in `metadata(exposomicset)$sample_clustering`
#' when `action="add"`.
#'
#' @return If `action="add"`, returns the updated `exposomicset`.
#' If `action="get"`, returns a list with:
#' \item{sample_cluster}{A hierarchical clustering object (`hclust`).}
#' \item{sample_groups}{A named vector of sample cluster assignments.}
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # determine sample clusters
#' mae <- run_cluster_samples(
#'     exposomicset = mae,
#'     exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi"),
#'     clustering_approach = "diana"
#' )
#'
#' @importFrom dplyr select mutate left_join all_of everything across
#' @importFrom tibble column_to_rownames
#' @importFrom cluster daisy diana clusGap maxSE
#' @importFrom factoextra hcut fviz_nbclust
#' @export
run_cluster_samples <- function(exposomicset,
                                exposure_cols = NULL,
                                dist_method = NULL,
                                user_k = NULL,
                                cluster_method = "ward.D",
                                clustering_approach = "diana",
                                action = "add") {
    message("Starting clustering analysis...")
    .check_suggested(pkg = "dynamicTreeCut")
    .check_suggested(pkg = "densityClust")

    # Validate exposure_cols input
    if (is.null(exposure_cols) || length(exposure_cols) == 0) {
        stop("Please specify `exposure_cols` is a a column name in `colData`.")
    }

    # Ensure selected exposure columns exist in colData
    available_variables <- intersect(
        exposure_cols,
        colnames(MultiAssayExperiment::colData(exposomicset))
    )

    if (length(available_variables) == 0) {
        stop("None of the specified `exposure_cols` are available in `colData`.")
    }

    # Extract numeric exposure data for clustering
    exposure_data <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame() |>
        dplyr::select(dplyr::all_of(available_variables))

    # Remove missing/infinite values
    exposure_data <- exposure_data |>
        na.omit() |>
        dplyr::mutate(
            dplyr::across(
                dplyr::everything(),
                ~ ifelse(is.infinite(.),
                    NA,
                    .
                )
            )
        ) |>
        na.omit()

    if (nrow(exposure_data) == 0) {
        stop("All selected exposure data contains missing or infinite values.")
    }

    # Determine appropriate distance metric
    if (is.null(dist_method)) {
        if (all(vapply(exposure_data, is.numeric, FUN.VALUE = logical(1)))) {
            dist_method <- "euclidean" # Continuous data
        } else {
            dist_method <- "gower" # Mixed data types
        }
    }

    # Compute distance matrix
    sample_dist <- if (dist_method == "gower") {
        cluster::daisy(exposure_data, metric = "gower")
    } else {
        dist(exposure_data, method = dist_method)
    }

    if (!is.null(user_k)) {
        k_samples <- user_k
    } else {
        # Determine optimal k clusters
        k_samples <- .determine_k(
            dist_matrix = sample_dist,
            cluster_method = cluster_method,
            clustering_approach = clustering_approach
        )
    }

    # Perform hierarchical clustering
    sample_cluster <- hclust(as.dist(sample_dist),
        method = cluster_method
    )

    # Cut dendrogram using optimal k
    sample_groups <- cutree(sample_cluster, k = k_samples)

    message("Optimal number of clusters for samples: ", k_samples)

    # add in sample groups for samples to the colData
    col_data <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame() |>
        (\(df){
            df$id_to_map <- rownames(df)
            df
        })() |>
        dplyr::left_join(
            data.frame(
                id_to_map = names(sample_groups),
                sample_group = paste0("Group_", as.character(sample_groups))
            ),
            by = "id_to_map"
        ) |>
        tibble::column_to_rownames("id_to_map")

    MultiAssayExperiment::colData(exposomicset) <- col_data

    if (action == "add") {
        # Save clustering results in metadata
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$quality_control$sample_clustering <- list(
            sample_cluster = sample_cluster,
            sample_groups = sample_groups
        )

        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        # Add analysis steps taken to metadata
        step_record <- list(
            run_cluster_samples = list(
                timestamp = Sys.time(),
                params = list(
                    exposure_cols = exposure_cols,
                    dist_method = dist_method,
                    user_k = user_k,
                    cluster_method = cluster_method,
                    clustering_approach = clustering_approach
                ),
                notes = paste0("Optimal number of clusters for samples: ", k_samples)
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else if (action == "get") {
        return(list(
            sample_cluster = sample_cluster,
            sample_groups = sample_groups
        ))
    } else {
        stop("Invalid action specified. Use 'add' or 'get'.")
    }
}

# --- Determine K Clusters Function -------------
#' Determine Optimal Number of Clusters (k)
#'
#' Internal function to determine the optimal number of clusters based on a
#' selected clustering approach. Supports multiple methods such as gap statistic,
#' elbow method, DIANA, dynamic tree cut, and density-based clustering.
#'
#' @param dist_matrix A distance matrix (or object coercible to `dist`) representing
#'   pairwise sample distances.
#' @param cluster_method The hierarchical clustering method to use (e.g., "ward.D2",
#'   "average") if applicable.
#' @param clustering_approach The clustering approach used to determine number
#' of clusters.
#'
#' @return An integer indicating the estimated optimal number of clusters.
#' @keywords internal
#' @importFrom cluster diana clusGap maxSE
#' @importFrom factoextra hcut fviz_nbclust
#' @noRd
.determine_k <- function(
  dist_matrix,
  cluster_method,
  clustering_approach
) {
    if (clustering_approach == "diana") {
        # Determine optimal k using the height difference method
        sample_cluster <- cluster::diana(as.dist(dist_matrix))
        height_diffs <- diff(sample_cluster$height)
        cutoff_index <- which.max(height_diffs)
        return(length(sample_cluster$height) - cutoff_index)
    } else if (clustering_approach == "gap") {
        # Determine optimal k using the gap statistic
        gap_stat <- cluster::clusGap(as.matrix(dist_matrix),
            FUN = factoextra::hcut,
            K.max = 20,
            B = 50
        )
        return(cluster::maxSE(
            gap_stat$Tab[, "gap"],
            gap_stat$Tab[, "SE.sim"]
        ))
    } else if (clustering_approach == "elbow") {
        # Determine optimal k using the elbow method
        wss_plot <- factoextra::fviz_nbclust(as.matrix(dist_matrix),
            FUN = factoextra::hcut,
            method = "wss"
        )

        # Identify the first significant drop and ensure it's a number we can use
        k_optimal <- which.min(diff(diff(wss_plot$data$y))) + 1
        if (is.na(k_optimal) || k_optimal < 2) k_optimal <- 3
        return(k_optimal)
    } else if (clustering_approach == "dynamic") {
        .check_suggested(pkg = "dynamicTreeCut")
        # Determine optimal k using dynamic tree cut
        sample_cluster <- hclust(as.dist(dist_matrix), method = cluster_method)
        cut_clusters <- dynamicTreeCut::cutreeDynamic(
            dendro = sample_cluster,
            distM = as.matrix(as.dist(dist_matrix)),
            deepSplit = 2
        )
        return(length(unique(cut_clusters)))
    } else if (clustering_approach == "density") {
        .check_suggested(pkg = "densityClust")
        # Determine optimal k using density-based clustering
        dclust <- densityClust::densityClust(as.dist(dist_matrix),
            gaussian = TRUE
        )
        dclust <- densityClust::findClusters(dclust,
            rho = quantile(dclust$rho, 0.90),
            delta = quantile(dclust$delta, 0.90)
        )
        return(length(unique(dclust$clusters)))
    } else {
        stop("Invalid clustering approach selected.")
    }
}

#' Impute Missing Exposure and Omics Data in a MultiAssayExperiment
#'
#' Performs missing data imputation on both exposure variables
#' (from `colData`) and omics datasets (from `experiments`) within
#' a `MultiAssayExperiment` object.
#'
#' For exposures, numeric columns in `colData` are imputed using
#' the selected method. For omics data, assays are selected and
#' imputed individually.
#'
#' Supported imputation methods include:
#' \itemize{
#'   \item \code{"median"}: Median imputation using
#'   \code{naniar::impute_median_all}
#'   \item \code{"mean"}: Mean imputation using
#'   \code{naniar::impute_mean_all}
#'   \item \code{"knn"}: k-nearest neighbor imputation using
#'   \code{impute::impute.knn}
#'   \item \code{"mice"}: Multiple imputation using chained equations
#'   (\code{mice::mice})
#'   \item \code{"missforest"}: Random forest-based imputation using
#'   \code{missForest::missForest}
#'   \item \code{"lod_sqrt2"}: Substitution of missing values with
#'   LOD/sqrt(2), where LOD is the smallest non-zero value per variable
#' }
#'
#' @param exposomicset A \code{MultiAssayExperiment} object containing
#' exposures and omics data.
#' @param exposure_impute_method Character. Imputation method to use for
#' exposure variables. Defaults to \code{"median"}.
#' @param exposure_cols Character vector. Names of columns in
#' \code{colData} to impute. If \code{NULL}, all numeric columns are used.
#' @param omics_impute_method Character. Imputation method to use for
#'  omics data. Defaults to \code{"knn"}.
#' @param omics_to_impute Character vector. Names of omics datasets to impute.
#' If \code{NULL}, all omics datasets are included.
#'
#' @return A \code{MultiAssayExperiment} object with imputed exposure
#' and/or omics data.
#'
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Introduce some missingness
#' MultiAssayExperiment::colData(mae)$exposure_pm25[sample(1:20, 5)] <- NA
#'
#' # Filter features and exposures with high missingness
#' mae <- run_impute_missing(
#'     exposomicset = mae,
#'     exposure_impute_method = "median"
#' )
#'
#' @importFrom naniar impute_median_all impute_mean_all
#' @importFrom S4Vectors DataFrame
#' @importFrom SummarizedExperiment assays
#' @export
run_impute_missing <- function(
  exposomicset,
  exposure_impute_method = "median",
  exposure_cols = NULL,
  omics_impute_method = NULL,
  omics_to_impute = NULL
) {
    # Helper for LOD/sqrt(2) imputation
    impute_lod_sqrt2 <- function(data) {
        data[] <- lapply(data, function(col) {
            if (is.numeric(col)) {
                lod <- min(col[col > 0], na.rm = TRUE)
                imputed <- ifelse(is.na(col), lod / sqrt(2), col)
                return(imputed)
            } else {
                return(col)
            }
        })
        return(as.data.frame(data))
    }

    # General imputation helper
    impute_data <- function(data, method) {
        if (method == "median") {
            return(naniar::impute_median_all(data))
        } else if (method == "mean") {
            return(naniar::impute_mean_all(data))
        } else if (method == "knn") {
            .check_suggested(pkg = "impute")
            return(as.data.frame(impute::impute.knn(as.matrix(data))$data))
        } else if (method == "mice") {
            .check_suggested(pkg = "mice")
            return(mice::complete(mice::mice(data,
                m = 5, maxit = 50,
                method = "pmm", seed = 500
            )))
        } else if (method == "missforest") {
            .check_suggested(pkg = "missForest")
            return(missForest::missForest(data)$ximp)
        } else if (method == "lod_sqrt2") {
            return(impute_lod_sqrt2(data))
        } else {
            stop("Unsupported imputation method: ", method)
        }
    }

    # Impute selected exposure columns
    if (!is.null(exposure_impute_method)) {
        message("Imputing exposure data using method: ", exposure_impute_method)

        metadata_df <- as.data.frame(MultiAssayExperiment::colData(exposomicset))
        # if (is.null(exposure_cols)) {
        #   exposure_cols <- names(metadata_df)[sapply(metadata_df, is.numeric)]
        # }
        if (is.null(exposure_cols)) {
            exposure_cols <- names(metadata_df)[
                vapply(metadata_df, is.numeric, FUN.VALUE = logical(1))
            ]
        }


        data_to_impute <- metadata_df[, exposure_cols, drop = FALSE]
        imputed <- impute_data(data_to_impute, exposure_impute_method)

        metadata_df[, exposure_cols] <- imputed
        MultiAssayExperiment::colData(exposomicset) <- S4Vectors::DataFrame(
            metadata_df
        )
    }

    if (!is.null(omics_to_impute)) {
        # Impute omics data
        all_omics <- setdiff(names(
            MultiAssayExperiment::experiments(exposomicset)
        ), "exposure")
        omics_to_use <- if (is.null(omics_to_impute)) {
            all_omics
        } else {
            intersect(all_omics, omics_to_impute)
        }

        for (omics_name in omics_to_use) {
            message(
                "Imputing omics dataset: ",
                omics_name,
                " using method: ",
                omics_impute_method
            )
            experiment <- MultiAssayExperiment::experiments(exposomicset)[[omics_name]]
            assay_data <- as.data.frame(SummarizedExperiment::assays(experiment)[[1]])
            imputed_data <- impute_data(assay_data, omics_impute_method)
            SummarizedExperiment::assays(experiment)[[1]] <- as.matrix(imputed_data)
            MultiAssayExperiment::experiments(exposomicset)[[omics_name]] <- experiment
        }
    }

    # Add analysis steps taken to metadata
    step_record <- list(run_impute_missing = list(
        timestamp = Sys.time(),
        params = list(
            exposure_impute_method = exposure_impute_method,
            exposure_cols = exposure_cols,
            omics_impute_method = omics_impute_method,
            omics_to_impute = omics_to_impute
        ),
        notes = ""
    ))

    MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
        MultiAssayExperiment::metadata(exposomicset)$summary$steps,
        step_record
    )

    return(exposomicset)
}

#' Plot Normality Summary of Exposure Variables
#'
#' Generates a bar plot summarizing the number of exposure variables that
#' pass or fail normality
#' tests (e.g., Shapiro-Wilk) before or after transformation.
#'
#' @param exposomicset A `MultiAssayExperiment` object with quality control
#'  metadata.
#' @param transformed Logical; if `TRUE`, use results after transformation.
#' Default is `FALSE`.
#'
#' @return A `ggplot` object summarizing the number of exposures
#' classified as normal or not normal.
#'
#' @details
#' This function assumes that `run_normality_check()` has been executed and
#'  that the results are
#' stored in `metadata(exposomicset)$quality_control$normality`.
#'  If `transformed = TRUE`, the function will
#' instead plot the transformation summary stored in `metadata(exposomicset)$quality_control$transformation$norm_summary`,
#' which is populated by `transform_exposure()`.
#'
#' The plot includes both bar heights and overlaid line segments to
#' reinforce the counts.
#'
#' @examples
#' # Create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' # Test for normality
#' mae <- mae |>
#'     run_normality_check() |>
#'     transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))
#'
#' # plot the normality summary
#' norm_p <- mae |>
#'     plot_normality_summary()
#'
#' @importFrom MultiAssayExperiment metadata
#' @importFrom ggplot2 ggplot aes geom_bar geom_segment labs scale_fill_manual scale_color_manual theme
#' @importFrom ggpubr theme_pubr get_palette
#' @importFrom purrr pluck
#' @export
plot_normality_summary <- function(
  exposomicset,
  transformed = FALSE
) {
    # require(ggplot2)

    # Check if "normality" is a name in metadata
    if (!("normality" %in% names(
        MultiAssayExperiment::metadata(exposomicset)$quality_control
    ))) {
        stop("Please run `run_normality_check() first.`")
    }

    # Check if "transformation" is a name in metadata
    if (transformed) {
        if (!("transformation" %in% names(
            MultiAssayExperiment::metadata(exposomicset)$quality_control
        ))) {
            stop("Please run `transform_exposure() first.`")
        }
    }

    # Plot normality results
    if (transformed) {
        norm_plot <- MultiAssayExperiment::metadata(exposomicset)$quality_control$transformation$norm_summary |>
            ggplot(aes(
                x = var,
                y = value,
                fill = var
            )) +
            geom_bar(stat = "identity", alpha = 0.5) +
            geom_segment(
                aes(
                    x = as.numeric(as.factor(var)) - 0.45,
                    xend = as.numeric(as.factor(var)) + 0.45,
                    y = value,
                    yend = value,
                    color = var
                ),
                linewidth = 1
            ) +
            ggpubr::theme_pubr(legend = "right") +
            scale_fill_manual(
                values = ggpubr::get_palette("uchicago", k = 2)[c(2, 1)]
            ) +
            scale_color_manual(
                values = ggpubr::get_palette("uchicago", k = 2)[c(2, 1)]
            ) +
            guides(color = "none") +
            theme(
                plot.title = element_text(face = "bold.italic"),
                plot.subtitle = element_text(face = "italic")
            ) +
            labs(
                x = "",
                y = "No. of Exposures",
                fill = "",
                title = "Normality of Exposure Variables",
                subtitle = "Shapiro-Wilk Test"
            )
    } else {
        norm_plot <- MultiAssayExperiment::metadata(exposomicset) |>
            purrr::pluck(
                "quality_control",
                "normality",
                "norm_summary"
            ) |>
            ggplot(aes(
                x = var,
                y = value,
                fill = var
            )) +
            geom_bar(stat = "identity", alpha = 0.5) +
            geom_segment(
                aes(
                    x = as.numeric(as.factor(var)) - 0.45,
                    xend = as.numeric(as.factor(var)) + 0.45,
                    y = value,
                    yend = value,
                    color = var
                ),
                linewidth = 1
            ) +
            ggpubr::theme_pubr(legend = "right") +
            scale_fill_tidy_exp() +
            scale_color_tidy_exp(guide = "none") +
            theme(
                plot.title = element_text(face = "bold.italic"),
                plot.subtitle = element_text(face = "italic")
            ) +
            labs(
                x = "",
                y = "No. of Exposures",
                fill = "",
                title = "Normality of Exposure Variables",
                subtitle = "Shapiro-Wilk Test"
            )
    }

    return(norm_plot)
}

#' Perform enrichment analysis on selected features from a exposomicset object
#'
#' This function performs enrichment analysis using selected features derived
#' from differential expression, correlation analysis,
#' or multi-omics factor features across experiments in an `exposomicset`.
#' It supports multiple enrichment databases (e.g., GO, KEGG, Reactome),
#'  applies FDR correction, and optionally clusters GO terms by Jaccard overlap.
#'
#' @param exposomicset An `exposomicset` (a `MultiAssayExperiment`
#' object with metadata) containing omics and metadata.
#' @param feature_type Character string indicating the feature source.
#' One of `"degs"`, `"degs_robust"`,
#'   `"omics"`, `"factor_features"`, `"degs_cor"`, `"omics_cor"`,
#'    or `"factor_features_cor"`.
#' @param score_col Column name used for stability score filtering
#' (only for `degs_robust`).
#' @param score_threshold Optional numeric threshold for filtering
#' stability scores. If `NULL`, the default
#'   threshold stored in the metadata will be used.
#' @param variable_map A data frame with `exp_name` and `variable` columns,
#'  used when `feature_type = "omics"`.
#' @param factor_type Character string for selecting factor features:
#'  `"common_top_factor_features"` or
#'   `"top_factor_features"`.
#' @param feature_col The name of the feature column used to extract
#'  gene identifiers.
#' @param deg_pval_col Column name for adjusted p-values from DEG analysis.
#' @param deg_pval_threshold Threshold to select significant DEGs
#' (default: 0.05).
#' @param deg_logfc_col Column name for log-fold changes from DEG analysis.
#' @param deg_logfc_threshold Threshold to select DEGs by absolute logFC
#' (default: `log2(1.5)`).
#' @param db Enrichment database to use. One of `"GO"`,
#'  `"KEGG"`, `"Reactome"`.
#' @param species Species name (required for GO enrichment,
#' e.g., `"Homo sapiens"`). Ignored for other databases.
#' @param fenr_col Column name for gene IDs used by `fenr`
#'  (e.g., `"gene_symbol"`).
#' @param padj_method Method for p-value adjustment (default: `"fdr"`).
#' @param pval_thresh Adjusted p-value threshold for filtering enriched terms
#' (default: 0.1).
#' @param min_set Minimum number of selected genes overlapping an enriched term
#' (default: 3).
#' @param max_set Maximum number of selected genes overlapping an enriched term
#' (default: 800).
#' @param clustering_approach Clustering method for GO term grouping.
#'  Defaults to `"diana"`.
#' @param action Either `"add"` to store results in the object's metadata
#'  or `"get"` to return results as a data frame.
#'
#' @return If `action = "add"`, returns the modified `exposomicset` with
#' enrichment results added to metadata.
#' If `action = "get"`, returns a `data.frame` of enrichment results
#' with GO term clusters (if applicable).
#'
#' @details
#' The function identifies selected features based on the chosen
#' `feature_type`, determines the gene universe
#' for each experiment, and performs enrichment analysis using the
#' `fenr` package. Results are adjusted for
#' multiple testing and optionally clustered by gene set overlap (for GO terms).
#'
#' If `feature_type` includes correlation-based results
#' (ending in `_cor`), enrichment is performed for each
#' exposure category separately.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 30,
#'     return_mae = TRUE
#' )
#'
#' # perform differential abundance analysis
#' mae <- run_differential_abundance(
#'     exposomicset = mae,
#'     formula = ~ smoker + sex,
#'     abundance_col = "counts",
#'     method = "limma_voom",
#'     action = "add"
#' )
#'
#' # perform enrichment analysis
#' mae <- run_enrichment(
#'     exposomicset = mae,
#'     feature_type = "degs",
#'     feature_col = "symbol",
#'     species = "goa_human",
#'     deg_logfc_threshold = log2(1),
#'     deg_pval_col = "P.Value",
#'     deg_pval_threshold = 0.2,
#'     action = "add"
#' )
#'
#' @importFrom MultiAssayExperiment metadata experiments
#' @importFrom SummarizedExperiment rowData
#' @importFrom dplyr filter inner_join pull mutate summarise bind_rows
#' select distinct arrange
#' @importFrom tidyr unnest pivot_wider
#' @importFrom purrr pluck map map2
#' @importFrom tibble column_to_rownames rownames_to_column
#' @importFrom stringr str_split str_trim
#' @importFrom rlang sym set_names .data
#' @importFrom stats p.adjust
#' @import fenr
#' @export
run_enrichment <- function(
  exposomicset,
  feature_type = c(
      "degs",
      "degs_robust",
      "omics",
      "factor_features",
      "degs_cor",
      "omics_cor",
      "factor_features_cor"
  ),
  score_col = "stability_score",
  score_threshold = NULL,
  variable_map = NULL,
  factor_type = c(
      "common_top_factor_features",
      "top_factor_features"
  ),
  feature_col = "feature",
  deg_pval_col = "adj.P.Val",
  deg_pval_threshold = 0.05,
  deg_logfc_col = "logFC",
  deg_logfc_threshold = log2(1.5),
  db = c("GO", "KEGG", "Reactome"),
  species = NULL,
  fenr_col = "gene_symbol",
  padj_method = "fdr",
  pval_thresh = 0.1,
  min_set = 3,
  max_set = 800,
  clustering_approach = "diana",
  action = "add"
) {
    # require(fenr)
    db <- match.arg(db)
    factor_type <- match.arg(factor_type)
    feature_type <- match.arg(feature_type)

    # Grab all feature meta data
    fdata <- exposomicset |>
        pivot_feature()

    if (feature_type %in% c(
        "degs",
        "degs_robust",
        "omics",
        "factor_features"
    )) {
        enr_res <- .run_feature_enr(
            exposomicset = exposomicset,
            feature_type = feature_type,
            db = db,
            fdata = fdata,
            species = species,
            fenr_col = fenr_col,
            feature_col = feature_col,
            deg_pval_col = deg_pval_col,
            deg_pval_threshold = deg_pval_threshold,
            deg_logfc_col = deg_logfc_col,
            deg_logfc_threshold = deg_logfc_threshold,
            score_col = score_col,
            score_threshold = score_threshold,
            factor_type = factor_type,
            variable_map = variable_map,
            padj_method = padj_method,
            pval_thresh = pval_thresh,
            min_set = min_set,
            max_set = max_set
        )
    }

    if (feature_type %in% c(
        "degs_cor",
        "omics_cor",
        "factor_features_cor"
    )) {
        enr_res <- .run_feature_cor_enr(
            exposomicset = exposomicset,
            feature_type = feature_type,
            db = db,
            fdata = fdata,
            species = species,
            fenr_col = fenr_col,
            feature_col = feature_col,
            deg_pval_col = deg_pval_col,
            deg_pval_threshold = deg_pval_threshold,
            deg_logfc_col = deg_logfc_col,
            deg_logfc_threshold = deg_logfc_threshold,
            score_col = score_col,
            score_threshold = score_threshold,
            factor_type = factor_type,
            variable_map = variable_map,
            padj_method = padj_method,
            pval_thresh = pval_thresh,
            min_set = min_set,
            max_set = max_set
        )
    }

    # --- Group Terms ------------
    if (is.data.frame(enr_res) && nrow(enr_res) > 0L &&
        all(c("term_name", "ids") %in% names(enr_res))) {
        enr_res <- .group_enr_res(
            enr_res,
            clustering_approach = clustering_approach
        )
    }

    # --- Store or return ---
    if (action == "add") {
        all_metadata <- MultiAssayExperiment::metadata(exposomicset)
        all_metadata$enrichment[[feature_type]] <- enr_res
        MultiAssayExperiment::metadata(exposomicset) <- all_metadata

        step_record <- list(
            run_enrichment = list(
                timestamp = Sys.time(),
                params = list(
                    feature_type = feature_type,
                    score_col = score_col,
                    score_threshold = score_threshold,
                    variable_map = variable_map,
                    factor_type = factor_type,
                    feature_col = feature_col,
                    deg_pval_col = deg_pval_col,
                    deg_pval_threshold = deg_pval_threshold,
                    deg_logfc_col = deg_logfc_col,
                    deg_logfc_threshold = deg_logfc_threshold,
                    db = db,
                    species = species,
                    fenr_col = fenr_col,
                    padj_method = padj_method,
                    pval_thresh = pval_thresh,
                    clustering_approach = clustering_approach
                ),
                notes = paste0(
                    "Performed ",
                    db,
                    " enrichment on ",
                    feature_type,
                    " features."
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else if (action == "get") {
        return(enr_res)
    } else {
        stop("Invalid action. Choose from 'add' or 'get'.")
    }
}

# --- Run Fenr Enrichment Function ---------
#' @noRd
#'
#' @title Internal wrapper to run functional enrichment using `fenr`
#'
#' @description
#' Helper function to fetch database-specific term data,
#' prepare term mappings,
#' and perform enrichment analysis using the `fenr` package.
#'
#' @param selected_genes Character vector of selected gene identifiers.
#' @param universe_genes Character vector of all background/universe
#' gene identifiers.
#' @param db Character string specifying the database.
#' One of `"GO"`, `"KEGG"`, `"Reactome"`.
#' @param species Optional character string specifying species name
#' (required for GO).
#' @param fenr_col Column name in the term mappings corresponding to gene IDs
#' (default `"gene_symbol"`).
#'
#' @return A data frame of enrichment results from
#' `fenr::functional_enrichment()`, or `NULL` if enrichment fails.
#'
#' @importFrom MultiAssayExperiment metadata experiments
#' @importFrom SummarizedExperiment rowData
#' @importFrom dplyr filter inner_join pull mutate summarise
#' bind_rows select distinct arrange
#' @importFrom tidyr unnest pivot_wider
#' @importFrom purrr pluck map map2
#' @importFrom tibble column_to_rownames rownames_to_column
#' @importFrom stringr str_split str_trim
#' @importFrom rlang sym set_names .data
#' @importFrom stats p.adjust
#' @import fenr
#' @keywords internal
.run_fenr <- function(
  selected_genes,
  universe_genes,
  db = c("GO", "KEGG", "Reactome"),
  species = NULL,
  fenr_col = "gene_symbol"
) {
    # require(fenr)

    db <- match.arg(db)

    # avoid small selected gene sets
    if (length(selected_genes) <= 1L ||
        is.null(universe_genes) || all(is.na(universe_genes))) {
        return(tibble::tibble())
    }

    # Fetch functional terms + mapping
    fetch_fun <- switch(db,
        GO = fenr::fetch_go,
        KEGG = fenr::fetch_kegg,
        Reactome = fenr::fetch_reactome
    )

    # Fix for fenr given it is trying to assert a dead link exists
    if (db == "GO") {
        # url <- paste0("http://current.geneontology.org/annotations/", species, ".gaf.gz")
        url <- "http://current.geneontology.org/annotations"
        options(GO_ANNOTATION_URL = url)
    }

    # Handle species if required
    if (db == "GO" && is.null(species)) {
        stop("Please specify a species designation for GO.")
    }

    term_data <- tryCatch(
        {
            if (db == "GO") fetch_fun(species = species) else fetch_fun()
        },
        error = function(e) {
            message(e$message)
            return(NULL)
        }
    )
    if (is.null(term_data)) {
        return(tibble::tibble())
    }

    # Prepare for enrichment
    terms_obj <- tryCatch(
        {
            # may fail if there are no mappings
            fenr::prepare_for_enrichment(
                terms = term_data$terms,
                mapping = term_data$mapping,
                all_features = universe_genes,
                feature_name = fenr_col
            )
        },
        error = function(e) {
            message(e$message)
            return(NULL)
        }
    )
    if (is.null(terms_obj)) {
        return(tibble::tibble())
    }


    # Run enrichment
    out <- tryCatch(
        fenr::functional_enrichment(
            feat_all = universe_genes,
            feat_sel = selected_genes,
            term_data = terms_obj
        ),
        error = function(e) {
            # e.g., internal size assertions
            message(e$message)
            tibble::tibble()
        }
    )

    tibble::as_tibble(out)
}

# --- Run Enrichment on Features ------------
#' Run enrichment for feature-based selections across experiments
#'
#' This internal helper performs enrichment analysis for a given
#' \code{feature_type}
#' across all assays/experiments in an \code{exposomicset}
#' (a \code{MultiAssayExperiment}).
#' It selects features per experiment according to \code{feature_type},
#' computes the gene universe from \code{rowData()}, runs \code{.run_fenr()},
#' then applies multiple testing correction and size filtering.
#'
#' Supported \code{feature_type} values (no \code{*_cor} here):
#' \itemize{
#'   \item \code{"degs"} - select DEGs by adjusted p-value and absolute value
#'   of the logFC
#'   \item \code{"degs_robust"} - select features using
#'   sensitivity/stability scores
#'   \item \code{"factor_features"} - select features from multi-omics
#'    integration
#'   \item \code{"omics"} - select features using a \code{variable_map}
#' }
#'
#' @param exposomicset A \code{MultiAssayExperiment} with experiments and
#'  metadata.
#' @param feature_type Character; one of \code{"degs"}, \code{"degs_robust"},
#'   \code{"factor_features"}, \code{"omics"}.
#' @param db Enrichment database: one of \code{"GO"}, \code{"KEGG"},
#' \code{"Reactome"}.
#' @param species Species identifier required by the underlying database
#' (used by \code{.run_fenr}).
#' @param fenr_col Column name of gene IDs expected by the enrichment backend
#'  (e.g., \code{"gene_symbol"}).
#' @param feature_col Column in \code{rowData()} providing gene identifiers
#' for the universe and mapping.
#' @param deg_pval_col Column of adjusted p-values in DEG results
#'  (for \code{feature_type = "degs"}).
#' @param deg_pval_threshold Numeric threshold for adjusted p-values in
#' DEG selection.
#' @param deg_logfc_col Column of log-fold change values in DEG results
#' (for \code{feature_type = "degs"}).
#' @param deg_logfc_threshold Numeric threshold for absolute log-fold change
#' in DEG selection.
#' @param score_col Column name for stability/sensitivity score
#'  (for \code{"degs_robust"}).
#' @param score_threshold Optional numeric threshold; if \code{NULL},
#'  uses value determined in \code{"run_sensitivity_analysis"}.
#' @param factor_type Character; one of \code{"common_top_factor_features"}
#' or \code{"top_factor_features"}
#'   (for \code{"factor_features"}).
#' @param variable_map A data.frame with columns \code{exp_name} and
#' \code{variable} used when \code{feature_type = "omics"} to select
#'  features per experiment.
#' @param padj_method Multiple-testing correction method passed to
#' \code{stats::p.adjust} (default \code{"fdr"}).
#' @param pval_thresh Adjusted p-value filter applied to enrichment results
#' (default \code{0.1}).
#' @param min_set Minimum overlap size filter on enriched terms
#' (default \code{3}).
#' @param max_set Maximum overlap size filter on enriched terms
#' (default \code{800}).
#'
#' @return A \code{data.frame} (tibble) of enrichment results across
#' experiments with columns returned by \code{.run_fenr()} plus
#' \code{exp_name} and \code{padj}, filtered by \code{pval_thresh},
#' \code{min_set}, and \code{max_set}. If no results, returns an empty tibble.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom MultiAssayExperiment experiments metadata
#' @importFrom SummarizedExperiment rowData
#' @importFrom dplyr filter inner_join pull mutate bind_rows select
#' @importFrom rlang sym `%||%`
#' @importFrom stats p.adjust
#' @import fenr
.run_feature_enr <- function(
  exposomicset,
  feature_type,
  db,
  fdata,
  species,
  fenr_col,
  feature_col,
  deg_pval_col,
  deg_pval_threshold,
  deg_logfc_col,
  deg_logfc_threshold,
  score_col,
  score_threshold,
  factor_type,
  variable_map,
  padj_method,
  pval_thresh,
  min_set,
  max_set
) {
    # Get unique experiments
    exps <- names(MultiAssayExperiment::experiments(exposomicset))

    # Set variable for enrichment results
    enr_res <- list()

    for (exp in exps) {
        sprintf(
            "Working on '%s' %s %s enrichment analysis",
            feature_type, exp, db
        )

        # Subset universe from rowData of experiment
        universe_genes <- SummarizedExperiment::rowData(
            exposomicset[[exp]]
        )[[feature_col]] |>
            unique()

        # Select features per method
        selected_genes <- switch(feature_type,
            "degs" = {
                exposomicset@metadata |>
                    purrr::pluck("differential_analysis") |>
                    purrr::pluck("differential_abundance") |>
                    dplyr::filter(exp_name == exp) |>
                    dplyr::filter(
                        !!rlang::sym(deg_pval_col) < deg_pval_threshold,
                        abs(!!rlang::sym(deg_logfc_col)) > deg_logfc_threshold
                    ) |>
                    pull(!!rlang::sym(feature_col)) |>
                    unique()
            },
            "degs_robust" = {
                sens <- exposomicset@metadata |>
                    purrr::pluck("differential_analysis") |>
                    purrr::pluck("sensitivity_analysis")

                sens |>
                    purrr::pluck("feature_stability") |>
                    dplyr::filter(
                        exp_name == exp,
                        !!rlang::sym(score_col) > (
                            score_threshold %||% sens$score_thresh)
                    ) |>
                    dplyr::select(feature, exp_name) |>
                    dplyr::inner_join(
                        fdata |>
                            dplyr::filter(.exp_name == exp),
                        by = c(
                            "feature" = ".feature",
                            "exp_name" = ".exp_name"
                        )
                    ) |>
                    dplyr::pull(!!dplyr::sym(feature_col)) |>
                    unique()
            },
            "factor_features" = {
                exposomicset@metadata |>
                    purrr::pluck("differential_analysis") |>
                    purrr::pluck("multiomics_integration") |>
                    purrr::pluck(factor_type) |>
                    dplyr::filter(exp_name == exp) |>
                    dplyr::inner_join(
                        fdata |>
                            dplyr::filter(.exp_name == exp),
                        by = c(
                            "feature" = ".feature",
                            "exp_name" = ".exp_name"
                        )
                    ) |>
                    dplyr::pull(!!dplyr::sym(feature_col)) |>
                    unique()
            },
            "omics" = {
                variable_map |>
                    dplyr::filter(exp_name == exp) |>
                    dplyr::inner_join(
                        fdata |>
                            dplyr::filter(.exp_name == exp),
                        by = c(
                            "variable" = ".feature",
                            "exp_name" = ".exp_name"
                        )
                    ) |>
                    dplyr::pull(!!dplyr::sym(feature_col)) |>
                    unique()
            }
        )

        if (length(selected_genes) < 2) {
            enr <- NULL
        } else if (is.null(universe_genes) | all(is.na(universe_genes))) {
            enr <- NULL
        } else {
            # Run enrichment
            enr <- .run_fenr(
                selected_genes = selected_genes,
                universe_genes = universe_genes,
                db = db,
                species = species,
                fenr_col = fenr_col
            ) |>
                dplyr::mutate(exp_name = exp)
        }

        enr_res[[exp]] <- enr
    }

    if (all(purrr::map_lgl(enr_res, is.null))) {
        return(tibble::tibble())
    }

    # Multiple-hypothesis correction after all tests are done
    enr_res <- enr_res |>
        dplyr::bind_rows()

    if (nrow(enr_res) == 0L) {
        return(tibble::tibble())
    }

    enr_res <- enr_res |>
        dplyr::mutate(padj = p.adjust(p_value, method = padj_method)) |>
        dplyr::filter(padj < pval_thresh) |>
        dplyr::filter(
            n_with_sel > min_set,
            n_with_sel < max_set
        )

    return(enr_res)
}
# --- Run Enrichment on Exposure-Correlated Features ---------
#' Run correlation-based enrichment analysis for each (experiment, category)
#'
#' This internal helper performs enrichment analysis using correlation-derived
#' features across experiments and exposure categories.
#' It handles feature types ending in \code{"_cor"}:
#' \itemize{
#'   \item \code{"degs_cor"}: correlation between DEGs and exposures
#'   \item \code{"omics_cor"}: correlation between omics features and exposures
#'   \item \code{"factor_features_cor"}: correlation between factor features and exposures
#' }
#'
#' For each experiment by category pair, it:
#' \enumerate{
#'   \item Extracts the gene universe from \code{rowData()}.
#'   \item Selects features based on correlation metadata and
#'   \code{pivot_feature()}.
#'   \item Runs \code{.run_fenr()} with the selected genes.
#'   \item Annotates results with \code{exp_name} and \code{category}.
#' }
#' Then, it applies multiple testing correction and filters enriched
#' terms by adjusted p-value and gene set size.
#'
#' @inheritParams .run_feature_enr
#' @param feature_type One of \code{"degs_cor"}, \code{"omics_cor"},
#'  \code{"factor_features_cor"}.
#'
#' @return A data frame of enrichment results across experiments and
#' categories,
#' with columns returned by \code{.run_fenr()} plus \code{exp_name},
#'  \code{category}, and \code{padj}.
#'
#' @keywords internal
#' @noRd
#'
#' @importFrom purrr pluck
#' @importFrom MultiAssayExperiment metadata experiments
#' @importFrom SummarizedExperiment rowData
#' @importFrom dplyr filter inner_join pull mutate bind_rows select
#' @importFrom rlang sym
#' @importFrom stats p.adjust
#' @import fenr
.run_feature_cor_enr <- function(
  exposomicset,
  feature_type,
  db,
  fdata,
  species,
  fenr_col,
  feature_col,
  deg_pval_col,
  deg_pval_threshold,
  deg_logfc_col,
  deg_logfc_threshold,
  score_col,
  score_threshold,
  factor_type,
  variable_map,
  padj_method,
  pval_thresh,
  min_set,
  max_set
) {
    # Set the categories and experiment names to loop through
    cor_table <- exposomicset@metadata |>
        purrr::pluck("correlation") |>
        purrr::pluck(gsub("_.*", "", feature_type))

    exps <- unique(cor_table$exp_name)
    categories <- unique(cor_table$category)

    # Set variable for enrichment results
    enr_res <- list()

    for (cat_i in categories) {
        for (exp in exps) {
            sprintf(
                "Working on '%s' %s %s %s enrichment analysis",
                feature_type, exp, cat_i, db
            )
            # Subset universe from rowData of experiment
            universe_genes <- SummarizedExperiment::rowData(
                exposomicset[[exp]]
            )[[feature_col]] |>
                unique()

            # Select features per feature type
            selected_genes <- switch(feature_type,
                "degs_cor" = {
                    exposomicset@metadata |>
                        purrr::pluck("correlation") |>
                        purrr::pluck("degs") |>
                        dplyr::filter(
                            exp_name == exp,
                            category == cat_i
                        ) |>
                        dplyr::inner_join(
                            fdata |>
                                dplyr::filter(.exp_name == exp),
                            by = c(
                                "feature" = ".feature",
                                "exp_name" = ".exp_name"
                            )
                        ) |>
                        dplyr::pull(!!dplyr::sym(feature_col)) |>
                        unique()
                },
                "omics_cor" = {
                    exposomicset@metadata |>
                        purrr::pluck("correlation") |>
                        purrr::pluck("omics") |>
                        dplyr::filter(
                            exp_name == exp,
                            category == cat_i
                        ) |>
                        dplyr::inner_join(
                            fdata |>
                                dplyr::filter(.exp_name == exp),
                            by = c(
                                "feature" = ".feature",
                                "exp_name" = ".exp_name"
                            )
                        ) |>
                        dplyr::pull(!!dplyr::sym(feature_col)) |>
                        unique()
                },
                "factor_features_cor" = {
                    exposomicset@metadata |>
                        purrr::pluck("correlation") |>
                        purrr::pluck("factor_features") |>
                        dplyr::filter(
                            exp_name == exp,
                            category == cat_i
                        ) |>
                        dplyr::inner_join(
                            fdata |>
                                dplyr::filter(.exp_name == exp),
                            by = c(
                                "feature" = ".feature",
                                "exp_name" = ".exp_name"
                            )
                        ) |>
                        dplyr::pull(!!dplyr::sym(feature_col)) |>
                        unique()
                }
            )

            if (length(selected_genes) < 2) {
                enr <- NULL
            } else if (is.null(universe_genes) | all(is.na(universe_genes))) {
                enr <- NULL
            } else {
                # Run enrichment
                enr <- .run_fenr(
                    selected_genes = selected_genes,
                    universe_genes = universe_genes,
                    db = db,
                    species = species,
                    fenr_col = fenr_col
                ) |>
                    dplyr::mutate(exp_name = exp) |>
                    dplyr::mutate(category = cat_i)
            }

            enr_res[[exp]] <- enr
        }
    }

    if (all(purrr::map_lgl(enr_res, is.null))) {
        return(tibble::tibble())
    }

    # Multiple-hypothesis correction after all tests are done
    enr_res <- enr_res |>
        dplyr::bind_rows()

    if (nrow(enr_res) == 0L) {
        return(tibble::tibble())
    }

    enr_res <- enr_res |>
        dplyr::mutate(padj = p.adjust(p_value, method = padj_method)) |>
        dplyr::filter(padj < pval_thresh) |>
        dplyr::filter(
            n_with_sel > min_set,
            n_with_sel < max_set
        )

    return(enr_res)
}

# --- Group Enrichment Terms Function ------------
#' Group enriched GO terms into clusters based on gene overlap
#'
#' This internal function clusters enriched GO terms based on pairwise Jaccard
#' overlap of their gene lists. It is typically used downstream of enrichment
#' analysis to simplify redundant terms into groups.
#'
#' @param enr_res A data frame containing enrichment results. Must contain:
#'   - `term_name`: term identifier (e.g. GO term)
#'   - `ids`: comma-separated list of associated genes
#' @param clustering_approach Clustering method to use. Defaults to `"hclust"`.
#'
#' @return A data frame with an added `go_group` column indicating the cluster
#'   assignment for each term.
#'
#' @import dplyr
#' @import tidyr
#' @import stringr
#' @import tibble
#' @importFrom rlang set_names
#' @keywords internal
#' @noRd
.group_enr_res <- function(enr_res, clustering_approach = "diana") {
    if (nrow(enr_res) < 2) {
        message("GO clustering skipped - fewer than 2 enriched terms.")
        enr_res$go_group <- rep(NA_character_, nrow(enr_res))
        return(enr_res)
    }

    message("Determining Number of GO Term Clusters...")

    go_term_list <- enr_res |>
        dplyr::mutate(gene_list = stringr::str_split(ids, ",")) |>
        tidyr::unnest(gene_list) |>
        dplyr::mutate(gene_list = stringr::str_trim(gene_list)) |>
        dplyr::distinct(term_name, gene_list) |>
        dplyr::group_by(term_name) |>
        dplyr::summarise(genes = list(unique(gene_list)), .groups = "drop")

    if (nrow(go_term_list) < 2) {
        message("GO clustering skipped - fewer than 2 unique terms.")
        enr_res$go_group <- rep(NA_character_, nrow(enr_res))
        return(enr_res)
    }

    go_groups <- go_term_list$genes |>
        rlang::set_names(go_term_list$term_name) |>
        .get_pairwise_overlaps() |>
        dplyr::select(source, target, jaccard) |>
        tidyr::pivot_wider(names_from = "target", values_from = "jaccard") |>
        tibble::column_to_rownames("source") |>
        as.matrix() |>
        .cluster_mat(clustering_approach = clustering_approach) |>
        (\(x) data.frame(
            term_name = names(x),
            go_group = paste0("Group ", as.numeric(x))
        ))()

    if (is.null(go_groups) || nrow(go_groups) == 0) {
        message("GO clustering failed - no clusters found.")
        enr_res$go_group <- rep(NA_character_, nrow(enr_res))
        return(enr_res)
    }

    dplyr::inner_join(enr_res, go_groups, by = "term_name")
}

#' Plot Correlation Tilemap
#'
#' Visualizes a correlation matrix as a heatmap tile plot using
#' correlation results stored in the metadata of a
#' `MultiAssayExperiment` object. When `feature_type = "pcs"`,
#' the function forces PCs to appear on the x-axis and exposures on the y-axis,
#' and it adds a barplot showing how many PCs are significantly
#' associated with each exposure. It also suppresses nonsignificant tiles
#'  based on a specified p-value threshold.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing correlation
#' results in metadata.
#' @param feature_type Type of correlation results to plot.
#'  One of `"pcs"`, `"degs"`, `"omics"`,
#' `"factors"`, `"factor_features"`, or `"exposures"`.
#'  Must match the key used in
#' `metadata(exposomicset)$correlation[[feature_type]]`.
#' @param pval_cutoff Numeric p-value cutoff below which
#' correlations are displayed.
#' Nonsignificant tiles are rendered in the `na_color`.
#' Default is `0.05`.
#' @param na_color Color used to represent nonsignificant or
#'  missing correlations. Default is `"grey100"`.
#' @param fill_limits Numeric vector of length 2 defining the scale
#' limits for correlation values. Default is `c(-1, 1)`.
#' @param midpoint Numeric value for centering the fill gradient.
#' Default is `0`.
#'
#' @return A `ggplot2` tile plot (or a combined tile + barplot if
#'  `feature_type = "pcs"`).
#'
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run pca
#' mae <- mae |>
#'     run_pca()
#'
#' # correlate with exposures
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "pcs",
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # make the correlation tile plot
#' cor_tile_p <- mae |>
#'     plot_correlation_tile(
#'         feature_type = "pcs"
#'     )
#'
#' @export
plot_correlation_tile <- function(
  exposomicset,
  feature_type = c(
      "pcs",
      "degs",
      "omics",
      "factors",
      "factor_features",
      "exposures"
  ),
  pval_cutoff = 0.05,
  na_color = "grey100",
  fill_limits = c(-1, 1),
  midpoint = 0
) {
    .check_suggested(pkg = "patchwork")
    feature_type <- match.arg(feature_type)

    correlation_list <- MultiAssayExperiment::metadata(exposomicset)$correlation
    if (is.null(correlation_list) || !feature_type %in% names(correlation_list)) {
        stop(
            "Correlation results for feature_type '",
            feature_type,
            "' not found in metadata."
        )
    }

    correlation_df <- correlation_list[[feature_type]]

    if (!all(c("var1", "var2", "correlation", "p.value") %in%
        colnames(correlation_df))) {
        cols_needed <- "var1, var2, correlation, p.value."
        stop("Correlation data must contain columns: ", cols_needed)
    }

    extract_pc_number <- function(x) {
        as.numeric(stringr::str_extract(x, "\\d+"))
    }

    if (feature_type == "pcs") {
        # Identify PCs in each var
        is_pc1 <- grepl("^PC\\d+$", correlation_df$var1)
        is_pc2 <- grepl("^PC\\d+$", correlation_df$var2)

        # Keep only PC vs non-PC pairs
        keep <- xor(is_pc1, is_pc2)
        df <- correlation_df[keep, , drop = FALSE]

        # Assign PC to x-axis, exposure to y-axis
        df <- df |>
            dplyr::mutate(
                var_pc = dplyr::case_when(is_pc1 ~ var1,
                    is_pc2 ~ var2,
                    .default = NA_character_
                ),
                var_exp = dplyr::case_when(is_pc1 ~ var2,
                    is_pc2 ~ var1,
                    .default = NA_character_
                ),
                correlation = dplyr::case_when(
                    p.value < pval_cutoff ~ correlation,
                    .default = NA_real_
                )
            ) |>
            dplyr::filter(!is.na(var_pc), !is.na(var_exp)) |>
            dplyr::mutate(
                var_pc = factor(var_pc, levels = unique(var_pc)[
                    order(extract_pc_number(unique(var_pc)))
                ])
            )

        # Count significant PC associations per exposure
        bar_df <- df |>
            dplyr::filter(!is.na(correlation)) |>
            dplyr::count(var_exp, name = "n")

        # Order exposures by number of PC associations
        bar_df <- bar_df |>
            dplyr::arrange(n) |>
            dplyr::mutate(var_exp = factor(var_exp, levels = var_exp))

        # Reorder var_exp in df accordingly
        df <- df |>
            dplyr::filter(var_exp %in% bar_df$var_exp) |>
            dplyr::mutate(var_exp = factor(var_exp,
                levels = levels(bar_df$var_exp)
            ))

        # Create heatmap
        heatmap <- ggplot2::ggplot(df, ggplot2::aes(
            x = var_pc,
            y = var_exp,
            fill = correlation
        )) +
            ggplot2::geom_tile() +
            ggplot2::scale_fill_gradient2(
                low = "blue4", high = "red4", mid = "white",
                midpoint = midpoint, limits = fill_limits,
                na.value = na_color, name = "Correlation"
            ) +
            ggplot2::theme_bw() +
            ggplot2::labs(
                x = "",
                y = ""
            ) +
            ggpubr::rotate_x_text(90)

        # Create barplot
        bar_plot <- ggplot2::ggplot(
            bar_df,
            ggplot2::aes(
                x = n,
                y = var_exp
            )
        ) +
            ggplot2::geom_col(fill = "grey40") +
            ggplot2::theme_classic() +
            ggplot2::labs(
                x = "No. of PCs",
                y = NULL
            ) +
            ggplot2::theme(
                axis.text.y = ggplot2::element_blank(),
                axis.ticks.y = ggplot2::element_blank()
            )

        # Combine plots
        combined_plot <- (heatmap + bar_plot) +
            patchwork::plot_layout(
                widths = c(4, 1),
                guides = "collect"
            ) &
            ggplot2::theme(legend.position = "right")

        return(combined_plot)
    } else {
        # Handle symmetric correlation matrices
        df <- correlation_df |>
            dplyr::filter(var1 != var2) |>
            dplyr::mutate(
                var_min = pmin(var1, var2),
                var_max = pmax(var1, var2)
            ) |>
            dplyr::select(-var1, -var2) |>
            dplyr::rename(var1 = var_min, var2 = var_max) |>
            dplyr::mutate(
                correlation = dplyr::case_when(
                    p.value < pval_cutoff ~ correlation,
                    .default = NA_real_
                )
            )

        p <- ggplot2::ggplot(df, ggplot2::aes(
            x = var2,
            y = var1,
            fill = correlation
        )) +
            ggplot2::geom_tile() +
            ggplot2::scale_fill_gradient2(
                low = "blue4", high = "red4", mid = "white",
                midpoint = midpoint, limits = fill_limits,
                na.value = na_color, name = "Correlation"
            ) +
            ggplot2::theme_bw() +
            ggplot2::labs(x = NULL, y = NULL) +
            ggpubr::rotate_x_text(angle = 90)

        return(p)
    }
}

#' Plot Summary of Factor Contributions from Multi-Omics Integration
#'
#' Generates a summary plot of factor contributions from multi-omics
#' integration results stored in a `MultiAssayExperiment` object.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing integration
#' results in
#'   `metadata(exposomicset)$multiomics_integration$integration_results`.
#' @param low Color for low values in the fill gradient.
#' Default is `"#006666"`.
#' @param mid Color for midpoint in the fill gradient.
#' Default is `"white"`.
#' @param high Color for high values in the fill gradient.
#' Default is `"#8E0152"`.
#' @param midpoint Midpoint value for the gradient color scale.
#' Default is `0.5`.
#'
#' @details
#' This function visualizes factor contributions based on the integration method:
#'
#' - **MOFA**: Variance explained per factor and view.
#' - **MCIA**: Block score weights per omic.
#' - **DIABLO**: Mean absolute sample score per omic and factor
#' (from block-specific variates).
#' - **RGCCA**: Mean absolute sample score per omic and factor
#' (from aligned block scores).
#'
#' The color gradient can be customized using the `low`, `mid`, `high`,
#' and `midpoint` parameters.
#'
#' @return A `ggplot` object showing factor contributions based on
#'  the integration method.
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 20,
#'     return_mae = TRUE
#' )
#'
#' mae <- run_multiomics_integration(
#'     mae,
#'     method = "DIABLO",
#'     outcome = "smoker",
#'     n_factors = 3
#' )
#'
#' factor_sum_plot <- mae |>
#'     plot_factor_summary()
#'
#' @importFrom ggplot2 ggplot aes geom_tile scale_fill_gradient2 labs
#'  theme_minimal element_text
#' @importFrom ggpubr theme_pubr rotate_x_text
#' @importFrom tidyr pivot_longer
#' @importFrom tibble rownames_to_column
#' @importFrom dplyr mutate bind_rows select distinct
#' @importFrom purrr discard imap map
#' @export
plot_factor_summary <- function(
  exposomicset,
  low = "#006666",
  mid = "white",
  high = "#8E0152",
  midpoint = 0.5
) {
    # require("ggplot2")
    # require("ggpubr")

    integration <- MultiAssayExperiment::metadata(exposomicset) |>
        purrr::pluck(
            "multiomics_integration",
            "integration_results"
        )

    if (is.null(integration) || is.null(integration$result)) {
        stop("Please run `run_multiomics_integration()` first.")
    }

    method <- integration$method
    result <- integration$result

    factor_contrib_plot <- switch(method,
        "MOFA" = {
            .check_suggested("MOFA2")
            MOFA2::plot_variance_explained(
                result,
                x = "view",
                y = "factor"
            ) +
                ggpubr::rotate_x_text(45) +
                scale_fill_gradient2(
                    low = low,
                    mid = mid,
                    high = high,
                    midpoint = midpoint
                )
        },
        "MCIA" = {
            result@block_score_weights |>
                as.data.frame() |>
                tibble::rownames_to_column("omic") |>
                tidyr::pivot_longer(!omic,
                    names_to = "factor",
                    values_to = "weight"
                ) |>
                dplyr::mutate(
                    factor = gsub("V", "", factor),
                    factor = factor(as.numeric(factor),
                        levels = sort(unique(as.numeric(factor)))
                    )
                ) |>
                ggplot(aes(
                    x = factor,
                    y = omic,
                    fill = weight
                )) +
                geom_tile() +
                ggpubr::theme_pubr(legend = "right") +
                scale_fill_gradient2(
                    low = low,
                    mid = mid,
                    high = high,
                    midpoint = midpoint
                ) +
                labs(
                    x = "Factor",
                    y = NULL,
                    fill = "Weight"
                )
        },

        # "MCCA" = {
        #   result$sample_scores |>
        #     purrr::map(~ apply(.x, 2, function(x) mean(abs(x))) |>
        #                  as.data.frame() |>
        #                  tibble::rownames_to_column("factor") |>
        #                  setNames(c("factor", "weight"))) |>
        #     dplyr::bind_rows(.id = "omic") |>
        #     dplyr::mutate(
        #       factor = factor(as.numeric(factor),
        #                       levels = sort(unique(as.numeric(factor))))
        #     ) |>
        #     ggplot(aes(x = factor, y = omic, fill = weight)) +
        #     geom_tile() +
        #     ggpubr::theme_pubr(legend = "right") +
        #     scale_fill_gradient2(low = low, mid = mid, high = high, midpoint = midpoint) +
        #     labs(x = "Factor", y = "", fill = "Avg |Score|")
        # },
        "DIABLO" = {
            result$variates |>
                (\(lst) lst[names(lst) != "Y"])() |>
                purrr::discard(~ is.null(.x) || is.character(.x) || is.factor(.x)) |>
                purrr::imap(~ apply(.x, 2, function(x) mean(abs(x))) |>
                    as.data.frame() |>
                    tibble::rownames_to_column("factor") |>
                    setNames(c("factor", "weight"))) |>
                dplyr::bind_rows(.id = "omic") |>
                dplyr::mutate(
                    factor = gsub("comp", "", factor),
                    factor = factor(as.numeric(factor),
                        levels = sort(unique(as.numeric(factor)))
                    )
                ) |>
                ggplot(aes(x = factor, y = omic, fill = weight)) +
                geom_tile() +
                ggpubr::theme_pubr(legend = "right") +
                scale_fill_gradient2(
                    low = low,
                    mid = mid,
                    high = high,
                    midpoint = midpoint
                ) +
                labs(
                    x = "Factor",
                    y = "",
                    fill = "Avg. |Score|"
                )
        },
        "RGCCA" = {
            result$Y |>
                purrr::imap(~ apply(.x, 2, function(x) mean(abs(x))) |>
                    as.data.frame() |>
                    tibble::rownames_to_column("factor") |>
                    setNames(c("factor", "weight"))) |>
                dplyr::bind_rows(.id = "omic") |>
                dplyr::mutate(
                    factor = gsub("comp", "", factor),
                    factor = factor(as.numeric(factor),
                        levels = sort(unique(as.numeric(factor)))
                    )
                ) |>
                ggplot(aes(x = factor, y = omic, fill = weight)) +
                geom_tile() +
                ggpubr::theme_pubr(legend = "right") +
                scale_fill_gradient2(
                    low = low,
                    mid = mid,
                    high = high,
                    midpoint = midpoint
                ) +
                labs(
                    x = "Factor",
                    y = "",
                    fill = "Avg. |Score|"
                )
        },
        stop("Integration method not supported in plot_factor_summary().")
    )

    return(factor_contrib_plot)
}

#' Plot Correlation Summary from Exposure-Feature Correlations
#'
#' Generates a bar plot summary of exposure-feature correlations
#' using customizable modes.
#'
#' @param exposomicset A `MultiAssayExperiment` object with
#' correlation results in metadata.
#' @param feature_type One of `"degs"`, `"factors"`,
#' `"omics"`, or `"exposures"`.
#' @param mode One of:
#'   - `"top_exposures"`: Top exposures by assay
#'   - `"top_features"`: Top features by exposure category
#'   - `"exposure_category"`: Total associations by exposure category
#'   - `"assay"`: Total associations by omics assay
#'   - `"summary"`: Patchwork layout combining all
#' @param top_n Number of top exposures or features to display
#' (for top modes). Default is `15`.
#'
#' @return A `ggplot2` object or a `patchwork` object (if `mode = "summary"`).
#'
#' @examples
#'
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # correlate with exposures
#' mae <- mae |>
#'     run_correlation(
#'         feature_type = "omics",
#'         variable_map = mae |>
#'             pivot_feature() |>
#'             dplyr::select(
#'                 variable = .feature,
#'                 exp_name = .exp_name
#'             ),
#'         exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
#'     )
#'
#' # create the correlation summary plot
#' cor_summary_plot <- mae |>
#'     plot_correlation_summary(
#'         feature_type = "omics",
#'         mode = "summary"
#'     )
#'
#' @importFrom dplyr group_by reframe mutate ungroup arrange pull filter select
#' @importFrom ggplot2 ggplot aes geom_bar geom_segment labs theme element_text
#' @export
plot_correlation_summary <- function(
  exposomicset,
  feature_type = c(
      "degs",
      "omics",
      "factors",
      "factor_features",
      "exposures",
      "pcs"
  ),
  mode = c(
      "top_exposures",
      "top_features",
      "exposure_category",
      "assay",
      "summary"
  ),
  top_n = 15
) {
    .check_suggested(pkg = "janitor")
    .check_suggested(pkg = "forcats")
    .check_suggested(pkg = "patchwork")

    feature_type <- match.arg(feature_type)
    mode <- match.arg(mode)

    # Select correlation data
    cor_list <- MultiAssayExperiment::metadata(exposomicset)$correlation
    cor_data <- cor_list[[feature_type]]
    if (is.null(cor_data)) {
        stop("No correlation results found for type: ", feature_type)
    }

    # Helper plots
    plot_top_exposures <- function() {
        cor_data |>
            dplyr::group_by(exposure, exp_name) |>
            dplyr::reframe(n_exp_name = dplyr::n()) |>
            dplyr::group_by(exposure) |>
            dplyr::mutate(total = sum(n_exp_name)) |>
            dplyr::ungroup() |>
            dplyr::filter(exposure %in% head(
                cor_data |>
                    janitor::tabyl(exposure) |>
                    dplyr::arrange(desc(n)) |>
                    dplyr::pull(exposure),
                top_n
            )) |>
            ggplot2::ggplot(ggplot2::aes(
                x = n_exp_name,
                y = forcats::fct_reorder(exposure, total),
                fill = exp_name
            )) +
            ggplot2::geom_bar(stat = "identity", alpha = 0.7) +
            scale_fill_tidy_exp(rev = TRUE) +
            ggpubr::theme_pubr() +
            ggplot2::labs(
                title = "Top Exposures by Assay",
                x = "No. of Associations", y = "", fill = "Assay"
            ) +
            theme(plot.title = element_text(face = "bold.italic"))
    }

    plot_top_features <- function() {
        cor_data |>
            dplyr::group_by(feature, category) |>
            dplyr::reframe(n_category = dplyr::n()) |>
            dplyr::group_by(feature) |>
            dplyr::mutate(total = sum(n_category)) |>
            dplyr::ungroup() |>
            dplyr::filter(feature %in% head(
                cor_data |>
                    janitor::tabyl(feature) |>
                    dplyr::arrange(desc(n)) |>
                    dplyr::pull(feature),
                top_n
            )) |>
            ggplot2::ggplot(ggplot2::aes(
                x = n_category,
                y = forcats::fct_reorder(feature, total),
                fill = category
            )) +
            ggplot2::geom_bar(stat = "identity", alpha = 0.7) +
            scale_fill_tidy_exp() +
            ggpubr::theme_pubr() +
            ggplot2::labs(
                title = "Top Features by Exposure Category",
                x = "No. of Associations", y = "", fill = "Category"
            ) +
            theme(plot.title = element_text(face = "bold.italic"))
    }

    plot_exposure_categories <- function() {
        cor_data |>
            janitor::tabyl(category) |>
            ggplot2::ggplot(ggplot2::aes(
                x = n,
                y = reorder(category, n),
                fill = category
            )) +
            ggplot2::geom_bar(stat = "identity", alpha = 0.7) +
            ggplot2::geom_segment(ggplot2::aes(
                xend = n, x = n,
                yend = as.numeric(reorder(category, n)) + 0.45,
                y = as.numeric(reorder(category, n)) - 0.45,
                color = category
            ), linewidth = 1) +
            scale_fill_tidy_exp() +
            scale_color_tidy_exp() +
            ggpubr::theme_pubr() +
            ggplot2::labs(
                title = "Exposure Associations by Category",
                x = "Count",
                y = ""
            ) +
            theme(plot.title = element_text(face = "bold.italic")) +
            labs(
                fill = "Category",
                color = "Category"
            )
    }

    plot_assays <- function() {
        cor_data |>
            janitor::tabyl(exp_name) |>
            ggplot2::ggplot(ggplot2::aes(
                x = n,
                y = reorder(exp_name, n),
                fill = exp_name
            )) +
            ggplot2::geom_bar(stat = "identity", alpha = 0.7) +
            ggplot2::geom_segment(ggplot2::aes(
                xend = n, x = n,
                yend = as.numeric(reorder(exp_name, n)) + 0.45,
                y = as.numeric(reorder(exp_name, n)) - 0.45,
                color = exp_name
            ), linewidth = 1) +
            scale_fill_tidy_exp(rev = TRUE) +
            scale_color_tidy_exp(rev = TRUE) +
            ggpubr::theme_pubr() +
            ggplot2::labs(
                title = "Omics Associations by Assay",
                x = "Count",
                y = ""
            ) +
            theme(plot.title = element_text(face = "bold.italic")) +
            labs(
                fill = "Assay",
                color = "Assay"
            )
    }

    # Return appropriate plot
    switch(mode,
        top_exposures = plot_top_exposures(),
        top_features = plot_top_features(),
        exposure_category = plot_exposure_categories(),
        assay = plot_assays(),
        summary = {
            left <- (plot_top_features() +
                theme(legend.position = "none")) /
                (plot_exposure_categories() +
                    theme(
                        legend.position = "bottom",
                        legend.direction = "vertical"
                    )) +
                patchwork::plot_layout(heights = c(3, 1))

            right <- (plot_top_exposures() +
                theme(legend.position = "none")) /
                (plot_assays() +
                    theme(
                        legend.position = "bottom",
                        legend.direction = "vertical"
                    )) +
                patchwork::plot_layout(heights = c(3, 1))

            left | right
        }
    )
}

#' Perform Principal Component Analysis (PCA)
#'
#' Runs PCA on the feature and sample spaces of a `MultiAssayExperiment` object,
#' identifying outliers based on Mahalanobis distance.
#'
#' @param exposomicset A `MultiAssayExperiment` object containing omics
#' and exposure data.
#' @param log_trans_exp A boolean value specifying whether to log2
#' transform the exposure data
#' @param log_trans_omics a boolean value specifying whether to log2
#' transform the omics data
#' @param action A character string specifying whether to store
#' (`"add"`) or return (`"get"`) the results. Default is `"add"`.
#'
#' @details
#' This function:
#' - Identifies **common samples** across all assays and exposure data.
#' - Performs **PCA on features** (transformed and standardized).
#' - Performs **PCA on samples** and computes Mahalanobis distance to
#' detect outliers.
#'
#' @return
#' If `action = "add"`, the function returns the input
#' `MultiAssayExperiment` with:
#'
#' - **PC scores** added as columns in `colData(exposomicset)`, and
#' - **PCA objects** stored under
#'   `metadata(exposomicset)$quality_control$pca`.
#'
#' If `action = "get"`, the function returns a list containing:
#' \item{pca_df}{A tibble of the transformed input data.}
#' \item{pca_feature}{A `prcomp` object containing PCA results for features.}
#' \item{pca_sample}{A `prcomp` object containing PCA results for samples.}
#' \item{outliers}{A character vector of detected sample outliers.}
#'
#' @examples
#' # create example data
#' mae <- make_example_data(
#'     n_samples = 10,
#'     return_mae = TRUE
#' )
#'
#' # run pca
#' mae <- mae |>
#'     run_pca()
#'
#' @export
run_pca <- function(
  exposomicset,
  log_trans_exp = FALSE,
  log_trans_omics = TRUE,
  action = "add"
) {
    # Identify common samples across all data
    message("Identifying common samples.")

    common_samples <- rownames(MultiAssayExperiment::colData(exposomicset))
    for (omics_name in names(MultiAssayExperiment::experiments(exposomicset))) {
        common_samples <- intersect(
            common_samples,
            colnames(MultiAssayExperiment::experiments(
                exposomicset
            )[[omics_name]])
        )
    }

    if (length(common_samples) == 0) {
        stop("No common samples found across exposure and omics data.")
    }

    # Subset colData to common samples
    message("Subsetting exposure data.")

    exposure_data <- MultiAssayExperiment::colData(
        exposomicset
    )[common_samples, ] |>
        as.data.frame() |>
        dplyr::select_if(is.numeric) |>
        t() |>
        as.data.frame()
    exposure_data <- transform(exposure_data, category = "exposure")

    # Remove exposures with any NA values
    exposure_data <- exposure_data[rowSums(is.na(exposure_data)) == 0, , drop = FALSE]

    if (log_trans_exp) {
        exposure_data <- exposure_data |>
            dplyr::mutate(
                dplyr::across(dplyr::where(is.numeric), ~ .safe_log2(. + abs(min(.)) + 1))
            )
    }

    # Subset omics data to common samples
    message("Subsetting omics data.")

    omics_data <- lapply(
        names(MultiAssayExperiment::experiments(exposomicset)),
        function(omics_name) {
            SummarizedExperiment::assays(
                MultiAssayExperiment::experiments(exposomicset)[[omics_name]]
            )[[1]][
                , common_samples,
                drop = FALSE
            ] |>
                as.data.frame() |>
                transform(category = omics_name)
        }
    ) |>
        dplyr::bind_rows()

    if (log_trans_omics) {
        omics_data <- omics_data |>
            dplyr::mutate(
                dplyr::across(dplyr::where(is.numeric), ~ .safe_log2(. + abs(min(.)) + 1))
            )
    }

    # Combine datasets
    dat <- exposure_data |>
        dplyr::bind_rows(omics_data)
    dat <- transform(dat, id = rownames(dat))

    # Remove zero-variance columns
    feature_data <- dat |>
        dplyr::select(-c(category, id)) |>
        dplyr::select(where(\(x) var(x, na.rm = TRUE) > 0))

    # PCA analysis: feature space
    message("Performing PCA on Feature Space.")
    pca_feature <- prcomp(feature_data, center = TRUE, scale. = TRUE)


    # PCA analysis: sample space
    message("Performing PCA on Sample Space.")
    sample_data <- dat |>
        dplyr::select(-c(category, id)) |>
        t() |>
        as.data.frame()

    sample_vars <- apply(sample_data, 2, \(x) var(x, na.rm = TRUE))
    sample_data <- sample_data |>
        dplyr::select(names(sample_vars[sample_vars > 0]))

    pca_sample <- prcomp(sample_data, center = TRUE, scale. = TRUE)

    # Compute Mahalanobis distance
    pca_coords <- pca_sample$x[, c("PC1", "PC2")] # Adjust if using more PCs
    cov_mat <- cov(pca_coords) # Covariance matrix
    inv_cov <- solve(cov_mat) # Inverse covariance
    center <- colMeans(pca_coords) # Mean vector

    mahal_dist <- apply(pca_coords, 1, function(x) {
        sqrt(t(x - center) %*% inv_cov %*% (x - center))
    })

    # Determine threshold (Chi-square with 2 degrees of freedom, p < 0.01)
    threshold <- qchisq(0.99, df = 2)
    outliers <- which(mahal_dist^2 > threshold)

    # Print outlier sample names
    if (length(outliers) > 0) {
        message(paste(
            "Outliers detected:",
            paste(rownames(pca_sample$x)[outliers],
                collapse = ", "
            )
        ))
    } else {
        message("No outliers detected.")
    }

    # add in PCs for samples to the colData
    col_data <- MultiAssayExperiment::colData(exposomicset) |>
        as.data.frame() |>
        (\(df){
            df$id_to_map <- rownames(df)
            df
        })() |>
        dplyr::left_join(
            pca_sample$x |>
                as.data.frame() |>
                (\(df){
                    df$id_to_map <- rownames(df)
                    df
                })(),
            by = "id_to_map"
        ) |>
        tibble::column_to_rownames("id_to_map")

    MultiAssayExperiment::colData(exposomicset) <- col_data

    if (action == "add") {
        # Store results
        MultiAssayExperiment::metadata(exposomicset)$quality_control$pca <- list(
            pca_df = tibble(dat),
            pca_feature = pca_feature,
            pca_sample = pca_sample,
            outliers = rownames(pca_sample$x)[outliers]
        )

        # Add analysis steps taken to metadata
        step_record <- list(
            run_pca = list(
                timestamp = Sys.time(),
                params = list(),
                notes = paste("Outliers: ",
                    paste(rownames(pca_sample$x)[outliers], collapse = ", "),
                    collapse = ""
                )
            )
        )

        MultiAssayExperiment::metadata(exposomicset)$summary$steps <- c(
            MultiAssayExperiment::metadata(exposomicset)$summary$steps,
            step_record
        )

        return(exposomicset)
    } else if (action == "get") {
        return(list(
            pca_df = tibble(dat),
            pca_feature = pca_feature,
            pca_sample = pca_sample,
            outliers = rownames(pca_sample$x)[outliers]
        ))
    } else {
        stop("Invalid action. Use 'add' or 'get'.")
    }
}


#' Safe Log2 Transformation
#'
#' This helper function ensures that negative, infinite, or `NaN` values
#' do not cause errors during log transformation. It converts all
#' negative values to zero, replaces `Inf`/`NaN` with `NA`, and performs
#' `log2(x + 1)` on the adjusted input.
#'
#' @param x A numeric vector to be log2-transformed.
#' @return A numeric vector of the same length as `x`, with safely
#' log2-transformed values.
#'
#' @noRd
#' @keywords internal
.safe_log2 <- function(x) {
    x <- as.numeric(x)
    x <- ifelse(is.infinite(x) | is.nan(x), NA, x)
    log2(pmax(x, 0) + 1)
}

---
title: "Exposure Annotation"
author: "Jason Laird"
header-includes:
   - \usepackage{amsmath}
   - \usepackage{amsfonts}
output:
  BiocStyle::html_document:
    toc_float: true
    toc: true
vignette: >
  %\VignetteIndexEntry{tidyexposomics: Exposure Annotation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r setup,include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Exposure Metadata and Ontology Annotation
<hr>

## Codebook Setup

Before starting an exposomics data analysis we recommend having a codebook, with information on your exposure variables. Some suggestions:

- **Variable Name**: The name of the variable in the data set.

- **Variable Description**: A concise description of what the variable measures, including units (e.g., “urinary bisphenol A (ng/mL)”).

- **Variable Type**: The type of variable, such as continuous, categorical, or binary.

- **Variable Period**: The period of time over which the variable was measured, such as “lifetime”, “year”, “month”, or “day”.

- **Variable Location**: The location where the variable was measured, such as “home”, “work”, “school”, or “geospatial code”.

- **Variable Ontology**: The ontology term associated with the variable.

## Ontology Choices

Variables captured in the codebook should be annotated with ontology terms to provide a standardized vocabulary for the variables. We recommend using the following ontologies for exposure and outcome variables:

- [Environment Exposure Ontology](https://www.ebi.ac.uk/ols4/ontologies/ecto) 
to annotate your exposure variables. 

- [Human Phenotype Ontology](https://www.ebi.ac.uk/ols4/ontologies/hp) to
annotate your outcome variables and phenotypic data.

- [Chemical Entities of Biological Interest](https://www.ebi.ac.uk/ols4/ontologies/chebi) 
to annotate your chemical exposure variables.

**Why annotate with ontologies?**

- **Interpretability**: Ontology labels clarify ambiguous or inconsistently named variables.

- **Harmonization**: You can compare and combine variables across datasets when they map to the same term.

- **Grouping**: Ontologies allow you to collapse fine-grained exposures into broader categories.

- **Integration**: Many public tools, knowledge graphs, and repositories are ontology-aware. This can make your results more interoperable and reusable.

## Ontology Annotation App

To help annotate exposure variables, we provide a lightweight shiny app:

```{r ontology-app,eval=FALSE}
# Launch the shiny app to annotate exposure variables
shiny::runApp(build_ont_annot_app())
```

**To use the app:**

- Click **Browse** to select your exposure metadata file.

- Then you can click the variable you’d like to link to an annotation term and search in the **Choose Ontology Term** dropdown.

- After you select a term, you will see a short description of the term.

- After you are done, click **Apply Annotate** to save the annotation.

- Now you can group exposures into larger categories by selecting each line and then choose your ontology and root depth level (where a lower number means a more general term).

- Then you can click **Apply Categorization** to apply the selected categorization to the selected rows.

- If the ontology has nothing to do with your variable, you may manually enter a category in the **Category** column. This will change the **Category Source** to manual and will not be linked to the ontology.

- Once you have annotated all your variables click **Download Annotated CSV** to save the annotated metadata file.


```{r ontology-app-img, echo=FALSE,out.width="150%",fig.align='center',fig.cap="Screenshot of the ontology annotation app. The sidebar has an upload button to load your exposure metadata file. The main panel displays the uploaded exposure metadata where users can select variables to annotate and categorize. The sidebar also contains buttons to apply annotations/categorizations and download the annotated file."}
knitr::include_graphics("./ont_annot.png")
```

## Session Info

```{r session_info}

sessionInfo()
```

---
title: "Exposome Scores"
author: "Jason Laird"
header-includes:
   - \usepackage{amsmath}
   - \usepackage{amsfonts}
output:
  BiocStyle::html_document:
    toc_float: true
    toc: true
vignette: >
  %\VignetteIndexEntry{tidyexposomics: Exposure Scores}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r setup,include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## Load Data and Libraries

```{r load-libraries,message=FALSE,warning=FALSE}
# Load Libraries
library(tidyverse)
library(tidyexposomics)
```

We will start off with our example dataset pulled from the [ISGlobal Exposome Data Challenge 2021](https://doi.org/10.1016/j.envint.2022.107422) (Maitre et al., 2022). 

```{r load-data}
# Load example data
data("tidyexposomics_example")

# Create exposomic set object
expom <- create_exposomicset(
    codebook = tidyexposomics_example$annotated_cb,
    exposure = tidyexposomics_example$meta,
    omics = list(
        "Gene Expression" = tidyexposomics_example$exp_filt,
        "Methylation" = tidyexposomics_example$methyl_filt
    ),
    row_data = list(
        "Gene Expression" = tidyexposomics_example$exp_fdata,
        "Methylation" = tidyexposomics_example$methyl_fdata
    )
)
```

We will focus on a few exposure variable categories.

```{r exp-vars}
# Grab exposure variables
exp_vars <- tidyexposomics_example$annotated_cb |>
    filter(category %in% c(
        "aerosol",
        "main group molecular entity",
        "polyatomic entity"
    )) |>
    pull(variable) |>
    as.character()
```

# Quality Control
<hr>

As in the main vignette, we will impute exposure data using `missforest`.

```{r impute-missing}

# Impute missing values
expom <- run_impute_missing(
    exposomicset = expom,
    exposure_impute_method = "missforest",
    exposure_cols = exp_vars
)
```


And we will transform our exposure data to ensure it is more normally distributed using the `boxcox_best` method.

```{r trasform-vars}

# Transform variables
expom <- transform_exposure(
    exposomicset = expom,
    transform_method = "boxcox_best",
    exposure_cols = exp_vars
)
```

## Exposome Scores
<hr>

We can calculate exposome scores, which are a summary measure of exposure. The `run_exposome_score` function is used to calculate the exposome score. The `exposure_cols` argument is used to set the columns to use for the exposome score. The `score_type` argument is used to set the type of score to calculate. Here we could use:

- `median`: Calculates the median of the exposure variables.

- `mean`: Calculates the mean of the exposure variables.

- `sum`: Calculates the sum of the exposure variables.

- `pca`: Calculates the first principal component of the exposure variables.

- `irt`: Uses Item Response Theory to calculate the exposome score.

- `quantile`: Calculates the quantile of the exposure variables.

- `var`: Calculates the variance of the exposure variables.

The `score_column_name` argument is used to set the name of the column to store the exposome score in. Here we will define a score for aerosols using a variety of different methods and demonstrate their use in association with asthma status.

```{r calc-exposome, results='hide'}

# determine which aerosol variables to use
aerosols <- c("h_pm25_ratio_preg_None", "h_pm10_ratio_preg_None")

# Create exposome scores
expom <- expom |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "median",
        score_column_name = "exposome_median_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "pca",
        score_column_name = "exposome_pca_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "irt",
        score_column_name = "exposome_irt_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "quantile",
        score_column_name = "exposome_quantile_score"
    ) |>
    run_exposome_score(
        exposure_cols = aerosols,
        score_type = "var",
        score_column_name = "exposome_var_score"
    )
```


We can then associate these exposome scores with asthma status using the `run_association` function, just like we did before. However, this time we specify our `feature_set` to be the exposome scores we just calculated.

```{r associate-exposome-score}

# Associate exposome scores with outcome
expom <- run_association(
    exposomicset = expom,
    outcome = "hs_asthma",
    source = "exposures",
    feature_set = c(
        "exposome_median_score",
        "exposome_pca_score",
        "exposome_irt_score",
        "exposome_quantile_score",
        "exposome_var_score"
    ),
    action = "add",
    family = "binomial"
)
```

```{r plot-exposome-scores, fig.height=2.5, fig.width=5,fig.cap="Associations of aerosol exposome scores with asthma status. The variance-based score has the strongest association with asthma status."}

# Plot the association forest plot
plot_association(
    exposomicset = expom,
    source = "exposures",
    terms = c(
        "exposome_median_score",
        "exposome_pca_score",
        "exposome_irt_score",
        "exposome_quantile_score",
        "exposome_var_score"
    ),
    filter_col = "p.value",
    filter_thresh = 0.05,
    r2_col = "r2"
)
```

## Session Info

```{r session_info}

sessionInfo()
```

---
title: "Custom Analysis"
author: "Jason Laird"
header-includes:
   - \usepackage{amsmath}
   - \usepackage{amsfonts}
output:
  BiocStyle::html_document:
    toc_float: true
    toc: true
vignette: >
  %\VignetteIndexEntry{tidyexposomics: Custom Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{r setup,include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## Load Data and Libraries

```{r load-libraries,message=FALSE,warning=FALSE}
# Load Libraries
library(tidyverse)
library(tidyexposomics)
```

We will start off with our example dataset pulled from the [ISGlobal Exposome Data Challenge 2021](https://doi.org/10.1016/j.envint.2022.107422) (Maitre et al., 2022). 

```{r load-data}
# Load example data
data("tidyexposomics_example")

# Create exposomic set object
expom <- create_exposomicset(
    codebook = tidyexposomics_example$annotated_cb,
    exposure = tidyexposomics_example$meta,
    omics = list(
        "Gene Expression" = tidyexposomics_example$exp_filt,
        "Methylation" = tidyexposomics_example$methyl_filt
    ),
    row_data = list(
        "Gene Expression" = tidyexposomics_example$exp_fdata,
        "Methylation" = tidyexposomics_example$methyl_fdata
    )
)
```

# Custom Analysis

We provide functionality to access the underlying data in the MultiAssayExperiment object and to construct tibbles for your own analysis:

- `pivot_sample`: Pivot the sample data to a tibble with samples as rows and exposures as columns.

- `pivot_feature`: Pivot the feature metadata to a tibble with features as rows and feature metadata as columns.

- `pivot_exp`: Pivot the sample and experiment assay data to a tibble with samples as rows and sample metadata as columns. Additionally, there will be a column for values for specified features in specified assays.

## Pivot Sample

Let’s check out the `pivot_sample` function. This function pivots the sample data to a tibble with samples as rows and exposures as columns.

```{r pivot-sample}

# Pivot sample data to a tibble
expom |>
    pivot_sample() |>
    head()
```

We could use this functionality to count the number of asthmatics per sex:

```{r pivot-sample-example}

# Count the number of asthma cases by sex status
expom |>
    pivot_sample() |>
    group_by(hs_asthma, e3_sex_None) |>
    reframe(n = n())
```

## Pivot Feature

The `pivot_feature` function pivots the feature metadata to a tibble with features as rows and feature metadata as columns. This can be useful for exploring the feature metadata in a more flexible way.

```{r pivot-feature}

# Pivot feature data to a tibble
expom |>
    pivot_feature() |>
    head()
```

We could use this functionality to count the number of features per omics layer, or to filter features based on their metadata. For example, we can count the number of features per omics layer:


```{r pivot-feature-example}

# Count the number of features per omic layer
expom |>
    pivot_feature() |>
    group_by(.exp_name) |>
    summarise(n = n())
```

## Pivot Experiment

Now if we want to grab assay data from a particular experiment, we can do that with the `pivot_exp.` Let’s try grabbing assay values for the `TC01004453.hg.1` probe (probe for *IL23R*) in the `Gene Expression` experiment:

```{r pivot-exp}

# Pivot experiment data to a tibble
expom |>
    pivot_exp(
        exp_name = "Gene Expression",
        features = "TC01004453.hg.1"
    ) |>
    head()
```

We can use this functionality to create custom plots or analyses based on the exposure and feature data. For example, we can plot the expression levels of *IL23R* by asthma status:

```{r pivot-exp-example, fig.height=4, fig.width=4, fig.align='center',fig.cap="IL23R gene expression levels (probe TC01004453.hg.1) by asthma status."}

# Plot expression of IL23R
expom |>
    pivot_exp(
        exp_name = "Gene Expression",
        features = "TC01004453.hg.1"
    ) |>
    ggplot(aes(
        x = as.character(hs_asthma),
        y = log2(counts + 1),
        color = as.character(hs_asthma),
        fill = as.character(hs_asthma)
    )) +
    geom_boxplot(alpha = 0.5) +
    geom_jitter(alpha = 0.1) +
    ggpubr::geom_pwc(label = "{p.adj.format}{p.adj.signif}") +
    theme_minimal() +
    ggpubr::rotate_x_text(angle = 45) +
    ggsci::scale_color_cosmic() +
    ggsci::scale_fill_cosmic() +
    labs(
        x = "",
        y = expression(Log[2] * "Abd."),
        fill = "Asthma Status",
        color = "Asthma Status"
    )
```

## Session Info

```{r session_info}

sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_results_excel.R
\name{extract_results_excel}
\alias{extract_results_excel}
\title{Export tidyexposomics Results to Excel}
\usage{
extract_results_excel(
  exposomicset,
  file = "tidyexposomics_results.xlsx",
  result_types = c("correlation", "association", "mixture_analysis",
    "differential_analysis", "multiomics_integration", "network", "enrichment",
    "exposure_summary", "pipeline"),
  include_empty_tabs = FALSE
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object with results stored
in \verb{@metadata}, typically created by the \code{tidyexposomics} pipeline.}

\item{file}{Character. Path to the output Excel file.}

\item{result_types}{Character vector specifying which result categories
to export. Options include:
\itemize{
\item \code{"correlation"}: Correlation results.
\item \code{"association"}: Association results.
\item \code{"mixture_analysis"}: Mixture Analysis results.
\item \code{"differential_analysis"}: Differential abundance results,
including sensitivity analysis if available.
\item \code{"multiomics_integration"}: Common top features contributing to
latent factors from multi-omics integration.
\item \code{"network"}: Exposure impact metrics from network analyses.
\item \code{"enrichment"}: Enrichment results by omic and exposure category.
\item \code{"exposure_summary"}: Summary statistics for exposure variables.
\item \code{"pipeline"}: Overview of steps completed in the pipeline.
}

Use \code{"all"} to export all of the above categories.}

\item{include_empty_tabs}{Logical. If \code{TRUE}, adds placeholder sheets for
any missing result types. Default is \code{FALSE}.}
}
\value{
An Excel file is written to the specified path.
A message is printed with the file location.
}
\description{
Exports selected results stored in a \code{MultiAssayExperiment} object created
by the \code{tidyexposomics} pipeline to an Excel workbook.
Users can select which result types to include,
and optionally add placeholder sheets for missing data.
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# run correlation analysis
mae <- mae |>
    run_correlation(
        feature_type = "exposures",
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# file path of the output file
tmp <- tempfile(fileext = ".xlsx")

# extract the correlation results
extract_results_excel(
    exposomicset = mae,
    result_types = "correlation",
    file = tmp
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_normality_check.R
\name{run_normality_check}
\alias{run_normality_check}
\title{Assess Normality of Exposure Variables}
\usage{
run_normality_check(exposomicset, action = "add")
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing exposure
data in \code{colData}.}

\item{action}{A character string specifying whether to store (\code{"add"})
or return (\code{"get"}) the results. Default is \code{"add"}.}
}
\value{
A \code{MultiAssayExperiment} object with normality results added to
metadata (if \code{action = "add"}) or a list with:
\item{norm_df}{A data frame of Shapiro-Wilk test results for each
exposure variable.}
\item{norm_plot}{A ggplot object showing the distribution of normal
and non-normal exposures.}
}
\description{
Performs Shapiro-Wilk tests to check the normality of numeric exposure
variables in \code{colData} of a \code{MultiAssayExperiment} object.
}
\details{
This function:
\itemize{
\item Extracts \strong{numeric, non-constant} exposure variables from \code{colData}.
\item Runs \strong{Shapiro-Wilk tests} to assess normality.
\item Summarizes the number of normally and non-normally distributed exposures.
\item Generates a bar plot visualizing the normality results.
\item \strong{Output Handling}:
\itemize{
\item \code{"add"}: Stores results in \code{metadata(exposomicset)$normality}.
\item \code{"get"}: Returns a list containing the normality test results and plot.
}
}
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Test for normality
mae <- mae |>
    run_normality_check() |>
    transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_exposome_score.R
\name{run_exposome_score}
\alias{run_exposome_score}
\title{Compute Composite Exposome Scores}
\usage{
run_exposome_score(
  exposomicset,
  score_type,
  exposure_cols = NULL,
  scale = TRUE,
  score_column_name = NULL
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing exposure
data in its \code{colData}.}

\item{score_type}{Character. The method used to compute the score.
Options are:
\code{"mean"}, \code{"sum"}, \code{"median"}, \code{"pca"}, \code{"irt"}, \code{"quantile"}, \code{"var"}.}

\item{exposure_cols}{Optional character vector. Specific exposure
column names to include. If \code{NULL}, all numeric columns are used.}

\item{scale}{Logical. Whether to scale the exposures before computing
the score. Default is \code{TRUE}.}

\item{score_column_name}{Optional name for the resulting score column.
If \code{NULL}, an automatic name is used (e.g., \code{"exposome_score_pca"}).}
}
\value{
A \code{MultiAssayExperiment} object with the exposome score added
to \code{colData()}.
}
\description{
Calculates a summary exposome score per sample using one of several methods
including mean, sum, median, PCA (first principal component),
IRT (Item Response Theory), quantile binning, or row-wise variance.
The resulting score is added to the \code{colData} of the
\code{MultiAssayExperiment} object.
}
\details{
\itemize{
\item \code{"pca"} uses the first principal component from \code{prcomp()}.
\item \code{"irt"} uses the \code{mirt} package to fit a graded response model to
discretized exposures.
\item \code{"quantile"} assigns decile bins (1-10) to each variable and sums
them row-wise.
\item \code{"var"} computes the row-wise variance across exposures.
}
}
\examples{
# create the example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# create the air pollution score
mae <- run_exposome_score(
    mae,
    score_type = "pca",
    exposure_cols = c("exposure_pm25", "exposure_no2"),
    scale = TRUE,
    score_column_name = "air_pollution_score"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_ont_annot.R
\name{.get_ols_description}
\alias{.get_ols_description}
\title{Internal - fetch OLS description for a term}
\usage{
.get_ols_description(ontology_id, ontology_prefix)
}
\arguments{
\item{ontology_id}{Term ID (e.g., \code{"HP:0004322"}).}

\item{ontology_prefix}{Prefix (e.g., \code{"HP"}).}
}
\value{
A character scalar description or \code{NA_character_}.
}
\description{
Internal - fetch OLS description for a term
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_results.R
\name{extract_results}
\alias{extract_results}
\title{Extract Results from \code{MultiAssayExperiment} Metadata}
\usage{
extract_results(
  exposomicset,
  result = c("codebook", "quality_control", "correlation", "association",
    "differential_analysis", "multiomics_integration", "network", "enrichment")
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object.}

\item{result}{A character string indicating which result to
extract from metadata. Must be one of:
\code{"codebook"}, \code{"quality_control"}, \code{"correlation"}, \code{"association"},
\code{"mixture_analysis"}, \code{"differential_analysis"}, \code{"multiomics_integration"},
\code{"network"}, \code{"enrichment"}.}
}
\value{
The corresponding result object stored in \code{metadata(exposomicset)},
or \code{NULL} if not present.
}
\description{
Retrieves a specific analysis result
from the metadata slot of a \code{MultiAssayExperiment} object.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# extract results
res <- extract_results(
    exposomicset = mae,
    result = "codebook"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_ont_annot.R
\name{.can_use_network}
\alias{.can_use_network}
\title{Internal - should we allow network access now?}
\usage{
.can_use_network()
}
\value{
Logical scalar.
}
\description{
Controlled by the R option \code{tidyexposomics.allow_network} and \code{interactive()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_network.R
\name{plot_network}
\alias{plot_network}
\title{Plot Network Graph of Features or Exposures}
\usage{
plot_network(
  exposomicset,
  network = c("degs", "omics", "factors", "factor_features", "exposures",
    "degs_feature_cor", "omics_feature_cor", "factor_features_feature_cor"),
  include_stats = TRUE,
  nodes_to_include = NULL,
  centrality_thresh = NULL,
  top_n_nodes = NULL,
  cor_thresh = NULL,
  label = FALSE,
  label_top_n = 5,
  nodes_to_label = NULL,
  facet_var = NULL,
  foreground = "steelblue",
  fg_text_colour = "grey25",
  node_colors = NULL,
  node_color_var = NULL,
  alpha = 0.5,
  size_lab = "Centrality",
  color_lab = "Group"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing network results
in metadata.}

\item{network}{Character string specifying the network type.
One of \code{"degs"}, \code{"omics"}, \code{"factors"},
\code{"factor_features"}, \code{"exposures"}, \code{"degs_feature_cor"},
\code{"omics_feature_cor"}, \code{"factor_features_feature_cor"}.}

\item{include_stats}{Logical; if \code{TRUE}, include edge weights and
node centrality metrics in the plot aesthetics. Default is \code{TRUE}.}

\item{nodes_to_include}{Optional character vector of node names to
include (subset of \code{name}).}

\item{centrality_thresh}{Optional numeric threshold to filter nodes
by centrality degree.}

\item{top_n_nodes}{Optional integer to keep only the top N nodes by
centrality.}

\item{cor_thresh}{Optional numeric threshold to filter edges by
minimum absolute correlation.}

\item{label}{Logical; whether to label nodes.
If \code{TRUE}, top nodes will be labeled.}

\item{label_top_n}{Integer; number of top-centrality nodes to
label if \code{label = TRUE}. Default is \code{5}.}

\item{nodes_to_label}{Optional character vector of specific nodes to label.}

\item{facet_var}{Optional node metadata column to facet the
network layout by.}

\item{foreground}{Color for node outlines and edge borders.
Default is \code{"steelblue"}.}

\item{fg_text_colour}{Color of node label text. Default is \code{"grey25"}.}

\item{node_colors}{Optional named vector of colors for node groups.}

\item{node_color_var}{Optional node attribute used for node color mapping.}

\item{alpha}{Alpha transparency for nodes and edges. Default is \code{0.5}.}

\item{size_lab}{Legend title for node size (typically centrality).
Default is \code{"Centrality"}.}

\item{color_lab}{Legend title for node color group. Default is \code{"Group"}.}
}
\value{
A \code{ggraph} plot object.
}
\description{
Visualizes network structures created by \code{run_create_network()} from
the metadata of a \code{MultiAssayExperiment} object.
Nodes can represent features (e.g., genes or factors) or exposures,
and edges represent correlations or shared connections.
}
\details{
This function retrieves the stored graph object and optionally filters or
labels nodes based on:
centrality, correlation, user input, or group-specific attributes.
It supports layout faceting,
custom color mappings, and highlights highly central nodes.

Large graphs (> 500 nodes) will prompt the user before plotting.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# perform correlation analyses
# correlate with exposures
mae <- mae |>
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# create the networks
mae <- mae |>
    run_create_network(
        feature_type = "omics",
        action = "add"
    )

# plot the network
network_p <- mae |>
    plot_network(
        network = "omics"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_exposures.R
\name{plot_exposures}
\alias{plot_exposures}
\title{Plot Exposure Distributions by Category or Group}
\usage{
plot_exposures(
  exposomicset,
  exposure_cat = "all",
  exposure_cols = NULL,
  group_by = NULL,
  plot_type = "boxplot",
  alpha = 0.3,
  panel_sizes = rep(1, 100),
  title = "Exposure Levels by Category",
  xlab = "",
  ylab = "",
  facet_cols = NULL,
  group_cols = NULL,
  box_width = 0.1,
  fill_lab = ""
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing exposure data.}

\item{exposure_cat}{A character string or vector specifying exposure
category names (from \code{codebook$category}) to include.
Use \code{"all"} to include all exposures.}

\item{exposure_cols}{Optional character vector specifying exact
exposure variables to plot.}

\item{group_by}{A string specifying the column in \code{colData(exposomicset)}
used to fill the plot (e.g., \code{"sex"}). Defaults to \code{NULL}, in which case
exposures are colored by \code{category}.}

\item{plot_type}{Type of plot: \code{"boxplot"} (default) or \code{"ridge"}.}

\item{alpha}{Transparency level for background facet color strips.
Default is \code{0.5}.}

\item{panel_sizes}{A numeric vector passed to \code{ggh4x::force_panelsizes()}
for controlling facet widths or heights.}

\item{title}{Plot title. Default is \code{"Exposure Levels by Category"}.}

\item{xlab}{X-axis label. Default is an empty string.}

\item{ylab}{Y-axis label. Default is an empty string.}

\item{facet_cols}{Optional vector of colors to use as background for
facet categories. If \code{NULL}, a default palette is used.}

\item{group_cols}{Optional named vector of colors for \code{group_by} levels.
If \code{NULL}, a default palette is used.}

\item{box_width}{A numeric value specifying the width of the boxplots.
Only used when \code{plot_type = "boxplot"}. Default is \code{0.1}.}

\item{fill_lab}{Legend title for the fill aesthetic
(e.g., \code{"Sex"} or \code{"Exposure Group"}). Default is \code{""}.}
}
\value{
A \code{ggplot2} object showing exposure distributions,
optionally grouped.
}
\description{
Visualizes exposure variable distributions using \strong{boxplots}
or \strong{ridge plots}.
}
\details{
This function:
\itemize{
\item Filters exposure data based on category or selected columns.
\item Merges variable metadata from \code{metadata(exposomicset)$codebook}.
\item Supports either \strong{boxplot} (vertical distributions per variable)
or \strong{ridgeplot} (horizontal density plots per variable).
\item If \code{group_by} is specified, that variable defines the plot fill color;
otherwise, the fill is based on exposure \code{category}.
\item Facets by \code{category} using \code{ggh4x::facet_grid2()}
with color-coded strip backgrounds.
\item The \code{box_width} argument controls the width of the boxplots when
\code{plot_type = "boxplot"}.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# plot exposure data
exposure_plot <- mae |>
    plot_exposures(
        exposure_cols = c("exposure_pm25", "exposure_no2"),
        box_width = 0.2
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_pipeline_summary.R
\name{run_pipeline_summary}
\alias{run_pipeline_summary}
\title{Summarize and Visualize Analysis Pipeline Steps}
\usage{
run_pipeline_summary(
  exposomicset,
  show_index = TRUE,
  console_print = TRUE,
  diagram_print = FALSE,
  include_notes = TRUE
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object that contains a
"summary"
entry in its metadata, which includes a list named \code{steps}.}

\item{show_index}{Logical, default \code{TRUE}. If \code{TRUE},
prefixes each step with its index.}

\item{console_print}{Logical, default \code{TRUE}. If \code{TRUE},
prints the step list to the console.}

\item{diagram_print}{Logical, default \code{FALSE}. If \code{TRUE},
renders a Mermaid diagram of the steps.}

\item{include_notes}{Logical, default \code{TRUE}. If \code{TRUE},
appends any "notes" associated with each step to the label.}
}
\value{
No return value. This function is called for its side effects:
console output and/or diagram rendering.
}
\description{
This function prints and visualizes the analysis steps stored in the
metadata of a \code{MultiAssayExperiment} object. The steps are optionally
printed to the console as a numbered list and can be rendered as a
left-to-right Mermaid flowchart.
The flowchart connects steps with arrows and includes step notes
if requested.
}
\details{
The Mermaid flowchart is rendered left-to-right and connects
each step in sequence. Each node is labeled using the step name and,
optionally,
any attached notes.
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Test for normality
mae <- mae |>
    run_normality_check() |>
    transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))

# Run the pipeline summary
run_pipeline_summary(mae)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_exposomicset.R
\name{create_exposomicset}
\alias{create_exposomicset}
\title{Create an Exposomicset Object}
\usage{
create_exposomicset(codebook, exposure, omics = NULL, row_data = NULL)
}
\arguments{
\item{codebook}{A data frame containing variable information metadata.}

\item{exposure}{A data frame containing exposure data,
with rows as samples and columns as variables.}

\item{omics}{An optional list of matrices or a single matrix representing
omics data. Each matrix should have samples as columns and features as rows.
If \code{NULL}, creates an exposure-only exposomicset. Default is \code{NULL}.}

\item{row_data}{An optional list of \code{DataFrame} objects providing
feature metadata for each omics dataset. If \code{NULL},
row metadata is generated automatically. Default is \code{NULL}.}
}
\value{
A \code{MultiAssayExperiment} object containing the formatted exposure
and optionally omics datasets.
}
\description{
Constructs a \code{MultiAssayExperiment} object from exposure data and optionally
omics datasets, ensuring proper formatting and alignment of samples and
features. For epidemiology-only workflows, omics data can be omitted.
}
\details{
The function validates inputs and creates a \code{MultiAssayExperiment} object.
If omics data is provided, it converts matrices into \code{SummarizedExperiment}
objects with proper sample alignment. If omics is \code{NULL}, the function
creates an exposure-only object suitable for epidemiological analyses
using \code{run_association()} with \code{source = "exposures"}.
}
\examples{
# Epi user workflow
# so no omics data
epi_data <- data.frame(
    pm25 = rnorm(10),
    outcome = rbinom(10, 1, 0.5),
    age = rnorm(10, 45, 10),
    row.names = paste0("subj_", 1:10)
)

codebook <- data.frame(
    variable = c("pm25", "outcome", "age"),
    category = c("exposure", "outcome", "covariate")
)

mae <- create_exposomicset(
    codebook = codebook,
    exposure = epi_data
)

# Multi-omics workflow
tmp <- make_example_data(n_samples = 10)

mae <- create_exposomicset(
    codebook = tmp$codebook,
    exposure = tmp$exposure,
    omics = tmp$omics,
    row_data = tmp$row_data
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_association.R
\name{run_association}
\alias{run_association}
\title{Run Association Analysis}
\usage{
run_association(
  exposomicset,
  outcome,
  source = c("omics", "exposures", "factors"),
  covariates = NULL,
  feature_set = NULL,
  log_trans = TRUE,
  top_n = NULL,
  family = "gaussian",
  correction_method = "fdr",
  action = "add",
  feature_col = NULL,
  mirna_assays = NULL
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing data
and metadata.}

\item{outcome}{The outcome variable name (must be in \code{colData}).}

\item{source}{Source of features to test. One of \code{"omics"},
\code{"exposures"}, \code{"factors"}.}

\item{covariates}{Optional vector of covariate names to include in the model.}

\item{feature_set}{Optional character vector of exposure or GO terms to test.}

\item{log_trans}{Optional boolean value dictating whether or not to log2
transform omics features.}

\item{top_n}{Optional integer: if using omics source, select top \code{n}
most variable features.}

\item{family}{GLM family; \code{"gaussian"} or \code{"binomial"}.}

\item{correction_method}{Method for p-value adjustment (default: \code{"fdr"}).}

\item{action}{If \code{"add"} (default), saves results to metadata;
else returns results as list.}

\item{feature_col}{The column in \code{rowData}
for matching gene symbols or IDs.}

\item{mirna_assays}{Optional character vector of assays to exclude
when extracting GO terms.}
}
\value{
If \code{action = "add"}, returns updated \code{MultiAssayExperiment}.
Otherwise, returns a list of:
\itemize{
\item \code{results_df}: tidy summary of associations
\item \code{covariates}: the covariates used
\item \code{model_data}: model matrix used in the GLMs
}
}
\description{
Perform GLM-based association testing between a specified outcome and
features from exposures, omics, or latent factors. Automatically adjusts
for covariates and supports both Gaussian and binomial models.
}
\examples{

#' # create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run association tests
mae <- mae |>
    run_association(
        source = "exposures",
        feature_set = c("exposure_pm25", "exposure_no2"),
        outcome = "smoker",
        covariates = c("age"),
        family = "binomial"
    )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_exposure.R
\name{transform_exposure}
\alias{transform_exposure}
\title{Transform Exposure Variables for Normality}
\usage{
transform_exposure(
  exposomicset,
  exposure_cols = NULL,
  transform_method = "boxcox_best"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing exposure
variables in \code{colData}.}

\item{exposure_cols}{Optional character vector of exposure variable
names to transform.
If \code{NULL}, uses exposures found in \code{metadata(exposomicset)$quality_control$normality$norm_df$exposure}.}

\item{transform_method}{Character. Transformation method to apply. Options:
\itemize{
\item \code{"none"}: no transformation
\item \code{"log2"}: log base 2 transformation
\item \code{"x_1_3"}: cube-root transformation
\item \code{"sqrt"}: square-root transformation
\item \code{"boxcox_best"}: data-driven Box-Cox approximation with
heuristic labeling
}}
}
\value{
A \code{MultiAssayExperiment} object with transformed exposures
in \code{colData}, and transformation details stored in:
\itemize{
\item \code{metadata(exposomicset)$quality_control$transformation$norm_df}:
Shapiro-Wilk test results
\item \code{metadata(exposomicset)$quality_control$transformation$norm_summary}:
Summary of normality
\item \code{metadata(exposomicset)$codebook}: Updated with transformation info
per variable
\item \code{metadata(exposomicset)$summary$steps}: Updated with step record
}
}
\description{
Applies a transformation to selected numeric exposure variables in the
\code{colData} of a \code{MultiAssayExperiment} to improve their normality
(e.g., log, Box-Cox, sqrt). Transformation results and normality
statistics are stored in metadata for tracking.
}
\details{
For \code{transform_method = "boxcox_best"}, the function automatically shifts
values to be strictly positive and chooses from a discrete set of
transformations (e.g., \code{1/x}, \code{log(x)}, \code{x^2})
based on estimated Box-Cox lambda. Each variable may receive a
different transformation.
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Test for normality
mae <- mae |>
    run_normality_check() |>
    transform_exposure(
        exposure_cols = c("age", "bmi", "exposure_pm25"),
        transform_method = "boxcox_best"
    )

}
\seealso{
\code{\link[MASS]{boxcox}}, \code{\link[stats]{shapiro.test}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_ont_annot.R
\name{.run_categorize_ontology}
\alias{.run_categorize_ontology}
\title{Internal - categorizes to ontology roots / depth}
\usage{
.run_categorize_ontology(
  data,
  id_col,
  ontologyDF,
  root_level = 0,
  assign_label = TRUE
)
}
\arguments{
\item{data}{Data frame with an ID column.}

\item{id_col}{Column name containing ontology term IDs.}

\item{ontologyDF}{Processed ontology data.frame (with list columns for relationships and \code{depth}).}

\item{root_level}{Either a character vector of explicit root IDs, a numeric depth, or (default) top-level roots.}

\item{assign_label}{If TRUE, return input with new columns; otherwise return assigned IDs.}
}
\value{
Data frame with \code{root_id}, \code{root_label}, \code{category}, \code{category_source} (if \code{assign_label}).
}
\description{
Internal - categorizes to ontology roots / depth
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidyexposomics_example.R
\docType{data}
\name{tidyexposomics_example}
\alias{tidyexposomics_example}
\title{Example exposome multi-omics dataset}
\format{
An object of class \code{list} of length 6.
}
\usage{
data("tidyexposomics_example")
}
\description{
A downsampled version of the ISGlobal Exposome Data Challenge 2021
dataset (Maitre \emph{et al.}, \emph{Environment International}, 2022; DOI: 10.1016/j.envint.2022.107422).
This example dataset is included with \strong{tidyexposomics} for vignette
evaluation. This subset represents samples from children with
low socioeconomic status in the first cohort of the original dataset.
}
\details{
The data have been filtered and transformed to illustrate
the \code{tidyexposomics} workflow. Only a small subset of variables, and
the top 500 most variable features per omic layer are retained.

\strong{Contents}
\describe{
\item{meta}{A data frame of selected exposure, demographic, and
outcome variables for a subset of participants.}
\item{annotated_cb}{A data frame providing ontology-linked annotation
for the exposures in \code{meta}.}
\item{exp_filt}{A numeric matrix of gene expression values
(500 features, 48 samples) representing the top-variance transcripts.}
\item{exp_fdata}{Feature-level metadata for \code{exp_filt}, including
cleaned gene symbols.}
\item{methyl_filt}{A numeric matrix of DNA methylation M-values
(500 CpG sites, 48 samples).}
\item{methyl_fdata}{Feature-level metadata for \code{methyl_filt}.}
}

\strong{Source}

Derived from the ISGlobal Exposome Data Challenge 2021
(Maitre \emph{et al.}, \emph{Environment International}, 2022;
DOI: 10.1016/j.envint.2022.107422),licensed under CC-BY 4.0.
The original data are available on Figshare (Project 98813)
and GitHub (\code{isglobal-exposomeHub/ExposomeDataChallenge2021}).
This example dataset was processed and downsampled by the
tidyexposomics authors and is not a replacement for the full dataset.
}
\examples{
data("tidyexposomics_example")

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_pca.R
\name{plot_pca}
\alias{plot_pca}
\title{Plot PCA Results for Features and Samples}
\usage{
plot_pca(
  exposomicset,
  feature_col = "#00a9b2",
  sample_col = "#8a4f77",
  sample_outlier_col = "firebrick"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing PCA results
in \code{metadata(exposomicset)$pca}.}

\item{feature_col}{A character string specifying the color for the
feature scree plot.
Default is \code{"#00a9b2"}.}

\item{sample_col}{A character string specifying the color for the
sample scree plot.
Default is \code{"#8a4f77"}.}

\item{sample_outlier_col}{A character string specifying the color
for sample outlier labels.
Default is \code{"firebrick"}.}
}
\value{
A combined \code{ggplot} object containing the four PCA plots.
}
\description{
Generates PCA plots for both feature space and sample space,
including scatter plots and scree plots.
}
\details{
This function creates four PCA visualizations:
\itemize{
\item \strong{Feature Space PCA Plot}: Colored by category (e.g., omics, exposure).
\item \strong{Feature Scree Plot}: Displays the variance explained by each
principal component.
\item \strong{Sample Space PCA Plot}: Highlights outlier samples.
\item \strong{Sample Scree Plot}: Displays variance explained in the sample PCA.
}

Outliers are labeled based on \code{metadata(exposomicset)$pca$outliers}.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run pca
mae <- mae |>
    run_pca()

# create the pca plot
pca_p <- mae |>
    plot_pca()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_summarize_exposures.R
\name{run_summarize_exposures}
\alias{run_summarize_exposures}
\title{Summarize Exposure Variables}
\usage{
run_summarize_exposures(exposomicset, exposure_cols = NULL, action = "add")
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing exposure
data in the sample metadata.}

\item{exposure_cols}{A character vector of exposure variable names
to summarize. If \code{NULL}, all numeric exposure variables are included.}

\item{action}{A string specifying the action to take. Use \code{"add"}
to attach the summary table to \code{metadata(exposomicset)} or \code{"get"}
to return the summary table directly. Default is \code{"add"}.}
}
\value{
A modified \code{MultiAssayExperiment} object (if \code{action = "add"}),
or a data frame of summary statistics (if \code{action = "get"}).
}
\description{
Computes summary statistics for numeric exposure variables and
optionally stores the results in the \code{MultiAssayExperiment} metadata.
}
\details{
This function:
\itemize{
\item Extracts sample-level exposure data using \code{pivot_sample()}.
\item Filters to user-specified exposures (\code{exposure_cols}) if provided.
\item Computes descriptive statistics for each numeric variable:
\itemize{
\item Number of values (\code{n_values})
\item Number of NAs (\code{n_na})
\item Minimum, maximum, and range
\item Sum, median, mean
\item Standard error of the mean
\item 95\% confidence interval of the mean
\item Variance, standard deviation
\item Coefficient of variation (\code{sd / mean})
}
\item Merges the result with variable metadata stored in
\code{metadata(exposomicset)$codebook}.
}
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Summarize exposure data
exp_sum <- mae |>
    run_summarize_exposures(
        exposure_cols = c("age", "bmi", "exposure_pm25"),
        action = "get"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_create_network.R
\name{run_create_network}
\alias{run_create_network}
\title{Create Correlation Network from Feature Data}
\usage{
run_create_network(
  exposomicset,
  feature_type = c("degs", "omics", "factors", "factor_features", "exposures",
    "degs_feature_cor", "omics_feature_cor", "factor_features_feature_cor"),
  action = c("add", "get")
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing correlation
results in metadata.}

\item{feature_type}{Type of correlation result to convert to a network.
One of:
\code{"degs"}, \code{"omics"}, \code{"factors"}, \code{"factor_features"}, \code{"exposures"},
\code{"degs_feature_cor"}, \code{"omics_feature_cor"},
or \code{"factor_features_feature_cor"}.}

\item{action}{Whether to \code{"add"} the network to the object or
\code{"get"} it as a list.}
}
\value{
If \code{action = "add"}, returns the updated \code{MultiAssayExperiment}
with a new \code{network} entry in metadata.
If \code{action = "get"}, returns a list with \code{graph}
(an \code{igraph} object) and \code{summary} (a tibble).
}
\description{
Constructs an undirected feature-feature or feature-exposure
correlation network from correlation results stored in a
\code{MultiAssayExperiment} object. The function
supports multiple correlation formats depending on \code{feature_type},
and stores or returns an \code{igraph} object with associated node
and edge metadata.
}
\details{
The function detects the appropriate edge and node structure
based on column names in the correlation results.
Edge weights are based on correlation coefficients and include FDR values.
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# perform correlation analyses
# correlate with exposures
mae <- mae |>
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    ) |>
    # correlate omics features with themselves
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        feature_cors = TRUE,
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# create the networks
mae <- mae |>
    run_create_network(
        feature_type = "omics_feature_cor",
        action = "add"
    ) |>
    run_create_network(
        feature_type = "omics",
        action = "add"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_pca.R
\name{run_pca}
\alias{run_pca}
\title{Perform Principal Component Analysis (PCA)}
\usage{
run_pca(
  exposomicset,
  log_trans_exp = FALSE,
  log_trans_omics = TRUE,
  action = "add"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing omics
and exposure data.}

\item{log_trans_exp}{A boolean value specifying whether to log2
transform the exposure data}

\item{log_trans_omics}{a boolean value specifying whether to log2
transform the omics data}

\item{action}{A character string specifying whether to store
(\code{"add"}) or return (\code{"get"}) the results. Default is \code{"add"}.}
}
\value{
If \code{action = "add"}, the function returns the input
\code{MultiAssayExperiment} with:
\itemize{
\item \strong{PC scores} added as columns in \code{colData(exposomicset)}, and
\item \strong{PCA objects} stored under
\code{metadata(exposomicset)$quality_control$pca}.
}

If \code{action = "get"}, the function returns a list containing:
\item{pca_df}{A tibble of the transformed input data.}
\item{pca_feature}{A \code{prcomp} object containing PCA results for features.}
\item{pca_sample}{A \code{prcomp} object containing PCA results for samples.}
\item{outliers}{A character vector of detected sample outliers.}
}
\description{
Runs PCA on the feature and sample spaces of a \code{MultiAssayExperiment} object,
identifying outliers based on Mahalanobis distance.
}
\details{
This function:
\itemize{
\item Identifies \strong{common samples} across all assays and exposure data.
\item Performs \strong{PCA on features} (transformed and standardized).
\item Performs \strong{PCA on samples} and computes Mahalanobis distance to
detect outliers.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run pca
mae <- mae |>
    run_pca()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_enrichment.R
\name{run_enrichment}
\alias{run_enrichment}
\title{Perform enrichment analysis on selected features from a exposomicset object}
\usage{
run_enrichment(
  exposomicset,
  feature_type = c("degs", "degs_robust", "omics", "factor_features", "degs_cor",
    "omics_cor", "factor_features_cor"),
  score_col = "stability_score",
  score_threshold = NULL,
  variable_map = NULL,
  factor_type = c("common_top_factor_features", "top_factor_features"),
  feature_col = "feature",
  deg_pval_col = "adj.P.Val",
  deg_pval_threshold = 0.05,
  deg_logfc_col = "logFC",
  deg_logfc_threshold = log2(1.5),
  db = c("GO", "KEGG", "Reactome"),
  species = NULL,
  fenr_col = "gene_symbol",
  padj_method = "fdr",
  pval_thresh = 0.1,
  min_set = 3,
  max_set = 800,
  clustering_approach = "diana",
  action = "add"
)
}
\arguments{
\item{exposomicset}{An \code{exposomicset} (a \code{MultiAssayExperiment}
object with metadata) containing omics and metadata.}

\item{feature_type}{Character string indicating the feature source.
One of \code{"degs"}, \code{"degs_robust"},
\code{"omics"}, \code{"factor_features"}, \code{"degs_cor"}, \code{"omics_cor"},
or \code{"factor_features_cor"}.}

\item{score_col}{Column name used for stability score filtering
(only for \code{degs_robust}).}

\item{score_threshold}{Optional numeric threshold for filtering
stability scores. If \code{NULL}, the default
threshold stored in the metadata will be used.}

\item{variable_map}{A data frame with \code{exp_name} and \code{variable} columns,
used when \code{feature_type = "omics"}.}

\item{factor_type}{Character string for selecting factor features:
\code{"common_top_factor_features"} or
\code{"top_factor_features"}.}

\item{feature_col}{The name of the feature column used to extract
gene identifiers.}

\item{deg_pval_col}{Column name for adjusted p-values from DEG analysis.}

\item{deg_pval_threshold}{Threshold to select significant DEGs
(default: 0.05).}

\item{deg_logfc_col}{Column name for log-fold changes from DEG analysis.}

\item{deg_logfc_threshold}{Threshold to select DEGs by absolute logFC
(default: \code{log2(1.5)}).}

\item{db}{Enrichment database to use. One of \code{"GO"},
\code{"KEGG"}, \code{"Reactome"}.}

\item{species}{Species name (required for GO enrichment,
e.g., \code{"Homo sapiens"}). Ignored for other databases.}

\item{fenr_col}{Column name for gene IDs used by \code{fenr}
(e.g., \code{"gene_symbol"}).}

\item{padj_method}{Method for p-value adjustment (default: \code{"fdr"}).}

\item{pval_thresh}{Adjusted p-value threshold for filtering enriched terms
(default: 0.1).}

\item{min_set}{Minimum number of selected genes overlapping an enriched term
(default: 3).}

\item{max_set}{Maximum number of selected genes overlapping an enriched term
(default: 800).}

\item{clustering_approach}{Clustering method for GO term grouping.
Defaults to \code{"diana"}.}

\item{action}{Either \code{"add"} to store results in the object's metadata
or \code{"get"} to return results as a data frame.}
}
\value{
If \code{action = "add"}, returns the modified \code{exposomicset} with
enrichment results added to metadata.
If \code{action = "get"}, returns a \code{data.frame} of enrichment results
with GO term clusters (if applicable).
}
\description{
This function performs enrichment analysis using selected features derived
from differential expression, correlation analysis,
or multi-omics factor features across experiments in an \code{exposomicset}.
It supports multiple enrichment databases (e.g., GO, KEGG, Reactome),
applies FDR correction, and optionally clusters GO terms by Jaccard overlap.
}
\details{
The function identifies selected features based on the chosen
\code{feature_type}, determines the gene universe
for each experiment, and performs enrichment analysis using the
\code{fenr} package. Results are adjusted for
multiple testing and optionally clustered by gene set overlap (for GO terms).

If \code{feature_type} includes correlation-based results
(ending in \verb{_cor}), enrichment is performed for each
exposure category separately.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 30,
    return_mae = TRUE
)

# perform differential abundance analysis
mae <- run_differential_abundance(
    exposomicset = mae,
    formula = ~ smoker + sex,
    abundance_col = "counts",
    method = "limma_voom",
    action = "add"
)

# perform enrichment analysis
mae <- run_enrichment(
    exposomicset = mae,
    feature_type = "degs",
    feature_col = "symbol",
    species = "goa_human",
    deg_logfc_threshold = log2(1),
    deg_pval_col = "P.Value",
    deg_pval_threshold = 0.2,
    action = "add"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_exposure_omics_association.R
\name{.plot_exp_omic_category}
\alias{.plot_exp_omic_category}
\title{Plot Significant Omics Associations per Exposure Category}
\usage{
.plot_exp_omic_category(
  exposomicset,
  pval_col = "p_adjust",
  pval_thresh = 0.05
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing association results.}

\item{pval_col}{Character. Name of the column used for p-value filtering.
Defaults to \code{"p_adjust"}.}

\item{pval_thresh}{Numeric. Significance threshold applied to \code{pval_col}.
Rows with values below this threshold are retained. Defaults to \code{0.05}.}
}
\value{
A \code{ggplot} object.
}
\description{
Internal helper that counts and plots the number of significant omics
associations for each exposure category.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_loading.R
\name{download_dataset}
\alias{download_dataset}
\title{Download and cache a tidyexposomics dataset}
\usage{
download_dataset(
  name = c("omics_list", "fdata", "meta", "annotated_cb", "expom_1", "chebi", "ecto",
    "hpo"),
  verbose = TRUE,
  validate = TRUE
)
}
\arguments{
\item{name}{Dataset name: one of "omics_list", "fdata", "meta",
"annotated_cb", "expom_1", "chebi", "ecto", "hpo".}

\item{verbose}{Logical; print messages.}

\item{validate}{Logical; validate MD5 checksum.}
}
\value{
A list or object loaded from the cached .RData.
}
\description{
Download and cache a tidyexposomics dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pivot_exp.R
\name{pivot_exp}
\alias{pivot_exp}
\title{Pivot a selected omics dataset from a MultiAssayExperiment into tidybulk format}
\usage{
pivot_exp(exposomicset, exp_name, features = NULL)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
one or more omics assays.}

\item{exp_name}{A character string. The name of the omics dataset
to extract (e.g., "Proteomics").}

\item{features}{Optional character vector of row (feature) names to retain.
If \code{NULL}, all features are included.}
}
\value{
A tibble in tidy format with one row per feature/sample pair,
including all metadata and a new column \code{exp_name}
indicating the assay source. Assay values are provided in separate columns
named after the assay slot(s).
}
\description{
Extracts a specified omics dataset from a \code{MultiAssayExperiment},
optionally filters by feature (row) names,
and returns a tidy tibble. The output includes assay values along with
sample metadata and feature metadata.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# pivot experiment
exp_data <- pivot_exp(
    exposomicset = mae,
    exp_name = "mRNA",
    features = c("feat_42")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_non_normal.R
\name{filter_non_normal}
\alias{filter_non_normal}
\title{Filter Non-Normal Exposure Variables}
\usage{
filter_non_normal(exposomicset, p_thresh = 0.05)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
exposure and omics data.}

\item{p_thresh}{A numeric value specifying the p-value
threshold for normality. Variables with \code{p.value < p_thresh} are removed.
Default is \code{0.05}.}
}
\value{
A \code{MultiAssayExperiment} object with non-normal
exposure variables removed.
}
\description{
Removes exposure variables that deviate significantly from a
normal distribution based on
normality test results stored in metadata.
}
\details{
The function identifies exposure variables that fail a normality test
using \code{metadata(exposomicset)$transformation$norm_df}.
\itemize{
\item Exposure variables with \code{p.value < p_thresh} are removed from
\code{colData(exposomicset)}.
\item The same filtering is applied to \code{colData} in each experiment
within \code{experiments(exposomicset)}.
}
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Test for normality
mae <- mae |>
    run_normality_check() |>
    transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))

# Remove non-normal variables
mae_filtered <- mae |>
    filter_non_normal()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_multiomics_integration.R
\name{run_multiomics_integration}
\alias{run_multiomics_integration}
\title{Run Multi-Omics Integration}
\usage{
run_multiomics_integration(
  exposomicset,
  method = "MCIA",
  n_factors = 10,
  scale = TRUE,
  outcome = NULL,
  max.iter = 500,
  near.zero.var = TRUE,
  action = "add"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object with at least two assays.}

\item{method}{Character. Integration method to use. Options are
\code{"MOFA"}, \code{"MCIA"}, \code{"RGCCA"}, or \code{"DIABLO"}.}

\item{n_factors}{Integer. Number of latent factors/components to compute.
Default is 10.}

\item{scale}{Logical. Whether to scale each assay before integration.
Default is \code{TRUE}.}

\item{outcome}{Character. Required if \code{method = "DIABLO"}.
Name of outcome variable in \code{colData} used for supervised integration.}

\item{max.iter}{numeric. Option to increase the number of iterations for
\code{mixOmics::block.splsda} the default is 500.}

\item{near.zero.var}{Logical. Option to remove variables with near zero
variance for \code{mixOmics::block.splsda}, default is \code{TRUE} .}

\item{action}{Character. Whether to \code{"add"} results to the metadata or
\code{"get"} them as a list. Default is \code{"add"}.}
}
\value{
If \code{action = "add"}, returns a \code{MultiAssayExperiment} with
integration results
stored in \code{metadata(exposomicset)$multiomics_integration$integration_results}.
If \code{action = "get"}, returns a list with integration \code{method} and \code{result}.
}
\description{
Performs multi-omics integration using one of several available methods:
MOFA, MCIA, RGCCA, or DIABLO. This function takes a \code{MultiAssayExperiment}
object with two or more assays and computes shared latent factors
across omics layers.
}
\details{
\itemize{
\item \code{"MOFA"} runs Multi-Omics Factor Analysis using the \code{MOFA2} package and
returns a trained model.
\item \code{"MCIA"} runs multi-co-inertia analysis using the \code{nipalsMCIA} package.
\item \code{"RGCCA"} runs Regularized Generalized Canonical Correlation Analysis
using the \code{RGCCA} package.
\item \code{"DIABLO"} performs supervised integration using the \code{mixOmics} package
and a specified outcome.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# perform multiomics integration
mae <- run_multiomics_integration(
    mae,
    method = "DIABLO",
    outcome = "smoker",
    n_factors = 3
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_omics_exposure_df.R
\name{extract_omics_exposure_df}
\alias{extract_omics_exposure_df}
\title{Extract Merged Omics and Exposure Data Frame}
\usage{
extract_omics_exposure_df(exposomicset, variable_map = NULL, log2_trans = TRUE)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
omics and exposure data.}

\item{variable_map}{A data frame with columns \code{"variable"} and \code{"exp_name"},
indicating which variables belong to each omics or exposure domain.}

\item{log2_trans}{Logical; whether to log2-transform omics data.
Default is \code{TRUE}.}
}
\value{
A data frame where rows correspond to samples,
and columns contain exposure variables and log2-transformed omics features.
Columns from different omics types are disambiguated using prefixes.
}
\description{
This function extracts and merges exposure variables from \code{colData}
with selected features from omics datasets
in a \code{MultiAssayExperiment} object. Optionally applies log2 transformation
to omics data and restricts features based on a variable map.
}
\details{
If \code{variable_map} is provided, it is used to select variables from
both exposures and omics. If not provided, all numeric \code{colData} variables
are used as exposures (excluding variables matching \verb{^PC}),
and all omics features are included.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)
# export the omics exposure df
merged_df <- extract_omics_exposure_df(
    mae,
    log2_trans = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_manhattan.R
\name{plot_manhattan}
\alias{plot_manhattan}
\title{Plot a Manhattan-style ExWAS summary across omics categories}
\usage{
plot_manhattan(
  exposomicset,
  pval_thresh = 0.05,
  feature_col = "term",
  alpha = 0.5,
  min_per_cat = 1,
  vars_to_label = NULL,
  sig_color = "magenta2",
  non_sig_cols = c("grey25", "grey75"),
  pval_thresh_line_col = "grey25",
  panel_sizes = c(1, 1, 1, 1, 1),
  linetype = "dashed",
  facet_cols = NULL,
  label_size = 3.5,
  facet_angle = 90,
  facet_text_face = "bold.italic"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object that has
already been processed by \code{associate_all_outcome()}.}

\item{pval_thresh}{Numeric threshold for significance (default = 0.05).}

\item{feature_col}{A character string indicating the column name to use
for feature labeling and highlighting (e.g., \code{"term"} or \code{"feature"}).
Default is \code{"term"}.}

\item{alpha}{Transparency applied to facet strip colors (default = 0.5).}

\item{min_per_cat}{Minimum number of features per category to be shown
(default = 1).}

\item{vars_to_label}{Optional character vector of variable names to label
explicitly, matched against the \code{feature_col} column.}

\item{sig_color}{Color used for significant points (default = \code{"magenta2"}).}

\item{non_sig_cols}{Character vector of alternating colors for
non-significant points (default = \code{c("grey25", "grey75")}).}

\item{pval_thresh_line_col}{Color of the horizontal significance
threshold line (default = \code{"grey25"}).}

\item{panel_sizes}{Numeric vector passed to \code{ggh4x::force_panelsizes()}
to control panel widths (default = \code{c(1,1,1,1,1)}).}

\item{linetype}{Line type for the horizontal threshold (default = \code{"dashed"}).}

\item{facet_cols}{Optional vector of colors to use for
facet strip backgrounds.}

\item{label_size}{Numeric size of the feature label text (default = 3.5).}

\item{facet_angle}{Angle (in degrees) for strip text rotation (default = 90).}

\item{facet_text_face}{Font face for facet strip labels
(default = \code{"bold.italic"}).}
}
\value{
A \code{ggplot} object showing the Manhattan-style faceted plot.
}
\description{
This function generates a multi-faceted Manhattan plot from the results of
\code{associate_all_outcome()}, visualizing the significance of associations
across omics features, grouped by category. Significant features can be
highlighted and labeled, and strip backgrounds can be colored per facet.
}
\details{
\itemize{
\item This function expects \code{associate_all_outcome()} to have been run first.
\item Facets represent omics categories, and points represent features.
\item Points below the significance threshold are colored using \code{non_sig_cols},
while significant ones are colored with \code{sig_color} and optionally labeled.
\item Uses \code{ggrepel} to avoid overlapping labels and \code{ggh4x} for
enhanced faceting.
\item The \code{feature_col} argument allows customization of which column is used
to label or identify features, enabling compatibility with different
result formats.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run association tests
mae <- mae |>
    run_association(
        source = "omics",
        top_n = 20,
        feature_set = c("exposure_pm25", "exposure_no2"),
        outcome = "smoker",
        covariates = c("age"),
        family = "binomial"
    )

# create the manhattan plot
manhattan_p <- mae |>
    plot_manhattan(feature_col = "term")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_exposure_omics_association.R
\name{run_exposure_omics_association}
\alias{run_exposure_omics_association}
\title{Run Exposure-Omics Association}
\usage{
run_exposure_omics_association(
  exposomicset,
  exposures = NULL,
  exp_name = NULL,
  covariates = NULL,
  scaling_method = "none",
  correction_method = "fdr",
  top_pct = NULL,
  filter_by = c("variance", "mean"),
  action = "add"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing exposome
and omics data.}

\item{exposures}{Character vector of exposure variable names to test.
If \code{NULL}, uses all variables from the codebook.}

\item{exp_name}{Name(s) of the omics assay(s) to test against. If \code{NULL},
uses all assays.}

\item{covariates}{Optional character vector of covariate names to include
in the model.}

\item{scaling_method}{Character. Scaling method to apply before modeling.
Options include \code{"none"} (default).}

\item{correction_method}{Method for p-value adjustment. Default is \code{"fdr"}.}

\item{top_pct}{Top X\% of features to retain using either mean or variance
which is specified by \code{filter_by}. If \code{NULL}, no features will be filtered.}

\item{filter_by}{Determination of how to filter omics features either by
mean or variance.}

\item{action}{If \code{"add"} (default), saves results to metadata,
if \code{"get"}, returns results as a data frame.}
}
\value{
If \code{action = "add"}, returns updated \code{MultiAssayExperiment}.
Otherwise, returns a tibble with association results.
}
\description{
Test associations between each exposure and each omics feature using
limma's linear modeling framework.
}
\details{
This function uses limma to test associations between multiple exposures
and omics features. For each exposure, a linear model is fit with the
exposure as the predictor and each omics feature as the outcome,
adjusting for covariates.

\code{omics_feature ~ exposure + covariate1 + covariate2 + ...}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run exposure-omics association
mae <- mae |>
    run_exposure_omics_association(
        exposures = c("exposure_pm25", "exposure_no2"),
        covariates = c("age", "sex")
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_enrichment.R
\name{plot_enrichment}
\alias{plot_enrichment}
\title{Plot Enrichment Results from exposomicset}
\usage{
plot_enrichment(
  exposomicset,
  feature_type = c("degs", "degs_robust", "omics", "factor_features", "degs_cor",
    "omics_cor", "factor_features_cor"),
  plot_type = c("dotplot", "cnet", "network", "heatmap", "summary"),
  top_n = 5,
  n_per_group = 5,
  add_top_genes = TRUE,
  top_n_genes = 5,
  heatmap_fill = TRUE,
  logfc_thresh = log2(1),
  pval_col = "P.Value",
  pval_thresh = 0.05,
  score_metric = "stability_score",
  score_thresh = NULL,
  overlap_thresh = 0.2,
  node_radius = 0.2,
  pie_colors = NULL,
  label_top_n = NULL,
  label_colour = "black",
  net_facet_by = NULL,
  max_terms = 30,
  node_size = 1,
  term_node_correction = 0.2,
  gene_node_correction = 3,
  go_groups = NULL,
  layout_algo = "fr",
  edge_alpha = 0.3,
  label_size = 3,
  feature_col = "feature",
  logfc_col = "logFC"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object with enrichment results
added via \code{run_enrichment()}.}

\item{feature_type}{Character; one of \code{"degs"}, \code{"degs_robust"},
\code{"omics"}, \code{"factor_features"},
\code{"degs_cor"}, \code{"omics_cor"}, or \code{"factor_features_cor"}.
Defines which enrichment results to use.}

\item{plot_type}{Type of plot to generate. One of \code{"dotplot"},
\code{"cnet"}, \code{"network"}, \code{"heatmap"}, or \code{"summary"}.}

\item{top_n}{Integer; number of top \code{go_group}s to include
(used in \code{"dotplot"}). Default is \code{5}.}

\item{n_per_group}{Integer; number of terms per group to plot
(used in \code{"dotplot"}). Default is \code{5}.}

\item{add_top_genes}{Logical; if \code{TRUE}, appends top shared genes to
dotplot facets. Default is \code{TRUE}.}

\item{top_n_genes}{Integer; number of top genes to show in each group
(used in \code{"dotplot"}). Default is \code{5}.}

\item{heatmap_fill}{Logical; whether to fill tiles by logFC in the heatmap.
Default is \code{TRUE}.}

\item{logfc_thresh}{Numeric; log2 fold change threshold for filtering
(heatmap only). Default is \code{log2(1)}.}

\item{pval_col}{Column name of the p-value used for filtering in \code{"degs"}
heatmap. Default is \code{"P.Value"}.}

\item{pval_thresh}{Threshold for \code{pval_col} (heatmap only).
Default is \code{0.05}.}

\item{score_metric}{Column for stability score
(used in \code{"degs_robust"} heatmap). Default is \code{"stability_score"}.}

\item{score_thresh}{Numeric; threshold for \code{score_metric}
(heatmap only). Default is \code{NULL}.}

\item{overlap_thresh}{Numeric; Jaccard threshold for edges in
the network plot. Default is \code{0.2}.}

\item{node_radius}{Numeric; node size in network plot. Default is \code{0.2}.}

\item{pie_colors}{Optional named vector of colors for pie charts
(network and cnet).}

\item{label_top_n}{Integer; number of top nodes to label in network.
Default is \code{NULL}.}

\item{label_colour}{Color of node labels in network. Default is \code{"black"}.}

\item{net_facet_by}{Column used to facet the network plot
(e.g., \code{"category"}). Default is \code{NULL}.}

\item{max_terms}{Integer; max number of terms to include in the cnet plot.
Default is \code{30}.}

\item{node_size}{Numeric; base node size for cnet plot. Default is \code{1}.}

\item{term_node_correction}{Scaling factor for term nodes in cnet plot.
Default is \code{0.2}.}

\item{gene_node_correction}{Scaling factor for gene nodes in cnet plot.
Default is \code{3}.}

\item{go_groups}{Optional character vector of GO group names to subset
enrichment results (all plots).}

\item{layout_algo}{Graph layout algorithm to use in \code{"network"} and \code{"cnet"}
plots. Default is \code{"fr"}.}

\item{edge_alpha}{Transparency of network/cnet plot edges.
Default is \code{0.3}.}

\item{label_size}{Font size for labels in network and cnet plots.
Default is \code{3}.}

\item{feature_col}{Column name used to join gene-level metadata.
Default is \code{"feature"}.}

\item{logfc_col}{Column name used for log2 fold change values.
Default is \code{"logFC"}.}
}
\value{
A \code{ggplot} or \code{patchwork} object corresponding to the
requested plot type.
}
\description{
Visualize enrichment results stored in a \code{MultiAssayExperiment} object.
Supports dotplots, heatmaps, cnetplots, networks,
and multi-panel summary plots.
}
\details{
This function visualizes results from \code{run_enrichment()}
using one of several plot types:
\itemize{
\item \code{"dotplot"}: Enrichment terms grouped by GO group, colored by
significance.
\item \code{"heatmap"}: Term - gene matrix with optional logFC fill and
shared gene highlighting.
\item \code{"network"}: Graph of term overlap based on shared genes,
faceted by metadata if desired.
\item \code{"cnet"}: Gene - term bipartite graph with gene logFC values
and term pie slices.
\item \code{"summary"}: Multi-panel figure with GO group ridgeplots,
gene counts, and Venn diagram.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 30,
    return_mae = TRUE
)

# perform differential abundance analysis
mae <- run_differential_abundance(
    exposomicset = mae,
    formula = ~ smoker + sex,
    abundance_col = "counts",
    method = "limma_voom",
    action = "add"
)

# perform enrichment analysis
mae <- run_enrichment(
    exposomicset = mae,
    feature_type = "degs",
    feature_col = "symbol",
    species = "goa_human",
    deg_logfc_threshold = log2(1),
    deg_pval_col = "P.Value",
    deg_pval_threshold = 0.2,
    action = "add"
)

# create an enrichment plot
enr_plot <- plot_enrichment(
    exposomicset = mae,
    feature_type = "degs",
    plot_type = "network"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pivot_feature.R
\name{pivot_feature}
\alias{pivot_feature}
\title{Extract Feature Metadata from a MultiAssayExperiment}
\usage{
pivot_feature(exposomicset)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object.}
}
\value{
A tibble with feature metadata from all assays,
with an added \code{.exp_name} column.
}
\description{
Extracts feature-level metadata across all assays in a
\code{MultiAssayExperiment} and returns a combined tibble.
}
\details{
This function:
\itemize{
\item Iterates over all assays in the \code{MultiAssayExperiment}.
\item Updates each assay's sample metadata (\code{colData}) using
\code{.update_assay_colData()}.
\item Extracts feature-level metadata using \code{pivot_transcript()} from \code{tidybulk}.
\item Combines results across assays into a single tibble,
adding a \code{.exp_name} column.
}
}
\examples{

#' # create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# pivot experiment
feature_data <- mae |>
    pivot_feature()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_top_factor_features.R
\name{extract_top_factor_features}
\alias{extract_top_factor_features}
\title{Extract Top Contributing Features for Factors}
\usage{
extract_top_factor_features(
  exposomicset,
  factors = NULL,
  pval_col = "p_adjust",
  pval_thresh = 0.05,
  method = "percentile",
  percentile = 0.9,
  threshold = 0.3,
  action = "add"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
integration results.}

\item{factors}{A character vector specifying the factors of interest.
If \code{NULL}, factors are automatically selected from the association results
using the \code{pval_col} and \code{pval_thresh} criteria.}

\item{pval_col}{A string specifying the column name of the p-value or
adjusted p-value used for factor selection if \code{factors} is \code{NULL}.
Default is \code{"p_adjust"}.}

\item{pval_thresh}{A numeric value specifying the significance threshold
for selecting factors from association results when \code{factors} is \code{NULL}.
Default is \code{0.05}.}

\item{method}{A character string specifying the feature selection method
(\code{"percentile"} or \code{"threshold"}). Default is \code{"percentile"}.}

\item{percentile}{A numeric value between 0 and 1 indicating the
percentile threshold for feature selection when \code{method = "percentile"}.
Default is \code{0.9}.}

\item{threshold}{A numeric value specifying the absolute loading cutoff
for feature selection when \code{method = "threshold"}. Default is \code{0.3}.}

\item{action}{A character string indicating whether to return results
(\code{"get"}) or add them to metadata (\code{"add"}). Default is \code{"add"}.}
}
\value{
If \code{action = "add"}, returns the modified \code{exposomicset} with
selected features stored in metadata.
If \code{action = "get"}, returns a data frame containing:
\item{feature}{The selected feature contributing to the factor.}
\item{factor}{The factor to which the feature contributes.}
\item{loading}{The factor loading value of the feature.}
\item{exp_name}{The experiment from which the feature originated.}
}
\description{
Identifies the most influential features for specified factors
using multiomics integration results.
Features are selected based on either a percentile cutoff
or an absolute loading threshold.
}
\details{
The function extracts factor loadings from \code{metadata(exposomicset)},
applies filtering based on
the selected method, and identifies top contributing features for
each specified factor.

If \code{factors} is not provided, the function will automatically select
statistically significant factors from \code{metadata(exposomicset)$association$assoc_factors$results_df}
using the specified \code{pval_col} and \code{pval_thresh} as criteria.

Features can be selected using:
\itemize{
\item \strong{Percentile-based filtering} (\code{method = "percentile"}): Selects
features with absolute loadings above a specified percentile.
\item \strong{Threshold-based filtering} (\code{method = "threshold"}): Selects
features with absolute loadings exceeding a fixed value.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# perform multiomics integration
mae <- run_multiomics_integration(
    mae,
    method = "DIABLO",
    outcome = "smoker",
    n_factors = 3
)

top_feats <- extract_top_factor_features(
    mae,
    factors = c("V1", "V2", "V3"),
    method = "percentile",
    percentile = 0.9,
    action = "get"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_ont_annot.R
\name{.load_ontologies}
\alias{.load_ontologies}
\title{Internal - load ontologies}
\usage{
.load_ontologies(use_demo = TRUE, ...)
}
\arguments{
\item{use_demo}{Logical; if TRUE, load packaged demo objects \code{hpo}, \code{ecto}, \code{chebi}.}

\item{...}{Optional named overrides: \code{hpo}, \code{ecto}, \code{chebi}.}
}
\value{
A list with elements \code{hpo}, \code{ecto}, \code{chebi}.
}
\description{
Internal - load ontologies
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server_ont_annot.R
\name{.ont_annot_build_server}
\alias{.ont_annot_build_server}
\title{Internal - builds ontology annotation server}
\usage{
.ont_annot_build_server(ontologies)
}
\arguments{
\item{ontologies}{A list with \code{hpo}, \code{ecto}, \code{chebi} data.frames.}
}
\value{
A Shiny server function.
}
\description{
Internal - builds ontology annotation server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_sample_clusters.R
\name{plot_sample_clusters}
\alias{plot_sample_clusters}
\title{Plot Sample Clusters}
\usage{
plot_sample_clusters(exposomicset, exposure_cols = NULL)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing sample
clustering results
in \code{metadata(exposomicset)$sample_clustering}.}

\item{exposure_cols}{A character vector specifying columns from \code{colData}
to include
in the summary. Default is \code{NULL}, which includes all available columns.}
}
\value{
A \code{ComplexHeatmap} plot displaying sample clustering results.
}
\description{
Generates a heatmap of sample clustering results and
summarizes sample group assignments.
}
\details{
This function:
\itemize{
\item Extracts sample cluster assignments from
\code{metadata(exposomicset)$sample_clustering}.
\item Merges cluster labels with \code{colData(exposomicset)}.
\item Plots the heatmap stored in
\code{metadata(exposomicset)$sample_clustering$heatmap}.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 30,
    return_mae = TRUE
)

# determine sample clusters
mae <- run_cluster_samples(
    exposomicset = mae,
    exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi"),
    clustering_approach = "diana"
)

# plot sample clusters
sample_cluster_p <- mae |>
    plot_sample_clusters(
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app_ont_annot.R
\name{build_ont_annot_app}
\alias{build_ont_annot_app}
\title{Build the Ontology Annotation Shiny app}
\usage{
build_ont_annot_app(use_demo = TRUE, ...)
}
\arguments{
\item{use_demo}{Logical, if TRUE, load packaged lightweight demo ontologies.}

\item{...}{Optional named overrides passed as data.frames with columns
\code{id}, \code{name}, \code{ancestors}, \code{parents}, \code{children}:
\code{hpo}, \code{ecto}, \code{chebi}.}
}
\value{
A \code{shiny.appobj}.
}
\description{
Returns a Shiny app object for ontology annotation. This function does not
launch the app. Please call \code{shiny::runApp()} yourself (see examples).
}
\examples{
if (interactive()) {
    app <- build_ont_annot_app()
    shiny::runApp(app)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_exposure_omics_association.R
\name{plot_exposure_omics_association}
\alias{plot_exposure_omics_association}
\title{Plot Exposure-Omics Associations}
\usage{
plot_exposure_omics_association(
  exposomicset,
  plot_type = c("exposures", "category"),
  pval_col = "p_adjust",
  pval_thresh = 0.05
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing association results.}

\item{plot_type}{Character. One of \code{"exposures"} or \code{"category"}. Controls
whether associations are summarized per exposure or per exposure category.
Defaults to \code{"exposures"}.}

\item{pval_col}{Character. Name of the column used for p-value filtering.
Defaults to \code{"p_adjust"}.}

\item{pval_thresh}{Numeric. Significance threshold applied to \code{pval_col}.
Rows with values below this threshold are retained. Defaults to \code{0.05}.}
}
\value{
A \code{ggplot} object.
}
\description{
Plots the number of significant exposure-omics associations, grouped
either by exposure or the exposure category.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# run exposure-omics association
mae <- mae |>
    run_exposure_omics_association(
        exposures = c("exposure_pm25", "exposure_no2"),
        covariates = c("age", "sex")
    )

plot_exposure_omics_association(
    exposomicset = mae,
    plot_type = "exposures"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_cluster_samples.R
\name{run_cluster_samples}
\alias{run_cluster_samples}
\title{Cluster Samples Based on Exposure Data}
\usage{
run_cluster_samples(
  exposomicset,
  exposure_cols = NULL,
  dist_method = NULL,
  user_k = NULL,
  cluster_method = "ward.D",
  clustering_approach = "diana",
  action = "add"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing omics
and exposure data.}

\item{exposure_cols}{A character vector of column names in
\code{colData(exposomicset)} to use for clustering.}

\item{dist_method}{A character string specifying the distance metric
(\code{"euclidean"}, \code{"gower"}, etc.). If \code{NULL}, it is automatically determined.}

\item{user_k}{An integer specifying the number of clusters.
If \code{NULL}, an optimal \code{k} is determined.}

\item{cluster_method}{A character string specifying the hierarchical
clustering method. Default is \code{"ward.D"}.}

\item{clustering_approach}{A character string specifying the method
for determining \code{k} (\code{"diana"}, \code{"gap"}, \code{"elbow"}, \code{"dynamic"},
or \code{"density"}). Default is \code{"diana"}.}

\item{action}{A character string specifying \code{"add"}
(store results in metadata) or \code{"get"} (return clustering results).
Default is \code{"add"}.}
}
\value{
If \code{action="add"}, returns the updated \code{exposomicset}.
If \code{action="get"}, returns a list with:
\item{sample_cluster}{A hierarchical clustering object (\code{hclust}).}
\item{sample_groups}{A named vector of sample cluster assignments.}
}
\description{
Performs hierarchical clustering of samples using exposure data from
\code{colData(exposomicset)}.
}
\details{
This function:
\itemize{
\item Extracts \strong{numeric exposure data} from \code{colData(exposomicset)}.
\item Computes a \strong{distance matrix} (\code{"gower"} for mixed data,
\code{"euclidean"} for numeric).
\item Determines the \strong{optimal number of clusters (\code{k})} using the
specified method.
\item Performs \strong{hierarchical clustering} (\code{hclust}) and assigns
samples to clusters.
\item Generates a \strong{heatmap} of scaled exposure values.
\item Stores results in \code{metadata(exposomicset)$sample_clustering}
when \code{action="add"}.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# determine sample clusters
mae <- run_cluster_samples(
    exposomicset = mae,
    exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi"),
    clustering_approach = "diana"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_normality_summary.R
\name{plot_normality_summary}
\alias{plot_normality_summary}
\title{Plot Normality Summary of Exposure Variables}
\usage{
plot_normality_summary(exposomicset, transformed = FALSE)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object with quality control
metadata.}

\item{transformed}{Logical; if \code{TRUE}, use results after transformation.
Default is \code{FALSE}.}
}
\value{
A \code{ggplot} object summarizing the number of exposures
classified as normal or not normal.
}
\description{
Generates a bar plot summarizing the number of exposure variables that
pass or fail normality
tests (e.g., Shapiro-Wilk) before or after transformation.
}
\details{
This function assumes that \code{run_normality_check()} has been executed and
that the results are
stored in \code{metadata(exposomicset)$quality_control$normality}.
If \code{transformed = TRUE}, the function will
instead plot the transformation summary stored in \code{metadata(exposomicset)$quality_control$transformation$norm_summary},
which is populated by \code{transform_exposure()}.

The plot includes both bar heights and overlaid line segments to
reinforce the counts.
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Test for normality
mae <- mae |>
    run_normality_check() |>
    transform_exposure(exposure_cols = c("age", "bmi", "exposure_pm25"))

# plot the normality summary
norm_p <- mae |>
    plot_normality_summary()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_correlation_tile.R
\name{plot_correlation_tile}
\alias{plot_correlation_tile}
\title{Plot Correlation Tilemap}
\usage{
plot_correlation_tile(
  exposomicset,
  feature_type = c("pcs", "degs", "omics", "factors", "factor_features", "exposures"),
  pval_cutoff = 0.05,
  na_color = "grey100",
  fill_limits = c(-1, 1),
  midpoint = 0
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing correlation
results in metadata.}

\item{feature_type}{Type of correlation results to plot.
One of \code{"pcs"}, \code{"degs"}, \code{"omics"},
\code{"factors"}, \code{"factor_features"}, or \code{"exposures"}.
Must match the key used in
\code{metadata(exposomicset)$correlation[[feature_type]]}.}

\item{pval_cutoff}{Numeric p-value cutoff below which
correlations are displayed.
Nonsignificant tiles are rendered in the \code{na_color}.
Default is \code{0.05}.}

\item{na_color}{Color used to represent nonsignificant or
missing correlations. Default is \code{"grey100"}.}

\item{fill_limits}{Numeric vector of length 2 defining the scale
limits for correlation values. Default is \code{c(-1, 1)}.}

\item{midpoint}{Numeric value for centering the fill gradient.
Default is \code{0}.}
}
\value{
A \code{ggplot2} tile plot (or a combined tile + barplot if
\code{feature_type = "pcs"}).
}
\description{
Visualizes a correlation matrix as a heatmap tile plot using
correlation results stored in the metadata of a
\code{MultiAssayExperiment} object. When \code{feature_type = "pcs"},
the function forces PCs to appear on the x-axis and exposures on the y-axis,
and it adds a barplot showing how many PCs are significantly
associated with each exposure. It also suppresses nonsignificant tiles
based on a specified p-value threshold.
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run pca
mae <- mae |>
    run_pca()

# correlate with exposures
mae <- mae |>
    run_correlation(
        feature_type = "pcs",
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# make the correlation tile plot
cor_tile_p <- mae |>
    plot_correlation_tile(
        feature_type = "pcs"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_top_factor_features.R
\name{plot_top_factor_features}
\alias{plot_top_factor_features}
\title{Plot Top Features by Factor from Integration Results}
\usage{
plot_top_factor_features(
  exposomicset,
  feature_col = "feature",
  factors = NULL,
  top_n = 5,
  facet_cols = NULL,
  exp_name_cols = NULL,
  alpha = 0.5
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
integration results in the \code{metadata} slot (must include \code{integration_results}).}

\item{feature_col}{A character string indicating the column name to use
for y-axis feature labels (e.g., \code{"feature"}, \code{"gene_symbol"}). This should
match a column in the output of \code{pivot_feature()}. Default is \code{"feature"}.}

\item{factors}{Character vector of factors to include
(e.g., "Factor1", "Factor2"). If \code{NULL}, all factors are plotted.}

\item{top_n}{Integer specifying the number of top features to show
per factor. Default is \code{5}.}

\item{facet_cols}{Optional color palette for facet strip backgrounds
(one per \code{exp_name}), used to distinguish factors.}

\item{exp_name_cols}{Optional color palette for experiment labels
in the plot (\code{exp_name}), passed to \code{scale_color_manual()}.}

\item{alpha}{Numeric value between 0 and 1 controlling the
transparency of facet strip background fill. Default is \code{0.5}.}
}
\value{
A \code{ggplot2} object with one facet per factor, showing the
top features and their loadings by experiment.
}
\description{
Visualizes the top loading features for each factor from multi-omics
integration results (e.g., MOFA, MCIA, DIABLO, RGCCA).
}
\details{
This function supports the following integration methods:
\itemize{
\item \code{"MOFA"}: Uses feature weights from MOFA2 (\code{get_weights()}).
\item \code{"MCIA"}: Uses block loadings from MCIA (\verb{@block_loadings}).
\item \code{"DIABLO"}: Extracts block-specific loadings from \code{loadings}.
\item \code{"RGCCA"}: Extracts block-specific loadings from \code{a}.
}

For each factor, it:
\itemize{
\item Selects the top \code{top_n} features by \strong{absolute loading}.
\item Merges with feature metadata using \code{pivot_feature()}.
\item Creates a point-range plot showing the loading magnitude.
\item Facets each factor with a customizable strip background.
}

The \code{feature_col} argument allows you to control which feature-level
metadata column (e.g., gene symbols, metabolite names) is used for
labeling the y-axis.

If palettes are not provided, defaults are chosen using
\code{ggpubr::get_palette()}.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

mae <- run_multiomics_integration(
    mae,
    method = "DIABLO",
    outcome = "smoker",
    n_factors = 3
)

# plot top features using default `feature` column
top_feature_p <- mae |>
    plot_top_factor_features()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_circos_correlation.R
\name{plot_circos_correlation}
\alias{plot_circos_correlation}
\title{Plot Circular Network of Exposure Relationships}
\usage{
plot_circos_correlation(
  exposomicset,
  feature_type = c("degs", "omics", "factors", "factor_features", "exposures", "pcs"),
  exposure_cols = NULL,
  corr_threshold = NULL,
  shared_cutoff = 10,
  annotation_colors = NULL,
  low = "#006666",
  mid = "white",
  high = "#8E0152",
  midpoint = NULL
)
}
\arguments{
\item{exposomicset}{A MultiAssayExperiment object.}

\item{feature_type}{One of "exposures", "degs", or "factors".}

\item{exposure_cols}{Character vector of exposures to include
(only for "exposures").}

\item{corr_threshold}{Minimum |correlation| (only for "exposures").}

\item{shared_cutoff}{Minimum number of shared features
(only for "degs" or "factors"). Default = 10.}

\item{annotation_colors}{Optional named vector of colors for categories.}

\item{low}{low value color for edges.}

\item{mid}{middle value color for edges.}

\item{high}{high value color for edges.}

\item{midpoint}{Midpoint for edge color gradient. Defaults to 0
(for correlations) or mean shared features.}
}
\value{
A ggplot object (ggraph circular plot).
}
\description{
Generates a circular network plot to visualize relationships between
exposures, either based on correlation ("exposures") or
shared features ("degs", "factors").
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run correlation analysis
mae <- mae |>
    run_correlation(
        feature_type = "exposures",
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# create the circos plot
circos_plot <- mae |>
    plot_circos_correlation(
        feature_type = "exposures"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{.onLoad}
\alias{.onLoad}
\title{Internal - onLoad hook to register www assets}
\usage{
.onLoad(libname, pkgname)
}
\value{
Invisibly returns \code{NULL}
}
\description{
Registers \code{inst/app/www} as a Shiny resource path \code{"www"} if present.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/make_example_data.R
\name{make_example_data}
\alias{make_example_data}
\title{Generate Example Data for Testing}
\usage{
make_example_data(
  n_samples = 12,
  n_proteins = 80,
  use_batch = FALSE,
  return_mae = FALSE
)
}
\arguments{
\item{n_samples}{Integer. Number of samples to simulate (default: 12).}

\item{n_proteins}{Integer. Number of proteins to simulate (default: 80).}

\item{use_batch}{Logical. If \code{TRUE},
include a "batch" variable in the exposure data (default: FALSE).}

\item{return_mae}{Logical. If \code{TRUE},
return a \code{MultiAssayExperiment} created using
\code{create_exposomicset()} (default: FALSE).}
}
\value{
Either:
\itemize{
\item A named list containing \code{codebook},
\code{exposure}, \code{omics}, and \code{row_data},
if \code{return_mae = FALSE}.
\item A \code{MultiAssayExperiment}, if \code{return_mae = TRUE}.
}
}
\description{
This helper function generates a reproducible dummy dataset
containing exposures, mRNA data, and proteomics data. It can
optionally return the data as a \code{MultiAssayExperiment} using
\code{\link{create_exposomicset}}.
}
\examples{
# Return as a list
dummy <- make_example_data()

# Return as a MultiAssayExperiment
mae <- make_example_data(return_mae = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_volcano.R
\name{plot_volcano}
\alias{plot_volcano}
\title{Volcano Plot of Differential Abundance}
\usage{
plot_volcano(
  exposomicset,
  pval_col = "adj.P.Val",
  pval_thresh = 0.05,
  logFC_col = "logFC",
  logFC_thresh = log2(1.5),
  plot_n_sig = TRUE,
  top_n_label = NULL,
  features_to_label = NULL,
  feature_col = "feature",
  xlab = expression(Log[2] * "FC"),
  ylab = expression(-Log[10] * "P"),
  title = "Volcano Plot of Differential Abundance",
  nrow = 2
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing differential
abundance results in \code{metadata(exposomicset)$differential_abundance}.}

\item{pval_col}{A character string specifying the column containing p-values.
Default is \code{"adj.P.Val"}.}

\item{pval_thresh}{A numeric threshold for significance. Features with
p-values below this are considered significant. Default is \code{0.05}.}

\item{logFC_col}{A character string specifying the column for
log fold changes. Default is \code{"logFC"}.}

\item{logFC_thresh}{A numeric threshold for absolute
log fold change significance. Default is \code{log2(1.5)}.}

\item{plot_n_sig}{Logical; if \code{TRUE}, appends the number of
significant features to facet titles. Default is \code{TRUE}.}

\item{top_n_label}{Optional integer. If provided, the top \code{n}
most significant features per assay will be labeled on the plot.}

\item{features_to_label}{Optional character vector.
Specific features to label regardless of significance.}

\item{feature_col}{A character string naming the feature ID
column to use for labeling. Default is \code{"feature"}.}

\item{xlab}{Label for the x-axis. Default is \code{expression(Log[2]*"FC")}.}

\item{ylab}{Label for the y-axis. Default is \code{expression(-Log[10]*"P")}.}

\item{title}{Plot title. Default is
\code{"Volcano Plot of Differential Abundance"}.}

\item{nrow}{Number of rows in the \code{facet_wrap()} layout.
Default is \code{2}.}
}
\value{
A \code{ggplot2} object representing the volcano plot.
}
\description{
Generates a \strong{volcano plot} to visualize differential abundance results
across one or more omics layers.
}
\details{
The function:
\itemize{
\item Extracts differential abundance results from
\code{metadata(exposomicset)$differential_abundance}.
\item Assigns each feature a direction of change:
\strong{Upregulated}, \strong{Downregulated}, or \strong{Not-Significant}.
\item Uses \code{logFC_thresh} and \code{pval_thresh} to define thresholds.
\item Adds dashed lines to indicate cutoffs for fold change and significance.
\item Uses \code{facet_wrap()} to display each assay (\code{exp_name}) separately.
\item Optionally labels the most significant features or user-defined ones.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# perform differential abundance analysis
mae <- run_differential_abundance(
    exposomicset = mae,
    formula = ~ smoker + sex,
    abundance_col = "counts",
    method = "limma_voom",
    action = "add"
)

# create the volcano plot
volcano_p <- mae |>
    plot_volcano()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_association.R
\name{plot_association}
\alias{plot_association}
\title{Plot Association Results (Unified Forest Plot)}
\usage{
plot_association(
  exposomicset,
  source = c("omics", "exposures", "factors", "go_pcs"),
  terms = NULL,
  filter_col = "p.value",
  filter_thresh = 0.05,
  direction_filter = "all",
  add_r2_tile = TRUE,
  r2_col = "adj_r2",
  facet = FALSE,
  nrow = 1,
  subtitle = NULL
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing association
results in metadata.}

\item{source}{Character string indicating the association source.
One of \code{"omics"}, \code{"exposures"}, \code{"factors"}, or \code{"go_pcs"}.}

\item{terms}{Optional character vector of term names to subset the plot to.
Default is \code{NULL} (include all).}

\item{filter_col}{Column used to assess statistical significance
(default: \code{"p.value"}).}

\item{filter_thresh}{Numeric threshold applied to \code{filter_col}
(default: \code{0.05}).}

\item{direction_filter}{Direction of associations to retain.
One of \code{"all"} (default), \code{"up"}, or \code{"down"}.}

\item{add_r2_tile}{Logical; if \code{TRUE}, includes a tile plot for \code{r2_col}
(default: \code{TRUE}).}

\item{r2_col}{Column used for coloring the tile plot (default: \code{"adj_r2"}).}

\item{facet}{Logical; if \code{TRUE} and \code{source == "go_pcs"},
apply nested faceting by experiment and GO cluster (default: \code{FALSE}).}

\item{nrow}{Integer; number of rows for facet layout if enabled (default: \code{1}).}

\item{subtitle}{Optional subtitle for the plot. If \code{NULL},
automatically generated from covariates used in the model.}
}
\value{
A \code{ggplot2} object: either a single forest plot or a composite plot
with an R^2 tile strip.
}
\description{
Generates a forest plot for association results from any source stored
in the metadata of a \code{MultiAssayExperiment} object.
Supports faceting and visual augmentation with R^2 tiles when available.
}
\details{
This function visualizes effect size estimates and confidence intervals from
association analyses. It allows filtering by
direction (\code{"up"} for positive, \code{"down"} for negative) and by significance.
For \code{source = "go_pcs"}, it supports special
formatting by splitting term labels into nested facets.

The R^2 tile (if enabled) adds a side heatmap indicating model fit for
each association. This can be useful for model diagnostics.
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run association tests
mae <- mae |>
    run_association(
        source = "exposures",
        feature_set = c("exposure_pm25", "exposure_no2"),
        outcome = "smoker",
        covariates = c("age"),
        family = "binomial"
    )

assoc_plot <- mae |>
    plot_association(
        source = "exposures"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_loading.R
\name{load_annotation_data}
\alias{load_annotation_data}
\title{Load Ontology Data}
\usage{
load_annotation_data(
  to_load = c("all", "chebi", "ecto", "hpo"),
  verbose = TRUE,
  validate = TRUE
)
}
\arguments{
\item{to_load}{Character vector indicating which ontology to load.}

\item{verbose}{Logical; print messages.}

\item{validate}{Logical; validate MD5 checksum.}
}
\value{
A named list of ontology objects.
}
\description{
Downloads and loads \code{chebi}, \code{ecto}, and \code{hpo}.
}
\examples{
\dontrun{
# load single ontology
onts <- load_annotation_data(to_load = "ecto")

# load all ontologies
onts <- load_annotation_data(to_load = "all")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_correlation.R
\name{run_correlation}
\alias{run_correlation}
\title{Run Correlation Analysis}
\usage{
run_correlation(
  exposomicset,
  feature_type = c("degs", "omics", "factors", "factor_features", "exposures", "pcs"),
  exposure_cols = NULL,
  variable_map = NULL,
  n_pcs = NULL,
  feature_cors = FALSE,
  robust = FALSE,
  score_col = "stability_score",
  score_thresh = NULL,
  correlation_method = "spearman",
  correlation_cutoff = 0.3,
  cor_pval_column = "p.value",
  pval_cutoff = 0.05,
  deg_pval_col = "adj.P.Val",
  deg_logfc_col = "logFC",
  deg_pval_thresh = 0.05,
  deg_logfc_thresh = log2(1.5),
  batch_size = 1500,
  action = c("add", "get")
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object.}

\item{feature_type}{Type of features to correlate. One of \code{"degs"},
\code{"omics"}, \code{"factors"}, \code{"factor_features"}, \code{"exposures"}, or \code{"pcs"}.}

\item{exposure_cols}{Optional vector of exposure column names
(from \code{colData}) to use.}

\item{variable_map}{Optional mapping of features to include by assay
for \code{omics} mode.}

\item{n_pcs}{Number of PCs to use when \code{feature_type = "pcs"}.}

\item{feature_cors}{Logical; if \code{TRUE}, compute correlations between
features rather than with exposures.}

\item{robust}{Logical; restrict DEGs to those passing sensitivity threshold.}

\item{score_col}{Column name in sensitivity analysis with feature
stability score.}

\item{score_thresh}{Threshold for filtering robust features.}

\item{correlation_method}{One of \code{"pearson"}, \code{"spearman"}, or \code{"kendall"}.}

\item{correlation_cutoff}{Minimum absolute correlation to retain.}

\item{cor_pval_column}{Column in output to filter by p-value
(default: \code{"p.value"}).}

\item{pval_cutoff}{Maximum p-value or FDR threshold to retain a correlation.}

\item{deg_pval_col}{Column with DEG adjusted p-values.}

\item{deg_logfc_col}{Column with DEG log fold-changes.}

\item{deg_pval_thresh}{P-value cutoff for DEGs.}

\item{deg_logfc_thresh}{Log fold-change cutoff for DEGs.}

\item{batch_size}{Number of features to process per batch (default: 1500).}

\item{action}{Whether to \code{"add"} results to metadata or \code{"get"}
as a data frame.}
}
\value{
If \code{action = "add"}, returns updated \code{MultiAssayExperiment}
with results added to metadata.
If \code{action = "get"}, returns a tidy \code{data.frame} of correlations.
}
\description{
Computes correlations between exposures and feature types including DEGs,
omics, latent factors,
top factor features, or principal components (PCs). Optionally computes
feature-feature correlations
to support network analysis.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run correlation analysis
mae <- mae |>
    run_correlation(
        feature_type = "exposures",
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_sensitivity_analysis.R
\name{run_sensitivity_analysis}
\alias{run_sensitivity_analysis}
\title{Run Sensitivity Analysis for Differential Abundance}
\usage{
run_sensitivity_analysis(
  exposomicset,
  base_formula,
  abundance_col = "counts",
  methods = c("limma_trend", "limma_voom", "DESeq2", "edgeR_quasi_likelihood"),
  scaling_methods = c("none", "TMM", "quantile"),
  contrasts = NULL,
  covariates_to_remove = NULL,
  pval_col = "adj.P.Val",
  logfc_col = "logFC",
  pval_threshold = 0.05,
  logFC_threshold = log2(1),
  score_thresh = NULL,
  score_quantile = 0.9,
  stability_metric = "stability_score",
  action = "add",
  bootstrap_n = 1
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} containing the assays to analyze.}

\item{base_formula}{The base model formula used for differential analysis.}

\item{abundance_col}{Character. Name of the column in the assays representing
abundance. Default is \code{"counts"}.}

\item{methods}{Character vector of differential expression methods.
Options include \code{"limma_trend"} ,\code{"limma_voom"}, \code{"DESeq2"}, and \code{"edgeR_quasi_likelihood"}.}

\item{scaling_methods}{Character vector of normalization methods to try.
Options include \code{"none"}, \code{"TMM"}, and \code{"quantile"}.}

\item{contrasts}{Optional list of contrasts to apply for differential testing.}

\item{covariates_to_remove}{Optional character vector of covariates
to remove from the base formula to generate model variants.}

\item{pval_col}{Name of the column containing p-values or adjusted
p-values used to define significance.}

\item{logfc_col}{Name of the column containing log fold changes.}

\item{pval_threshold}{Numeric threshold for significance. Default is 0.05.}

\item{logFC_threshold}{Numeric threshold for absolute log fold change.
Default is \code{log2(1)} (i.e., 0).}

\item{score_thresh}{Optional threshold for the selected stability metric.
If not provided, calculated using \code{score_quantile}.}

\item{score_quantile}{Quantile used to define the threshold
if \code{score_thresh} is not provided. Default is 0.9.}

\item{stability_metric}{Character. Name of the column in
\code{feature_stability} to use as the scoring metric. Default is \code{"stability_score"}.}

\item{action}{Whether to \code{"add"} results to \code{metadata()} or
\code{"get"} them as a list. Default is \code{"add"}.}

\item{bootstrap_n}{Integer. Number of bootstrap iterations.
If 0, no resampling is performed. Default is 1.}
}
\value{
If \code{action = "add"}, returns a \code{MultiAssayExperiment}
with results stored in
\code{metadata(exposomicset)$differential_analysis$sensitivity_analysis}.
If \code{action = "get"},
returns a list with three elements:
\describe{
\item{\code{sensitivity_df}}{Data frame of all differential results
across model/method combinations.}
\item{\code{feature_stability}}{Data frame summarizing feature
stability scores.}
\item{\code{score_thresh}}{The threshold used to define stable features.}
}
}
\description{
Performs sensitivity analysis by systematically varying statistical methods,
scaling strategies, and model formulas (with optional bootstrap sampling) to
assess the stability of differentially abundant features.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)


# Run differential abundance
mae <- run_differential_abundance(
    exposomicset = mae,
    formula = ~ smoker + sex,
    abundance_col = "counts",
    method = "limma_voom",
    action = "add"
)

# Run the sensitivity analysis
mae <- run_sensitivity_analysis(
    exposomicset = mae,
    base_formula = ~ smoker + sex,
    methods = c("limma_voom"),
    scaling_methods = c("none"),
    covariates_to_remove = "sex",
    pval_col = "P.Value",
    logfc_col = "logFC",
    pval_threshold = 0.05,
    logFC_threshold = 0,
    bootstrap_n = 3,
    action = "add"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_factor_summary.R
\name{plot_factor_summary}
\alias{plot_factor_summary}
\title{Plot Summary of Factor Contributions from Multi-Omics Integration}
\usage{
plot_factor_summary(
  exposomicset,
  low = "#006666",
  mid = "white",
  high = "#8E0152",
  midpoint = 0.5
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing integration
results in
\code{metadata(exposomicset)$multiomics_integration$integration_results}.}

\item{low}{Color for low values in the fill gradient.
Default is \code{"#006666"}.}

\item{mid}{Color for midpoint in the fill gradient.
Default is \code{"white"}.}

\item{high}{Color for high values in the fill gradient.
Default is \code{"#8E0152"}.}

\item{midpoint}{Midpoint value for the gradient color scale.
Default is \code{0.5}.}
}
\value{
A \code{ggplot} object showing factor contributions based on
the integration method.
}
\description{
Generates a summary plot of factor contributions from multi-omics
integration results stored in a \code{MultiAssayExperiment} object.
}
\details{
This function visualizes factor contributions based on the integration method:
\itemize{
\item \strong{MOFA}: Variance explained per factor and view.
\item \strong{MCIA}: Block score weights per omic.
\item \strong{DIABLO}: Mean absolute sample score per omic and factor
(from block-specific variates).
\item \strong{RGCCA}: Mean absolute sample score per omic and factor
(from aligned block scores).
}

The color gradient can be customized using the \code{low}, \code{mid}, \code{high},
and \code{midpoint} parameters.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

mae <- run_multiomics_integration(
    mae,
    method = "DIABLO",
    outcome = "smoker",
    n_factors = 3
)

factor_sum_plot <- mae |>
    plot_factor_summary()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_differential_abundance.R
\name{run_differential_abundance}
\alias{run_differential_abundance}
\title{Run Differential Abundance Analysis}
\usage{
run_differential_abundance(
  exposomicset,
  formula,
  abundance_col = "counts",
  method = "limma_trend",
  contrasts = NULL,
  scaling_method = "none",
  action = "add"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} containing assays to test.}

\item{formula}{A model formula for the differential analysis
(e.g., ~ group + batch).}

\item{abundance_col}{Character. The name of the assay matrix to use
for abundance values. Default is \code{"counts"}.}

\item{method}{Character. Differential analysis method to use.
Currently supports \code{"limma_trend"} (default).}

\item{contrasts}{A named list of contrasts for pairwise comparisons.
Default is \code{NULL} (uses default group comparisons).}

\item{scaling_method}{Character. Scaling method to apply before modeling.
Options include \code{"none"} (default), \code{"zscore"}, etc.}

\item{action}{Character. Whether to \code{"add"} results to \code{exposomicset}
metadata or \code{"get"} the results as a data frame. Default is \code{"add"}.}
}
\value{
Either the updated \code{MultiAssayExperiment} (if \code{action = "add"})
or a tibble with differential abundance results (if \code{action = "get"}).
}
\description{
Performs differential abundance testing across all assays in a
\code{MultiAssayExperiment} object using a specified statistical method.
The function updates each assay with its
corresponding \code{colData}, fits the model using the provided formula,
and combines the results into a unified table.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# perform differential abundance analysis
mae <- run_differential_abundance(
    exposomicset = mae,
    formula = ~ smoker + sex,
    abundance_col = "counts",
    method = "limma_trend",
    action = "add"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_omics.R
\name{filter_omics}
\alias{filter_omics}
\title{Filter low-quality features in omics assays}
\usage{
filter_omics(
  exposomicset,
  method = c("variance", "expression"),
  assays = NULL,
  assay_name = 1,
  min_var = 1e-05,
  min_value = 5,
  min_prop = 0.7,
  verbose = TRUE
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing omics assays.}

\item{method}{Filtering method: either \code{"variance"} or \code{"expression"}.}

\item{assays}{Character vector of assay names to filter.
If \code{NULL}, all assays are filtered.}

\item{assay_name}{Name or index of the assay within each
\code{SummarizedExperiment} to use.}

\item{min_var}{Minimum variance threshold (used if \code{method = "variance"}).}

\item{min_value}{Minimum expression value (used if \code{method = "expression"}).}

\item{min_prop}{Minimum proportion of samples exceeding \code{min_value}
(used if \code{method = "expression"}).}

\item{verbose}{Whether to print messages for each assay being filtered.}
}
\value{
A filtered \code{MultiAssayExperiment} object with
updated assays and step record.
}
\description{
This function applies variance- or expression-based filtering
across one or more assays within a \code{MultiAssayExperiment} object.
It is useful for removing low-quality or uninformative features
before downstream analysis.
}
\examples{
# Filter the proteomics assay by variance
filtered_mae <- filter_omics(
    exposomicset = make_example_data(return_mae = TRUE),
    method = c("variance"),
    assays = "proteomics",
    assay_name = 1,
    min_var = 0.01,
    verbose = TRUE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_factor_overlap.R
\name{run_factor_overlap}
\alias{run_factor_overlap}
\title{Identify and Annotate Shared Top Features Across Integration Factors}
\usage{
run_factor_overlap(
  exposomicset,
  robust = TRUE,
  stability_score = NULL,
  score_col = "stability_score",
  pval_thresh = 0.05,
  logfc_thresh = log2(1.5),
  pval_col = "padj",
  logfc_col = "logFC",
  action = "add"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} with integration results
and top factor features.}

\item{robust}{Logical; if \code{TRUE}, uses sensitivity score. Otherwise,
uses DEG thresholds.}

\item{stability_score}{Optional numeric threshold
(overrides default from metadata).}

\item{score_col}{Column name for sensitivity score.
Default is \code{"stability_score"}.}

\item{pval_thresh}{DEG p-value threshold (if \code{robust = FALSE}).
Default is \code{0.05}.}

\item{logfc_thresh}{DEG logFC threshold (if \code{robust = FALSE}).
Default is \code{log2(1.5)}.}

\item{pval_col}{Column name for p-value. Default is \code{"padj"}.}

\item{logfc_col}{Column name for logFC. Default is \code{"logFC"}.}

\item{action}{\code{"add"} to return modified object, \code{"get"} to
return data.frame.}
}
\value{
Modified \code{MultiAssayExperiment} or \code{data.frame} of
shared top features.
}
\description{
Identifies top features shared across factors based on integration method.
For MOFA/MCIA, takes intersection across factors. For DIABLO/RGCCA,
takes features recurring in more than 2 block-specific components.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# perform multiomics integration
mae <- run_multiomics_integration(
    mae,
    method = "DIABLO",
    outcome = "smoker",
    n_factors = 3
)


# identify the features that contribute most to the factors
mae <- extract_top_factor_features(
    mae,
    factors = c("V1", "V2", "V3"),
    method = "percentile",
    percentile = 0.5,
    action = "add"
)

# perform differential abundance analysis
mae <- run_differential_abundance(
    exposomicset = mae,
    formula = ~ smoker + sex,
    abundance_col = "counts",
    method = "limma_voom",
    action = "add"
)

# determine the overlap in features
mae <- mae |>
    run_factor_overlap(
        robust = FALSE,
        pval_col = "adj.P.Val"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_sensitivity_summary.R
\name{plot_sensitivity_summary}
\alias{plot_sensitivity_summary}
\title{Plot Sensitivity Analysis Summary}
\usage{
plot_sensitivity_summary(
  exposomicset,
  stability_score_thresh = NULL,
  stability_metric = "stability_score",
  title = "Distribution of Stability Scores"
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing sensitivity
analysis results
in \code{metadata(exposomicset)$sensitivity_analysis}.}

\item{stability_score_thresh}{A numeric threshold for stability scores.
Default is \code{NULL},
which uses the threshold stored in
\code{metadata(exposomicset)$sensitivity_analysis$score_thresh}.}

\item{stability_metric}{A character string specifying which
stability metric to plot (e.g., "stability_score",
"logp_weighted_score"). Default is "stability_score".}

\item{title}{A character string specifying the title of the ridge plot.
Default is
"Distribution of Stability Scores".}
}
\value{
A \code{patchwork} object combining a ridge plot and a bar chart.
}
\description{
Generates a ridge plot and bar chart summarizing feature stability
scores across assays.
}
\details{
This function:
\itemize{
\item Extracts feature stability scores from \code{metadata(exposomicset)$sensitivity_analysis$feature_stability}.
\item Displays a \strong{ridge plot} of stability score distributions per assay.
\item Displays a \strong{bar chart} of the number of features per assay.
\item Prints the number of features with stability scores above the threshold.
}
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)


# Run differential abundance
mae <- run_differential_abundance(
    exposomicset = mae,
    formula = ~ smoker + sex,
    abundance_col = "counts",
    method = "limma_voom",
    action = "add"
)

# Run the sensitivity analysis
mae <- run_sensitivity_analysis(
    exposomicset = mae,
    base_formula = ~ smoker + sex,
    methods = c("limma_voom"),
    scaling_methods = c("none"),
    covariates_to_remove = "sex",
    pval_col = "P.Value",
    logfc_col = "logFC",
    pval_threshold = 0.05,
    logFC_threshold = 0,
    bootstrap_n = 3,
    action = "add"
)

# create the sensitivity summary plot
sens_sum_p <- mae |>
    plot_sensitivity_summary()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_impute_missing.R
\name{run_impute_missing}
\alias{run_impute_missing}
\title{Impute Missing Exposure and Omics Data in a MultiAssayExperiment}
\usage{
run_impute_missing(
  exposomicset,
  exposure_impute_method = "median",
  exposure_cols = NULL,
  omics_impute_method = NULL,
  omics_to_impute = NULL
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
exposures and omics data.}

\item{exposure_impute_method}{Character. Imputation method to use for
exposure variables. Defaults to \code{"median"}.}

\item{exposure_cols}{Character vector. Names of columns in
\code{colData} to impute. If \code{NULL}, all numeric columns are used.}

\item{omics_impute_method}{Character. Imputation method to use for
omics data. Defaults to \code{"knn"}.}

\item{omics_to_impute}{Character vector. Names of omics datasets to impute.
If \code{NULL}, all omics datasets are included.}
}
\value{
A \code{MultiAssayExperiment} object with imputed exposure
and/or omics data.
}
\description{
Performs missing data imputation on both exposure variables
(from \code{colData}) and omics datasets (from \code{experiments}) within
a \code{MultiAssayExperiment} object.
}
\details{
For exposures, numeric columns in \code{colData} are imputed using
the selected method. For omics data, assays are selected and
imputed individually.

Supported imputation methods include:
\itemize{
\item \code{"median"}: Median imputation using
\code{naniar::impute_median_all}
\item \code{"mean"}: Mean imputation using
\code{naniar::impute_mean_all}
\item \code{"knn"}: k-nearest neighbor imputation using
\code{impute::impute.knn}
\item \code{"mice"}: Multiple imputation using chained equations
(\code{mice::mice})
\item \code{"missforest"}: Random forest-based imputation using
\code{missForest::missForest}
\item \code{"lod_sqrt2"}: Substitution of missing values with
LOD/sqrt(2), where LOD is the smallest non-zero value per variable
}
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Introduce some missingness
MultiAssayExperiment::colData(mae)$exposure_pm25[sample(1:20, 5)] <- NA

# Filter features and exposures with high missingness
mae <- run_impute_missing(
    exposomicset = mae,
    exposure_impute_method = "median"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interface_ont_annot.R
\name{.ont_annot_build_ui}
\alias{.ont_annot_build_ui}
\title{Internal - builds ontology annotation UI}
\usage{
.ont_annot_build_ui()
}
\value{
A \code{shiny.tag} UI definition.
}
\description{
Internal - builds ontology annotation UI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_missing.R
\name{filter_missing}
\alias{filter_missing}
\title{Filter Features and Variables with High Missingness}
\usage{
filter_missing(exposomicset, na_thresh = 20, na_plot_thresh = 5)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
exposure and omics data.}

\item{na_thresh}{A numeric value specifying the percentage of
missing data allowed before a variable or feature is removed. Default is \code{20}.}

\item{na_plot_thresh}{A numeric value specifying the
minimum missing percentage for inclusion in QC plots. Default is \code{5}.}
}
\value{
A \code{MultiAssayExperiment} object with filtered exposure
variables and omics features.
QC results, including missingness summaries and plots,
are stored in \code{metadata(exposomicset)$na_qc}.
}
\description{
Removes exposure variables and omics features with
missing values above a specified threshold.
Generates missing data summaries and quality control (QC) plots.
}
\details{
The function assesses missingness in both \code{colData(exposomicset)}
(exposure data) and \code{experiments(exposomicset)} (omics data).
\itemize{
\item Exposure variables with more than \code{na_thresh}\% missing values are removed.
\item Omics features (rows in assay matrices) exceeding \code{na_thresh}\%
missing values are filtered.
\item Missingness summaries and QC plots are generated using
\code{naniar::gg_miss_var()} and stored in metadata.
}
}
\examples{
# Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Introduce some missingness
MultiAssayExperiment::colData(mae)$exposure_pm25[sample(1:20, 5)] <- NA

# Filter features and exposures with high missingness
mae_filtered <- filter_missing(
    exposomicset = mae,
    na_thresh = 20,
    na_plot_thresh = 5
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_missing.R
\name{plot_missing}
\alias{plot_missing}
\title{Plot Missing Data Across Exposure and Omic Layers}
\usage{
plot_missing(
  exposomicset,
  threshold = 5,
  plot_type = c("summary", "lollipop"),
  layers = NULL
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing exposure
and omics assays. Missing data is inferred directly from the assays.}

\item{threshold}{Numeric. The percentage threshold (0-100) above which
features are counted as missing in the summary plot. Default is \code{5}.}

\item{plot_type}{Character. Type of plot to generate. Either \code{"summary"}
for a bar plot showing number of features above the missing threshold,
or \code{"lollipop"} for a per-feature lollipop plot with layer annotations.
Default is \code{"summary"}.}

\item{layers}{Optional character vector. If specified, filters the plot
to include only selected layers (e.g., \code{"Exposure"}, \code{"Transcriptome"}).}
}
\value{
A \code{ggplot} or \code{patchwork} object depending on the selected
\code{plot_type}.
}
\description{
Visualizes missing data patterns in a \code{MultiAssayExperiment} object using
summary bar plots or feature-level lollipop plots.
}
\details{
The function calculates missing data per feature (or variable) across all
assays (including exposure variables) and generates:
\itemize{
\item \strong{Summary plot (\verb{plot_type = "summary})}: A bar plot showing the number
of variables in each assay exceeding the specified missingness threshold.
\item \strong{Lollipop plot (\verb{plot_type = "lollipop})}: A feature-level plot where
each feature's percent missingness is shown, along with a color-coded tile
on the side indicating its layer of origin.
}

The tile colors in the lollipop plot match the experiment colors used in
other visualizations (e.g., via \code{scale_color_tidy_exp()}).
}
\examples{

#' # Create example data
mae <- make_example_data(
    n_samples = 20,
    return_mae = TRUE
)

# Introduce some missingness
MultiAssayExperiment::colData(mae)$exposure_pm25[sample(1:20, 5)] <- NA

# Summary bar plot of missing data
summary_p <- plot_missing(
    mae,
    threshold = 10,
    plot_type = "summary"
)

# Lollipop plot for all features with any missingness
lollipop_p <- plot_missing(
    mae,
    plot_type = "lollipop"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_exposure_impact.R
\name{plot_exposure_impact}
\alias{plot_exposure_impact}
\title{Plot Exposure Impact on Network Centrality Metrics}
\usage{
plot_exposure_impact(
  exposomicset,
  feature_type = c("degs", "omics", "factors"),
  min_per_group = 5,
  facet_cols = NULL,
  bar_cols = NULL,
  alpha = 0.3,
  ncol = 2,
  nrow = 1,
  heights = c(1, 1),
  widths = c(2, 1)
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object with results
from \code{run_exposure_impact()}.}

\item{feature_type}{Character string specifying the feature type.
One of \code{"degs"}, \code{"omics"}, or \code{"factors"}.}

\item{min_per_group}{Minimum number of features per exposure for
inclusion (not currently used). Default is \code{5}.}

\item{facet_cols}{Optional named vector of colors for exposure categories.}

\item{bar_cols}{Optional vector of colors for bar plots (if enabled).}

\item{alpha}{Transparency level for category strips (if enabled).
Default is \code{0.3}.}

\item{ncol, nrow}{Layout for optional patchwork combination
(currently unused). Default: \code{ncol = 2}, \code{nrow = 1}.}

\item{heights}{Relative heights and widths for combined plots
(currently unused). Default: \code{c(1,1)}.}

\item{widths}{Relative widths for combined plots
(currently unused). Default: \code{c(2,1)}.}
}
\value{
A \code{ggplot}/\code{patchwork} object showing a heatmap of
scaled network centrality scores per exposure, annotated by category.
}
\description{
Visualizes the impact of exposures on network centrality measures of
associated features (e.g., genes or latent factors) as a heatmap.
Each exposure is scored by four centrality metrics,
scaled within metric, and grouped by exposure category.
}
\details{
This function uses the output of \code{run_exposure_impact()} to
calculate and visualize the mean centrality
values for each exposure across its associated features.
The following network centrality metrics are shown:
\itemize{
\item Degree centrality
\item Eigenvector centrality
\item Closeness centrality
\item Betweenness centrality
}
All values are scaled within metric across exposures.
A side bar indicates the category of each exposure.
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# perform correlation analyses
# correlate with exposures
mae <- mae |>
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    ) |>
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        feature_cors = TRUE,
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# create the networks
mae <- mae |>
    run_create_network(
        feature_type = "omics_feature_cor",
        action = "add"
    ) |>
    run_create_network(
        feature_type = "omics",
        action = "add"
    )

# perform impact analysis
mae <- mae |>
    run_exposure_impact(
        feature_type = "omics"
    )

# create the exposure impact plot
exposure_impact_p <- mae |>
    plot_exposure_impact(
        feature_type = "omics"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_exposure_impact.R
\name{run_exposure_impact}
\alias{run_exposure_impact}
\title{Calculate Exposure Impact from Feature-Exposure Correlation Networks}
\usage{
run_exposure_impact(
  exposomicset,
  feature_type = c("degs", "omics", "factor_features"),
  pval_col = "adj.P.Val",
  pval_thresh = 0.1,
  action = c("add", "get")
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object with correlation
and network metadata.}

\item{feature_type}{One of \code{"degs"}, \code{"omics"}, or \code{"factor_features"}.}

\item{pval_col}{Column in differential abundance results to filter DEGs.
Default = \code{"adj.P.Val"}.}

\item{pval_thresh}{DEG p-value threshold. Ignored unless
\code{feature_type == "degs"}.}

\item{action}{Either \code{"add"} (store in metadata) or \code{"get"} (return list).}
}
\value{
Either an updated MultiAssayExperiment (if action = "add") or a list.
}
\description{
Generalized centrality-based exposure impact analysis using DEG, omics,
or factor features.
}
\examples{
# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# perform correlation analyses
# correlate with exposures
mae <- mae |>
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    ) |>
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        feature_cors = TRUE,
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# create the networks
mae <- mae |>
    run_create_network(
        feature_type = "omics_feature_cor",
        action = "add"
    ) |>
    run_create_network(
        feature_type = "omics",
        action = "add"
    )

# perform impact analysis
mae <- mae |>
    run_exposure_impact(
        feature_type = "omics"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_correlation_summary.R
\name{plot_correlation_summary}
\alias{plot_correlation_summary}
\title{Plot Correlation Summary from Exposure-Feature Correlations}
\usage{
plot_correlation_summary(
  exposomicset,
  feature_type = c("degs", "omics", "factors", "factor_features", "exposures", "pcs"),
  mode = c("top_exposures", "top_features", "exposure_category", "assay", "summary"),
  top_n = 15
)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object with
correlation results in metadata.}

\item{feature_type}{One of \code{"degs"}, \code{"factors"},
\code{"omics"}, or \code{"exposures"}.}

\item{mode}{One of:
\itemize{
\item \code{"top_exposures"}: Top exposures by assay
\item \code{"top_features"}: Top features by exposure category
\item \code{"exposure_category"}: Total associations by exposure category
\item \code{"assay"}: Total associations by omics assay
\item \code{"summary"}: Patchwork layout combining all
}}

\item{top_n}{Number of top exposures or features to display
(for top modes). Default is \code{15}.}
}
\value{
A \code{ggplot2} object or a \code{patchwork} object (if \code{mode = "summary"}).
}
\description{
Generates a bar plot summary of exposure-feature correlations
using customizable modes.
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# correlate with exposures
mae <- mae |>
    run_correlation(
        feature_type = "omics",
        variable_map = mae |>
            pivot_feature() |>
            dplyr::select(
                variable = .feature,
                exp_name = .exp_name
            ),
        exposure_cols = c("exposure_pm25", "exposure_no2", "age", "bmi")
    )

# create the correlation summary plot
cor_summary_plot <- mae |>
    plot_correlation_summary(
        feature_type = "omics",
        mode = "summary"
    )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pivot_sample.R
\name{pivot_sample}
\alias{pivot_sample}
\title{Extract Sample Metadata from MultiAssayExperiment or SummarizedExperiment}
\usage{
pivot_sample(x, ...)
}
\arguments{
\item{x}{A \code{MultiAssayExperiment} or \code{SummarizedExperiment} object.}

\item{...}{Additional arguments passed to \code{pivot_sample()} from \code{tidybulk}
for \code{SummarizedExperiment} objects.}
}
\value{
A tibble containing sample metadata with an added \code{.sample} column.
}
\description{
Extracts and formats sample-level metadata (\code{colData}) from a
\code{MultiAssayExperiment} or \code{SummarizedExperiment} object.
}
\details{
This function:
\itemize{
\item Extracts \strong{sample metadata} from \code{MultiAssayExperiment} using \code{colData()},
converting it to a tibble.
\item Calls \code{pivot_sample()} from \code{tidybulk} when applied to a
\code{SummarizedExperiment} object.
\item \strong{Error Handling}: Returns an error if \code{x} is not a
\code{MultiAssayExperiment} or \code{SummarizedExperiment}.
}
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

sample_data <- mae |>
    pivot_sample()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter_sample_outliers.R
\name{filter_sample_outliers}
\alias{filter_sample_outliers}
\title{Filter Sample Outliers}
\usage{
filter_sample_outliers(exposomicset, outliers = NULL)
}
\arguments{
\item{exposomicset}{A \code{MultiAssayExperiment} object containing
omics and exposure data.}

\item{outliers}{An optional character vector specifying
sample names to be removed.
If \code{NULL}, the function uses outliers identified in
\code{metadata(exposomicset)$pca$outliers}. Default is \code{NULL}.}
}
\value{
A \code{MultiAssayExperiment} object with the specified outliers removed.
}
\description{
Removes sample outliers from a \code{MultiAssayExperiment} object
based on PCA analysis.
}
\details{
The function checks for the presence of PCA results in
\code{metadata(exposomicset)}. If \code{outliers} is not provided,
it retrieves precomputed outliers from \code{metadata(exposomicset)$pca$outliers}.
The identified samples are removed
from the dataset.
}
\examples{

# create example data
mae <- make_example_data(
    n_samples = 10,
    return_mae = TRUE
)

# run PCA
mae <- mae |>
    run_pca()

# filter outliers if present
mae <- mae |>
    filter_sample_outliers()

}
